/*
 * $Id: LCD.h 251 2010-04-11 09:05:06Z gingko $
 * Copyright (C) 2007 lolo_32 <pouchintv-dev@baysse.fr>
 *
 * This file is part of Pouchin TV Mod, a free DVB-T viewer.
 * See http://www.pouchintv.fr/ for updates.
 *
 * Pouchin TV Mod is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Pouchin TV Mod is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 *
 *
 * Most files of this project have been changed from the Pouchin TV
 * project (Copyright (C) 2006 Pouchin <pouchinteve@free.fr>),
 * to use with a modified version of this software.
 * See http://pouchinteve.free.fr/ for the original project.
 */

/** \file
 *	\brief Dfinitions pour affichage sur clavier Logitech
 *
 *	Ce fichier inclut les dfinitions ncessaires pour pouvoir utiliser l'affichage
 *	existant sur les crans LCD des claviers de type Logitech G15 et drivs qui
 *	incluent un affichage LCD.
 *
 *	Cet affichage est utilis pour rpliquer les informations dj affiches sur la
 *	fentre, dans l'OSD et dans la zone de notification.
 *	\see http://en.wikipedia.org/wiki/Logitech_G15
 *	\see http://www.logitech.com/index.cfm/keyboards/keyboard/devices/3498
 **/

#pragma once

#include "LCD_Logitech/lglcd.h"

/**
 * Classe permettant de grer, et modifier l'affichage sur les crans LCD
 **/
class CLcd {
protected:
	/// 0 => pas de LCD trouv ou erreur activation
	/// 1 => LCD trouv et activ
	bool m_initialise;

	/// Contexte de la connexion du LCD
	/// C'est  partir de lui que se fait la config de l'affichage
	lgLcdConnectContext	m_connectContext;
	/// Contexte de l'ouverture du LCD
	lgLcdOpenContext	m_openContext;
	/// Descriptifs de l'cran
	lgLcdDeviceDesc		m_lcdDeviceDesc;

	/// Handle pour dessiner le bitmap
	HDC					m_hdc;
	/// Police  utiliser
	HFONT				m_police, m_police_heure;
	/// Image  afficher
	HBITMAP				m_bitmap;

	/// Nom de la chane actuelle
	char				m_chaine[256];

	/// Emission actuelle et suivante
	ActAndNxtProg		m_act_next;

	/// Volume
	long				m_volume;
	/// Mode "muet"
	bool				m_mute;

	/** Initialise le Lcd pour utilisation
	 * \retval  true   en cas de succs
	 * \retval  false  en cas d'erreur
	 **/
	bool InitialiseLcd();
	/// Met  jour les textes : Chane, Volume, programmes
	void UpdateTexte();

	/** Dfinit le nom de la chane actuelle
	 * \param[in]  nomChaine  Nom de la chane
	 **/
	void SetChaine(const char *nomChaine);

public:

	enum {
		DEFAULT_POINTSIZE  = 8,
		DEFAULT_DPI	       = 96,
	};

	CLcd();
	~CLcd();

	/** Raffiche l'cran
	 * \retval  ERROR_SUCCESS  en cas de succs
	 * \return  toute autre erreur en cas d'chec
	 **/
	DWORD ActualiseLcd();

	/** Dfinit les informations sur la chane actuelle
	 * \param[in]  canal  Canal contenant toutes les informations
	 **/
	void SetCanal(const Chaine &canal);

	/// Dfinit le volume  afficher
	void SetVolume(const long &volume) {
		this->m_volume = volume; ActualiseLcd();}
	/// On est en mode "muet"
	void Mute(const bool &muted) { this->m_mute = muted; ActualiseLcd(); }

	/// Indique si le LCD est initialis
	bool IsInitialise() {
		return m_initialise; }
};
