/*
 * $Id: chanutils.cpp 260 2010-08-04 20:42:33Z gingko $
 * Copyright (C) 2008 gingko - http://gingko.homeip.net/
 *
 * This file is part of Pouchin TV Mod, a free DVB-T viewer.
 * See http://www.pouchintv.fr/ for updates.
 *
 * Pouchin TV Mod is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Pouchin TV Mod is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 *
 *
 * Most files of this project have been changed from the Pouchin TV
 * project (Copyright (C) 2006 Pouchin <pouchinteve@free.fr>),
 * to use with a modified version of this software.
 * See http://pouchinteve.free.fr/ for the original project.
 */

/** \file
 *	\brief Module de gestion des pistes associes  une chane.
 *
 *	Ce fichier implmente des fonctionnalits utilitaires lies  l'identification des flux
 *	TS utiliss par les chanes TV.
 *
 *	Ces lments sont utiliss, d'une part pour la gestion des structures de sauvegarde des
 *	informations lies  ces chanes, et d'autre part pour la slection des pistes dans les
 *	enregistrements programms.
 **/

#include "stdafx.h"
#include "chanutils.h"

/// Comparaison pour galit
bool liste_pistes::operator == (const liste_pistes & lp) const
{
	if (nbr!=lp.nbr || nbr_tot!=lp.nbr_tot)
		return false;
	for (int i=0; i<nbr_tot; i++) {
		if (tbl[i]!=lp.tbl[i])
			return false;
	}
	return true;
}

/**
 * Ajout d'une piste  la liste, avec dtermination de la cattorie ("son" ou "autre")
 * selon le type de la piste
 * \param[in]	piste	Piste  ajouter
 * \return		Index de la piste qui vient d'tre ajout
 * \note	L'insertion d'une piste "son" va dcaler les index des pistes "autre" qui
 *			pourraient exister pralablement
 **/
int liste_pistes::ajoute(const une_piste & piste)
{
	if (nbr_tot<_countof(tbl)) {
		if (piste.type >= tst_Other_BEG) {
			tbl[nbr_tot] = piste;
			return nbr_tot++;
		} else {
			for (int nInx = nbr_tot; nInx > nbr; --nInx)
				tbl[nInx] = tbl[nInx-1];
			++nbr_tot;
			tbl[nbr] = piste;
			return nbr++;
		}
	}
	return -1;
}

/**
 * Recherche de la Nme piste audio correspondant  un critre donn
 * \param[in]	pfn		Pointeur sur une fonction membre de test de critre
 * \param[in]	nCount	Valeur de "N", numro de piste recherch
 * \return		Index de la piste trouve, ou -1 si aucune
 **/
int liste_pistes::cherche_piste(bool (une_piste::*pfn)() const, int nCount) const
{
	int	nbr_trouve = 0;		// Comptage des pistes trouves

	for (int i=0; i < nbr; i++) {
		if (CALL_MEMBER_FN(tbl[i], pfn)() && ++nbr_trouve >= nCount)
			return i;
	}
	return -1;
}

/**
 * Trouver la piste audio la mieux adapte au choix de l'utilisateur
 * \param[in]	eAudio	Code de prfrence de l'utilisateur
 * \return		Index de la piste audio slectionne, ou bien -1 en cas d'chec
 **/
int liste_pistes::selection_son(AudioMode eAudio) const
{
	int	nRes = -1;

	switch(eAudio) {

	case am_Mpeg_1:		// On retient la 1re piste MPEG2 trouve
		nRes = cherche_piste(&une_piste::isMPEG2);
		break;

	case am_Mpeg_2:		// On retient la 2nde piste MPEG2 trouve
		nRes = cherche_piste(&une_piste::isMPEG2, 2);
		break;

	case am_AC3:		// On retient la 1re piste AC3 quelle qu'elle soit
		nRes = cherche_piste(&une_piste::isAC3);
		break;

	case am_AC3_non_HC:		// On cherche d'abord la 1re piste AC3 non HC, et si rien trouv, on retient la 1re piste AC3
		nRes = cherche_piste(&une_piste::isAC3_non_HC);
		if (nRes < 0)
			nRes = cherche_piste(&une_piste::isAC3);
		break;

	case am_AC3_HC:		// On cherche d'abord la 1re piste AC3 HC, et si rien trouv, on retient la 1re piste AC3
		nRes = cherche_piste(&une_piste::isAC3_HC);
		if (nRes < 0)
			nRes = cherche_piste(&une_piste::isAC3);
		break;

	case am_VO:			// am_VO : on retient la 1re piste non francophone disponible
		nRes = cherche_piste(&une_piste::isVO);
	}

	if (nRes >= 0)
		return nRes;
	// Rien trouv : on retient la 1re piste audio, quelle qu'elle soit
	// (dans l'espoir que a marchera quand mme, sait-on jamais ...)
	return 0;
}

/// Comparaison pour galit
bool une_piste::operator == (const une_piste & up) const
{
	if (pid!=up.pid || type!=up.type)
		return false;
	return strcmp(lang, up.lang) == 0;
}

/**
 * Obtenir une chane reprsentant la dfinition de la fiche, pour affichage ou journalisation
 * \param[in,out]	pszBuf	Pointeur sur une chane recevant le rsultat
 * \param[in]		nBufSiz	Longueur de cette chane
 **/
void une_piste::getAsLogString(LPTSTR pszBuf, UINT32 nBufSiz) const
{
	_stprintf_s(pszBuf, nBufSiz, TEXT("pid=%4u, type="), pid);

	if (type<tst_Audio_END)
		strcat_T(pszBuf, nBufSiz, EnumToString(type, "Mpeg2\0AC3\0E-AC3\0"));
	else if (type>=tst_Other_BEG && type<=tst_Other_END)
		strcat_T(pszBuf, nBufSiz, EnumToString(type-tst_Other_BEG, "\0\0"));
	else
		strcat_T(pszBuf, nBufSiz, TEXT(""));

	if (*lang) {
		strcat_T(pszBuf, nBufSiz, TEXT(", lang="));
		strcat_T(pszBuf, nBufSiz, lang);
	}
}

/// Constructeur par dfaut
ChainePMT::ChainePMT() :
	pmt_pid(0),
	video_pid(0),
	video_type(vst_Unknown),
	pcr_pid(0)
{
}

/// Comparaison pour galit
bool ChainePMT::operator == (const ChainePMT & cpmt) const
{
	if (video_pid!=cpmt.video_pid || video_type!=cpmt.video_type || pcr_pid!=cpmt.pcr_pid)
		return false;
	return liste_pistes::operator == (cpmt);
}

/**
 * Rechercher si un "pid" donn fait partie de la dfinition des flux TS d'une chane
 * \param[in]	pid	Numro de "pid" recherch
 * \return		\p true si le "pid" est dfini dans la chane
 **/
bool ChainePMT::cherche_pid(UINT16 pid) const
{
	if (pid == pmt_pid || pid == video_pid)
		return true;
	for (int i=0; i<nbr_tot; i++) {
		if (pid == tbl[i].pid)
			return true;
	}
	return false;
}

/// Obtenir une chane de caractres reprsentant le codage vido utilis
/// pour la chane, pour affichage
LPCTSTR ChainePMT::norme() const
{
	switch (video_type) {
	case vst_MPEG2:
		return TEXT("Mpeg2");
	case vst_H264:
		return TEXT("H264");
	case vst_MPEG2_encrypted:
	case vst_H264_encrypted:
		return TEXT("");
	}
	if (nbr > 0)
		return TEXT("");
	return TEXT("");
}

/// Fonction retournant \p true si les identifiants \p sCIds dsignent une chane identique
///  ceux correspondants  la chane \p this.
/// Toute valeur nulle (sauf SID) vaut identit dans la comparaison.
bool ChaineIDs::EquIds(const ChaineIDs &sCIds) const {
	return
		(SID==sCIds.SID) &&
		(TSID==0 || sCIds.TSID==0 || TSID==sCIds.TSID) &&
		(canal_no==0 || sCIds.canal_no==0 || canal_no==sCIds.canal_no) &&
		(ONID==0 || sCIds.ONID==0 || ONID==sCIds.ONID);
}

/// Gnration d'une cl sur 32 bits visant  identifier la chane d'une faon (qu'on
/// espre raisonnablement) unique
UINT32 ChaineIDs::cle() const
{
	// (lire note concernant l'alignement des champs de l'enregistrement)
	UINT32 dwCle = crc32(LPBYTE(this), sizeof(*this));

	// Certaines valeurs sont interdites :
	if (dwCle==0 || dwCle==STREAM_PSEUDO_CLE)
		dwCle ^= SID << 9;

	return dwCle;
}
