/*
 * $Id$
 * Copyright (C) 2011 gingko - http://gingko.homeip.net/
 *
 * This file is part of Pouchin TV Mod, a free DVB-T viewer.
 * See http://www.pouchintv.fr/ for updates.
 *
 * Pouchin TV Mod is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Pouchin TV Mod is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */


// ====================================================================================

#pragma once

// ====================================================================================

class CCmdLine
{
protected:

	/// Mthode virtuelle appele par \p ProcessAll
	virtual bool	ProcessArg(tstring & strArg) = 0;

	virtual void	PostProcess() {
						}

public:
	tstring						strCommand;	//!< Commande invoque (argv[0])
	vector<tstring>				vCmdArgs;	//!< Liste des paramtres suivants
	vector<tstring>::iterator	itArgs;
	vector<tstring>::iterator	itEnd;
	bool						bParamError;

	/// (R)initialisation du traitement des paramtres chargs
	void			Reset() {
						itArgs = vCmdArgs.begin(); itEnd = vCmdArgs.end(); }

	bool			Next(tstring & strArg);
	tstring			Next();

	/// Chargement avec les paramtres de la fonction 'wmain'
	CCmdLine &		Load(int argc, LPWSTR argv[]);

	/// Chargement avec les paramtres de la fonction 'main'
	CCmdLine &		Load(int argc, LPSTR argv[]);

	/// Chargement avec une ligne de commande WCHAR brute
	CCmdLine &		Load(LPCWSTR pszCmdLine);

	/// Chargement avec une ligne de commande CHAR brute
	CCmdLine &		Load(LPCSTR pszCmdLine);

	/**
	 * \brief Traitement de tous les paramtres chargs
	 *
	 * Appel de la mthode virtuelle pure \p ProcessArg pour chaque paramtre, un par un.
	 * Tient compte de la possibilit que cette fonction appelle \p Next en interne pour
	 * acqurir des paramtres en tant qu'argument : dans ce cas, ces paramtres seront sauts.
	 **/
	bool			ProcessAll();

	/// Constructeur par dfaut
	CCmdLine() :
		bParamError(false)
	{
	}

	/// Constructeur avec les paramtres de la fonction 'wmain'
	CCmdLine(int argc, LPWSTR argv[]) :
		bParamError(false)
	{
		Load(argc, argv);
	}

	/// Constructeur avec les paramtres de la fonction 'main'
	CCmdLine(int argc, LPSTR argv[]) :
		bParamError(false)
	{
		Load(argc, argv);
	}

	/// Constructeur avec une ligne de commande WCHAR brute
	CCmdLine(LPCWSTR pszCmdLine) :
		bParamError(false)
	{
		Load(pszCmdLine);
	}

	/// Constructeur avec une ligne de commande CHAR brute
	CCmdLine(LPCSTR pszCmdLine) :
		bParamError(false)
	{
		Load(pszCmdLine);
	}
};

// ====================================================================================
