/*
 * $Id: dlgutils.cpp 276 2011-03-06 10:17:01Z gingko $
 * Copyright (C) 2010 gingko - http://gingko.homeip.net/
 *
 * This file is part of Pouchin TV Mod, a free DVB-T viewer.
 * See http://www.pouchintv.fr/ for updates.
 *
 * Pouchin TV Mod is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Pouchin TV Mod is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 *
 *
 * Most files of this project have been changed from the Pouchin TV
 * project (Copyright (C) 2006 Pouchin <pouchinteve@free.fr>),
 * to use with a modified version of this software.
 * See http://pouchinteve.free.fr/ for the original project.
 */

/** \file
 *	\brief Fonctions utilitaires pour botes de dialogue (implmentation)
 *
 *	Ce fichier implmente des fonctionnalits utilitaires d'assistance  la cration
 *	ainsi qu' la gestion des botes de dialogue.
 **/

#include "stdafx.h"
#include "dlgutils.h"
#include <shellapi.h>

// ====================================================================================

// Probablement daj dfini dans un en-tte de l'application, mais le redfinir ici
// simplifie l'ventuelle rutilisation de ce module dans un projet driv.
extern HINSTANCE	hAppInstance;		//!< Handle d'instance de l'application

// ====================================================================================
// Fonctions utilisables dans toutes les boites de dialogue
// ====================================================================================

/**
 * Recherche, parmi une suite de boutons radio, lequel est slectionn
 *
 * \param[in]  hDlg		handle de la boite de dialogue contenant l'lment  cocher
 * \param[in]  nFirstID	identifiant du premier bouton  tester
 * \param[in]  nLastID	identifiant du dernier bouton  tester
 * \param[in]  nDefault	valeur (rang) par dfaut, si aucun n'est trouv coch
 * \param[in]  nBase	base de numrotation (1er item) de la valeur retourne
 * \retval     rang du bouton coch, base \p nBase
 **/
int get_radio_check(HWND hDlg, int nFirstID, int nLastID, int nDefault, int nBase)
{
	for (int nID = nFirstID; nID <= nLastID; ++nID) {
		if (IsDlgButtonChecked(hDlg, nID) == BST_CHECKED)
			return nID-nFirstID+nBase;
	}
	return nDefault;
}

/**
 * Extraction d'une valeur "int" d'un item texte de dialogue, si son contenu est valide
 *
 * \param[in]  hDlg		handle de la boite de dialogue contenant l'lment  cocher
 * \param[in]  nID		identifiant de l'lment
 * \param[out] val		valeur extraite
 * \param[in]  bSigned  \p TRUE si la valeur  extraire doit tre signe
 * \return     \p true ou \p false selon que la valeur tait valide ou pas
 **/
bool get_int(HWND hDlg, int nID, int & val, BOOL bSigned)
{
	BOOL bValid = FALSE;

	// La valeur suivante n'est pas prise en compte dans le rsultat de la fonction
	int res = (int)GetDlgItemInt(hDlg, nID, &bValid, bSigned);
	if (bValid)
		val = res;
	return bValid==TRUE;
}

/**
 * \brief Initialisation d'un contrle combo box
 * \param[in]	hCtl		Handle du contrle  initialiser
 * \param[in]	pszInitList	Pointeur sur un tableau de chanes d'initialisation
 *							(termin par un pointeur \p NULL)
 * \param[in]	nSel		Item initialement slectionn (si >= 0)
 **/
void ComboBox_Init(HWND hCtl, const LPCTSTR * pszInitList, int nSel)
{
	if (pszInitList) {
		while (*pszInitList)
			SendMessage(hCtl, CB_ADDSTRING, 0, (LPARAM)*pszInitList++);
	}
	if (nSel >= 0)
		SendMessage(hCtl, CB_SETCURSEL, nSel, 0);
}

/**
 * Change le style de la liste
 *
 * \param[in]  hwndLV     handle de la liste  modifier
 * \param[in]  dwExStyle  style  appliquer
 **/
inline void ListView_SetExStyle(HWND hwndLV, DWORD dwExStyle)
{
	ListView_SetExtendedListViewStyleEx(hwndLV, dwExStyle, dwExStyle);
}

/**
 * Affiche les bordures horizontales et verticales  la liste passe, et rajoute les colonnes
 *
 * \param[in]  hList     handle de la liste traite
 * \param[in]  t_cols    tableau dcrivant les colonnes
 **/
void ListView_AjouteColonnes(HWND hList, const Tab_cols *t_cols)
{
	int i = 0;
	ListView_SetExStyle(hList, LVS_EX_GRIDLINES | LVS_EX_FULLROWSELECT | LVS_EX_HEADERDRAGDROP);

	// Ajoute les diffrentes colonnes
	while( t_cols[i].nom != NULL || t_cols[i].taille != 0) {
		const Tab_cols & col = t_cols[i];
		LVCOLUMN colonne = {
			LVCF_TEXT|LVCF_WIDTH,		// mask
			0,							// fmt
			col.taille,					// cx
			const_cast<LPTSTR>(col.nom)	// pszText
			// le reste  zro (implicite)
		};

		ListView_InsertColumn(hList, i, &colonne);
		i++;
	}
}

/**
 * Ajoute un lment dans la liste, et fait dfiler la fentre vers cet lment
 *
 * \param[in]	hList	Handle de la liste devant recevoir l'lment
 * \param[in]	str		Nom de l'lment  rajouter
 * \retval		\p LB_ERR en cas d'erreur
 **/
LRESULT AddLineToList(HWND hList, LPCTSTR str) {
	return
		SendMessage(hList, LB_SETTOPINDEX,
			(WPARAM)SendMessage(hList, LB_ADDSTRING, 0, (LPARAM)str),
			0);
}

/**
 * Recherche de l'item et du sous-item se trouvant  une position donne
 * \param[in]	hList		Handle de la liste dans laquelle rechercher
 * \param[in]	sPt			Position de l'item recherch
 * \param[in]	bRelative	\p true si la position donne est relative  la liste
 * \param[out]	iItem		Numro (base 0) de l'item de liste recherch
 * \param[out]	iSubItem	Numro (base 0) du sous-item de liste recherch
 * \return		\p true si succs, \p false si erreur
 **/
bool FindListViewSubItem(HWND hList, POINT sPt, bool bRelative, INT & iItem, INT & iSubItem)
{
	if (!bRelative)
		ScreenToClient(hList, &sPt);

	LVHITTESTINFO sHti = {
		sPt		// pt
		// Le reste  zro (implicite)
	};

	// On ne fait rien si on n'est pas dans la seconde/troisime colonne
	if (ListView_SubItemHitTest(hList, &sHti) >= 0) {
		iItem		= sHti.iItem;
		iSubItem	= sHti.iSubItem;
		return true;
	}
	return false;
}

/**
 * Ajout d'un item  une combo box, en lui associant une valeur binaire
 * \param[in]	hCtl	Handle de la combo box concerne
 * \param[in]	pszTxt	Texte de l'item ajout
 * \param[in]	dwData	Donne  lui associer
 * \return		index de l'item ajout, ou bien CB_ERR si erreur
 **/
LRESULT AddComboItemWithData(HWND hCtl, LPCTSTR pszTxt, DWORD dwData)
{
	LRESULT nRes = SendMessage(hCtl, CB_ADDSTRING, 0, (LPARAM)pszTxt);

	if (nRes >= 0) {
		// Utilisation du SID de la chane pour identifier la slection.
		if (SendMessage(hCtl, CB_SETITEMDATA, nRes, (LPARAM)dwData) == CB_ERR)
			return CB_ERR;
	}
	return nRes;
}

// ====================================================================================
// Implmentation des mthodes de CDialogBase
// ====================================================================================

/// Fonction "DialogProc" statique commune  tous les dialogues
INT_PTR CALLBACK CDialogBase::__DialogProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	// Obtenir le pointeur sur la classe de gestion, en principe pralablement charg
	// avec le message \p WM_INITDIALOG :
	CDialogBase * pDlg = reinterpret_cast<CDialogBase *>(GetWindowLongPtr(hDlg, DWLP_USER));

	if (pDlg) {
		// Prendre cette valeur avant d'excuter DialogProc : l'appel pouvant tre rcursif, il
		// se peut que la destruction survienne  un niveau de rcursion plus profond.
		bool	bDeleteNow = pDlg->mustDeleteNow(uMsg);

		// Excution de la fonction \p DialogProc de la classe
		INT_PTR nRes = pDlg->DialogProc(uMsg, wParam, lParam);

		// Destruction des donnes du dialogue si requis
		if (bDeleteNow) {
			// myprintf(TEXT("CDialogBase - delete 0x%08x\n"), pDlg->hDlg);
			delete pDlg;
		}
		return nRes;
	}
	return 0;
}

/// Fonction "DialogProc" statique d'initialisation commune  tous les dialogues
INT_PTR CALLBACK CDialogBase::__DialogInitProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	// Tant qu'on n'a pas reu le message WM_INITDIALOG, on ne peut rien traiter
	if (uMsg != WM_INITDIALOG)
		return 0;

	if (lParam == NULL)
		return 0;	// lParam ne devrait pas tre NULL, cependant

	// Le message \p WM_INITDIALOG contient en paramtre le pointeur sur la classe de
	// gestion de la bote de dialogue.
	SetWindowLongPtr(hDlg, DWLP_USER, LONG_PTR(lParam));

	// Mmoriser la "handle" de la bote de dialogue
	reinterpret_cast<CDialogBase *>(lParam)->hDlg = hDlg;

	// Le pointeur sur les donnes du dialogue ayant t charg, on peut maintenant
	// remplacer la fonction par celle qui va y accder :
	if (!SetWindowLongPtr(hDlg, DWLP_DLGPROC, LONG_PTR(__DialogProc)))
		return 0;

	return __DialogProc(hDlg, uMsg, wParam, lParam);
}

/**
 * \brief Constructeur
 * \note Aussi longtemps que cette classe pourra tre utilise virtuellement, il sera
 * considrablement plus commode de faire en sorte qu'elle n'ait qu'un constructeur par
 * dfaut (donc sans paramtres)
 **/
CDialogBase::CDialogBase() :
	hDlg(NULL)
{
	ShowCursor(TRUE);		// Suspend la disparition temporise du curseur
}

/**
 * \brief Limitation de la valeur entre dans un item de dialogue numrique ditable
 *
 * \param[in]  nID    identifiant de l'lment
 * \param[in]  nMin   valeur minimale
 * \param[in]  nMax   valeur maximale
 * \return	\p true si le contenu a fait l'objet d'un ajustement
 **/
bool CDialogBase::LimitIntItemValue(int nID, int nMin, int nMax)
{
	BOOL	valid = false;
	int		res = (int)GetDlgItemInt(hDlg, nID, &valid, TRUE);

	if (valid) {
		if (res < nMin) {
			res = nMin;
			valid = FALSE;
		} else if (res > nMax) {
			res = nMax;
			valid = FALSE;
		}
	}
	if (!valid) {
		SetDlgItemInt(hDlg, nID, (UINT)res, TRUE);
		return true;
	}
	return false;
}

/**
 * \brief Initialisation d'une paire d'items Edit + Up/Down
 *
 * \param[in]  nEditID	identifiant de l'item ditable
 * \param[in]  nUdnID	identifiant de l'item "up/down"
 * \param[in]  nVal		valeur initiale
 * \param[in]  nMin		valeur minimale
 * \param[in]  nMax		valeur maximale
 **/
void CDialogBase::InitUpDownItems(int nEditID, int nUdnID, LONG nVal, LONG nMin, LONG nMax)
{
	SendDlgItemMessage(hDlg, nUdnID, UDM_SETRANGE32, (WPARAM)nMin, (LPARAM)nMax);
	SetDlgItemInt(hDlg, nEditID, (UINT)nVal, nVal<0);
}

/**
 * \brief Vrification du contenu d'une paire d'items Edit + Up/Down
 *
 * \param[in]  nEditID	identifiant de l'item ditable
 * \param[in]  nUdnID	identifiant de l'item "up/down"
 * \return	\p true si le contenu a fait l'objet d'un ajustement
 **/
bool CDialogBase::CheckUpDownValue(int nEditID, int nUdnID)
{
	LONG nMin;
	LONG nMax;

	SendDlgItemMessage(hDlg, nUdnID, UDM_GETRANGE32, (WPARAM)&nMin, (LPARAM)&nMax);
	return LimitIntItemValue(nEditID, nMin, nMax);
}

/**
 * Dplacement d'un contrle de dialogue sans modifier sa taille,
 * en spcifiant la diffrence entre l'ancienne position et la nouvelle.
 * \param[in]	nId		Identifiant du contrle
 * \param[in]	dx		Dplacement horizontal
 * \param[in]	dy		Dplacement vertical
 **/
void CDialogBase::ChgCtlPos(INT nId, int dx, int dy)
{
	HWND	hCtl = GetDlgItem(hDlg, nId);
	RECT	sRect;

	GetWindowRect(hCtl, &sRect);

	POINT	sPt		= {sRect.left+dx, sRect.top+dy};

	ScreenToClient(hDlg, &sPt);
	SetWindowPos(hCtl, 0, sPt.x, sPt.y, 0, 0,
		SWP_NOSIZE|SWP_NOZORDER|SWP_NOOWNERZORDER);

	// Le contrle vient d'tre redessin, mais on l'invalide quand mme, car il
	// arrive que l'actualisation du contrle voisin provoque une corruption sur
	// les bords en cas de redimensionnement rapide :
	InvalidateRect(hCtl, NULL, false);
}

/**
 * Redimensionnement d'un contrle de dialogue sans modifier sa position,
 * en spcifiant la diffrence entre l'ancienne taille et la nouvelle.
 * \param[in]	nId		Identifiant du contrle
 * \param[in]	dx		Diffrence de taille horizontale
 * \param[in]	dy		Diffrence de taille verticale
 **/
void CDialogBase::ChgCtlSiz(INT nId, int dx, int dy)
{
	HWND	hCtl = GetDlgItem(hDlg, nId);
	RECT	sRect;

	GetWindowRect(hCtl, &sRect);
	SetWindowPos(hCtl, 0, 0, 0, RWdt(sRect)+dx, RHgt(sRect)+dy,
		SWP_NOMOVE|SWP_NOZORDER|SWP_NOOWNERZORDER);
}

/// Destructeur
CDialogBase::~CDialogBase()
{
	SetWindowLongPtr(hDlg, DWLP_USER, LONG_PTR(NULL));
	ShowCursor(FALSE);	// Rtablit la disparition temporise du curseur
}

// ====================================================================================
// Implmentation des mthodes de CModelessDialogBase
// ====================================================================================

HWND CModelessDialogBase::CreateDlg(HWND hParentWnd, LPCTSTR lpTemplateName)
{
	return CreateDialogParam(hAppInstance, lpTemplateName, hParentWnd,
			__DialogInitProc, reinterpret_cast<LPARAM>(dynamic_cast<CDialogBase *>(this)));
}

HWND CModelessDialogBase::CreateDlg(HWND hParentWnd, LPCDLGTEMPLATE lpTemplate)
{
	return CreateDialogIndirectParam(hAppInstance, lpTemplate, hParentWnd,
			__DialogInitProc, reinterpret_cast<LPARAM>(dynamic_cast<CDialogBase *>(this)));
}

/// Liste des dialogues modeless couramment ouverts
vector<HWND> CModelessDialogBase::vOpenDialogs;

bool CModelessDialogBase::__ShowAndAdd(HWND hNewDlg)
{
	if (hNewDlg) {
		hDlg = hNewDlg;
		ShowWindow(hDlg, SW_SHOW);

		// Ajouter la fentre  la liste des dialogues modeless ouverts
		vOpenDialogs.push_back(hDlg);
		return true;
	}
	return false;
}

/// Destructeur
CModelessDialogBase::~CModelessDialogBase()
{
	// Suppression de la handle du dialogue de la liste
	vector<HWND>::iterator	itBeg = vOpenDialogs.begin();
	vector<HWND>::iterator	itEnd = vOpenDialogs.end();
	vector<HWND>::iterator	itDel = find(itBeg, itEnd, hDlg);

	if (itDel != itEnd)
		vOpenDialogs.erase(itDel);
}

// ====================================================================================

/**
 * \brief Traitement des vnements lis  n'importe quel dialogue modeless ouvert
 *
 *  placer dans la boucle de gestion gnrale des messages.
 * \param[in,out]	msg		Message  traiter
 * \retval			true	si le message \p msg a t trait, auquel cas il ne faut pas le
 *							passer  \p TranslateMessage et \p DispatchMessage.
 * \retval			false	dans le cas contraire
 **/
bool IsModelessDialogMessage(MSG & msg)
{
	for (ITERATE_VECTOR(CModelessDialogBase::vOpenDialogs, HWND, it)) {
		if (::IsDialogMessage(*it, &msg) == TRUE)
			return true;
	}
	return false;
}

// ====================================================================================
// Implmentation des mthodes de CPropSheetPageBase
// ====================================================================================

/// Raison de la dernire notification \p PSN_KILLACTIVE
CPropSheetPageBase::PSN_KILLACTIVE_discriminator CPropSheetPageBase::ePSN_KILLACTIVE_discriminator = epkd_undef;

/// Fonction "DialogProc" statique d'initialisation commune des feuilles de proprits
INT_PTR CALLBACK CPropSheetPageBase::__PspInitProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	// Tant qu'on n'a pas reu le message WM_INITDIALOG, on ne peut rien traiter
	if (uMsg != WM_INITDIALOG)
		return 0;

	const PROPSHEETPAGE &	cPSP	= (*reinterpret_cast<PROPSHEETPAGE *>(lParam));
	NewPspProc				newProc	= reinterpret_cast<NewPspProc>(cPSP.lParam);
	CDialogBase *			pPsp	= newProc(cPSP);

	return __DialogInitProc(hDlg, uMsg, wParam, (LPARAM)pPsp);
}

/**
 * \brief Construction d'un descripteur de page de proprits
 * \param[in]	nResId		Identifiant (numrique) de la ressource utilise pour le dialogue
 * \param[in]	pszTitle	Titre de la page de proprits
 * \param[in]	newProc		Pointeur sur la fonction de cration de l'objet contenant les donnes
 **/
PROPSHEETPAGE CPropSheetPageBase::MakePropSheetPage(INT nResId, LPCTSTR pszTitle, NewPspProc newProc)
{
	PROPSHEETPAGE sPsp =
	{
		sizeof(PROPSHEETPAGE),				// dwSize
		PSP_USETITLE,						// dwFlags
		hAppInstance,						// hInstance
		{MAKEINTRESOURCE(nResId)},			// pszTemplate (union)
		{NULL},								// pszIcon (union)
		pszTitle,							// pszTitle
		__PspInitProc,						// pfnDlgProc
		reinterpret_cast<LPARAM>(newProc),	// lParam
		NULL								// pfnCallback
	};

	return sPsp;
}

/**
 * Assistance au retour de valeur aprs notification dans une feuille de proprits
 *
 * \param[in]  res    valeur devant tre retourne
 * \retval     true
 **/
BOOL CPropSheetPageBase::psn_result(LONG res)
{
	SetWindowLongPtr(hDlg, DWLP_MSGRESULT, res);
	return TRUE;
}

// ====================================================================================
// Implmentation des mthodes de CPropSheetBase
// ====================================================================================

/// Nom de la proprit de fentre  utiliser
static const LPCTSTR PROP_PROPSHEET_DATA = TEXT("_Propsheet_This_Ptr_");

/// Fonction "callback" statique de substitution  celle du dialogue de feuilles de proprits
LRESULT CALLBACK CPropSheetBase::__SubDlgProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	CPropSheetBase * pCl = reinterpret_cast<CPropSheetBase *>(GetProp(hDlg, PROP_PROPSHEET_DATA));

	if (!pCl)
		return 0;
	return pCl->SubProc(uMsg, wParam, lParam);
}

/// Fonction "callback" statique du dialogue de feuilles de proprits
int CALLBACK CPropSheetBase::__PshProc(HWND hDlg, UINT uMsg, LPARAM lParam)
{
	if (uMsg==PSCB_INITIALIZED) {
		HWND				hParentWnd = GetParent(hDlg);
		CPropSheetBase *	pCl = reinterpret_cast<CPropSheetBase *>(GetProp(hParentWnd, PROP_PROPSHEET_DATA));

		if (pCl && (WNDPROC)GetWindowLongPtr(hDlg, DWLP_DLGPROC) != __SubDlgProc) {
			RemoveProp(hParentWnd, PROP_PROPSHEET_DATA);
			SetProp(hDlg, PROP_PROPSHEET_DATA, (HANDLE)pCl);
			pCl->hDlg = hDlg;
			pCl->pDlgProc = (WNDPROC)SetWindowLongPtr(hDlg, DWLP_DLGPROC, (LONG_PTR)__SubDlgProc);
			pCl->Init();
		}
	}
	return 0;
}

LRESULT CPropSheetBase::SubProc(UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	// Ceci relve des choses stupides  faire, juste parce que la notification
	// "PSN_KILLACTIVE" ne fournit aucun moyen de savoir si elle a t provoque par
	// un changement de page ou bien par une pression sur un bouton "OK" ou "Appliquer".

	// Une observation empirique ayant dtermin que les changements de page sont prcds d'un
	// message de type WM_NOTIFY, tandis que les pressions sur bouton le sont par un message
	// de type WM_COMMAND, on considre que le dernier des deux survenu dtermine le type
	// de notification qui va ensuite tre envoye  la page de proprits.
	switch (uMsg) {

	case WM_NOTIFY:
		CPropSheetPageBase::ePSN_KILLACTIVE_discriminator = CPropSheetPageBase::epkd_pagechange;
		break;
	case WM_COMMAND:
		CPropSheetPageBase::ePSN_KILLACTIVE_discriminator = CPropSheetPageBase::epkd_buttonpressed;
	}

	// Appel de la fonction normale de feuille de proprits
	return CallWindowProc(pDlgProc, hDlg, uMsg, wParam, lParam);
}

// Ajout d'un descripteur de page de feuille de proprits
void CPropSheetBase::AddPage(const PROPSHEETPAGE & sPsp)
{
	vPsp.push_back(sPsp);
}

/**
 * Ouverture et traitement effectifs de la feuille de proprits
 * \param[in]	hParentWnd	\p HANDLE de la fentre parente
 * \param[in]	pszCaption	Titre de la feuille de proprits
 **/
void CPropSheetBase::Do(HWND hParentWnd, LPCTSTR pszCaption)
{
	const PROPSHEETHEADER psh = {
		sizeof(PROPSHEETHEADER),		// dwSize
		PSH_PROPSHEETPAGE | PSH_NOCONTEXTHELP | PSH_PROPTITLE | PSH_USECALLBACK, // dwFlags
		hParentWnd,						// hwndParent
		hAppInstance,					// hInstance
		{NULL},							// pszIcon (union)
		pszCaption,						// pszCaption
		(UINT)vPsp.size(),				// nPages
		{0},							// nStartPage (union)
		{&vPsp[0]},						// ppsp (union)
		__PshProc						// pfnCallback
	};

	// Utilisation des proprits de fentre pour passer le pointeur "this".
	// On ne peut pas utiliser la fentre courante comme support car elle n'existe pas encore,
	// donc on utilise la fentre parente, qui doit donc exister.
	SetProp(hParentWnd, PROP_PROPSHEET_DATA, (HANDLE)this);

	// Appel du dialogue
	PropertySheet(&psh);

	// En principe dj enlev, mais par prcaution ...
	RemoveProp(hParentWnd, PROP_PROPSHEET_DATA);
}

// ====================================================================================

/**
 * Classe d'excution de \p ShellExecute dans un thread (apparemment ncessaire
 * dans les applications qui utilisent la bibliothre COM en mode multithread
 * (initialis par "CoInitializeEx(NULL, COINIT_MULTITHREADED);").
 * \see http://support.microsoft.com/?scid=kb%3Ben-us%3B287087&x=16&y=15
 **/
class CShellExThread : public WorkingThread
{
	tstring	strFile;
	tstring	strArgs;
	HWND	hParWnd;

	/// Fonction d'excution principale effective du 'thread'.
	virtual DWORD ThreadProc() {
		/*int nRes = (int)*/ShellExecute(hParWnd, TEXT("open"), strFile.c_str(), strArgs.c_str(), NULL, SW_SHOWNORMAL);

		// myprintf(CMyP ERRX(TEXT("chec ShellExecute(%s), code=%i")), nRes<=32, strFile.c_str(), nRes);
		return 0;
	}

public:
	CShellExThread(HWND	hPWnd, LPCTSTR pszFile, LPCTSTR pszArgs) :
		WorkingThread(true),	// true -> autodestruction
		strFile(pszFile),
		strArgs(pszArgs),
		hParWnd(hPWnd)
	{
		StartThread();
	}
};

/**
 * Excution d'une commande via le "shell" de Windows
 *
 * \param[in]	hParentWnd	\p HANDLE de la fentre parente
 * \param[in]	pszFile		Commande  excuter
 * \param[in]	pszArg		Argument de la commande
 **/
void ShellOpen(HWND	hParentWnd, LPCTSTR pszFile, LPCTSTR pszArg)
{
	// Cration et lancement du thread (inutile de garder le pointeur, car l'objet sera
	// autodtruit  la sortie du thread)
	new CShellExThread(hParentWnd, pszFile, pszArg);
}

// ====================================================================================
