/*
 * $Id: dlgutils.h 273 2011-02-25 11:11:29Z gingko $
 * Copyright (C) 2010 gingko - http://gingko.homeip.net/
 *
 * This file is part of Pouchin TV Mod, a free DVB-T viewer.
 * See http://www.pouchintv.fr/ for updates.
 *
 * Pouchin TV Mod is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Pouchin TV Mod is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 *
 *
 * Most files of this project have been changed from the Pouchin TV
 * project (Copyright (C) 2006 Pouchin <pouchinteve@free.fr>),
 * to use with a modified version of this software.
 * See http://pouchinteve.free.fr/ for the original project.
 */

/** \file
 *	\brief Fonctions utilitaires pour botes de dialogue (dfinition)
 *
 *	Ce fichier dfinit l'accs  des objets et fonctions utilitaires d'assistance
 *	 la cration ainsi qu' la gestion des botes de dialogue.
 **/

#pragma once

// Macros utilitaires pour amliorer la lisibilit du code :
#define _cmd_ MAKEWPARAM
#define _ncode_(lParam) reinterpret_cast<LPNMHDR>(lParam)->code
#define _nidfr_(lParam) reinterpret_cast<LPNMHDR>(lParam)->idFrom

// ====================================================================================
// Fonctions utilisables dans toutes les boites de dialogue
// ====================================================================================

/**
 * Coche la case de l'lment checkbox correspondant
 *
 * \param[in]	hDlg	handle de la boite de dialogue contenant l'lment  cocher
 * \param[in]	nID		identifiant de l'lment
 * \param[in]	state	\p true ou \p false selon qu'on veut cocher ou dcocher
 **/
inline bool set_check(HWND hDlg, int nID, bool state)
{
	return CheckDlgButton(hDlg, nID, state ? BST_CHECKED : BST_UNCHECKED)==TRUE;
}

/**
 * Dfinit le texte de l'lment correspondant
 *
 * \param[in]	hElmnt	Handle de de l'lment dont le texte doit tre modifi
 * \param[in]	texte	Texte  afficher
 **/
inline bool set_text(HWND hElmnt, LPCTSTR texte)
{
	return SendMessage(hElmnt, WM_SETTEXT, 0, (LPARAM)texte)==TRUE;
}

/**
 * Rcupre le texte de l'lment correspondant
 *
 * \param[in]  hElmnt	handle de de l'lment dont le texte doit tre modifi
 * \param[in]  texte	tampon de rception du texte
 **/
template <size_t S>
	inline int get_text(HWND hElmnt, TCHAR(&texte)[S])
{
	return (int)SendMessage(hElmnt, WM_GETTEXT, (WPARAM)S, (LPARAM)texte);
}

/**
 * Indique si la case  cocher est, justement, coche
 *
 * \param[in]  hDlg   handle de la boite de dialogue contenant l'lment  cocher
 * \param[in]  nID    identifiant de l'lment
 * \retval     true   si la case est coche
 * \retval     false  si la case est dcoche
 **/
inline bool get_check(HWND hDlg, int nID)
{
	return IsDlgButtonChecked(hDlg, nID) == BST_CHECKED;
}

/**
 * Recherche, parmi une suite de boutons radio, lequel est slectionn
 *
 * \param[in]  hDlg		handle de la boite de dialogue contenant l'lment  cocher
 * \param[in]  nFirstID	identifiant du premier bouton  tester
 * \param[in]  nLastID	identifiant du dernier bouton  tester
 * \param[in]  nDefault	valeur (rang) par dfaut, si aucun n'est trouv coch
 * \param[in]  nBase	base de numrotation (1er item) de la valeur retourne
 * \retval     rang du bouton coch, base \p nBase
 **/
int get_radio_check(HWND hDlg, int nFirstID, int nLastID, int nDefault, int nBase = 0);

/**
 * Extraction d'une valeur "int" d'un item texte de dialogue, si son contenu est valide
 *
 * \param[in]  hDlg		handle de la boite de dialogue contenant l'lment  cocher
 * \param[in]  nID		identifiant de l'lment
 * \param[out] val		valeur extraite
 * \param[in]  bSigned  \p TRUE si la valeur  extraire doit tre signe
 * \return     \p true ou \p false selon que la valeur tait valide ou pas
 **/
bool get_int(HWND hDlg, int nID, int & val, BOOL bSigned);

/**
 * \brief Initialisation d'un contrle combo box
 * \param[in]	hCtl		Handle du contrle  initialiser
 * \param[in]	pszInitList	Pointeur sur un tableau de chanes d'initialisation
 *							(termin par un pointeur \p NULL)
 * \param[in]	nSel		Item initialement slectionn (si >= 0)
 **/
void ComboBox_Init(HWND hCtl, const LPCTSTR * pszInitList, int nSel = -1);

/**
 * Structure servant  dcrire les colonnes devant tre ajoutes dans la liste
 */
struct Tab_cols {
	LPCTSTR	nom;		///< Texte affich en nom de colonne
	int		taille;		///< Largeur initiale (en pixel) de la colonne
};

/**
 * Affiche les bordures horizontales et verticales  la liste passe, et rajoute les colonnes
 *
 * \param[in]  hList     handle de la liste traite
 * \param[in]  t_cols    tableau dcrivant les colonnes
 **/
void ListView_AjouteColonnes(HWND hList, const Tab_cols *t_cols);

/**
 * Ajoute un lment dans la liste, et fait dfiler la fentre vers cet lment
 *
 * \param[in]	hList	Handle de la liste devant recevoir l'lment
 * \param[in]	str		Nom de l'lment  rajouter
 * \retval		\p LB_ERR en cas d'erreur
 **/
LRESULT AddLineToList(HWND hList, LPCTSTR str);

/**
 * Recherche de l'item et du sous-item se trouvant  une position donne
 * \param[in]	hList		Handle de la liste dans laquelle rechercher
 * \param[in]	sPt			Position de l'item recherch
 * \param[in]	bRelative	\p true si la position donne est relative  la liste
 * \param[out]	iItem		Numro (base 0) de l'item de liste recherch
 * \param[out]	iSubItem	Numro (base 0) du sous-item de liste recherch
 * \return		\p true si succs, \p false si erreur
 **/
bool FindListViewSubItem(HWND hList, POINT sPt, bool bRelative, INT & iItem, INT & iSubItem);

/**
 * Ajout d'un item  une combo box, en lui associant une valeur binaire
 * \param[in]	hCtl	Handle de la combo box concerne
 * \param[in]	pszTxt	Texte de l'item ajout
 * \param[in]	dwData	Donne  lui associer
 * \return		index de l'item ajout, ou bien CB_ERR si erreur
 **/
LRESULT AddComboItemWithData(HWND hCtl, LPCTSTR pszTxt, DWORD dwData);

// ====================================================================================

/// Classe de base pour toutes les botes de dialogue gres en tant qu'objets
class CDialogBase
{
	/// Mthode virtuelle retournant \p true si, d'aprs le message \p uMsg, les
	/// donnes du dialogue doivent tre dtruites maintenant
	virtual bool	mustDeleteNow(UINT uMsg) const {
						return false; }	// (ce qui n'est pas le cas par dfaut)

protected:
	/// Fonction "DialogProc" statique commune  tous les dialogues
	static INT_PTR CALLBACK __DialogProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam);

	/// Fonction "DialogProc" statique d'initialisation commune  tous les dialogues
	static INT_PTR CALLBACK __DialogInitProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam);

public:

	/**
	 * \brief Obtenir la "handle" d'un item du dialogue.
	 * \param[in]	nIDDlgItem	Identifiant de l'item
	 * \return	"handle" de l'item, ou NULL si inexistant
	 **/
	HWND GetItem(int nIDDlgItem) {
		return GetDlgItem(hDlg, nIDDlgItem); }

	/**
	 * \brief Activation ou dsactivation d'un item du dialogue.
	 * \param[in]	nIDDlgItem	Identifiant de l'item
	 * \param[in]	bEnable		\p true ou \p false pour activer ou dsactiver
	 **/
	BOOL EnableDlgItem(int nIDDlgItem, bool bEnable) {
		return EnableWindow(GetItem(nIDDlgItem), bEnable ? TRUE : FALSE); }

	/**
	 * Dfinit le texte de l'item de dialogue
	 *
	 * \param[in]	nIDDlgItem	Identifiant de l'item
	 * \param[in]	pszTxt		Texte  afficher
	 **/
	LRESULT SetItemText(int nIDDlgItem, LPCTSTR pszTxt) {
		return SendDlgItemMessage(hDlg, nIDDlgItem, WM_SETTEXT, 0, (LPARAM)pszTxt); }

	/**
	 * \brief Limitation de la valeur entre dans un item de dialogue numrique ditable
	 *
	 * \param[in]  nID    identifiant de l'lment
	 * \param[in]  nMin   valeur minimale
	 * \param[in]  nMax   valeur maximale
	 * \return	\p true si le contenu a fait l'objet d'un ajustement
	 **/
	bool LimitIntItemValue(int nID, int nMin, int nMax);

	/**
	 * \brief Initialisation d'une paire d'items Edit + Up/Down
	 *
	 * \param[in]  nEditID	identifiant de l'item ditable
	 * \param[in]  nUdnID	identifiant de l'item "up/down"
	 * \param[in]  nVal		valeur initiale
	 * \param[in]  nMin		valeur minimale
	 * \param[in]  nMax		valeur maximale
	 **/
	void InitUpDownItems(int nEditID, int nUdnID, LONG nVal, LONG nMin, LONG nMax);

	/**
	 * \brief Vrification du contenu d'une paire d'items Edit + Up/Down
	 *
	 * \param[in]  nEditID	identifiant de l'item ditable
	 * \param[in]  nUdnID	identifiant de l'item "up/down"
	 * \return	\p true si le contenu a fait l'objet d'un ajustement
	 **/
	bool CheckUpDownValue(int nEditID, int nUdnID);

	/**
	 * \brief Vrification du contenu d'une paire d'items Edit + Up/Down
	 *
	 * \param[in]  wParam	paramtre \p wParam de l'item ditable (l'item "up/down"
	 *						doit avoir la mme valeur +1)
	 * \return	\p true si le contenu a fait l'objet d'un ajustement
	 **/
	bool CheckUpDownValue(WPARAM wParam) {
			return CheckUpDownValue(LOWORD(wParam), LOWORD(wParam+1)); }

	/**
	 * Dplacement d'un contrle de dialogue sans modifier sa taille,
	 * en spcifiant la diffrence entre l'ancienne position et la nouvelle.
	 * \param[in]	nId		Identifiant du contrle
	 * \param[in]	dx		Dplacement horizontal
	 * \param[in]	dy		Dplacement vertical
	 **/
	void ChgCtlPos(INT nId, int dx, int dy);

	/**
	 * Redimensionnement d'un contrle de dialogue sans modifier sa position,
	 * en spcifiant la diffrence entre l'ancienne taille et la nouvelle.
	 * \param[in]	nId		Identifiant du contrle
	 * \param[in]	dx		Diffrence de taille horizontale
	 * \param[in]	dy		Diffrence de taille verticale
	 **/
	void ChgCtlSiz(INT nId, int dx, int dy);

	/**
	 * \brief Fonction "DialogProc" virtuelle.
	 * Le paramtre \p HWND n'est pas inclus, car recopi dans la variable membre
	 * \p hDlg de la classe.
	 **/
	virtual INT_PTR DialogProc(UINT uMsg, WPARAM wParam, LPARAM lParam) = 0;

	/// \p HANDLE de la bote de dialogue, recharge  chaque appel de la procdure
	/// de dialogue
	HWND		hDlg;

	/**
	 * \brief Constructeur
	 * \note Aussi longtemps que cette classe pourra tre utilise virtuellement, il sera
	 * considrablement plus commode de faire en sorte qu'elle n'ait qu'un constructeur par
	 * dfaut (donc sans paramtres)
	 **/
	CDialogBase();

	bool PostDlgMessage(UINT uMsg, WPARAM wParam = 0, LPARAM lParam = NULL) const {
		return ::PostMessage(hDlg, uMsg, wParam, lParam) == TRUE; }

	/// Destructeur
	virtual ~CDialogBase();
};

// ====================================================================================

/**
 * \brief Classe de base pour grer un dialogue modal en tant qu'objet
 * \note On hrite "virtuellement" de \p CDialogBase  cause d'un dialogue
 * qui est utilis  la fois en tant que dialogue modal et en tant que
 * feuille de proprits. Si jamais ceci tait supprim, il conviendrait
 * alors de retirer aussi l'attribut "virtual".
 **/
template <typename P>
	class CModalDialogPatBase : public virtual CDialogBase
{
	/// Fonction d'ouverture par nom de ressource
	INT_PTR			__DlgBox(HWND hParentWnd, LPCTSTR lpTemplateName) {
						return DialogBoxParam(hAppInstance, lpTemplateName, hParentWnd,
							__DialogInitProc, reinterpret_cast<LPARAM>(dynamic_cast<CDialogBase *>(this))); }

	/// Fonction d'ouverture par template
	INT_PTR			__DlgBox(HWND hParentWnd, LPCDLGTEMPLATE hDialogTemplate) {
						return DialogBoxIndirectParam(hAppInstance, hDialogTemplate, hParentWnd,
							__DialogInitProc, reinterpret_cast<LPARAM>(dynamic_cast<CDialogBase *>(this))); }

	/// Fonction d'ouverture par ID de ressource
	INT_PTR			__DlgBox(HWND hParentWnd, WORD nResId) {
						return __DlgBox(hParentWnd, MAKEINTRESOURCE(nResId)); }

	P				creationPattern;	//!< Spcification du modle de dialogue
public:

	/**
	 * Constructeur.
	 * \param[in]	crPat	Information de cration
	 **/
	CModalDialogPatBase(P crPat) :
		creationPattern(crPat)
	{
	}

	/**
	 * Ouverture et traitement effectifs de la bote de dialogue.
	 * \param[in]	hParentWnd	\p HANDLE de la fentre parente
	 * \return		Valeur passe  la fonction \p EndDialog, ou 0 si erreur
	 **/
	INT_PTR			Do(HWND hParentWnd) {
						return __DlgBox(hParentWnd, creationPattern); }
};

/// Cas le plus frquent :
typedef CModalDialogPatBase<WORD>	CModalDialogBase;

// ====================================================================================

/**
 * Classe de base pour grer un dialogue modeless en tant qu'objet
 * \note Tous les membres sont \p private en dehors du constructeur et du destructeur,
 * car l'accs  cette classe est rserv au template qui maintient le pointeur sur son
 * instance, laquelle doit tre unique.
 **/
class CModelessDialogBase : public CDialogBase
{
	/// Liste des dialogues "modeless" couramment ouverts (utilis pour pouvoir
	/// automatiquement passer \p IsModelessDialogMessage  tous les dialogues ouverts)
	static vector<HWND>	vOpenDialogs;

	/// SingletonDlgPtr est notre amie
	template <typename P>
		friend class CModelessDialogPatBase;

	HWND			CreateDlg(HWND hParentWnd, LPCTSTR lpTemplateName);		//!< Fonction de cration par nom de ressource
	HWND			CreateDlg(HWND hParentWnd, WORD wResId) {				//!< Fonction de cration par ID de ressource
						return CreateDlg(hParentWnd, MAKEINTRESOURCE(wResId)); }
	HWND			CreateDlg(HWND hParentWnd, LPCDLGTEMPLATE lpTemplate);	//!< Fonction de cration par template
	bool			__ShowAndAdd(HWND hNewDlg);

	/// Mthode virtuelle retournant \p true si, d'aprs le message \p uMsg, les
	/// donnes du dialogue doivent tre dtruites maintenant
	virtual bool	mustDeleteNow(UINT uMsg) const {
						return uMsg == WM_NCDESTROY; }

	/// Cration de la fentre
	template <typename P>
		bool		__Create(HWND hParentWnd, P crPat) {
						return __ShowAndAdd(CreateDlg(hParentWnd, crPat)); }

protected:
	/// Protection contre une erreur commune
	BOOL		EndDialog(HWND hDlg, INT_PTR nResult) const {
					ASSERT(("EndDialog used in modeless dialog", !hDlg)); return FALSE; }

	/// Destructeur
	virtual ~CModelessDialogBase();

public:

	/// Restauration, ramener au premier plan
	void		Restore() {
					ShowWindow(hDlg, SW_SHOW);
					ShowWindow(hDlg, SW_RESTORE);
					BringWindowToTop(hDlg); }

	/// S'assure que la fentre a bien t cre
	bool		isValid() const {
					return hDlg != NULL; }

	/**
	 * \brief Traitement des vnements lis  n'importe quel dialogue modeless ouvert
	 *
	 *  placer dans la boucle de gestion gnrale des messages.
	 * \param[in,out]	msg		Message  traiter
	 * \retval			true	si le message \p msg a t trait, auquel cas il ne faut pas le
	 *							passer  \p TranslateMessage et \p DispatchMessage.
	 * \retval			false	dans le cas contraire
	 **/
	friend bool	IsModelessDialogMessage(MSG & msg);
};

// ====================================================================================

template <typename P = WORD>
	class CModelessDialogPatBase : public CModelessDialogBase
{
	P	creationPattern;
public:
	bool	__Create(HWND hParentWnd) {
				return __super::__Create(hParentWnd, creationPattern); }

	/**
	 * Constructeur.
	 * \param[in]	crPat	Identifiant de la ressource utilise pour le dialogue
	 **/
	CModelessDialogPatBase(P crPat) :
		creationPattern(crPat)
	{
	}
};

// ====================================================================================

/**
 * Classe "template" pour grer un dialogue modeless en tant qu'objet
 **/
template <typename T, typename P = WORD>
	class CModelessDialogHelper
{
	/// Pointeur statique garantissant l'unicit de chaque dialogue
	static CModelessDialogPatBase<P> * pDlgInstance;

public:
	/**
	 * \p brief	Ouverture / activation du dialogue
	 * \param[in]	hParentWnd	Handle de la fentre parente
	 **/
	static void ShowDialog(HWND hParentWnd)
	{
		if (!pDlgInstance) {
			pDlgInstance = dynamic_cast<CModelessDialogPatBase<P> *>(new T);	// Cration de la classe et de la fentre

			if (pDlgInstance && !pDlgInstance->__Create(hParentWnd)) {
				// La cration de la classe a eu lieu, mais pas celle de la fentre
				delete pDlgInstance;
				pDlgInstance = NULL;
			}
		} else if (pDlgInstance->isValid())
			pDlgInstance->Restore();
	}

	/// Envoi d'un message dans la file d'attente du dialogie
	static bool	PostDlgMessage(UINT uMsg, WPARAM wParam = 0, LPARAM lParam = NULL) {
					return pDlgInstance ? pDlgInstance->PostDlgMessage(uMsg, wParam, lParam) : false; }

	void clear_ptr() {
		pDlgInstance = NULL; }

	~CModelessDialogHelper() {
		clear_ptr(); }
};

template <typename T, typename P>
	CModelessDialogPatBase<P> * CModelessDialogHelper<T,P>::pDlgInstance = NULL;

// ====================================================================================

/**
 * Classe "template" pour grer un dialogue modeless en tant qu'objet
 **/
template <typename T, typename P = WORD>
	class CModelessDialog : public CModelessDialogPatBase<P>
{
	static CModelessDialogHelper<T, P> cHelper;
protected:
	CModelessDialog(P crPat) :
		CModelessDialogPatBase<P>(crPat)
	{
	}
public:
	static void	ShowDialog(HWND hParentWnd) {
					cHelper.ShowDialog(hParentWnd); }
	/// Envoi d'un message dans la file d'attente du dialogie
	static bool	PostDlgMessage(UINT uMsg, WPARAM wParam = 0, LPARAM lParam = NULL) {
					return cHelper.PostDlgMessage(uMsg, wParam, lParam); }

	virtual ~CModelessDialog() {
				cHelper.clear_ptr(); }
};

template <typename T, typename P>
	CModelessDialogHelper<T, P> CModelessDialog<T, P>::cHelper;

// ====================================================================================

/**
 * \brief Classe de base pour grer une page de feuille de proprits en tant qu'objet
 * \note On hrite "virtuellement" de \p CDialogBase  cause d'un dialogue
 * qui est utilis  la fois en tant que dialogue modal et en tant que
 * feuille de proprits. Si jamais ceci tait supprim, il conviendrait
 * alors de retirer aussi l'attribut "virtual".
 **/
class CPropSheetPageBase : public virtual CDialogBase
{
	friend class CPropSheetBase;
	template <typename T>
		friend static CPropSheetPageBase * CreatePspInstance(const PROPSHEETPAGE & sPsp);

	/// Mthode virtuelle retournant \p true si, d'aprs le message \p uMsg, les
	/// donnes du dialogue doivent tre dtruites maintenant
	virtual bool	mustDeleteNow(UINT uMsg) const {
						return uMsg == WM_NCDESTROY; }

	LPCTSTR			pszTemplate;

public:
	/// Fonction "DialogProc" statique d'initialisation commune des feuilles de proprits
	static INT_PTR CALLBACK __PspInitProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam);

	/// Type pointeur sur la fonction de cration
	typedef CPropSheetPageBase * (* NewPspProc)(const PROPSHEETPAGE & sPsp);

	/**
	 * \brief Construction d'un descripteur de page de proprits
	 * \param[in]	nResId		Identifiant (numrique) de la ressource utilise pour le dialogue
	 * \param[in]	pszTitle	Titre de la page de proprits
	 * \param[in]	newProc		Pointeur sur la fonction de cration de l'objet contenant les donnes
	 **/
	static PROPSHEETPAGE MakePropSheetPage(INT nResId, LPCTSTR pszTitle, NewPspProc newProc);

protected:
	/**
	 * Assistance au retour de valeur aprs notification dans une feuille de proprits
	 *
	 * \param[in]  res    valeur devant tre retourne
	 * \retval     true
	 **/
	BOOL psn_result(LONG res);

	/// numration des valeurs possibles de la fonction de discrimination entre les causes de
	/// la notification \p PSN_KILLACTIVE
	enum PSN_KILLACTIVE_discriminator {
		epkd_undef,			//!< La notification n'a pas encore t provoque
		epkd_pagechange,	//!< La notification a t provoque par un changement de page
		epkd_buttonpressed	/*!< La notification a t provoque par une pression sur un des boutons
								 \p OK ou \p Appliquer */
	};

	/// Raison de la dernire notification \p PSN_KILLACTIVE
	static PSN_KILLACTIVE_discriminator ePSN_KILLACTIVE_discriminator;

	CPropSheetPageBase() :
		pszTemplate(NULL)
	{
	}
};

/// Fonction de cration d'une nouvelle instance d'un dialogue de page de feuille
/// de proprits (normalement appele via pointeur sur fonction)
template <typename T>
	static CPropSheetPageBase * CreatePspInstance(const PROPSHEETPAGE & sPsp)
{
	CPropSheetPageBase * pDlg = new T();

	// Transfert de l'identifiant de ressource :
	if (pDlg)
		pDlg->pszTemplate = sPsp.pszTemplate;
	return pDlg;
}

/**
 * \brief Construction d'un descripteur de page de proprits (template)
 * \param[in]	nResId		Identifiant (numrique) de la ressource utilise pour le dialogue
 * \param[in]	pszTitle	Titre de la page de proprits
 **/
template <typename T>
	PROPSHEETPAGE MakePropSheetPage(INT nResId, LPCTSTR pszTitle)
{
	return CPropSheetPageBase::MakePropSheetPage(nResId, pszTitle, CreatePspInstance<T>);
}

// ====================================================================================

/**
 * \brief Classe de base pour grer une feuille de proprits en tant qu'objet
 **/
class CPropSheetBase
{
protected:
	/// Handle du dialogue
	HWND						hDlg;

	/// Adresse de la procdure d'origine du dialogue de feuilles de proprits
	WNDPROC						pDlgProc;

	/// Vecteur contenant les descripteurs de page de feuille de proprits ajouts
	vector<const PROPSHEETPAGE>	vPsp;

	/// Fonction "callback" statique de substitution  celle du dialogue de feuilles de proprits
	static LRESULT CALLBACK		__SubDlgProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam);

	/// Fonction "callback" statique du dialogue de feuilles de proprits
	static int CALLBACK			__PshProc(HWND hwndDlg, UINT uMsg, LPARAM lParam);

	/// Traitement des messages de la liste d'items surclasse
	virtual LRESULT				SubProc(UINT uMsg, WPARAM wParam, LPARAM lParam);

	/// Fonction d'initialisation par dfaut
	virtual void				Init() {
									/* rien par dfaut */ }
public:
	/// Constructeur
	CPropSheetBase() :
		hDlg(NULL),
		pDlgProc(NULL)
	{}

	/// Ajout d'un descripteur de page de feuille de proprits
	void AddPage(const PROPSHEETPAGE & sPsp);

	/**
	 * Ouverture et traitement effectifs de la feuille de proprits
	 * \param[in]	hParentWnd	\p HANDLE de la fentre parente
	 * \param[in]	pszCaption	Titre de la feuille de proprits
	 **/
	void Do(HWND hParentWnd, LPCTSTR pszCaption);
};

// ====================================================================================

/**
 * \brief "template" de surclassement d'une fentre, d'un dialogue ou d'un contrle de dialogue
 *
 * \param[in]	N		Une valeur numrique non signifiante en elle-mme, mais permettant de
 *						distinguer plusieurs instances du "template", considrant que celui-ci
 *						est destin  servir (ventuellement plusieurs fois) de classe de base
 *						 une classe de gestion de dialogue.
 * \param[in]	WT		\p false pour surclasser une fentre ou un contrle, \p true pour
 *						surclasser un dialogue.
 **/
template <int N = 0, int WT = false>
	class WinSubClass
{
	/// Index pour \p SetWindowLongPtr selon type de fentre  surclasser
	enum SWLP_index
	{
		SC_PROC = WT ? DWLP_DLGPROC : GWLP_WNDPROC,
		SC_DATA = WT ? DWLP_USER : GWLP_USERDATA
	};

public:
	WNDPROC	pfnProc;
	HWND	hSWnd;

	/// Fonction "WindowProc" statique de substitution
	static LRESULT CALLBACK __SubProc(HWND hSWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
	{
		WinSubClass * pCl = reinterpret_cast<WinSubClass *>(GetWindowLongPtr(hSWnd, SC_DATA));

		if (pCl)
			return pCl->SubProc(*pCl, uMsg, wParam, lParam);
		return 0;
	}

	/// Constructeur
	WinSubClass() :
		pfnProc(NULL),
		hSWnd(NULL)
	{
	}

	/**
	 * Installation du surclassement
	 * \param[in]	hWnd	"Handle" de la fentre surclasse
	 **/
	bool SubClass(HWND hWnd)
	{
		if (pfnProc == NULL) {
			hSWnd	= hWnd;
			pfnProc	= (WNDPROC)SetWindowLongPtr(hWnd, SC_PROC, (LONG_PTR)__SubProc);

			LONG_PTR nRes = SetWindowLongPtr(hWnd, SC_DATA, (LONG_PTR)this);

			// S'assurer, en debugging, que la zone de sauvegarde du pointeur n'tait pas dj utilise.
			ASSERT(nRes == 0);
			return true;
		}
		return false;
	}

	/**
	 * Fonction "WindowProc" de substitution
	 *
	 * Plusieurs peuvent exister dans la mme classe) drive, par variation de la
	 * dfinition du premier paramtre : \p WinSubClass<0>, \p WinSubClass<1>, etc.
	 **/
	virtual LRESULT SubProc(WinSubClass & sc, UINT uMsg, WPARAM wParam, LPARAM lParam) = 0;

	/// Appel de la procdure d'origine du contrle
	LRESULT CallWindowProc(UINT uMsg, WPARAM wParam, LPARAM lParam) {
		return ::CallWindowProc(pfnProc, hSWnd, uMsg, wParam, lParam);
	}

};

// ====================================================================================

/**
 * Excution d'une commande via le "shell" de Windows
 *
 * \param[in]	hParentWnd	\p HANDLE de la fentre parente
 * \param[in]	pszFile		Commande  excuter
 * \param[in]	pszArg		Argument de la commande
 **/
void ShellOpen(HWND	hParentWnd, LPCTSTR pszFile, LPCTSTR pszArg = TEXT(""));

// ====================================================================================
