/*
 * $Id$
 * Copyright (C) 2011 gingko - http://gingko.homeip.net/
 *
 * This file is part of Pouchin TV Mod, a free DVB-T viewer.
 * See http://www.pouchintv.fr/ for updates.
 *
 * Pouchin TV Mod is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Pouchin TV Mod is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

#include "stdafx.h"
#include "helpdlg.h"
#include <richedit.h>

#if LZMA_HELP
	#include "lzma.h"
#endif

// ====================================================================================

/// Extraction et dcompression de la ressource d'aide
bool CHelpDialogBase::ResReader::Extraction(UINT idRessource)
{
#if LZMA_HELP
	RawResource	vCompressed;

	if (!vCompressed.Extraction(NULL, idRessource))
		return false;
	if (!DecompressLzma(*this, vCompressed))
		return false;	// -> Dcompression impossible
#else
	if (!__super::Extraction(NULL, idRessource))
		return false;
#endif
	sMm = MemRng(&*begin(), (UINT)size());
	return true;
};

// ====================================================================================

CHelpDialogBase::HelpTEMPLATE CHelpDialogBase::sDlgTmpl =
{
	{
		1,				// dlgVer
		0xffff,			// signature
		0,				// helpID
		0,				// exStyle
		DS_SETFONT | DS_MODALFRAME | DS_FIXEDSYS | DS_CENTER | WS_MAXIMIZEBOX | WS_POPUP | WS_CAPTION | WS_SYSMENU | WS_THICKFRAME,	// style
		2,				// cDlgItems
		0, 0,			// x, y
		500,300,		// cx, cy
		0,				// menu
		0,				// windowClass
	#if defined(LANG) && LANG==LANG_FR
		L"Aide",		// title
	#else
		L"Help",		// title
	#endif
		8, 400,			// pointsize, weight
		0, 0x1,			// italic, charset
		L"MS Shell Dlg"	// typeface
	}, {
		0,				// helpID
		0,				// exStyle
		ES_MULTILINE | ES_READONLY | WS_BORDER | WS_VSCROLL | WS_TABSTOP | WS_CHILD | WS_VISIBLE,	// style
		4,4,			// x, y
		492,274,		// cx, cy
		eIDC_HELPTEXT,	// id
		L"RichEdit20A",	// windowClass
		0,				// title
		0				// extraCount
	}, {
		0,				// helpID
		0,				// exStyle
		BS_DEFPUSHBUTTON | WS_TABSTOP | WS_CHILD | WS_VISIBLE,	// style
		224,282,		// x, y
		50,14,			// cx, cy
		IDOK,			// id
		{0xffff,0x80},	// windowClass
		L"OK",			// title
		0				// extraCount
	}
};

DWORD CHelpDialogBase::ResReader::ESCallback(LPBYTE pbBuff, LONG cb, LONG *pcb)
{
	LONG		nLen	= min<LONG>(cb, sMm.siz);

	if (nLen > 0) {
		memcpy(pbBuff, sMm.ptr, nLen);
		*pcb = nLen;
		sMm += nLen;
	}
	return 0;
}

/// Traitement des messages de la bote de dialogue
INT_PTR CHelpDialogBase::DialogProc(UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch (uMsg) {

	case WM_INITDIALOG: {
		// Initialisation de la fentre
		SetWindowText(hDlg, strDlgTitle.c_str());
		hRtfCtl = GetItem(eIDC_HELPTEXT);
		WinSubClass<SC_0>::SubClass(hRtfCtl);

		ResReader	sResRdr;

		if (sResRdr.Extraction(idRtfResource)) {
			EDITSTREAM sEs = {(DWORD_PTR)&sResRdr, 0, sResRdr.ESCallback};

			SendMessage(hRtfCtl, EM_STREAMIN, SF_RTF, (LPARAM)&sEs);
		}
		return TRUE; }

	case WM_GETMINMAXINFO: {
		MINMAXINFO & sMinMax = *reinterpret_cast<MINMAXINFO *>(lParam);

		// Mmoriser l'ancienne taille pour calcul du changement :
		GetClientRect(hDlg, &sSavRect);
		// Dimensions minimales acceptables :
		sMinMax.ptMinTrackSize.x = 400;
		sMinMax.ptMinTrackSize.y = 180;
		break; }

	case WM_SIZE: {
		RECT	sRect;

		GetClientRect(hDlg, &sRect);

		// Calcul de la diffrence entre l'ancienne et la nouvelle taille :
		int		dx = RWdt(sRect)-RWdt(sSavRect);
		int		dy = RHgt(sRect)-RHgt(sSavRect);

		// Dplacement ou redimensionnement des contrles :
		ChgCtlSiz(eIDC_HELPTEXT,	dx,		dy);
		ChgCtlPos(IDOK,				dx/2,	dy);
		return TRUE; }

	case WM_CLOSE:
		DestroyWindow(hDlg);
		return TRUE;

	case WM_COMMAND:
		// Commande reue (clic, etc...)
		switch(wParam) {

		case eIDC_HELPTEXT:
			wParam = wParam; EN_CHANGE;
			break;

		case _cmd_(IDOK, BN_CLICKED):
			// Clic sur le bouton OK
			DestroyWindow(hDlg);
			return TRUE;
		}
	}

	// Autre message, trait par Windows
	return FALSE;
}

// ====================================================================================

