/*
 * $Id: sbuffer.cpp 262 2010-09-03 09:30:11Z gingko $
 * Copyright (C) 2008 gingko - http://gingko.homeip.net/
 *
 * This file is part of Pouchin TV Mod, a free DVB-T viewer.
 * See http://www.pouchintv.fr/ for updates.
 *
 * Pouchin TV Mod is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Pouchin TV Mod is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 *
 *
 * Most files of this project have been changed from the Pouchin TV
 * project (Copyright (C) 2006 Pouchin <pouchinteve@free.fr>),
 * to use with a modified version of this software.
 * See http://pouchinteve.free.fr/ for the original project.
 */

/** \file
 *	\brief Gestion d'un tampon de donnes  croissance automatique (implmentations)
 *
 *	Ce fichier implmente les classes utilises pour la construction d'un tampon
 *	dont la taille crot automatiquement en cas de dpassement de longueur.
 *	Ce tampon est utilis pour la conversion interne TS -> PS.
 **/

#include "stdafx.h"
#include "sbuffer.h"

void StreamBuffer::flush() throw(...)
{
	if (data_size > 0 && size() >= data_size) {
		if (!sent)
			send(data_ptr(), data_size);
		else
			extra_send(data_ptr(), data_size);
		sent = true;
		data_size = 0;
	}
}

void StreamBuffer::put_data(PCUINT8 ptr, UINT32 ptr_size, bool start) throw(...)
{
	if (ptr_size > 0) {
		if (start) {
			flush();
			sent = false;
		}
		UINT32 newIndex = data_size+ptr_size;

		if (newIndex > buf_size) {
			buf_size = ((newIndex/gran)+1)*gran;
			resize(buf_size);
		}
		memcpy(&(*this)[data_size], ptr, ptr_size);
		data_size = newIndex;
		if (test())
			flush();
	}
}

// change des donnes avec celles d'un autre tampon de mme type :
void StreamBuffer::swap(StreamBuffer & b2swap)
{
	std::swap(*dynamic_cast<std::vector<UINT8> *>(this), *dynamic_cast<std::vector<UINT8> *>(&b2swap));
	std::swap(gran, b2swap.gran);
	std::swap(buf_size, b2swap.buf_size);
	std::swap(sent, b2swap.sent);
	std::swap(data_size, b2swap.data_size);
}

StreamBuffer::~StreamBuffer()
{
}
