/*
 * $Id: sbuffer.h 262 2010-09-03 09:30:11Z gingko $
 * Copyright (C) 2008 gingko - http://gingko.homeip.net/
 *
 * This file is part of Pouchin TV Mod, a free DVB-T viewer.
 * See http://www.pouchintv.fr/ for updates.
 *
 * Pouchin TV Mod is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Pouchin TV Mod is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 *
 *
 * Most files of this project have been changed from the Pouchin TV
 * project (Copyright (C) 2006 Pouchin <pouchinteve@free.fr>),
 * to use with a modified version of this software.
 * See http://pouchinteve.free.fr/ for the original project.
 */

/** \file
 *	\brief Gestion d'un tampon de donnes  croissance automatique (dfinitions)
 *
 *	Ce fichier dfinit des classes utilises pour la construction d'un tampon
 *	dont la taille crot automatiquement en cas de dpassement de longueur.
 *	Ce tampon est utilis pour la conversion interne TS -> PS.
 **/

#pragma once


// ----------------------------------------------------------------------------

/*
 * Drivation de la classe "vector" pour recevoir des donnes fragmentes et rassembler celles-ci,
 * en utilisant un tampon dont la taille crot automatiquement en cas de dpassement de longueur.
 * Afin d'viter des rallocations mmoire continuelles, la taille du tampon ne se rduit jamais
 * en cours d'utilisation. Il est simplement entirement dtruit avec le destructeur de l'objet.
 */
class StreamBuffer : public std::vector<UINT8>
{
	size_t			gran;	//!< granularit (valeur qui spcifie par multiples
							//!< de combien la taille du tampon doit tre augmente)
	size_t			buf_size;
	bool			sent;	//!< 'true'  partir du moment o "send" a t appel ( cause d'un appel 'test'
							//!<  retournant 'true'), et jusqu' la rception d'un appel "put" avec 'start=true'
public:
	UINT32			data_size;

	StreamBuffer(size_t gr=0x200) :
		gran(gr),
		buf_size(0),
		sent(false),
		data_size(0)
	{}

	/// Ajout de donnes
	void			put_data(PCUINT8 ptr, UINT32 ptr_size, bool start=false) throw(...);

	/// Envoi et vidange du tampon
	virtual void	flush() throw(...);

	PCUINT8			data_ptr() const {
						return const_cast<PCUINT8>(&front());};

	/// Envoi des donnes compltes
	virtual void	send(PCUINT8 ptr, UINT32 ptr_size) throw(...) = 0;

	/// Envoi de donnes "hors paquet"
	virtual void	extra_send(PCUINT8 /*ptr*/, UINT32 /*ptr_size*/) {
						}	// donnes ignores par dfaut

	/// change des donnes avec celles d'un autre tampon de mme type
	void			swap(StreamBuffer & b2swap);

	/// Vrification du tampon aprs ajout de donnes, pour savoir s'il est complet,
	/// en utilisant les donnes elles-mmes
	virtual bool 	test() const {
						return false; }	// pas de vrification, par dfaut

	virtual ~StreamBuffer();
};

// ----------------------------------------------------------------------------
