/*
 * $Id: channels.h 273 2011-02-25 11:11:29Z gingko $
 * Copyright (C) 2007 gingko - http://gingko.homeip.net/
 *
 * This file is part of Pouchin TV Mod, a free DVB-T viewer.
 * See http://www.pouchintv.fr/ for updates.
 *
 * Pouchin TV Mod is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Pouchin TV Mod is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 *
 *
 * Most files of this project have been changed from the Pouchin TV
 * project (Copyright (C) 2006 Pouchin <pouchinteve@free.fr>),
 * to use with a modified version of this software.
 * See http://pouchinteve.free.fr/ for the original project.
 */

/** \file
 *	\brief Dfinitions lies  la gestion des chanes TV
 *
 *	Ce fichier dfinit les lments ncessaires au stockage et  la gestion
 *	des informations lies aux chanes TV identifies par l'application.
 **/

#pragma once

#include "chanutils.h"
#include <list>

#define DEF_IN_ACTUAL	1	//!< L'mission est dfinie dans le flux "actual"
#define	DEF_IN_SCHEDULE	2	//!< L'mission est dfinie dans le flux "schedule"

class Chaine;	// Forward

/// Structure dcrivant une mission pour l'EPG
class Emission
{
public:
	UINT64			nUtcBegT;	//!< Reprsentation simplifie de l'heure de dbut pour tests (0 si pas dfini)
	UINT64			nUtcEnd10;	//!< Heure d'expiration (= fin +10mn) pour tests
	UINT16			nEvtId;		//!< "event id"
	SYSTEMTIME		debut;		//!< Heure de dbut
	SYSTEMTIME		fin;		//!< Heure de fin
	tstring			strNom;		//!< Nom de l'mission (natif)
	tstring			strDesc;	//!< Description (native)
	tstring			strExtDesc;	//!< Description tendue (native)
private:
	mutable tstring	strSDCache;	//!< Description courte reformate (cache)
	mutable tstring	strLDCache;	//!< Description longue reformate (cache)
public:
	DWORD			nContent;	//!< Type de contenu \test Exprimental \todo Voir comment prsenter a
	mutable int		ixChaine;	//!< Index de la chane dans la table \p Canaux
	mutable bool	bUsed;		//!< Utilis pour marquer les missions dans l'EPG en vue de suppression
	BYTE			nFlags;		//!< #DEF_IN_ACTUAL / #DEF_IN_SCHEDULE (combinaison possible)
	UINT8			nRating;	//!< ge limite pour les mineurs, moins 3
	UINT32			cleChaine;	//!< Cl chane (pour retrouver une chane dont la position aurait chang)

	/// Constructeur
	Emission();

	/// Vrifie si l'mission est valide
	/// \retval \p true si valide, \p false si non valide
	/// \deprecated Probablement plus ncessaire
	bool	isOk() const {
		return nUtcBegT != 0; }

	/// Obtenir le nom de l'mission
	/// \retval Nom de l'mission si valide, chane vide si non valide
	tstring getName() const;

	/// Tester si l'mission a reu un nom
	bool hasName() const {
		return isOk() && !strNom.empty(); }

	/// Obtenir une description courte, limite  une ligne
	const tstring & getShortDescr() const;

	/// Obtenir une description longue, ventuellement multilignes
	const tstring & getLongDescr() const;

	/// Obtenir une rfrence sur la chane (d'aprs \p ixChaine vrifi par \p cleChaine)
	const Chaine & trouve_chaine() const;

	/// Obtenir le numro de la chane o l'mission est programme
	UINT16 channelNo() const;
};

/// Liste des missions rapportes par l'EPG
struct EmissionPtr : public shared_ptr<Emission>
{
	/// Constructeur
	EmissionPtr(Emission * pNewObj = NULL) :
		shared_ptr<Emission>(pNewObj)
	{
	}

	Emission & operator * () const {
				return *get(); }

	tstring getName(LPCTSTR pszDefault) const {
		if (*this)
			return (*this)->getName();
		return pszDefault; }

	/// Tester si l'mission a reu un nom
	bool hasName() const {
		return (*this) && (*this)->hasName(); }
};

struct  ActAndNxtProg
{
	EmissionPtr		apEmi[2];

	const Emission * get(int nEmis) {
		return apEmi[nEmis].get(); }
};

class ChanEventList : public std::list<EmissionPtr>
{
	/**
	 * \brief Section critique pour la synchronisation
	 *
	 * Sous forme de pointeur avec compteur de rfrence, car la liste entire est
	 * susceptible d'tre copie au moment de la construction de la liste des chanes.
	 **/
	mutable shared_ptr<CCritSec>	pCritSec;
public:

	ChanEventList() :
		pCritSec(new CCritSec)
	{
	}

	/// Obtenir un pointeur non const sur le descripteur de section critique
	CCritSec * crit_sec() const {
				return pCritSec.get(); }

	/**
	 * Recherche, dans les informations d'une chane, de l'mission correspondant  une cl d'mission donne
	 *
	 * \param[in]	nEvtId	ID d'mission recherche
	 * \return		Pointeur sur l'mission dans la liste, ou NULL si pas trouv
	 **/
	const Emission * cherche_par_id(UINT16 nEvtId) const;

	/**
	 * \brief Obtenir l'mission courante et la suivante, par rapport  une heure donne
	 * 
	 * \param[in]	localtime	Heure de rfrence (en principe l'heure systme)
	 * \param[in]	bStrict	si \p true, les horaires de l'mission doivent strictement englober
	 *				le temps courant ; si \p false, un nom d'mission non commence peut
	 *				ventuellement tre retourn.
	 * \retval		Structure contenant deux pointeurs (avec compteurs de rfrence),
	 *				l'un sur l'mission en cours, l'autre sur l'mission suivante.
	 *				Les deux pointeurs peuvent tre NULL si l'mission n'est pas dfinie.
	 * \todo	tant donn que la spcification \p bStrict ne semble pas faire l'unanimit parmi
	 *			les utilisateurs, tudier ventuellement un paramtre de configuration pour dcider.
	 **/
	ActAndNxtProg getCurrentAndNext(const SYSTEMTIME & localtime, bool bStrict=false) const;

	/**
	 * \brief Obtenir l'mission courante et la suivante
	 * 
	 * \param[in]	bStrict	si \p true, les horaires de l'mission doivent strictement englober
	 *				le temps courant ; si \p false, un nom d'mission non commence peut
	 *				ventuellement tre retourn.
	 * \retval		Structure contenant deux pointeurs (avec compteurs de rfrence),
	 *				l'un sur l'mission en cours, l'autre sur l'mission suivante.
	 *				Les deux pointeurs peuvent tre NULL si l'mission n'est pas dfinie.
	 **/
	ActAndNxtProg getCurrentAndNext(bool bStrict=false) const {
					SYSTEMTIME localtime;
					GetLocalTime(&localtime);
					return getCurrentAndNext(localtime, bStrict); }

	/**
	 * Obtenir l'mission couramment en cours de diffusion
	 *
	 * \param[in]	bStrict	si \p true, les horaires de l'mission doivent strictement englober
	 *				le temps courant ; si \p false, un nom d'mission non commence peut
	 *				ventuellement tre retourn.
	 * \retval	Pointeur (avec compteurs de rfrence) sur l'mission
	 **/
	EmissionPtr getCurrentProgram(bool bStrict=false) const {
					return getCurrentAndNext(bStrict).apEmi[0]; }

	/**
	 * Obtenir le temps restant avant le prochain changement d'tat EPG concernant cette chane.
	 *
	 * \param[in]	pLocalTime	Pointeur sur temps courant (rcupr en interne si NULL)
	 * \return		Temps restant, exprim en millisecondes, ou bien #DUREE_INFINIE si aucun temps
	 *				ne peut tre dtermin (aucune mission dfinie dans l'EPG ...).
	 **/
	long timeBeforeNextEvent(LPSYSTEMTIME pLocalTime = NULL) const;

	/// Effacement de la liste
	void clear();
};

enum EtatChaine {
	ec_inactive,	// Chane inactive (non slectionnable)
	ec_active,		// Chane active (slectionnable par menu et numro seulement)
	ec_preferee		// Chane active (slectionnable aussi par flches et roulette souris)
};

/// Identifiants de chane associs  des prfrences utilisateur
struct ChaineIdsUser : public ChaineIDs
{
	UINT16		nNumeroChaine;	//!< Numro de chane effectivement utilis
	EtatChaine	etat;			//!< tat d'activation

	ChaineIdsUser() :
		nNumeroChaine(0),
		etat(ec_preferee)
	{
	}
};

/**
 * \brief Structure contenant les "handle" d'icnes
 *
 * Structure destine  tre alloue en mmoire avec comptage de rfrence, pour
 * vider les problmes de destruction lors des copies de la classe parente.
 **/
struct IconHCache
{
	HBITMAP	hImage;				//!< "Handle" de l'icne si charge (format bitmap) (cache)
	HICON	hIcon;				//!< "Handle" de la conversion ventuelle au format icne (cache)

	IconHCache() :
		hImage(NULL),
		hIcon(NULL)
	{
	}

	~IconHCache() {
		if (hImage && hImage!=INVALID_HANDLE_VALUE)
			DeleteObject(hImage);
		if (hIcon && hIcon!=INVALID_HANDLE_VALUE)
			DeleteObject(hIcon);
	}
};

struct MappingData;

/// Structure dcrivant une chane
class Chaine : public ChainePMT, public ChaineIdsUser, public ChanEventList
{
public:
	char			nom[64];
	char			groupe[64];
	INT32			khz;			//!< Frquence retenue pour le multiplex
	UINT16			nNumeroNit;		//!< Numro de chane original dfini par le rseau
	UINT16			nNumeroAuto;	//!< Numro de chane automatiquement attribu par l'application
									//!< (mmoris car il existe une commande de restauration)
private:
#if USE_PMTFILTER
	static void CALLBACK PmtMonitor(const ChainePMT & sExpectedPmt, const ChainePMT & sFoundPmt, DWORD_PTR dwUserData);
#endif

	mutable shared_ptr<IconHCache>
					pIconCache;
public:

	/// Constructeur
	Chaine();

	/// Destructeur
	~Chaine();

#ifdef _XMLUTILS_H_INCLUDED_

	/// numration corresopondant aux tags XML utiliss pour la sauvegarde d'une chane
	enum TagListChaine {
		tlc_Nom,
		tlc_Canal,
		tlc_Groupe,
		tlc_Frequence,
		tlc_ONID,
		tlc_TSID,
		tlc_SID,
		tlc_Numero_initial,
		tlc_Numero_auto,
		tlc_Numero,
		tlc_Etat,
		tlc_PMT,
		tlc_PCR,
	  #if STREAMS_SAVE_ALL
		tlc_Video,
		tlc_Video_mpeg4,
		tlc_Video_type,
	  #endif
		tlc_END
	};

	/// Noms des tags XML utiliss pour la sauvegarde d'une chane
	static LPCSTR pszTagStrList;

	/**
	 * Chargement de la dfinition de la chane  partir d'un noeud TinyXML construit
	 * \param[in]	pXmlElm	Noeud XML contenant les donnes  charger
	 * \return		\p true ou \p false selon succs ou chec
	 **/
	bool SetFromXMLElement(const TiXmlElement * pXmlElm);

	/**
	 * Construction d'un noeud TinyXML reprsentant la dfinition de la chane
	 * \return	Noeud XML construit
	 **/
	TiXmlElement * GetAsXMLElement() const;

	/**
	 * Obtenir la base du tag XML requis, selon qu'on se rfre  une piste "son" ou "autre"
	 * \param[in]	bOther	\p false pour "son", \p true pour "autre"
	 **/
	LPCSTR	getTagBase(bool bOther) const {
		return bOther ? "Autre" : "Son";}

  #if STREAMS_SAVE_ALL
	/**
	 * Chargement de la dfinition d'un groupe de pistes  partir d'un noeud TinyXML construit
	 * \param[in]	pXmlElm	Noeud XML parent de celui contenant les informations recherches
	 * \param[in]	bOther	\p false pour pistes "son", \p true pour pistes "autre"
	 * \return		\p true ou \p false selon succs ou chec
	 **/
	bool SetStreamsFromXML(const TiXmlElement * pXmlElm, bool bOther);

	/**
	 * Ajout des lments de ce groupe de pistes  un noeud TinyXML
	 * \param[in]	pXmlElm	Noeud XML auquel ajouter les informations
	 * \param[in]	bOther	\p false pour pistes "son", \p true pour pistes "autre"
	 **/
	void GetStreamsAsXML(TiXmlElement * pXmlElm, bool bOther) const;
  #endif // #if STREAMS_SAVE_ALL

#endif // #ifdef _XMLUTILS_H_INCLUDED_

	/// Obtenir l'icne associe  la chane, au format bitmap
	HBITMAP GetIconAsBmp() const;

	/// Obtenir l'icne associe  la chane, au format ICO
	HICON GetIconAsIco() const;

	/**
	 * \brief Comparaison avec une autre chane pour tri.
	 *
	 * \retval	Une valeur ngative si cette chane a un rang infrieur  l'autre chane
	 * \retval	0 Si cette chane a un rang gal  l'autre chane
	 * \retval	Une valeur positive si cette chane a un rang suprieur  l'autre chane
	 **/
	int compare(const Chaine & canal2)	const;

	/**
	 * \brief Comparaison avec une autre chane pour tri.
	 *
	 * Utilis par la fonction std::sort
	 * \retval \p true si cette programmation se classe avant la programmation \p p2 dans le temps.
	 **/
	bool operator < (const Chaine & canal2) const {
		return compare(canal2) < 0; }

	/// Gnration du nom affichable pour le menu
	int toMenuString(LPTSTR pstr, size_t bufSize) const {
		return _stprintf_s(pstr, bufSize,
			TEXT("%4i : %") A2t, nNumeroChaine, nom, groupe); }

	/// Gnration du nom affichable pour le menu (avec TSID)
	int toMenuString_tsid(LPTSTR pstr, size_t bufSize) const {
		return _stprintf_s(pstr, bufSize,
			TEXT("%4i [%u] : %") A2t, nNumeroChaine, TSID, nom); }

	/**
	 * Gnration du nom affichable d'une piste associe pour le menu
	 * \param[in]	nTrack	Numro de la piste
	 * \param[in]	pstr	Pointeur sur le tampon qui recevra le rsultat
	 * \param[in]	bufSize	Taille du tampon
	 * \param[in]	nNum	Numro de piste  insrer (si >= 0)
	 **/
	void toTrackMenuString(int nTrack, LPTSTR pstr, size_t bufSize, int nNum = -1) const;

	/**
	 * Dtermination du mappage de tous les PIDs de la chane ;
	 * Si \p reset_son est \p true, alors le canal son est, en mme temps, remis  sa
	 * valeur par dfaut, selon prfrences globales de l'utilisateur
	 **/
	MappingData get_MappingData(bool reset_son) const;

	/**
	 * Mappage du PID pmt et dbranchement de tous les autres ;
	 **/
	HRESULT suspend() const;

	/**
	 * Mappage de tous les PIDs de la chane (la frquence est prsume dj syntonise) ;
	 * Si \p reset_son est \p true, alors le canal son est, en mme temps, remis  sa
	 * valeur par dfaut, selon prfrences globales de l'utilisateur
	 **/
	HRESULT branche(bool reset_son) const;

	/// Dterminer s'il s'agit d'une chane audio (radio numrique)
	bool isAudio() const {
		return video_type == vst_Unknown && video_pid == 0 && nbr > 0;}

	/// Dterminer si l'enregistrement en mode PS est disponible
	bool isPsRecordAvailable() const {
			return video_type==vst_MPEG2;}	// PS en MPEG2 seulement pour le moment

	/// Vrifier si le multiplex de cette chane est le mme que celui de la chane \p cChan
	bool sameMultiplex(const Chaine & cChan) const {
			return khz == cChan.khz; }

	/// Vrifier si le type vido de cette chane est le mme que celui de la chane \p cChan
	bool sameVideoType(const Chaine & cChan) const {
			return video_type == cChan.video_type; }

	/// Vrifier si le multiplex ET le type de la vido sont les mmes que ceux de la chane \p cChan
	bool sameMultiplexAndVideoType(const Chaine & cChan) const {
			return sameMultiplex(cChan) && sameVideoType(cChan); }
};

/// Type tableau des chanes
class Chaines : public std::vector<Chaine>
{
public:
	Chaines()
	{
	}

#ifdef _XMLUTILS_H_INCLUDED_
	/**
	 * Chargement de l'ensemble des chanes  partir d'un noeud TinyXML construit
	 * \param[in]	pXmlElm	Noeud XML contenant les donnes  charger
	 * \return		\p true ou \p false selon succs ou chec
	 **/
	bool SetFromXMLElement(const TiXmlElement * pXmlElm);

	/**
	 * Construction d'un noeud TinyXML reprsentant la dfinition de l'ensemble des chanes
	 * \return	Noeud XML construit
	 **/
	TiXmlElement * GetAsXMLElement() const;
#endif

	/**
	 * Tri des chanes par numro.
	 * En cas d'identit de numro (numro pas attribu = 0, par exemple),
	 * les noms de chanes sont compars.
	 **/
	void tri() {
		std::sort(begin(), end()); }

	/**
	 * Recherche d'une chane en partant de son numro
	 * \param[in]	nNumChaine	Numro de la chane, tel que dfini dans la configuration des
	 *							chanes et tel qu'affich  l'utilisateur
	 * \retval		Index de la chane dans la table #Canaux
	 * \retval		-1 si le numro n'a pas t trouv
	 **/
	int trouve_par_no(WORD nNumChaine) const;

	/**
	 * Recherche d'une chane en partant de son SID ("Service ID")
	 * \param[in]	sidChaine	SID de la chane, tel que spcifi dans le flux TS
	 * \retval		Index de la chane dans la table #Canaux
	 * \retval		-1 si le numro n'a pas t trouv
	 * \deprecated	Cette fonction n'identifie pas la chane de faon suffisamment
	 * unique (en particulier dans le cas de rception d'une mme chane par plusieurs
	 * metteurs) ; utiliser ChaineIDs::cle et Chaines::trouve_par_cle  la place.
	 **/
	int trouve_par_sid(WORD sidChaine) const;

	/**
	 * Recherche d'une chane en partant d'une cl 32 bits
	 * \param[in]	cleChaine	Cl 32 bits de la chane, gnre par ChaineIDs::cle
	 * \retval		Index de la chane dans la table #Canaux
	 * \retval		-1 si le numro n'a pas t trouv
	 **/
	int trouve_par_cle(UINT32 cleChaine) const;

	/**
	 * Recherche d'une restauration de chane en partant d'une cl 32 bits
	 * (identique  \p trouve_par_cle, mais retourne la premire chane de la liste
	 * si pas trouv, et ne retourne -1 que si aucune chane n'existe)
	 * \param[in]	cleChaine	Cl 32 bits de la chane, gnre par ChaineIDs::cle
	 * \retval		Index de la chane dans la table #Canaux
	 * \retval		-1 si aucune chane n'existe
	 **/
	int trouve_restauration_par_cle(UINT32 cleChaine) const;

	/**
	 * Recherche d'une chane avec son jeu d'identificateurs complet
	 * \param[in]	sIds	srie d'indentifiants de la chane
	 * \retval		Index de la chane dans la table #Canaux
	 * \retval		-1 si le numro n'a pas t trouv
	 **/
	int trouve_par_Ids(const ChaineIDs & sIds) const;

	/**
	 * Dtermine le temps restant avant le prochain vnement EPG concernant n'importe quelle
	 * chane.
	 * \return		Temps restant, exprim en millisecondes, ou bien #DUREE_INFINIE si aucun temps
	 *				ne peut tre dtermin (aucune mission dfinie dans l'EPG ...).
	 **/
	long timeBeforeNextEvent();

	/// Obtenir l'index d'une chane  partir d'un itrateur
	int	iterToIndex(const_iterator it) const {
			return (int)distance(begin(), it); }
};


/// Tableau des chanes
extern Chaines	Canaux;

extern int		ixChaineCourante;	//!< Index de la chane couramment utilise
extern int		ixChaineAvance;		/*!< Index de la chane pendant les transitions de zapping, ainsi qu'en
										 phase de dmarrage (normalement identique  \p ixChaineCourante, sauf
										 entre la slection d'une chane et l'application de cette slection) */
extern int		ixDerniereChaine;	//!< Mmorisation dernire chane regarde (avant chane courante)
extern int		ixSonCourant;		//!< Index de la piste sonore couramment utilise
extern bool		suspendu;			//!< \p true si vido et audio suspendus ( l'initialisation,
									//!< ou si minimis + \p suspend_minimized)

/**
 * Syntonisation d'une frquence dans le tuner.
 * \param[in]	nFreq		Frquence  syntoniser, exprime en kilohertz
 * \param[in]	nBandwidth	Largeur de bande, exprime en megahertz
 * \param[in]	nNoCanal	Numro du canal  explorer (en principe de 21  69)
 * \param[in]	pChIDs		Autres informations optionnelles concernant la chane
 * \param[in]	pstrErr		Pointeur sur une variable recevant un dtail des erreurs si non NULL
 **/
HRESULT change_frequence(LONG nFreq, LONG nBandwidth, int nNoCanal=-1, const ChaineIDs * pChIDs=NULL, tstring * pstrErr=NULL);

/**
 * Changement de chane, slection par index dans la table.
 * \param[in]	ixChaine	Index dans la table #Canaux.
 *							(mettre -1 pour dsactiver la slection de chane)
 * \param[in]	bForceTune	Si \p true, le  tuner est resyntonis mme si la frquence
 *							ne change pas
 * \retval		S_OK si pas d'erreur
 **/
HRESULT change_chaine(int ixChaine, bool bForceTune=false);

/**
 * Changement de piste son
 * \param[in]	ixSon	Index de la nouvelle piste audio dans la table \p sons
 *						des pistes de la chane
 **/
void switch_audio(int ixSon);

/**
 * Dconnexion globale des sorties vido et audio
 **/
bool suspend();

/**
 * Reconnexion globale des sorties vido et audio
 * \param[in]	reset_son	\p true si le canal son doit en mme temps tre remis  sa
 *							valeur par dfaut, selon les prfrences globales de l'utilisateur
 **/
bool rebranche(bool reset_son);

/**
 * Vrification de la validit d'un index de chane
 * \param[in]	ixChaine	Index dans la table #Canaux
 * \retval		\p true si l'index est valide.
 **/
inline bool ixChaine_ok(int ixChaine);

/**
 * Trouver le SID correspodant  un index de chane.
 * \param[in]	ixChaine	Index dans la table #Canaux
 * \note Si on passe \p STREAM_PSEUDO_INDEX en paramtre, la fonction retourne \p STREAM_PSEUDO_SID.
 * Ceci est utilis en interne pour grer les enregistrements de multiplex entier.
 * \deprecated Remplacer par #indexToCle
 **/
WORD indexToSID(int ixChaine);

/**
 * Trouver la cl correspodant  un index de chane.
 * \param[in]	ixChaine	Index dans la table #Canaux
 * \note Si on passe \p STREAM_PSEUDO_INDEX en paramtre, la fonction retourne \p STREAM_PSEUDO_CLE.
 * Ceci est utilis en interne pour grer les enregistrements de multiplex entier.
 **/
UINT32 indexToCle(int ixChaine);
