/*
 * $Id: config.h 271 2010-12-25 17:33:54Z gingko $
 * Copyright (C) 2009 gingko - http://gingko.homeip.net/
 *
 * This file is part of Pouchin TV Mod, a free DVB-T viewer.
 * See http://www.pouchintv.fr/ for updates.
 *
 * Pouchin TV Mod is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Pouchin TV Mod is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 *
 *
 * Most files of this project have been changed from the Pouchin TV
 * project (Copyright (C) 2006 Pouchin <pouchinteve@free.fr>),
 * to use with a modified version of this software.
 * See http://pouchinteve.free.fr/ for the original project.
 */

/** \file
 *	\brief Configuration de la compilation,  l'chelle de la solution "Pouchin TV Mod".
 *
 *	Dans ce fichier sont dfinies les macros permettant de d'activer ou de dsactiver
 *	certaines fonctionnalits dans la compilation.
 *	Ce fichier est normalement inclus depuis les proprits de chacun des projets de la solution.
 *	("Proprits de configuration", "C/C++", "Avanc", "Fichiers Include forcs").
 *  Certains projets de cette solution peuvent ventuellement inclure un fichier analogue,
 *	pour les dfinitions qui doivent leur rester strictement locales.
 **/

#pragma once

/** \defgroup MiscTestMacros Macros d'exprimentations diverses
 * Macros destines  activer ou dsactiver des fonctionnalits du logiciel,
 *  titre permanent ou bien en vue d'exprimentations pour des volutions futures,
 * ou encore pour pouvoir restaurer d'anciennes fonctionnalits.
 * @{ */

// ====================================================================================
// Options diverses
// ====================================================================================

/**
 * Activation / dsactivation du LCD Logitech
 **/
#ifdef USE_LOGITECH_LCD	// (si dfini par le projet)
	#undef USE_LOGITECH_LCD
	#define USE_LOGITECH_LCD	1
#else
  #ifdef _DEBUG
	#define USE_LOGITECH_LCD	1
  #else
	#define USE_LOGITECH_LCD	0
  #endif
#endif

/**
 * Activation (1) ou dsactivation (0) de la sauvegarde des informations
 * sur les flux vidos, son et autre/
 * \test La dsactivation semble poser problme, notamment vis  vis des
 * enregistrements programms. Pour le moment, laisser activ et ne se
 * servir de ceci que pour des tests d'volutions progressives visant 
 * permettre une dsactivation sre.
 * \test Exprimental
 **/
#define STREAMS_SAVE_ALL		1

/**
 * Mise  jour de l'application  l'aide d'un module externe
 **/
#define UPDATE_VIA_EXTERNAL		1

/**
 * La programmation d'enregistrements depuis l'EPG charge les donnes dans le
 * dialogue des enregistrements programms plutt que de crer directement la
 * programmation
 **/
#define EPG_RECORD_VERIFICATION	1

// ====================================================================================
// Options lies au graphe
// ====================================================================================

/**
 * Activation / dsactivation de la resyntonisation automatique.
 **/
#define USE_AUTO_RETUNE			1

/**
 * Si cette macro est  1, le logiciel tente d'utiliser les broches existantes du
 * dmultiplexeur (pour l'audio et la vido) et n'en cre de nouvelles que l o aucune
 * broche approprie n'existe.
 * \note La prsence de telles broches n'est possible que si le filtre
 * "BDA MPEG2 Transport Information Filter" est utilis, et elle pose un problme encore
 *  rsoudre qui est que ces broches sont sujets  des "MapPID/UnmapPID" automatiques
 * qui entrent en conflit avec les ntres. De plus, ceci est susceptible d'empcher la
 * dconnexion des sorties lorsque le programme est minimis.
 * \test Pour exprimentations
 **/
#define	USE_EXISTING_DEMUX_PINS	0

/**
 * Tentative de validation de l'option de dsentrelacement VMR qui n'a jamais t
 * rellement active  ce jour.
 **/
#define USE_VMR_DEINTERLACE		0

/**
 * Utilisation d'une seule broche de sortie audio sur le dmultiplexeur, ainsi que
 * d'un seul filtre de rendu audio.
 * \note	La slection de cette configuration, dans l'tat actuel du code, est
 *			susceptible d'induire un dlai si un changement de chane ou de piste audio
 *			ncessite un changement de codec audio (au mme titre actuellement que lors
 *			du changement de piste vido ; cependant, les deux dlais ne se cumulent pas
 *			si les deux codecs doivent tre changs en mme temps).
 **/
#define SINGLE_AUDIO_RENDERER	0

/**
 * Activation / dsactivation de l'utilisation du filtre PMT
 * \test Pour exprimentations
 **/
#define USE_PMTFILTER			1

/**
 * Macro permettant d'activer ou de dsactiver l'exportation du graphe
 * ( l'usage de Graphedit ou d'un logiciel quivalent, genre GraphStudio).
 **/
#define EXPORT_GRAPH			1

/**
 * Nouvelle version de la sortie VMR9. Dsactiv pour le moment car posant problme avec
 * certains protocoles de communication avec les codecs.
 **/
#define NEW_VMR9_OUTPUT			0

/**
 * Macro visant  simuler l'existence d'une radio numrique RNT du style de celles qui existent
 * en Belgique : si sa valeur est diffrente de 0, alors elle est un numro de PID vido qui ne
 * sera PAS reconnu (par exemple 620 pour Gulli en France  Paris), et la chane correspondante
 * sera vue comme si elle tait une radio.
 * \test Pour debugging
 **/
#ifdef _DEBUG
	#define TEST_RNT_RADIO		0 // 0 ou 620
#else
	#define TEST_RNT_RADIO		0
#endif

/**
 * Activation / dsactivation de la journalisation dans DirectShow
 **/
#ifdef _DIAGNOSTIC
	#define LOG_DSHOW			1
#else
	#define LOG_DSHOW			0
#endif

/**
 * \test Protection des mthodes \p IMPEG2PIDMap::UnmapPID par des appels \p IBDA_DeviceControl::StartChanges
 * dans l'espoir d'viter des deadlocks priodiques lors de l'appel  \p IMPEG2PIDMap::UnmapPID.
 * Ceci n'est aucunement document dans DirectShow, mais un examen du code du filtre Mpeg2 Demultiplexer montre
 * qu'il y semble tre procd ainsi.
 * \note Dernires nouvelles : a ne marche pas !
 **/
#define PROTECT_UNMAP_PID		1

/**
 * Certaines fonctions ou mthodes, qui ne sont plus (ou pas encore) utilises, sont tout de mme laisses
 * dans le code en prvision de leur (r)utilisation future ventuelle. Ces fonctions sont actives uniquement
 * en mode debugging, afin de s'assurer qu'elles restent compatibles avec les autres modifications du code.
 **/
#ifdef _DEBUG
	#define BUILD_UNUSED_CODE	1
#else
	#define BUILD_UNUSED_CODE	0
#endif

/**
 * Si cette valeur est gale  1, les objets OSD sont rendus visibles par ajout de trames, encadrements ou couleurs
 * (dans les compilations de debugging seulement !), facilitant le dbogage de ces objsts OSD.
 **/
#define OSD_DEBUGGING			0

/** @} */

// ====================================================================================

#if defined(_DEBUG) || defined(_DIAGNOSTIC)
	#define USE_CONSOLE 1
#endif
