/*
 * $Id: epg.cpp 276 2011-03-06 10:17:01Z gingko $
 * Copyright (C) 2006 Pouchin <pouchinteve@free.fr>
 *
 * This file is part of Pouchin TV Mod, a free DVB-T viewer.
 * See http://www.pouchintv.fr/ for updates.
 *
 * Pouchin TV Mod is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Pouchin TV Mod is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 *
 *
 * Most files of this project have been changed from the Pouchin TV
 * project (Copyright (C) 2006 Pouchin <pouchinteve@free.fr>),
 * to use with a modified version of this software.
 * See http://pouchinteve.free.fr/ for the original project.
 */

/** \file
 *	\brief Implmentation du dialogue EPG
 *
 *	Fichier contenant les fonctionnalits lies  l'interface utilisateur du
 *	guide lectronique de programmation (EPG).
 **/

#include "stdafx.h"
#include "dlgutils.h"
#include "record.h"
#include "recprog.h"
#include "channels.h"
#include "epg.h"

#define EPG_COL_NO			0	///< Colonne indiquant le numro de la chane
#define EPG_COL_CHAINE		1	///< Colonne indiquant le nom de la chane
#define EPG_COL_DEBUT		2	///< Colonne indiquant l'heure de dbut
#define EPG_COL_FIN			3	///< Colonne indiquant l'heure de fin
#define EPG_COL_PROGRAMME	4	///< Colonne indiquant le nom de du programme
#define EPG_COL_DESCRIPT	5	///< Colonne indiquant la description du programme

/**
 * Ajout de l'mission dcrite dans la liste du dialogue
 * \param[in]	pEmis		mission  ajouter
 * \param[in]	hListItem	HANDLE du contrle d'affichage
 * \param[in]	nItm		Index de cet enregistrement dans la table
 * \param[in]	bSel		\p true si l'item est  slectionner
 * \note		\p nItm correspondra au numro d'item de la liste au moment de son chargement, mais
 *				ce ne sera plus vrai une fois que cette liste aura t trie.
 * \note		Pour ajouter l'mission dans l'item de liste de dialogue, on place un pointeur sur
 *				le descripteur d'mission dans le champ "lParam" de l'item. Le problme est que ce
 *				champ ne peut contenir qu'un pointeur simple alors que l'item de l'mission est point
 *				par un "EmissionPtr" qui est en fait un "shared_ptr" (pointeur avec comptage de rfrence)
 *				qui ne rentre pas dans cet espace mmoire. Ceci oblige donc  allouer un pointeur sur
 *				l'objet "EmissionPtr" et  stocker ce pointeur sur pointeur dans le champ "lParam".
 *				Le pointeur sur pointeur sera lui-mme dtruit plus tard lors de la rception de la
 *				notification \p LVN_DELETEITEM. Ce n'est pas trs lgant mais bon, on fait comme on peut ...
 **/
void CEpgDialog::ajout_emission(const EmissionPtr & pEmis, HWND hListItem, int nItm, bool bSel)
{
	const Emission &	cEmis = *pEmis;
	const Chaine &		cChan = cEmis.trouve_chaine();
	tstring				strNomChaine;
	TCHAR				str[64];

	_stprintf_s(str, TEXT("%u"), cChan.nNumeroChaine);

	LVITEM	item = {
		LVIF_TEXT | LVIF_PARAM,			// mask
		nItm,							// iItem
		EPG_COL_NO,						// iSubItem
		0, 0,							// state, stateMask
		str,							// pszText
		0,								// cchTextMax
		0,								// iImage
		LPARAM(new EmissionPtr(pEmis))	// lParam
	};

	SendMessage(hListItem, LVM_INSERTITEM, 0, (LPARAM)&item);

	strcpy_T(str, cChan.nom);
	item.iSubItem = EPG_COL_CHAINE;
	SendMessage(hListItem, LVM_SETITEMTEXT, nItm, (LPARAM)&item);

	DateTimeToStr(cEmis.debut, str, _countof(str));
	item.iSubItem = EPG_COL_DEBUT;
	SendMessage(hListItem, LVM_SETITEMTEXT, nItm, (LPARAM)&item);

	TimeToStr(cEmis.fin, str, _countof(str));
	item.iSubItem = EPG_COL_FIN;
	SendMessage(hListItem, LVM_SETITEMTEXT, nItm, (LPARAM)&item);

	item.pszText = const_cast<LPTSTR>(cEmis.strNom.c_str());
	item.iSubItem = EPG_COL_PROGRAMME;
	SendMessage(hListItem, LVM_SETITEMTEXT, nItm, (LPARAM)&item);

	item.pszText = const_cast<LPTSTR>(cEmis.getShortDescr().c_str());
	item.iSubItem = EPG_COL_DESCRIPT;
	SendMessage(hListItem, LVM_SETITEMTEXT, nItm, (LPARAM)&item);

	cEmis.bUsed = true;	// Marquer que l'item d'mission existe dans la liste

	if (bSel) {
		item.stateMask	= LVIS_SELECTED|LVIS_FOCUSED;
		item.state		= LVIS_SELECTED|LVIS_FOCUSED;
		SendMessage(hListItem, LVM_SETITEMSTATE, nItm, (LPARAM)&item);
	}
}

/**
 * \brief Gnration du titre et du texte du "tooltip"
 * \param[in]	nItem		Numro de l'item de la liste
 * \param[in]	nSubItem	Numro du sous-item de la liste
 **/
bool CEpgDialog::SetToolTipText(int nItem, int nSubItem)
{
	tstring		strTitle;

	strToolTipMsg.clear();

	if (nItem >= 0 && nSubItem >= 0) {
		const Emission &	cEmis	= **GetItemData(nItem);

		switch (nSubItem) {

	#ifdef _DIAGNOSTIC
		case EPG_COL_CHAINE : {
			strTitle	= cEmis.getName().c_str();
			strToolTipMsg = tstr_printf(TEXT("Id = 0x%04x, content=0x%08x"), cEmis.nEvtId, cEmis.nContent);
			if (cEmis.nFlags & DEF_IN_ACTUAL)
				strToolTipMsg += TEXT(", ");
			if (cEmis.nFlags & DEF_IN_SCHEDULE)
				strToolTipMsg += TEXT(", ");
			break; }
	#endif

		case EPG_COL_DESCRIPT: {
			strTitle		= cEmis.getName().c_str();
			strToolTipMsg	= cEmis.getLongDescr();
			break; }
		}

		if (!strTitle.empty()) {
			const Chaine &	cChan = cEmis.trouve_chaine();
			HICON			hIcon = cChan.GetIconAsIco();	// Rcupration de l'icne
			INT				nLen  = (INT)strToolTipMsg.length();

			myprintf(TEXT("strToolTipMsg.length = %u") EOL, nLen);
			SendMessage(hToolTip, TTM_SETMAXTIPWIDTH, 0, 360+(INT)sqrt((float)nLen)*8);
			if (strTitle.length() > 99) {
				// Selon la documentation, un titre de tooltip ne peut excder 100 caractres, incluant le terminateur
				strTitle.resize(98);
				strTitle += TEXT("");	// points de suspension
			}
			SendMessage(hToolTip, TTM_SETTITLE, (WPARAM)(hIcon ? hIcon : (HICON)TTI_INFO), (LPARAM)strTitle.c_str());
			return true;
		}
	}
	return false;
}

void CEpgDialog::ProcessToolTipNotifications(LPARAM lParam)
{
	NMTTDISPINFO &	sNmDspInf	= *reinterpret_cast<LPNMTTDISPINFO>(lParam);
	POINT			sPt;
	INT				iItem, iSubItem;

	GetCursorPos(&sPt);

	if (FindListViewSubItem(hList, sPt, false, iItem, iSubItem)) {
		SetToolTipText(iItem, iSubItem);
		// myprintf(TEXT("TTN_GETDISPINFO") EOL);
		sNmDspInf.lpszText = const_cast<LPTSTR>(strToolTipMsg.c_str());
	}
}


BOOL AddTool(HWND hToolTip, HWND hDlg, HWND hCtl, LPCTSTR pszMsg, bool bSubClass)
{
	TOOLINFO sToolInfo = {
		sizeof(sToolInfo),			// cbSize
		(bSubClass ? TTF_SUBCLASS : 0)|TTF_IDISHWND,	// uFlags
		hDlg,						// hwnd
		(UINT_PTR)hCtl,				// uId
		{0},						// rect
		hAppInstance,				// hinst
		LPSTR_TEXTCALLBACK			// lpszText
	};

	return (BOOL)SendMessage(hToolTip, TTM_ADDTOOL, 0, (LPARAM)&sToolInfo);
}

/// Slection ou changement de l'icne systme
void CEpgDialog::SetSysIcon()
{
	if (hCurIcon != hSysIcon) {
		hSysIcon = hCurIcon;
		SetClassLongPtr(hDlg, GCLP_HICON, (LONG_PTR)hSysIcon);
	}
}

/// Initialisation
void CEpgDialog::InitializeList()
{
	static const Tab_cols t_cols[] = {
		{TEXT("N"),				28},
		{TEXT(""),			90},
		{TEXT(""),				102},
		{TEXT(""),				64},
		{TEXT("  "),	220},
		{TEXT(""),		1024},
		{NULL, 0}					// Terminateur
	};

	hList		= GetDlgItem(hDlg, IDC_LIST_EPG);

	// Initialisation des paramtres de tri par dfaut
	nColon		= EPG_COL_NO;
	bDescending	= FALSE;

	// Cration du tooltip
	hToolTip = CreateWindowEx(
		NULL, TOOLTIPS_CLASS, TEXT("ToolTip"), WS_POPUP | TTS_NOPREFIX | TTS_ALWAYSTIP | TTS_BALLOON,
		CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT,
		hList, NULL, hAppInstance, NULL);

	if (hToolTip) {
		// Surclassement de la liste et du tooltip
		WinSubClass<SC_LVW>::SubClass(hList);
		WinSubClass<SC_TTP>::SubClass(hToolTip);

		// Initialisation du tooltip
		SetWindowPos(hToolTip, HWND_TOPMOST, 0, 0, 0, 0,
			 SWP_NOMOVE | SWP_NOSIZE | SWP_NOACTIVATE);

		ListView_SetToolTips(hList, hToolTip);

		TOOLINFO sToolInfo = {
			sizeof(sToolInfo),			// cbSize
			TTF_SUBCLASS|TTF_IDISHWND,	// uFlags
			hDlg,						// hwnd
			(UINT_PTR)hList,			// uId
			{0},						// rect
			hAppInstance,				// hinst
			LPSTR_TEXTCALLBACK			// lpszText
		};

		SendMessage(hToolTip, TTM_ADDTOOL, 0, (LPARAM)&sToolInfo);
		SendMessage(hToolTip, TTM_SETTIPBKCOLOR, (WPARAM)RGB(255,255,255), NULL);				// Fond blanc
		SendMessage(hToolTip, TTM_SETDELAYTIME, TTDT_AUTOPOP, (LPARAM)MAKELONG(MAXSHORT, 0));	// MAXSHORT est le maximum admissible !
		SendMessage(hToolTip, TTM_SETDELAYTIME, TTDT_RESHOW, (LPARAM) MAKELONG(MAXSHORT, 0));
		SendMessage(hToolTip, TTM_SETMAXTIPWIDTH, 0, 480);
	}

	// Initialise la liste (ajout de la grille, des colonnes, etc.)
	ListView_AjouteColonnes(hList, t_cols);
	remplit_table_epg(epg_timelimit);
}

/**
 * Fonction utilise pour trier la liste des missions
 *
 * \param[in]  lParam1     Pointeur sur Pointeur (EmissionPtr) sur le 1er lment
 * \param[in]  lParam2     Pointeur sur Pointeur (EmissionPtr) sur le 2nd lment
 * \param[in]  lParamSort  paramtres de tri
 * \retval     <0          lParam1 < lParam2
 * \retval     0           lParam1 = lParam2
 * \retval     >0          lParam1 > lParam2
 **/
int CALLBACK CEpgDialog::LVEpgCompareProc(LPARAM lParam1, LPARAM lParam2,  LPARAM lParamSort)
{
	CEpgDialog &		cList	= *reinterpret_cast<CEpgDialog *>(lParamSort);
	const Emission &	cEmis1	= **reinterpret_cast<const EmissionPtr *>(lParam1);
	const Emission &	cEmis2	= **reinterpret_cast<const EmissionPtr *>(lParam2);
	int					res		= 0; // rsultat

	switch (cList.nColon) {

	case EPG_COL_NO:
		res = cEmis1.channelNo() - cEmis2.channelNo();
		if (res == 0)
			res = DiffTime(cEmis1.debut, cEmis2.debut);
		break;

	case EPG_COL_CHAINE:
		res = CompareStringA(LOCALE_NEUTRAL, NORM_IGNORECASE, cEmis1.trouve_chaine().nom, -1, cEmis2.trouve_chaine().nom, -1);
		if (res == CSTR_EQUAL || res == 0)
			res = DiffTime(cEmis1.debut, cEmis2.debut);
		else
			res -= 2;
		break;

	case EPG_COL_DEBUT:
		res = DiffTime(cEmis1.debut, cEmis2.debut);
		if (res == 0)
			res = cEmis1.channelNo() - cEmis2.channelNo();
		break;

	case EPG_COL_FIN:
		res = DiffTime(cEmis1.fin, cEmis2.fin);
		if (res == 0)
			res = cEmis1.channelNo() - cEmis2.channelNo();
		break;

	case EPG_COL_PROGRAMME:
		res = _tcsicmp(cEmis1.strNom.c_str(), cEmis2.strNom.c_str());
		break;

	case EPG_COL_DESCRIPT:
		res = _tcsicmp(cEmis1.strDesc.c_str(), cEmis2.strDesc.c_str());
	}

	// Tri descendant si pnml->lParam != 0
	if (cList.bDescending)
		res = -res;
	return res;
}

/// Chercher l'item slectionn
int CEpgDialog::GetSelection() const
{
	// S'assurer que l'item slectionn est visible
	int nCount = ListView_GetItemCount(hList);

	for (int nInx = 0; nInx < nCount; ++nInx) {
		if (ListView_GetItemState(hList, nInx, LVIS_SELECTED) != 0)
			return nInx;
	}
	return -1;	// Aucun item slectionn
}

/**
 * Obtenir les donnes de la ligne correspondant  un item de la liste EPG
 *
 * \param[in]	nItem	Index de l'item recherch
 * \return		Pointeur sur le pointeur partag de la ligne de donnes
 * \note		La fonction retourne un pointeur sur EmissionPtr car de dernier objet est allou
 *				sur le tas (heap) et est donc destin  tre dtruit par \p delete.
 **/
EmissionPtr * CEpgDialog::GetItemData(int nItem)
{
	ASSERT(nItem >= 0);

	LVITEM item = {
		LVIF_PARAM,	// mask
		nItem		// iItem
		// le reste  zro (implicite)
	};

	ListView_GetItem(hList, &item);
	return reinterpret_cast<EmissionPtr *>(item.lParam);
}

/**
 * \brief Trouver l'item correspondant  une mission dans la liste.
 *
 * S'il n'existe pas, trouver son point d'insertion, selon le critre de tri courant.
 * \param[in]	pEmis	mission recherche
 * \param[out]	nItem	Item trouv ou point d'insertion
 * \return	\p true si l'mission existe
 **/
bool CEpgDialog::Find(const EmissionPtr & pEmis, int & nItem)
{
	// Recherche dichotomique
	int nMin = 0;
	int nMax = ListView_GetItemCount(hList);
	int nCur = (nMin+nMax) / 2;
	//
	if (nMax > 0)
		do {
			const EmissionPtr * pTmp = GetItemData(nCur);

			// Pour l'identit, on peut comparer les pointeurs
			if (pTmp->get() == pEmis.get()) {
				nItem = nCur;
				return true;
			}

			// Utilisation de la fonction de comparaison dfinie pour le tri
			if (LVEpgCompareProc((LPARAM)pTmp, (LPARAM)&pEmis, (LPARAM)this) < 0)
				nMin = nCur+1;
			else
				nMax = nCur;
			nCur = (nMin+nMax) / 2;
		}
		while (nMin < nMax);
	//
	nItem = nCur;
	return false;
};

/**
 * \brief Remplit la liste des programmes via l'EPG
 * \param[in]	eEpgTime	Temps limite EPG
 **/
void CEpgDialog::remplit_table_epg(EpgTimeLimit eEpgTime)
{
	int		nMax = ListView_GetItemCount(hList);	// Nombre d'items existants
	int		ixChaine = 0;
	int		nItm;

	// Marquer tous les items pour pouvoir supprimer ceux qui n'existent plus
	for (nItm = 0; nItm < nMax; ++nItm)
		(*GetItemData(nItm))->bUsed = false;

	SYSTEMTIME	sCurrTime;

	GetLocalTime(&sCurrTime);

	static long nHorizTime [] = {
		0,				// eh_cur_next
		DUREE_1H,		// eh_1_hour
		2*DUREE_1H,		// eh_2_hours
		3*DUREE_1H,		// eh_3_hours
		4*DUREE_1H,		// eh_4_hours
		6*DUREE_1H,		// eh_6_hours
		9*DUREE_1H,		// eh_9_hours
		12*DUREE_1H,	// eh_12_hours
		18*DUREE_1H,	// eh_18_hours
		DUREE_24H,		// eh_24_hours
		2*DUREE_24H,	// eh_2_days
		3*DUREE_24H,	// eh_3_days
		MAXLONG			// eh_nolimit
	};

	long	nHorizon = nHorizTime[eEpgTime];

	ixChaine = 0;
	for (ITERATE_CONST_VECTOR(Canaux, Chaine, itc)) {
		const Chaine & cChan = *itc;

		if (cChan.etat != ec_inactive) {
			CAutoLock   cLock(cChan.crit_sec());

			for (ITERATE_CONST_LIST(cChan, EmissionPtr, it)) {
				const EmissionPtr & pEmis = *it;

				// Ignorer les enregistrements prims
				if (DiffTime(sCurrTime, pEmis->fin) > 10*DUREE_1MN)	// ... de plus de 10 minutes
					continue;
				if ((pEmis->nFlags & DEF_IN_ACTUAL) == 0) {
					// Ne pas filtrer les programmes "actuel/suivant"
					if (nHorizon == 0)
						continue;
					if (DiffTime(pEmis->debut, sCurrTime) > nHorizon)
						continue;
				}

				if (cleChaine == 0 && ixChaine == ixChaineCourante) {
					// Si aucune slection n'tait prcdemment dfinie,
					// alors on slectionne l'mission en cours
					cleChaine = cChan.cle();
					nEvtId = pEmis->nEvtId;
				}

				// Rechercher le point d'insertion de l'mission dans la liste
				if (Find(pEmis, nItm)) {
					// L'mission existe : on se contente de la marquer pour qu'elle
					// ne soit pas efface
					pEmis->bUsed = true;
				} else {
					// L'mission n'existe pas : l'insrer  l'emplacement trouv
					ajout_emission(*it, hList, nItm,
						pEmis->cleChaine == cleChaine && pEmis->nEvtId == nEvtId);
				}
			}
		}
		ixChaine++;	// Incrmenter index chane
	}

	// Supprimer les items qui n'existent plus
	nItm = ListView_GetItemCount(hList);
	while (nItm > 0) {
		--nItm;
		if (!(*GetItemData(nItm))->bUsed)
			ListView_DeleteItem(hList, nItm);
	}

	// S'assurer que l'item slectionn est visible
	// nItm = GetSelection();
	// if (nItm >= 0)
	// 	ListView_EnsureVisible(hList, nItm, FALSE);
}

/**
 * Dfinition d'un timer dont la dure est gale au temps avant l'heure la plus proche
 * dans les dbuts ou fins d'missions de l'EPG.
 **/
static void SetEpgEventTimer(HWND hDlg)
{
	long			nEvtDly	= Canaux.timeBeforeNextEvent();

	if (nEvtDly != DUREE_INFINIE) {
		myprintf(TEXT("SetEpgEventTimer = %.2f\n"), nEvtDly/1000.0);
		SetTimer(hDlg, TIMER_EPG_EVENT, nEvtDly, NULL);
	} else
		KillTimer(hDlg, TIMER_EPG_EVENT);
}

/// Traitement des messages de la liste d'items surclasse
LRESULT CEpgDialog::SubProc(WinSubClass<SC_LVW> & sc, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch (uMsg) {

	case WM_MOUSEMOVE: {
		POINT sPt = {GET_X_LPARAM(lParam), GET_Y_LPARAM(lParam)};

		// Filtrage des messages WM_MOUSEMOVE si le dplacement par rapport au prcdent est trop faible
		if (abs(sPt.x - ptSaveLvw.x) < 8 && abs(sPt.y - ptSaveLvw.y) < 8)
			return 0;

		ptSaveLvw = sPt;
		break; }
	}
	return sc.CallWindowProc(uMsg, wParam, lParam);
}

/// Traitement des messages du tooltip surclass
LRESULT CEpgDialog::SubProc(WinSubClass<SC_TTP> & sc, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch (uMsg) {

	case WM_MOUSEMOVE: {
		POINT sPt = {GET_X_LPARAM(lParam), GET_Y_LPARAM(lParam)};

		// Filtrage des messages WM_MOUSEMOVE si le dplacement par rapport au prcdent est trop faible
		if (abs(sPt.x - ptSaveTtp.x) < 8 && abs(sPt.y - ptSaveTtp.y) < 8)
			return 0;
		ptSaveTtp = sPt;
		break; }

	case TTM_RELAYEVENT:
		// Pas de tooltip si on a un message NC_MOUSEMOVE, transmis depuis le rectangle non-client
		// de la fentre parente (sinon, on a des tooltips mme au-dessus de la barre de dfilement).
		if (reinterpret_cast<const MSG *>(lParam)->message == WM_NCMOUSEMOVE) {
			// Et mme, enlever  ce moment-l tout tooltip rsiduel
			PostMessage(sc.hSWnd, TTM_POP, 0, NULL);
			return 0; }
	}
	return sc.CallWindowProc(uMsg, wParam, lParam);
}

/// Traitement des messages de la bote de dialogue
INT_PTR CEpgDialog::DialogProc(UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch (uMsg) {

	case WM_INITDIALOG: {
		static LPCTSTR aTimeInitList[] = {	// non static
			TEXT("/"),  // eh_cur_next
			TEXT("1 "),		 // eh_1_hour
			TEXT("2 "),		 // eh_2_hours
			TEXT("3 "),		 // eh_3_hours
			TEXT("4 "),		 // eh_4_hours
			TEXT("6 "),		 // eh_6_hours
			TEXT("9 "),		 // eh_9_hours
			TEXT("12 "),		 // eh_12_hours
			TEXT("18 "),		 // eh_18_hours
			TEXT("24 "),		 // eh_24_hours
			TEXT("2 "),		 // eh_2_days
			TEXT("3 "),		 // eh_3_days
			TEXT(" "),		 // eh_nolimit
			NULL
		};

		InitializeList();
		SetSysIcon();
		init_methodes(hDlg);
		ComboBox_Init(GetDlgItem(hDlg, IDC_EPG_TIME), aTimeInitList, epg_timelimit);
		SetEpgEventTimer(hDlg);
		return TRUE; }

	case WM_APPCOMMAND:
		switch (GET_APPCOMMAND_LPARAM(lParam)) {

		case APPCOMMAND_BROWSER_REFRESH:	// Actualise la liste des programmes
			PostDlgMessage(WM_COMMAND, IDC_RESET_EPG);
			return TRUE;

		}
		break;

	case WM_GETMINMAXINFO: {
		MINMAXINFO & sMinMax = *reinterpret_cast<MINMAXINFO *>(lParam);

		// Mmoriser l'ancienne taille pour calcul du changement :
		GetClientRect(hDlg, &sSavRect);
		// Dimensions minimales acceptables :
		sMinMax.ptMinTrackSize.x = 680;
		sMinMax.ptMinTrackSize.y = 150;
		break; }

	case WM_SIZE:{
		RECT	sRect;

		GetClientRect(hDlg, &sRect);

		// Calcul de la diffrence entre l'ancienne et la nouvelle taille :
		int		dx = RWdt(sRect)-RWdt(sSavRect);
		int		dy = RHgt(sRect)-RHgt(sSavRect);

		// Dplacement ou redimensionnement des contrles :
		ChgCtlPos(IDC_METHOD_S,		0,	dy);
		ChgCtlPos(IDC_METHOD,		0,	dy);
		ChgCtlPos(IDC_AUDIO_S,		0,	dy);
		ChgCtlPos(IDC_AUDIO,		0,	dy);
		ChgCtlPos(IDC_EPG_TIME_S,	0,	dy);
		ChgCtlPos(IDC_EPG_TIME,		0,	dy);
		ChgCtlPos(IDC_RECORD,		0,	dy);
		ChgCtlPos(IDC_RESET_EPG,	dx, dy);
		ChgCtlPos(IDOK,				dx,	dy);
		ChgCtlSiz(IDC_LIST_EPG,		dx, dy);
		return TRUE; }

	case WM_TIMER:
		switch (wParam) {

		case TIMER_EPG_UPDATE:
			PostDlgMessage(WM_APP_EPG_UPDATED);
			return TRUE;

		case TIMER_EPG_EVENT:
			PostDlgMessage(WM_APP_EPG_EVENT);
			return TRUE;
		}
		break;

	case WM_APP_EPG_UPDATED: {
		// Limiter les mises  jour  une par seconde
		#define EPG_UPDATE_LIMIT 1000	// 1 seconde

		DWORD			dwTicks = GetTickCount();
		UINT			nDiff	= dwTicks-dwLastEpgUpdate;

		KillTimer(hDlg, TIMER_EPG_UPDATE);
		if (nDiff < EPG_UPDATE_LIMIT) {
			SetTimer(hDlg, TIMER_EPG_UPDATE, EPG_UPDATE_LIMIT-nDiff, NULL);
			return TRUE;
		}
		dwLastEpgUpdate = dwTicks;

		remplit_table_epg(epg_timelimit);

		SetEpgEventTimer(hDlg);
		return TRUE; }


	case WM_APP_EPG_EVENT:
		// Au moins une couleur d'item a vraisemblablement chang :
		// on rafrachit la liste.
		InvalidateRect(hList, NULL, FALSE);
		SetEpgEventTimer(hDlg);

		// Mise  jour de l'icne du menu systme si l'tat d'enregistrement a chang :
		SetSysIcon();
		return TRUE;

	case WM_COMMAND:
		switch(wParam) {

		case _cmd_(IDC_METHOD, CBN_SELCHANGE):
		case _cmd_(IDC_AUDIO, CBN_SELCHANGE):
			update_methodes(hDlg);
			return TRUE;

		case _cmd_(IDC_EPG_TIME, CBN_SELCHANGE):
			epg_timelimit = (EpgTimeLimit)SendDlgItemMessage(hDlg, IDC_EPG_TIME, CB_GETCURSEL, 0, NULL);
			PostDlgMessage(WM_APP_EPG_UPDATED);

			if (epg_timelimit != eh_cur_next) {
				MessageBoxCheck(hDlg,
					TEXT("   ,        ")
					TEXT("        ")
					TEXT(".        ")
					TEXT("  ."),
					TEXT("  "), MB_OK|MB_ICONWARNING,
					MB_OK, TEXT("  EPG"), &bWarnEpgDone);
			}
			// Ne pas laisser le focus sur ce contrle (car cela a tendance  provoquer des
			// changements intempestifs)
			SetFocus(hList);
			return TRUE;

		case _cmd_(IDOK, BN_CLICKED):
		case _cmd_(IDCANCEL, BN_CLICKED):
			DestroyWindow(hDlg);
			return TRUE;

		case _cmd_(IDC_RESET_EPG, BN_CLICKED):
			for (ITERATE_VECTOR(Canaux, Chaine, it))
				it->clear();
			PostDlgMessage(WM_APP_EPG_UPDATED);
			dwLastEpgUpdate = 0;
			return TRUE;

		case _cmd_(IDC_RECORD, BN_CLICKED): {
			int nItem = GetSelection();

			if (nItem >= 0) {
				const Emission &	cEmis = **GetItemData(nItem);

#if EPG_RECORD_VERIFICATION
				MessageBoxCheck(hDlg,
					TEXT("              ")
					TEXT("        ")
					TEXT("(   ).") EOL
					TEXT("         ")
					TEXT("  蠻."),
					TEXT("    "), MB_OK|MB_ICONINFORMATION,

					MB_OK, TEXT("    EPG"), &bWarnRecDone);
				// Ouvrir la fentre des enregistrements programms, pour vrification
				CRecordDialog::ShowDialog(hMainWnd);
				CRecordDialog::PostDlgMessage(WM_APP_ADD_EPG_PROG, cEmis.nEvtId, cEmis.cleChaine);
#else
				const Chaine &		cChan = cEmis.trouve_chaine();
				Programme			prog;

				if (!chaineToProg(cChan, cEmis, prog) || !prog.verifie())
					return FALSE;

				// Ces deux valeurs ont t extraites de la bote de dialogue au moment de leur slection :
				prog.methode	= epg_typerecvideo;
				prog.audio		= epg_typerecaudio;

				Programmes.push_back(prog);
				finalize_prog_change();

				// Ouvrir la fentre des enregistrements programms, pour vrification
				PostMessage(hMainWnd, WM_COMMAND, IDM_DELAYED_RECORD, NULL);
#endif
			} else
				affiche_erreurs(TEXT("  "));
			return TRUE; }
		}
		break;

	case WM_NOTIFY:
		if (_nidfr_(lParam) == IDC_LIST_EPG) {
			// Notification envoye  la liste
			switch (_ncode_(lParam)) {

			case NM_CLICK:
			case NM_DBLCLK: {
				int nItem = reinterpret_cast<LPNMITEMACTIVATE>(lParam)->iItem;

				if (nItem >= 0) {
					const Emission & cEmis = **GetItemData(nItem);

					// Mmorisation de la nouvelle slection
					cleChaine = cEmis.cleChaine;
					nEvtId = cEmis.nEvtId;
					if (_ncode_(lParam) == NM_DBLCLK) {
						// Double clic : on change de chane
						if (cEmis.ixChaine != ixChaineCourante)
							zappe_index(cEmis.ixChaine);
					}
				}
				break; }

			case NM_CUSTOMDRAW: {
				NMLVCUSTOMDRAW &	sNlcd = *reinterpret_cast<LPNMLVCUSTOMDRAW>(lParam);

				switch(sNlcd.nmcd.dwDrawStage) {

				case CDDS_PREPAINT:
					SetWindowLongPtr(hDlg, DWLP_MSGRESULT, CDRF_NOTIFYITEMDRAW);
					return TRUE;

				case CDDS_ITEMPREPAINT:
					// Ajustement des couleurs du texte et du fond
					if (ixChaine_ok(ixChaineCourante)) {
						const Emission &	cEmis = **GetItemData((int)sNlcd.nmcd.dwItemSpec);
						const Chaine &		cChan = cEmis.trouve_chaine();
						SYSTEMTIME			localtime;

						if (cEmis.ixChaine == ixChaineCourante)
							sNlcd.clrTextBk = RGB(192,255,192);	// Chane courante : fond teint de vert
						else if (cChan.sameMultiplex(Canaux[ixChaineCourante]))
							sNlcd.clrTextBk = RGB(255,255,208);	// Mme multiplex : fond teint de jaune

						GetLocalTime(&localtime);

						if (DiffTime(cEmis.debut, localtime) <= 0 && DiffTime(localtime, cEmis.fin) < 0) {
							if (enregistrements_actuels.getRecordingDescrByKey(cEmis.cleChaine) >= 0)
								sNlcd.clrText = RGB(255,0,0);	// mission enregistre en rouge
							else
								sNlcd.clrText = RGB(0,128,255);	// mission en cours en bleu
						} else
							sNlcd.clrText = RGB(128,128,128);	// mission en attente en gris
					}
					return TRUE;
				}
				break; }

			case LVN_COLUMNCLICK: {		// Clic sur la colonne => trie les missions
				// Tri de la liste par clic sur un en-tte de colonne :
				NMLISTVIEW &	sNmLv		= *reinterpret_cast<LPNMLISTVIEW>(lParam);
				int				iCol		= sNmLv.iSubItem;	// N de colonne concerne

				if (nColon != iCol) {
					// Colonne diffrente : tri ascendant
					nColon		= iCol;
					bDescending	= false;
				} else
					// Mme colonne : inversion de l'ordre de tri
					bDescending	= !bDescending;
				SortItems();	// Tri
				break; }

			case LVN_DELETEITEM: {
				// Un item a t dtruit : dcrmenter le compteur de rfrence de l'mission correspondante.
				// Pour cela, tant donn que le pointeur EmissionPtr a lui-mme t allou sur le tas (heap),
				// il faut rcuprer le pointeur sur ce pointeur et dtruire l'objet EmissionPtr allou, ce
				// qui a pour effet de dcrmenter le compteur de rfrence.
				EmissionPtr * ppEmis = GetItemData(reinterpret_cast<NMLISTVIEW *>(lParam)->iItem);

				if (ppEmis)	// Normalement, ceci ne devrait jamais tre NULL ...
					delete ppEmis;
				// myprintf(TEXT("Destruction de d'item N%i") EOL, reinterpret_cast<NMLISTVIEW *>(lParam)->iItem);
				break; }

			case LVN_KEYDOWN:
				// On a appuy sur une touche dans la liste
				if (reinterpret_cast<LPNMLVKEYDOWN>(lParam)->wVKey == VK_F5)
					PostDlgMessage(WM_COMMAND, IDC_RESET_EPG);
			}
		} else if (_ncode_(lParam) == TTN_GETDISPINFO)
			ProcessToolTipNotifications(lParam);
		break;

	case WM_CLOSE:
		DestroyWindow(hDlg);
		return TRUE;

	}

	return FALSE;
}
