/*
 * $Id: epg.h 267 2010-11-19 19:56:02Z gingko $
 * Copyright (C) 2006 Pouchin <pouchinteve@free.fr>
 *
 * This file is part of Pouchin TV Mod, a free DVB-T viewer.
 * See http://www.pouchintv.fr/ for updates.
 *
 * Pouchin TV Mod is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Pouchin TV Mod is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 *
 *
 * Most files of this project have been changed from the Pouchin TV
 * project (Copyright (C) 2006 Pouchin <pouchinteve@free.fr>),
 * to use with a modified version of this software.
 * See http://pouchinteve.free.fr/ for the original project.
 */

/** \file
 *	\brief Accs au dialogue EPG
 *
 *	Fichier d'inclusion pour l'accs au dialogue du guide lectronique de
 *	programmation (EPG).
 **/

#pragma once

#include "dlgutils.h"
#include "resource.h"

#define SC_LVW 0	//!< Identifiant pour le template de surclassement de la liste
#define SC_TTP 1	//!< Identifiant pour le template de surclassement du tooltip

enum EpgTimeLimit {
	eh_cur_next,
	eh_1_hour,
	eh_2_hours,
	eh_3_hours,
	eh_4_hours,
	eh_6_hours,
	eh_9_hours,
	eh_12_hours,
	eh_18_hours,
	eh_24_hours,
	eh_2_days,
	eh_3_days,
	eh_nolimit
};

/**
 * Classe grant la bote de dialogue des programmes via l'EPG
 * \todo Amliorations  envisager :
 * - Envisager sauvegarde de la dimension de cette bote de dialogue,
 *   maintenant qu'elle est redimensionnable.
 * - Griser les chanes n'appartenant pas au mme multiplex lorsque des
 *   enregistrements sont en cours.
 **/
class CEpgDialog : public WinSubClass<SC_LVW>, public WinSubClass<SC_TTP>, public CModelessDialog<CEpgDialog>
{
	// Les deux itemps suivants permettent de reconstituer la slection dans le cas
	// o on rinitialise la liste : ds que l'mission qui tait slectionne
	// rapparat, elle est reslectionne  nouveau.
	UINT32		cleChaine;		//!< Cl de la chane slectionne
	UINT16		nEvtId;			//!< ID de l'mission slectionne

	HWND		hList;			//!< Handle du contrle de liste
	HWND		hToolTip;		//!< Handle du tooltip
	int			nColon;			//!< Numro de colonne du dernier tri
	bool		bDescending;	//!< \p true si le dernier tri tait descendant
	tstring		strToolTipMsg;	//!< Texte courant du tooltip

	RECT		sSavRect;		//!< Mmorisation du rectangle client dans WM_GETMINMAXINFO et avant WM_SIZE
	DWORD		dwLastEpgUpdate;//!< Instant de la dernire mise  jour de l'affichage EPG
	bool		bWarnEpgDone;	//!< \p true si avertissement EPG dj donn
	bool		bWarnRecDone;	//!< \p true si avertissement enregistrement dj donn

	POINT		ptSaveLvw;		//!< Sauvegarde de la position de la souris dans la liste
	POINT		ptSaveTtp;		//!< Sauvegarde de la position de la souris dans le tooltip

	HICON		hSysIcon;		//!< Icne du menu systme

	/// Traitement des messages de la liste d'items surclasse
	virtual LRESULT SubProc(WinSubClass<SC_LVW> & sc, UINT uMsg, WPARAM wParam, LPARAM lParam);

	/// Traitement des messages du tooltip surclass
	virtual LRESULT SubProc(WinSubClass<SC_TTP> & sc, UINT uMsg, WPARAM wParam, LPARAM lParam);

	/// Slection ou changement de l'icne systme
	void SetSysIcon();

	/// Initialisation
	void InitializeList();

	/**
	 * \brief Gnration du titre et du texte du "tooltip"
	 * \param[in]	nItem		Numro de l'item de la liste
	 * \param[in]	nSubItem	Numro du sous-item de la liste
	 **/
	bool SetToolTipText(int nItem, int nSubItem);

	void ProcessToolTipNotifications(LPARAM lParam);

	/**
	 * Ajout de l'mission dcrite dans la liste du dialogue
	 * \param[in]	pEmis		mission  ajouter
	 * \param[in]	hListItem	HANDLE du contrle d'affichage
	 * \param[in]	nItm		Index de cet enregistrement dans la table
	 * \param[in]	bSel		\p true si l'item est  slectionner
	 * \note		\p nItm correspondra au numro d'item de la liste au moment de son chargement, mais
	 *				ce ne sera plus vrai une fois que cette liste aura t trie.
	 * \note		Pour ajouter l'mission dans l'item de liste de dialogue, on place un pointeur sur
	 *				le descripteur d'mission dans le champ "lParam" de l'item. Le problme est que ce
	 *				champ ne peut contenir qu'un pointeur simple alors que l'item de l'mission est point
	 *				par un "EmissionPtr" qui est en fait un "shared_ptr" (pointeur avec comptage de rfrence)
	 *				qui ne rentre pas dans cet espace mmoire. Ceci oblige donc  allouer un pointeur sur
	 *				l'objet "EmissionPtr" et  stocker ce pointeur sur pointeur dans le champ "lParam".
	 *				Le pointeur sur pointeur sera lui-mme dtruit plus tard lors de la rception de la
	 *				notification \p LVN_DELETEITEM. Ce n'est pas trs lgant mais bon, on fait comme on peut ...
	 **/
	static void ajout_emission(const EmissionPtr & pEmis, HWND hListItem, int nItm, bool bSel);

	/**
	 * \brief Trouver l'item correspondant  une mission dans la liste.
	 *
	 * S'il n'existe pas, trouver son point d'insertion, selon le critre de tri courant.
	 * \param[in]	pEmis	mission recherche
	 * \param[out]	nItem	Item trouv ou point d'insertion
	 * \return	\p true si l'mission existe
	 **/
	bool Find(const EmissionPtr & pEmis, int & nItem);

	/**
	 * \brief Remplit la liste des programmes via l'EPG
	 * \param[in]	eEpgTime	Temps limite EPG
	 **/
	void remplit_table_epg(EpgTimeLimit eEpgTime);

	/**
	 * Obtenir les donnes de la ligne correspondant  un item de la liste EPG
	 *
	 * \param[in]	nItem	Index de l'item recherch
	 * \return		Pointeur sur le pointeur partag de la ligne de donnes
	 * \note		La fonction retourne un pointeur sur EmissionPtr car de dernier objet est allou
	 *				sur le tas (heap) et est donc destin  tre dtruit par \p delete.
	 **/
	EmissionPtr * GetItemData(int nItem);

	/// Chercher l'item slectionn
	int GetSelection() const;

	/**
	 * Fonction utilise pour trier la liste des missions
	 *
	 * \param[in]  lParam1     Pointeur sur Pointeur (EmissionPtr) sur le 1er lment
	 * \param[in]  lParam2     Pointeur sur Pointeur (EmissionPtr) sur le 2nd lment
	 * \param[in]  lParamSort  paramtres de tri
	 * \retval     <0          lParam1 < lParam2
	 * \retval     0           lParam1 = lParam2
	 * \retval     >0          lParam1 > lParam2
	 **/
	static int CALLBACK LVEpgCompareProc(LPARAM lParam1, LPARAM lParam2,  LPARAM lParamSort);

	void SortItems() {
			ListView_SortItems(hList, LVEpgCompareProc, this); }

	/// Traitement des messages de la bote de dialogue
	virtual INT_PTR DialogProc(UINT uMsg, WPARAM wParam, LPARAM lParam);

public:

	/// Constructeur
	CEpgDialog() :
		CModelessDialog(IDD_EPG),
		cleChaine(0),
		nEvtId(0),
		hList(NULL),
		hToolTip(NULL),
		nColon(0),
		bDescending(false),
		dwLastEpgUpdate(0),
		bWarnEpgDone(false),
		bWarnRecDone(false),
		ptSaveLvw(POINT()),
		ptSaveTtp(POINT()),
		hSysIcon(NULL)
	{
	}
};
