/*
 * $Id: errors_ui.cpp 259 2010-08-04 17:47:32Z gingko $
 * Copyright (C) 2010 gingko - http://gingko.homeip.net/
 *
 * This file is part of Pouchin TV Mod, a free DVB-T viewer.
 * See http://www.pouchintv.fr/ for updates.
 *
 * Pouchin TV Mod is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Pouchin TV Mod is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 *
 *
 * Most files of this project have been changed from the Pouchin TV
 * project (Copyright (C) 2006 Pouchin <pouchinteve@free.fr>),
 * to use with a modified version of this software.
 * See http://pouchinteve.free.fr/ for the original project.
 */

/** \file
 *	\brief Implmentation de la gestion de messages d'erreurs globaux
 *
 *	Ce fichier implmente la gestion des messages globaux dfinis dans le
 *	fichier d'en-ttes qui lui est associ.
 **/

#include "stdafx.h"

/**
 * \file
 * Fonctions pour grer les erreurs
 */

/// Chane de caractres contenant tous les messages d'erreurs concatns
static tstring	strErrBuf;

/// Dernier code d'erreur trait (pour viter la multiplication des messages identiques)
static HRESULT	lastRes = S_OK;

/**
 * Met en file d'attente le message d'erreur pass en paramtre pour un affichage ultrieur
 *
 * \param[in]	str	Le message  stocker, et pouvant tre pass  un printf
 *					ou si NULL, le message d'erreur sera rcupr automatiquement pour DirectShow
 * \param[in]	hr	valeur de retour devant tre teste (en cas d'erreur DShow),
 *					ou 0 si on doit l'ignorer. Ce sera la valeur retourne
 * \param[in]	...	variables optionnelles traites par printf
 * \return			la valeur du hr pass en paramtre
 **/
HRESULT erreur(LPCTSTR str, HRESULT hr, ...)
{
	// Utile simplement si on passe un message d'erreur personnalis
	if (str) {
		// Buffer de stockage temporaire
		FMT_BUF(buffer, 512, str, hr);

		if (!strErrBuf.empty())
			strErrBuf += EOL;
		strErrBuf += buffer;

		// Enregistre le message d'erreur
		myprintf(ERRX(TEXT("Erreur# %s")), buffer);
	}

	if (FAILED(hr) && hr!=lastRes) {
		erreur(TEXT("%s (code 0x%08x)"), S_OK, DShowErrorToString(hr).c_str(), hr);
		lastRes = hr;
	}

	return hr;
}

/**
 * Affiche les messages d'erreurs mis en file d'attente, ainsi que le nouveau message  la suite
 *
 * \param[in]	str		Le message  stocker, et  afficher en dernier
 * \param[in]	hr		Valeur de retour devant tre teste (en cas d'erreur DShow),
 *						ou 0 si on doit l'ignorer. Ce sera la valeur retourne.
 * \param[in]	title	Titre optionnel de la fentre
 * \param[in]	uType	Options du message (voir documentation de \p MessageBox)
 * \return		Code retour de la fentre de message
 **/
int affiche_erreurs(LPCTSTR str, HRESULT hr, LPCTSTR title, UINT uType)
{
	int nRet = 0;

	// Met le nouveau message d'erreur en file
	erreur(str, hr);

	// Affiche le(s) message(s) d'erreur(s), et vide la pile
	if (!strErrBuf.empty()) {
		nRet = MessageBox(hMainWnd, strErrBuf.c_str(), title, uType);
		strErrBuf.clear();
		lastRes = S_OK;
	}
	return nRet;
}
