/*
 * $Id: grabber.h 243 2010-03-17 08:36:42Z gingko $
 * Copyright (C) 2006 Pouchin <pouchinteve@free.fr>
 *
 * This file is part of Pouchin TV Mod, a free DVB-T viewer.
 * See http://www.pouchintv.fr/ for updates.
 *
 * Pouchin TV Mod is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Pouchin TV Mod is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 *
 *
 * Most files of this project have been changed from the Pouchin TV
 * project (Copyright (C) 2006 Pouchin <pouchinteve@free.fr>),
 * to use with a modified version of this software.
 * See http://pouchinteve.free.fr/ for the original project.
 */

/** \file
 *	\brief Dfinition de l'interface "callback" du "Sample Grabber"
 *
 *	Ce fichier dfinit l'interface "callback" qui se connecte au filtre
 *	\p DirectShow "Sample Grabber", utilis pour rcuprer les chantillons
 *	du flux TS destins aux enregistrements programms.
 **/

#pragma once

#include "capture.h"
#include "memfifo.h"

/// Classe implmentant les mthodes "callback"  associer  un "Sample grabber"
class CSampleGrabber : public CUnknown, public ISampleGrabberCB, public WorkingThread     // The Filter Interface
{
	//
	// --- Implmentation des mthodes abstraites de l'interface ISampleGrabberCB --
	//

	/**
	 * Fonction "callback" recevant un pointeur sur le tampon d'chantillons.
	 * Non utilis ici (car ce mode n'est pas configur), mais ncessaire car
	 * la classe de base est abstraite.
	 **/
	STDMETHODIMP BufferCB(double SampleTime, BYTE *pBuffer,long BufferLen) {
		myprintf(ERRX(TEXT("Appel BufferCB inattendu")));
		return S_OK;}

	/**
	 * Fonction "callback" transmettant les chantillons issus du tuner, renvoys par le graphe
	 *	param[in]	SampleTime	Temps de dpart de l'chantillon, en secondes.
	 *	param[in]	pSample		Pointeur sur l'interface permettant d'obtenir l'chantillon.
	 **/
	STDMETHODIMP SampleCB(double SampleTime, IMediaSample *pSample);

	//
	// --- Implmentation des mthodes abstraites de l'interface IUnknown ---
	//

	// Implmentation standard des mthodes (abstraites) de l'interface \p IUnknown
	DECLARE_IUNKNOWN;

	STDMETHODIMP NonDelegatingQueryInterface(REFIID riid, void **ppv)
	{
		if (riid == IID_ISampleGrabberCB)
			return GetInterface((ISampleGrabberCB*)this, ppv);
		return CUnknown::NonDelegatingQueryInterface(riid, ppv);
	}

	//
	// --- Donnes et mthodes propres  l'application ---
	//

	BYTE			taille_paquet_buf;	//!< Longueur du paquet partiel mmoris
	TS_packet		paquet_buf;			//!< Tampon de mmorisation de paquet partiel

	CMemFifo		cMemFifo;			//!< Buffer "First In First Out" de mmorisation des chantillons reus

	/// Fonction d'excution principale effective du 'thread'.
	virtual DWORD	ThreadProc();

public:

	CSyncMutex		cMutex;				//!< Mutex protgeant les dbuts et fins d'enregistrements

	/// Arrt du "thread" de capture
	bool			StopThread();

private:

	/**
	 * Traitement d'un chantillon reu du tuner
	 * param[in]	nSize	Taille de l'chantillon
	 * param[in]	pPtr	Pointeur sur les donnes de l'chantillon
	 **/
	void			traite_sample(size_t nSize, const BYTE * pPtr);

	/**
	 * Traitement d'un paquet TS reu du tuner (issu du dcoupage de l'chantillon, et mme
	 * ventuellement d'une reconstitution de celui-ci).
	 *
	 * param[in]	p		Rfrence sur le paquet reu (qui a une longueur fixe)
	 **/
	void			traite_paquet(const TS_packet & p);

public:

	/// Constructeur de base
	CSampleGrabber();

	/// Destructeur gnral
	virtual ~CSampleGrabber();
};
