/*
 * $Id: ini.cpp 277 2011-03-13 10:24:41Z gingko $
 * Copyright (C) 2006 Pouchin <pouchinteve@free.fr>
 *
 * This file is part of Pouchin TV Mod, a free DVB-T viewer.
 * See http://www.pouchintv.fr/ for updates.
 *
 * Pouchin TV Mod is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Pouchin TV Mod is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 *
 *
 * Most files of this project have been changed from the Pouchin TV
 * project (Copyright (C) 2006 Pouchin <pouchinteve@free.fr>),
 * to use with a modified version of this software.
 * See http://pouchinteve.free.fr/ for the original project.
 */

/** \file
 *	\brief Implmentation de l'accs aux fichiers de configuration
 *
 *	Ce module implmente les fonctionnalits permettant de sauvegarder et de rcuprer
 *	la configuration, les chanes reues, les enregistrements programms, etc.
 **/

#include "stdafx.h"
#include "resource.h"
#include "search.h"
#include "recprog.h"
#include "lzma.h"

// ====================================================================================
// Variables externes
// ====================================================================================

/**
 * Liste des raccourcis programmables
 *
 * \li \a Identifiant    :  valeur numrique, ou constante, permettant d'associer le raccourci  l'action
 * \li \a Nom \a textuel :  c'est le texte tel qu'il sera affich dans la fentre lors de la redfinition du raccourci.
 * \li \a Nom_config :      c'est le nom de l'option telle qu'elle apparatra dans le fichier de configuration.
 *                          Mettre NULL comme nom si on veut un raccourcis qui ne soit pas modifiable (et donc non affich) dans
 *                          la fentre de modification des raccourcis.
 *                          \warning pas plus de 32 caractres
 * \li \a Maj :             indique si la touche majuscule doit tre combine pour former le raccourci
 * \li \a Ctrl :            mme chose, mais avec la touche Ctrl
 * \li \a Alt :             pareil avec la touche Alt
 * \li \a Touche :          indique la touche devant tre combine avec, si besoin est, Maj, Ctrl et Alt
 **/
ShortcutItem shortcuts[] = {
	//   Identifiant                     nom textuel                            nom dans la config       maj    ctrl    alt    touche
	// Changement de chanes
	{IDM_CHAN_AUG,			TEXT(" "),						TEXT("Chane suiv"),		{false,	false,	false,	VK_DOWN}, {}, {true,	true,	false,	TEXT('N')}},
	{IDM_CHAN_DIM,			TEXT(" "),						TEXT("Chane prc"),		{false,	false,	false,	VK_UP}, {}, {false,	true,	false,	TEXT('N')}},
	{IDM_LAST_CHAN,			TEXT("   "),				TEXT("Chane retour"),		{false,	false,	false,	VK_BACK}, {}, {true,	true,	false,	TEXT('R')}},
	{IDM_CHAN_AUG_MPX,		TEXT("   "),			TEXT("Chane suiv mx"),		{true,	false,	false,	VK_DOWN}},
	{IDM_CHAN_DIM_MPX,		TEXT("   "),			TEXT("Chane prc mx"),		{true,	false,	false,	VK_UP}},
	{IDM_CHAN_TUNE,			TEXT(" "),				TEXT("Chane Resyntoniser"),{false,	true,	false,	TEXT('F')}},
	// EPG
	{IDM_EPG,				TEXT("  "),					TEXT("EPG Guide progs"),	{false,	false,	false,	TEXT('G')}, {}, {true,	true,	false,	TEXT('T')}},
	{IDM_SHOW_PROGRAMME,	TEXT(" "),						TEXT("EPG Affiche prog"),	{false,	false,	false,	TEXT('I')}},
	// Enregistrements
	{IDM_DELAYED_RECORD,	TEXT("  "),			TEXT("Enreg Programmer"),	{false,	false,	false,	TEXT('P')}},
	{IDM_RECORD_CHANNEL_PS,	TEXT("  PS"),			TEXT("Enreg PS"),			{false,	true,	false,	VK_SPACE}},
	{IDM_RECORD_CHANNEL_TS,	TEXT("  TS"),			TEXT("Enreg TS"),			{false,	true,	false,	TEXT('T')}, {}, {false,	true,	false,	TEXT('R')}},
	{IDM_RECORD_OTHER,		TEXT("   "),			TEXT("Enreg autre"),		{false,	true,	false,	TEXT('E')}},
	{IDM_SCREENSHOT,		TEXT("  "),						TEXT("Enreg Capure cran"),	{false,	true,	false,	TEXT('X')}},
	{IDM_STOP_RECORD,		TEXT("  "),				TEXT("Enreg Stop immdiat"),{false,	true,	false,	VK_RETURN}, {}, {false,	true,	false,	VK_END}},
	{IDM_DELAYED_STOP,		TEXT("   "),		TEXT("Enreg Stop diffr"),	{false,	false,	false,	TEXT('S')}},
	// Volume/Son
	{IDM_MUTE,				TEXT(" "),							TEXT("Son couper"),			{false,	false,	false,	TEXT('M')}},
	{IDM_VOL_AUG,			TEXT("  "),					TEXT("Son augmenter"),		{false,	false,	false,	VK_ADD}, {}, {true,	true,	false,	TEXT('B')}},
	{IDM_VOL_DIM,			TEXT("  "),						TEXT("Son diminuer"),		{false,	false,	false,	VK_SUBTRACT}, {}, {false,	true,	false,	TEXT('B')}},
	{IDM_SON_AUG,			TEXT("  "),					TEXT("Son suiv"),			{false,	false,	true,	VK_DOWN}},
	{IDM_SON_DIM,			TEXT("  "),					TEXT("Son prc"),			{false,	false,	true,	VK_UP}},
	// Affichage
	{IDM_ALWAYS_ON_TOP,		TEXT("  "),					TEXT("Aff Toujours devant"),{false,	false,	true,	TEXT('T')}},
	{IDM_FULLSCREEN,		TEXT(" "),							TEXT("Aff Plein cran"),	{false,	false,	false,	VK_SPACE}, {}, {true,	true,	false,	TEXT('F')}},
	{IDM_BORDERS,			TEXT(" "),							TEXT("Aff Bordures"),		{false,	false,	false,	VK_TAB}},
	{IDM_ZOOM,				TEXT(""),									TEXT("Aff Zoomer"),			{true,	false,	false,	TEXT('Z')}},
	{IDM_DEZOOM,			TEXT(""),								TEXT("Aff Dzoomer"),		{false,	false,	false,	TEXT('Z')}},
	{IDM_DEFAULT_ZOOM,		TEXT(" "),						TEXT("Aff Restaurer"),		{false,	false,	false,	TEXT('D')}},
	{IDM_ALLWDT,			TEXT("   "),				TEXT("Aff Pleine largeur"),	{false,	false,	false,	TEXT('W')}},
	{IDM_ALLHGT,			TEXT("   "),				TEXT("Aff Pleine hauteur"),	{false,	false,	false,	TEXT('H')}},
	{IDM_ETIRER,			TEXT("   "),						TEXT("Aff tirer"),			{false,	false,	false,	TEXT('E')}},
	{IDM_VIDEOCONTROL,		TEXT("  "),					TEXT("Aff Rglages"),		{false,	false,	false,	TEXT('V')}},
	// Ratios d'aspect
	{IDM_AR_AUTO,			TEXT("Ratio d'aspect Automatique"),				TEXT("Aff RA Auto")},		// Rien par dfaut
	{IDM_AR_1X1,			TEXT("Ratio d'aspect 1:1"),						TEXT("Aff RA 1:1")},		// Rien par dfaut
	{IDM_AR_5X4,			TEXT("Ratio d'aspect 5:4"),						TEXT("Aff RA 5:4")},		// Rien par dfaut
	{IDM_AR_4X3,			TEXT("Ratio d'aspect 4:3"),						TEXT("Aff RA 4:3")},		// Rien par dfaut
	{IDM_AR_16X10,			TEXT("Ratio d'aspect 16:10"),					TEXT("Aff RA 16:10")},		// Rien par dfaut
	{IDM_AR_16X9,			TEXT("Ratio d'aspect 16:9"),					TEXT("Aff RA 16:9")},		// Rien par dfaut
	// Divers
	{IDM_CONFIG,			TEXT(""),							TEXT("Opt Config"),			{false,	false,	false,	TEXT('C')}},
	{IDM_SIGNAL,			TEXT("  "),						TEXT("Info Signal"),		{true,	false,	false,	TEXT('S')}},
	{IDM_RECORD_INFO,		TEXT("  "),			TEXT("Info Enreg"),			{true,	false,	false,	TEXT('E')}},
#ifdef _DIAGNOSTIC
	{IDM_RUN_GRAPH,			TEXT(" "),				TEXT("Debug GrapheMarche"),	{false,	true,	false,	TEXT('R')}},
	{IDM_PAUSE_GRAPH,		TEXT(" "),				TEXT("Debug GraphePause"),	{false,	true,	true,	TEXT('R')}},
	{IDM_STOP_GRAPH,		TEXT(" "),						TEXT("Debug GrapheArrt"),	{true,	true,	false,	TEXT('R')}},
	{IDM_GRAPH_STATE,		TEXT("  "),					TEXT("Debug tatGraphe"),	{false,	false,	false,	TEXT('R')}},
	{IDM_BRANCHE,			TEXT("  "),					TEXT("Debug Connect"),		{false,	true,	false,	TEXT('B')}},
	{IDM_DEBRANCHE,			TEXT("  "),					TEXT("Debug Disconnect"),	{true,	true,	false,	TEXT('B')}},
#endif
	{IDM_APP_RESTART,		TEXT("  "),					TEXT("Graphe Relance"),		{false,	true,	false,	TEXT('Q')}},
	{IDM_CLOSE,				TEXT(""),									TEXT("Fichier Fermer")},	// Rien par dfaut
	{IDM_QUIT,				TEXT(""),								TEXT("Fichier Quitter"), {}, {}, {true,	true,	false,	TEXT('P')}},	// Rien par dfaut
	{IDM_HELPDOC,			TEXT(""),							TEXT("Aide Documentation"), {false,	false,	false,	VK_F1}},
	{IDM_UPDATE,			TEXT("   "),				TEXT("Aide Mise  jour")},	// Rien par dfaut
	{IDM_ABOUT,				TEXT(" "),		TEXT("Aide Crdits"),		{false,	true,	false,	VK_F1}},
};

/**
 * Obtenir le nombre de raccourcis dfini.
 **/
int get_shortcuts_count()
{
	// (ceci dispense d'avoir  spcifier la taille du tableau dans le fichier d'en-tte '.h', alors
	// mme que cette taille est variable selon des directives de compilation conditionnelle)
	return _countof(shortcuts);
}

/// Position de la fentre (enregistr) de PTVM
WindowPos		winPos;

/// Priorit du processus courant selon la configuration
DWORD			configPriority = IDM_ABOVE_NORMAL_PRIORITY;

/// tat de la fentre principale selon la configuration
EtatsFenetre	configState = etf_normal;

/// Rpertoire de Pouchin TV Mod
SZPathName		pouchindir_prog;

/// Rpertoire pour la sauvegarde des paramtres (Dossier App Data)
SZPathName		pouchindir_conf;

/// Rpertoire initial au lancement du programme
SZPathName		pouchindir_curr;

/// Prfixe optionnel aux noms de fichiers de configuration
TCHAR			prefix_conf[32] = TEXT("");

/**
 * Variable  mettre  'true' toutes les fois que quelque chose a t modifi
 * dans les enregistrements programms, ncessitant une sauvegarde.
 **/
bool			programmation_modifiee = false;

// ====================================================================================
// Variables de configuration
// ====================================================================================

SZFilterName	filtreMPEG2;
SZFilterName	filtreH264;
SZFilterName	filtreAudio;
SZFilterName	filtreAc3;
SZFilterName	filtreEac3;
TCHAR			choixCanaux[256];
TCHAR			liste_canaux[256];

bool			exact_match;

SZPathName		screenshots_dir;
SZPathName		video_dir;

UINT32			cleChaineConfig			= 0;
INT32			avance_enreg			= 8;
INT32			retard_enreg			= 16;
SearchMethod	methode_recherche		= INI_METHODE_RECH_DEFAULT;
OffsetsToUse	use_offsets				= INI_USE_OFFSETS_DEFAULT;
INT32			autonum_undef			= INI_AUTONUM_UNDEF_DEFAULT;
INT32			autonum_duplicate		= INI_AUTONUM_DUP_DEFAULT;
bool			suspend_minimized		= true;
bool			minimize_system_tray	= true;
bool			start_with_windows		= false;
bool			record_protect			= false;
bool			allow_wakeup			= false;
INT32			wakeup_delay			= 5;
INT32			duree_grace_quit		= 30;
INT32			time_before_shutdown	= 60;

#if USE_VMR_DEINTERLACE
bool			use_vmr_deinterlace		= false;
#endif

bool			allow_stream_record		= false;

bool			use_msn					= false;
TCHAR			titre_fenetre[128]		= TEXT("");
bool			on_top					= false;
AudioMode		audio_pref				= am_None;
bool			use_osd					= true;
COLORREF		osd_color				= INI_OSD_COLOR;
COLORREF		aCustomColors[16]		= {0};

INT32			osd_transparency		= INI_OSD_TRANSPARENCY;
RdrTypes		vmr_mode;

PanModes		video_pan_mode			= pm_Normal;
ArModes			aspect_ratio			= ar_Auto;
INT32			zoom_ratio				= 100;

extern INT32	video_ctl[4]			= {VIDEO_UNDEF,VIDEO_UNDEF,VIDEO_UNDEF,VIDEO_UNDEF};
extern INT32	video_sav[4]			= {VIDEO_UNDEF,VIDEO_UNDEF,VIDEO_UNDEF,VIDEO_UNDEF};

INT32			volumeCourant			= 0;

bool			use_multimedia			= true;
MouseWheelUsage	role_molette			= mwu_Channels;
bool			inversion_molette		= false;
INT32			duree_zapping			= INI_DUREE_ZAPPING;

SZPathName		motif_noms_enreg;				//!< Motifs pour les noms des enregistrements

MethodeEnregistrement
				epg_typerecvideo;				//!< Type d'enregistrement prcdement utilis dans la
												//!< fentre EPG pour la vido
AudioMode		epg_typerecaudio;				//!< Type d'enregistrement prcdement utilis dans la
												//!< fentre EPG pour l'audio
EpgTimeLimit	epg_timelimit;					//!< Limite d'exploration de l'EPG

SZFilterName	nom_tuner;
bool			autre_graphe			= false;//!< Utilisation du graphe des versions prcdentes

#ifdef _DIAGNOSTIC
INT32			mpx_frequence_diag;				//!< Frquence du multiplex enregistr (diagnostic slt.)
#endif


// Valeurs sauvegardes sparment pour chaque type de tuner
TunerProps sTunProps =
{
	{0},							// nom_receiver
	{0},							// szPathTuner
	{0},							// szPathDemod
	{0},							// szPathCapture
	0,								// offset_tuner
	INI_TRY_ADDING_5KHZ_DEFAULT,	// try_adding_5khz
	INI_DELAI_TUNER1_DEFAULT,		// delai_tuner1
	INI_DELAI_TUNER2_DEFAULT,		// delai_tuner2
	false,							// scan_ignore_presence
	false,							// scan_ignore_lock
	false,							// scan_ignore_quality
	0,								// force_signal_zero
	100,							// force_signal_max
	false,							// force_signal_aff_pct
	true							// auto_resyntonisation
};

// ====================================================================================
// Variables locales
// ====================================================================================

/// Nom de base du fichier dfinissant les listes de canaux  explorer
TCHAR scan_ini[]	= TEXT("canaux.ini");

/// Nom du fichier de configuration
TCHAR config_file[]	= TEXT("config.ini");

// ====================================================================================
// Fonctions
// ====================================================================================

/**
 * Dtermine si une dfinition de configuration est dfinie comme "Aucun" ("[Null]") ou
 * "non dfini" (chane vide).
 *
 * \param[in]  pszName  Nom du codec
 * \retval	\p true si aucun
 * \retval	\p false si dfini
 **/
bool IsNullName(LPCTSTR pszName)
{
	return pszName[0]==0 || _tcscmp(pszName, NULL_NAME)==0;
}

/**
 * Test Prsence debugger
 * \note Ncessaire car IsDebuggerPresent semble ne pas toujours retourner le mme rsultat d'un appel
 *  l'autre dans la mme application, avec pour rsultat des inconsistances dans la configuration.
 * Avec cette fonction, on considre que le premier rsultat retourn est le bon.
 **/
bool TestIfDebuggerPresent()
{
	static bool bDpTested = false;
	static bool	bPresent = false;

	if (!bDpTested) {
		bPresent = IsDebuggerPresent() == TRUE;
		bDpTested = true;
	}
	return bPresent;
}

// ====================================================================================
// Tableaux associatifs
// ====================================================================================

/**
 * Table d'associations pour nommer les tats possible des chanes.
 **/
const AssocElement aChStateTable[] =
{							// EtatChaine
	{TEXT(""),		ec_inactive},
	{TEXT(""),		ec_active},
	{TEXT(""),		ec_preferee},
	{NULL, 0}		// -> terminateur
};

/**
 * Tableau associatif convertissant les valeurs boolennes pour sauvegarde
 **/
const AssocElement aBoolTable[] =
{
	{TEXT(""),			0},
	{TEXT(""),			1},
	{NULL, 0}		// -> terminateur
};

/**
 * Tableau associatif dcrivant les priorites de PTVM pour sauvegarde
 **/
const AssocElement aPrioTable[] =
{
	{TEXT(" "),			IDLE_PRIORITY_CLASS},
	{TEXT(" "),	BELOW_NORMAL_PRIORITY_CLASS},
	{TEXT(" "),		NORMAL_PRIORITY_CLASS},
	{TEXT(" "),	ABOVE_NORMAL_PRIORITY_CLASS},
	{TEXT(" "),			HIGH_PRIORITY_CLASS},
	{TEXT(" "),			REALTIME_PRIORITY_CLASS},
	{NULL, 0}		// -> terminateur
};

/**
 * Tableau associatif dcrivant les diffrents affichages de la fentre pour sauvegarde
 **/
const AssocElement aWStateTable[] =
{							// EtatsFenetre
	{TEXT(""),		etf_normal},
	{TEXT(" "),	etf_noborders},
	{TEXT(""),		etf_maximized},
	{TEXT("  "),	etf_fullscreen},
	{TEXT(""),		etf_minimized},
	{NULL, 0}		// -> terminateur
};

/**
 * Tableau associatif dcrivant les diffrents types de flux vido reconnus
 **/
const AssocElement aVideoTypeTable[] =
{							// VideoStreamType
	{TEXT("MPEG2"),			vst_MPEG2},
	{TEXT("H264"),			vst_H264},
	{TEXT("MPEG2 "),	vst_MPEG2_encrypted},
	{TEXT("H264 "),	vst_H264_encrypted},
	{NULL, 0}		// -> terminateur
};

/**
 * Tableau associatif dcrivant le dernier type d'enregistrement dans la fentre de gestion de l'EPG pour la vido
 **/
const AssocElement aEpgRecVideo[] =
{							// MethodeEnregistrement
	{TEXT("TS"),			meth_TS},
	{TEXT("PS"),			meth_PS},
	{TEXT("Multiplex"),		meth_multiplex},
	{NULL, 0}		// -> terminateur
};

/**
 * Tableau associatif dcrivant le dernier type d'enregistrement dans la fentre de gestion de l'EPG pour l'audio
 **/
const AssocElement aEpgRecAudio[] =
{							// AudioMode
	{TEXT(" "),		am_None},
	{TEXT("Audio1"),		am_Mpeg_1},
	{TEXT("Audio2"),		am_Mpeg_2},
	{TEXT("AC3"),			am_AC3},
	{TEXT("AC3 2.0"),		am_AC3_non_HC},
	{TEXT("AC3 HC"),		am_AC3_HC},
	{TEXT("V.O."),			am_VO},
	{NULL, 0}		// -> terminateur
};

const AssocElement	aEpgHorizon[] =
{
	{TEXT(""),		eh_cur_next},
	{TEXT("1 "),		eh_1_hour},
	{TEXT("2 "),		eh_2_hours},
	{TEXT("3 "),		eh_3_hours},
	{TEXT("4 "),		eh_4_hours},
	{TEXT("6 "),		eh_6_hours},
	{TEXT("9 "),		eh_9_hours},
	{TEXT("12 "),		eh_12_hours},
	{TEXT("18 "),		eh_18_hours},
	{TEXT("24 "),		eh_24_hours},
	{TEXT("2 "),		eh_2_days},
	{TEXT("3 "),		eh_3_days},
	{TEXT(""),		eh_nolimit},
	{NULL, 0}		// -> terminateur
};

/**
 * Tableau associatif dcrivant les diffrentes configurations VMR disponibles
 **/
const AssocElement aRenderModeTable[] =
{									// RdrTypes
	{TEXT(" "),					rdt_null},
	{TEXT("VMR7 windowless"),		rdt_VMR7_windowless},
	{TEXT("VMR7 renderless"),		rdt_VMR7_renderless},
	{TEXT("VMR9 windowless"),		rdt_VMR9_windowless},
#if NEW_VMR9_OUTPUT
	{TEXT("VMR9 windowless (v2)"),	rdt_VMR9_windowless_new},
#endif
	{TEXT("EVR"),					rdt_EVR},
	{TEXT("EVR (v2)"),				rdt_EVR_new},
	{TEXT("VMR9 renderless"),		rdt_VMR9_renderless},	// (en attente d'implmentation)
	{NULL, 0}		// -> terminateur
};

/**
 * Tableau associatif dcrivant les diffrents modes panoramiques de la vido
 **/
const AssocElement aPanModeTable[] =
{								// PanModes
	{TEXT(""),			pm_Normal},
	{TEXT(""),			pm_Stretch},
	{TEXT(" "),	pm_AllWidth},
	{TEXT(" "),	pm_AllHeight},
	{NULL, 0}		// -> terminateur
};

#if 0
/**
 * Tableau associatif dcrivant les diffrents modes panoramiques de la vido
 * \note	Pas utilis pour le moment
 **/
const AssocElement aArModeTable[] =
{								// PanModes
	{TEXT("Auto"),				ar_Auto},
	{TEXT("1:1"),				ar_1x1},
	{TEXT("5:4"),				ar_5x4},
	{TEXT("4:3"),				ar_4x3},
	{TEXT("16:10"),				ar_16x10},
	{TEXT("16:9"),				ar_16x9},
	{NULL, 0}		// -> terminateur
};
#endif

/**
 * Tableau associatif dcrivant les mthodes de recherche des chanes
 **/
const AssocElement aSMethodTable[] =
{								// SearchMethod
	{TEXT(""),			sm_undef},
	{TEXT(""),		sm_incremental},
	{TEXT(" "),sm_prefs},
	{TEXT(" "),		sm_full},
	{NULL, 0}		// -> terminateur
};

/**
 * Tableau associatif dcrivant les offsets de recherche des chanes
 **/
const AssocElement aOffsetsToUse[] =
{								// OffsetsToUse
	{TEXT("0"),					(DWORD)uo_0},
	{TEXT("+167"),				(DWORD)uo_167},
	{TEXT("+333"),				(DWORD)uo_333},
	{TEXT("+500"),				(DWORD)uo_500},
	{TEXT("-167"),				(DWORD)uo_m167},
	{NULL, 0}		// -> terminateur
};

/**
 * Tableau associatif dcrivant les diffrents rles possibles de la molette de la souris
 **/
const AssocElement aMWheelTable[] =
{								// MouseWheelUsage
	{TEXT(" "),				(DWORD)mwu_None},
	{TEXT("  "),			(DWORD)mwu_Channels},
	{TEXT("  "), (DWORD)mwu_Channels_MX},
	{TEXT("   "),		(DWORD)mvu_Soundtrack},
	{TEXT("  "),			(DWORD)mwu_Volume},
	{TEXT("  "),				(DWORD)mvu_Zoom},
	{NULL, 0}		// -> terminateur
};

// ====================================================================================
// Descripteurs d'items de configuration
// ====================================================================================

/// Types d'items de configuration
enum ConfigItemTypes
{
	cit_INT,	//!< Valeur entire signe (utiliser type \p INT32)
	cit_HEX,	//!< Valeur entire hexa non signe (utiliser type \p UINT32 ou \p DWORD ou quiv.)
	cit_BOOL,	//!< Valeur boolenne
	cit_STR,	//!< Chane de caractres
	cit_DRV,	//!< Chane de caractres spcialise (nom de pilote ou de codec)
	cit_PATH,	//!< Chane de caractres spcialise (chemin de rpertoire)
	cit_ASSOC,	//!< Association entier <-> chane (typiquement numration)
	cit_WPOS	//!< Position de fentre
};

/// Description d'un item de configuration
struct ConfigItem
{
	LPCTSTR			pszName;	//!< Nom de l'item de configuration
	ConfigItemTypes	eType;		//!< Type de la variable
	LONG_PTR		nDef;		//!< Valeur par dfaut (utiliser un cast de type au besoin)
	LPVOID			pVar;		//!< Adresse de la variable
	LONG_PTR		nExtra;		//!< \brief Donnes complmentaires (utiliser un cast de type au besoin)
								//!< - \p cit_STR, \p cit_DRV ou \p cit_PATH : taille de la variable
								//!< - \p cit_ASSOC : adresse de la table
								//!< - \p cit_WPOS : adresse de la "handle" de la fentre)
};

#define CONFIG_STR(str)	&str, _countof(str)
#define CONFIG_VAL(val)	reinterpret_cast<LONG_PTR>(val)

// Sections dans le fichier de config
static LPCTSTR ini_config_section	= TEXT("Config");		//!< Nom de la section contenant la configuration
static LPCTSTR ini_shortcut_section	= TEXT("Raccourcis");	//!< Nom de la section contenant les raccourcis clavier
static LPCTSTR ini_dontshow_section	= TEXT("Dsactiver");	//!< Nom de la section des messages  ne plus afficher

static LPCTSTR ini_color_num		= TEXT("Couleur Perso %02u");	//!< Nom de sauvegarde des couleurs perso

static LPCTSTR ini_running			= TEXT("Excution");	//!< Nom de sauvegarde de l'tat "en cours d'excution"

/// Liste des items sauvegards globalement
static ConfigItem ini_general_items[] =
{ // Nom										Type		Dfaut							Variable				Extra
	{TEXT("Rendu vido"),						cit_ASSOC,	rdt_EVR_new,						& vmr_mode,				CONFIG_VAL(aRenderModeTable)},
	// (note : rendu vido en premier, pour faciliter l'ajustement sa valeur par dfaut selon la version de Windows)

	{TEXT("Utiliser l'OSD"),					cit_BOOL,	true,							& use_osd},
	{TEXT("Couleur OSD"),						cit_HEX,	INI_OSD_COLOR,					& osd_color},
	{TEXT("Transparence OSD"),					cit_INT,	INI_OSD_TRANSPARENCY,			& osd_transparency},
#if USE_VMR_DEINTERLACE
	{TEXT("Utilise le dsentrelacement du VMR"),cit_BOOL,	false,							& use_vmr_deinterlace},
#endif

	{TEXT("Filtre MPEG2"),						cit_DRV,	NULL,							CONFIG_STR(filtreMPEG2)},
	{TEXT("Filtre H264"),						cit_DRV,	NULL,							CONFIG_STR(filtreH264)},
	{TEXT("Filtre audio"),						cit_DRV,	NULL,							CONFIG_STR(filtreAudio)},
	{TEXT("Filtre AC3"),						cit_DRV,	NULL,							CONFIG_STR(filtreAc3)},
	{TEXT("Filtre EAC3"),						cit_DRV,	NULL,							CONFIG_STR(filtreEac3)},

	{TEXT("Recherche stricte"),					cit_BOOL,	true,							& exact_match},
	{TEXT("Graphe alternatif"),					cit_BOOL,	false,							& autre_graphe},
	{TEXT("Prfrence audio"),					cit_ASSOC,	false,							& audio_pref,			CONFIG_VAL(aEpgRecAudio)},

	{TEXT("Canaux  explorer"),					cit_STR,	CONFIG_VAL(NULL_NAME),			CONFIG_STR(choixCanaux)},
	{TEXT("Liste de canaux"),					cit_STR,	CONFIG_VAL(TEXT("21-69")),		CONFIG_STR(liste_canaux)},

	{TEXT("Mthode de recherche"),				cit_ASSOC,	INI_METHODE_RECH_DEFAULT,		& methode_recherche,	CONFIG_VAL(aSMethodTable)},
	{TEXT("Offsets"),							cit_ASSOC,	INI_USE_OFFSETS_DEFAULT,		& use_offsets,			CONFIG_VAL(aOffsetsToUse)},
	{TEXT("Base numrotation inconnus"),		cit_INT,	INI_AUTONUM_UNDEF_DEFAULT,		& autonum_undef},
	{TEXT("Dcalage numrotation duplicatas"),	cit_INT,	INI_AUTONUM_DUP_DEFAULT,		& autonum_duplicate},

	{TEXT("Identifiant chane courante"),		cit_HEX,	0,								& cleChaineConfig},
	{TEXT("Avance dbut enregistrements"),		cit_INT,	8,								& avance_enreg},
	{TEXT("Retard fin enregistrements"),		cit_INT,	16,								& retard_enreg},
	{TEXT("Enregistrement multiplex autoris"),	cit_BOOL,	false,							& allow_stream_record},

	// CSIDL_MYVIDEO : Rpertoire "Mes vidos" (utilis par dfaut)
	{TEXT("Rpertoire vido"),					cit_PATH,	CSIDL_MYVIDEO,					CONFIG_STR(video_dir)},

	// CSIDL_MYPICTURES : Rpertoire "Mes images" (utilis par dfaut)
	{TEXT("Rpertoire screenshots"),			cit_PATH,	CSIDL_MYPICTURES,				CONFIG_STR(screenshots_dir)},

	{TEXT("Nom du fichier d'enregistrement"),	cit_STR,	CONFIG_VAL(TEXT(MOTIF_NOMS_ENREG_DEFAULT)),
																							CONFIG_STR(motif_noms_enreg)},
	{TEXT("Type d'enregistrement video"),		cit_ASSOC,	meth_TS,						& epg_typerecvideo,		CONFIG_VAL(aEpgRecVideo)},
	{TEXT("Type d'enregistrement audio"),		cit_ASSOC,	am_Mpeg_1,						& epg_typerecaudio,		CONFIG_VAL(aEpgRecAudio)},
	{TEXT("Temps limite EPG"),					cit_ASSOC,	eh_cur_next,					& epg_timelimit,		CONFIG_VAL(aEpgHorizon)},

	{TEXT("Titre de la fentre"),				cit_STR,	CONFIG_VAL(szAppName),			CONFIG_STR(titre_fenetre)},
	{TEXT("Fentre toujours devant"),			cit_BOOL,	false,							& on_top},
	{TEXT("Envoie programme regard  MSN"),	cit_BOOL,	false,							& use_msn},

	{TEXT("Dmarrer avec Windows"),				cit_BOOL,	false,							& start_with_windows},
	{TEXT("Protection des enregistrements"),	cit_BOOL,	false,							& record_protect},
	{TEXT("Permettre le rveil"),				cit_BOOL,	false,							& allow_wakeup},
	{TEXT("Dlai rveil avant enregistrement"),	cit_INT,	5,								& wakeup_delay},
	{TEXT("Dure pour refus d'extinction"),		cit_INT,	30,								& duree_grace_quit},
	{TEXT("Temps avant extinction"),			cit_INT,	60,								& time_before_shutdown},
	{TEXT("Suspendre si minimis"),				cit_BOOL,	true,							& suspend_minimized},
	{TEXT("Minimiser en zone de notification"),	cit_BOOL,	true,							& minimize_system_tray},
	{TEXT("Panoramique"),						cit_ASSOC,	pm_Normal,						& video_pan_mode,		CONFIG_VAL(aPanModeTable)},
	{TEXT("Facteur de zoom"),					cit_INT,	100,							& zoom_ratio},

	{TEXT("Vido luminosit"),					cit_INT,	VIDEO_UNDEF,					& video_ctl[pat_Brightness]},
	{TEXT("Vido contraste"),					cit_INT,	VIDEO_UNDEF,					& video_ctl[pat_Contrast]},
	{TEXT("Vido teinte"),						cit_INT,	VIDEO_UNDEF,					& video_ctl[pat_Hue]},
	{TEXT("Vido saturation"),					cit_INT,	VIDEO_UNDEF,					& video_ctl[pat_Saturation]},

	{TEXT("Volume"),							cit_INT,	100,							& volumeCourant},
	{TEXT("Latence zapping"),					cit_INT,	INI_DUREE_ZAPPING,				& duree_zapping},

	{TEXT("Grer les commandes multimdia"),	cit_BOOL,	true,							& use_multimedia},
	{TEXT("Molette de la souris"),				cit_ASSOC,	mwu_Channels,					& role_molette,			CONFIG_VAL(aMWheelTable)},
	{TEXT("Inversion du sens de la molette"),	cit_BOOL,	false,							& inversion_molette},

	{TEXT("tat fentre"),						cit_ASSOC,	etf_normal,						& configState,			CONFIG_VAL(aWStateTable)},
	{TEXT("Position fentre"),					cit_WPOS,	MAKELONG(800, 600),				& winPos,				CONFIG_VAL(&hMainWnd)},

#ifdef _DIAGNOSTIC	// versions diagnostic seulement
	{TEXT("Frquence multiplex diagnostic"),	cit_INT,	474167,							& mpx_frequence_diag},
#endif
	{TEXT("Priorit"),							cit_ASSOC,	ABOVE_NORMAL_PRIORITY_CLASS,	& configPriority,		CONFIG_VAL(aPrioTable)}
};

/// Item de slection du tuner lui-mme
static ConfigItem ini_tuner = {
	TEXT("Tuner TNT"),							cit_DRV,	CONFIG_VAL(NULL_NAME),			CONFIG_STR(nom_tuner)
};

/// Liste des items sauvegards sparment pour chaque type de tuner
static ConfigItem ini_tuner_items[] =
{ // Nom										Type		Dfaut							Variable
	// Rcepteur TNT (en fait filtre de capture) en premire position ( cause d'une fonction qui l'utilise via l'index 0)
	{TEXT("Rcepteur TNT"),						cit_STR,	CONFIG_VAL(NULL_NAME),			CONFIG_STR(sTunProps.nom_receiver)},
	{TEXT("Filtre BDA Tuner"),					cit_STR,	CONFIG_VAL(NULL_NAME),			CONFIG_STR(sTunProps.szPathTuner)},
	{TEXT("Filtre BDA Dmodulateur"),			cit_STR,	CONFIG_VAL(NULL_NAME),			CONFIG_STR(sTunProps.szPathDemod)},
	{TEXT("Filtre BDA Capture"),				cit_STR,	CONFIG_VAL(NULL_NAME),			CONFIG_STR(sTunProps.szPathCapture)},
	//
	{TEXT("Offset tuner"),						cit_INT,	0,								& sTunProps.offset_tuner},
	{TEXT("Essayer +5 kHz si chec"),			cit_BOOL,	INI_TRY_ADDING_5KHZ_DEFAULT,	& sTunProps.try_adding_5khz},
	{TEXT("Dlai verrouillage tuner"),			cit_INT,	INI_DELAI_TUNER1_DEFAULT,		& sTunProps.delai_tuner1},
	{TEXT("Dlai recherche tuner"),				cit_INT,	INI_DELAI_TUNER2_DEFAULT,		& sTunProps.delai_tuner2},
	{TEXT("Ignorer signal prsent"),			cit_BOOL,	false,							& sTunProps.scan_ignore_presence},
	{TEXT("Ignorer signal verrouill"),			cit_BOOL,	false,							& sTunProps.scan_ignore_lock},
	{TEXT("Ignorer signal qualit"),			cit_BOOL,	false,							& sTunProps.scan_ignore_quality},
	{TEXT("Valeur force signal nulle"),			cit_INT,	0,								& sTunProps.force_signal_zero},
	{TEXT("Dcalage force signal maximale"),	cit_INT,	100,							& sTunProps.force_signal_max},
	{TEXT("Force du signal en pourcentage"),	cit_BOOL,	false,							& sTunProps.force_signal_aff_pct},
	{TEXT("Resyntonisation auto"),				cit_BOOL,	true,							& sTunProps.auto_resyntonisation}
};

// ====================================================================================
// Raccourcis (clavier)
// ====================================================================================

bool Raccourci::operator ==(const Raccourci &r) const
{
	return maj == r.maj && ctrl == r.ctrl && alt == r.alt && touche == r.touche;
}

bool Raccourci::operator !=(const Raccourci &r) const
{
	return maj != r.maj || ctrl != r.ctrl || alt != r.alt || touche != r.touche;
}

Raccourci &Raccourci::operator =(const Raccourci &r)
{
	maj = r.maj;
	ctrl = r.ctrl;
	alt = r.alt;
	touche = r.touche;

	return *this;
}

ACCEL Raccourci::MakeAccel(WORD action) const
{
	ACCEL sAccel = {
		FVIRTKEY,		// fVirt
		(WORD)touche,	// key
		action			// cmd
	};

	if (maj)
		sAccel.fVirt |= FSHIFT;
	if (ctrl)
		sAccel.fVirt |= FCONTROL;
	if (alt)
		sAccel.fVirt |= FALT;
	return sAccel;
}

ShortcutItem &ShortcutItem::operator =(const ShortcutItem &s) {
	action = s.action;
	nom = s.nom;
	defaut = s.defaut;
	raccourci = s.raccourci;
	alternatif = s.alternatif;

	return *this;
}

/**
 * Gnre une chane de caractres correspondant au raccourci
 *
 * \param[out] buffer    chane de caractres gnre
 * \param[in]  taille    nombre de caractres dans le buffer
 **/
void Raccourci::ToString(LPTSTR buffer, UINT taille) const
{
	TCHAR	szWrk[32] = TEXT("");

	buffer[0] = 0;

	if (!isSet())
		return;

	// Traite les touche spciales
	if (ctrl)
		_tcscat_s(buffer, taille, TEXT("Ctrl+"));
	if (alt)
		_tcscat_s(buffer, taille, TEXT("Alt+"));
	if (maj)
		_tcscat_s(buffer, taille, TEXT("Shift+"));

	/**
	 * Tableau associatif dcrivant les diffrents textes
	 * franciss pour certaines touches de fonction
	 **/
	static AssocElement aShtCutsTable[] =
	{							// EtatsFenetre
		{TEXT("Space"),		VK_SPACE},
		{TEXT("Tab"),			VK_TAB},
		{TEXT("Backspace"),		VK_BACK},
		{TEXT("Insert"),		VK_INSERT},
		{TEXT("Del"),			VK_DELETE},
		{TEXT("Home"),			VK_HOME},
		{TEXT("End"),			VK_END},
		{TEXT("Page Dn"),		VK_PRIOR},
		{TEXT("Page Up"),		VK_NEXT},
		{TEXT("Caps Lock"),		VK_CAPITAL},
		{TEXT("Pause"),			VK_PAUSE},
		{TEXT("Up"),			VK_UP},
		{TEXT("Down"),			VK_DOWN},
		{TEXT("Left"),		VK_LEFT},
		{TEXT("Right"),		VK_RIGHT},
		{TEXT("Enter"),		VK_RETURN},
		{TEXT("Prnt Scrn"),		VK_SNAPSHOT},
		{TEXT("Num Lock"),		VK_NUMLOCK},
		{TEXT("Scroll"),	VK_SCROLL},
		{TEXT("Num-5"),			VK_CLEAR},
		{NULL, 0}		// -> terminateur
	};

	LPCTSTR pszTouche = dword2str(aShtCutsTable, touche);

	if (pszTouche[0]==TCHAR('?')) {
		/*
		// Structure du 'codeTouche' (seuls certains champs sont utiliss ici) :

		struct {
			WORD cRepeat	: 16;	// 0-15
			BYTE ScanCode	: 8 ;	// 16-23
			BYTE fExtended	: 1 ;	// 24
			BYTE Reserved	: 4 ;	// 25-28
			BYTE fAltDown	: 1 ;	// 29
			BYTE fRepeat	: 1 ;	// 30
			BYTE fUp		: 1 ;	// 31
		} vTouche;
		*/

		LONG codeTouche = MAKELONG(1,
				MAKEWORD(
					MapVirtualKey(touche, MAPVK_VK_TO_VSC),
					(touche >= VK_F1 && touche <= VK_F24 ? 0 : 1) ));

		GetKeyNameText(codeTouche, szWrk, _countof(szWrk));
		if (szWrk[0]==0) {
			// Si toujours aucune appellation de touche  ce stade ...
			_stprintf_s(szWrk, TEXT("<0x%02x>"), touche);

		}
		pszTouche = szWrk;
	}
	_tcscat_s(buffer, taille, pszTouche);
}

// ====================================================================================
// Noms de fichiers avec motif
// ====================================================================================

/**
 * Constructeur permettant de gnrer le nom du fichier d'enregistrement
 *
 * \param[in]	dir			dossier pour l'enregistrement du fichier
 * \param[in]	pszPrgName	nom de l'enregistrement (seulement en cas de programmation d'enregistrement)
 * \param[in]	pszChanName	nom de la chane enregistre
 * \param[in]	nChanNo		numro de la chane enregistre
 * \param[in]	pszExt		extension du fichier (permet de dfinir le type du fichier (BMP, MPG, TS, ...)
 **/
NomFichierAvecMotif::NomFichierAvecMotif(LPCTSTR dir, LPCTSTR pszPrgName, LPCSTR pszChanName, int nChanNo, LPCTSTR pszExt)
{
	// Remplace les caractres invalides par des espaces
	NomProtege<TCHAR>	prefix_prot1(pszPrgName, TCHAR(' '));
	NomProtege<TCHAR>	prefix_prot2(pszChanName, TCHAR(' '));
	// Date/heure de l'enregistrement
	SYSTEMTIME			localtime;

	GetLocalTime(&localtime);

	// Met le chemin de sauvegarde
	_stprintf_s(nom, TEXT("%s\\"), dir);

	// Maintenant, parse le motif du nom
	//     %c = chane
	//     %n = numro de chane
	//     %e = nom de l'enregistrement
	//     %a = anne
	//     %m = mois
	//     %j = jour
	//     %h = heure
	//     %M = minute
	//     %s = secondes
	int			sav_pos	= -1;	// sauvegarde du dbut du groupe  ignorer si la prochaine zone est vide
								// aprs un point d'interrogation
	int			lit_pos	= -1;	// sauvegarde du dbut du groupe  ignorer si la zone prcdente tait vide
								// et qu'un point d'interrogation est ensuite rencontr
	int			lmotif	= (int)strlen_T(motif_noms_enreg);
	SZPathName	szTmp;

	for (int i=0; i<lmotif; i++) {
		szTmp[0] = 0;

		if (motif_noms_enreg[i] == TEXT('%')) {
			switch (motif_noms_enreg[i+1]) {

			case TCHAR('n'):	// numro de la chane
				if (nChanNo != -1 && nChanNo < 1000)
					_itot_s(nChanNo, szTmp, 10);
				break;

			case TCHAR('N'):	// numro de la chane (forc sur trois chiffres)
				if (nChanNo != -1 && nChanNo < 1000)
					_stprintf_s(szTmp, TEXT("%03u"), nChanNo);
				break;

			case TCHAR('c'):	// chane
				if (prefix_prot2.nom[0] != 0)
					prefix_prot2.Extract(szTmp, _countof(szTmp));
				break;

			case TCHAR('e'):	// nom de l'enregistrement
				if (prefix_prot1.nom[0] != 0)
					prefix_prot1.Extract(szTmp, _countof(szTmp));
				break;

			case TCHAR('a'):	// anne
				_stprintf_s(szTmp, TEXT("%04d"), localtime.wYear);
				break;

			case TCHAR('m'):	// mois
				_stprintf_s(szTmp, TEXT("%02d"), localtime.wMonth);
				break;

			case TCHAR('j'):	// jour
				_stprintf_s(szTmp, TEXT("%02d"), localtime.wDay);
				break;

			case TCHAR('h'):	// heure
				_stprintf_s(szTmp, TEXT("%02d"), localtime.wHour);
				break;

			case TCHAR('M'):	// minute
				_stprintf_s(szTmp, TEXT("%02d"), localtime.wMinute);
				break;

			case TCHAR('s'):	// secondes
				_stprintf_s(szTmp, TEXT("%02d"), localtime.wSecond);
				break;

			default:	// inconnu, on recopie
				_tcsncat_s(szTmp, &motif_noms_enreg[i], _countof(szTmp)-1);
				break;
			}
			i++;
			if (szTmp[0] == 0) {	// Aucune donne produite
				if (sav_pos>=0) {
					// On dtruit les caractres littraux prcdents
					// si un point d'interrogation les a prcds
					nom[sav_pos] = 0;
					sav_pos = -1;
				}
				lit_pos = (int)strlen_T(nom);	// mmoriser position pour effacement avant le '?'
			} else
				lit_pos = -1;
		} else if (motif_noms_enreg[i] == L'?') {
			// Le point d'interrogation indique que les caractres littraux prcdents sont ignors
			// si le champ "%" prcdent n'insrait aucune donne, et/ou que les caractres littraux
			// subsquents seront ignors si le champ "%" suivant n'insre aucune donne.
			if (lit_pos>=0) {
				// On dtruit les caractres littraux prcdents
				// si aucune donne n'avait t insre par le champ "%" prcdent
				nom[lit_pos] = 0;
				lit_pos = -1;
			}
			sav_pos = (int)strlen_T(nom); // -> on mmorise la position
		} else {
			szTmp[0] = motif_noms_enreg[i];
			szTmp[1] = 0;
		}
		if (szTmp[0] != 0)
			strcat_T(nom, szTmp);
	}
	// rajoute l'extension
	strcat_T(nom, TEXT("."));
	strcat_T(nom, pszExt);
}

/**
 * Ouvre le fichier (d'enregistrement) en criture
 *
 * \return  le handle d'ouverture du fichier
 **/
HANDLE NomFichierAvecMotif::Create() const
{
	return CreateFile(nom, GENERIC_WRITE, FILE_SHARE_READ, NULL,
		CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);
}

// ====================================================================================
/// Classe d'accs aux fichiers d'initialisation (.ini)
// ====================================================================================

class CIniFile
{
	NomFichierConf<>	sIniFilePathName;
protected:
	TCHAR				szSectionName[128];
	DWORD				dwError;
public:
	CIniFile(LPCTSTR pszFileName, bool bUsePrefix=true, LPCTSTR pszDir=pouchindir_conf) :
		sIniFilePathName(pszFileName, bUsePrefix, pszDir),
		dwError(0)
	{
		szSectionName[0] = 0;
	}

	LPCTSTR	getFileName() const {
		return sIniFilePathName; }

	/**
	 * Fonction retournant une table contenant une liste des noms de sections existants.
	 * \param[out]  vstrSections	liste des sections
	 **/
	void GetSectionNames(vector<tstring> & vstrSections);

	/**
	 * Slection d'une nouvelle section
	 **/
	CIniFile & NewSection(LPCTSTR pszSectionName) {
		strcpy_T(szSectionName, pszSectionName);
		return *this;
	}

	/**
	 * Sauve la chane de caractres passe en paramtre
	 *
	 * \param[in]  lpKeyName   nom de la cl devant tre utilise
	 * \param[in]  lpString    chane de caractres  enregistrer
	 * \retval  "une valeur non nulle" en cas de succs
	 * \retval  0  en cas d'erreur
	 **/
	BOOL save_str(LPCTSTR lpKeyName, LPCTSTR lpString);

	/**
	 * crit une valeur numrique
	 *
	 * \param[in]  lpKeyName   nom de la cl devant tre utilise
	 * \param[in]  value       valeur numrique  enregistrer
	 * \retval  "une valeur non nulle" en cas de succs
	 * \retval  0  en cas d'erreur
	 **/
	BOOL save_int(LPCTSTR lpKeyName, INT32 value);

	/**
	 * crit une valeur numrique au format hexadcimal 32 bits
	 *
	 * \param[in]  lpKeyName   nom de la cl devant tre utilise
	 * \param[in]  value       valeur numrique  enregistrer
	 * \retval  "une valeur non nulle" en cas de succs
	 * \retval  0  en cas d'erreur
	 * \note  Les valeurs de ce type sont rcuprables avec \p load_int
	 **/
	BOOL save_hex32(LPCTSTR lpKeyName, UINT32 value);

	/**
	 * criture de la chane de caractres associe  une valeur fournie
	 *
	 * \param[in]  lpKeyName   nom de la cl devant tre utilise
	 * \param[in]  assocTable  \p AssocElement contenant le tableau  traduire
	 * \param[in]  value       valeur numrique devant tre convertie en chaine de caractres
	 * \retval  "une valeur non nulle" en cas de succs
	 * \retval  0  en cas d'erreur
	 **/
	BOOL save_assoc(LPCTSTR lpKeyName, const AssocElement * assocTable, DWORD value);

	/**
	 * Sauvegarde un boolen
	 *
	 * \param[in]  lpKeyName  nom de la cl devant tre utilise
	 * \param[in]  value      valeur  enregistrer
	 * \retval  "une valeur non nulle" en cas de succs
	 * \retval  0  en cas d'erreur
	 **/
	BOOL save_bool(LPCTSTR lpKeyName, bool value);

	/**
	 * Cherche une chane de caractres dans le fichier de configuration
	 *
	 * \param[in]	lpKeyName	nom de l'option  rechercher
	 * \param[out]	lpValue		valeur recherche
	 * \param[in]	bufSize		nombre de caractres dans la chane de retours
	 * \param[in]	lpDefault	valeur par dfaut, si on n'a rien trouv
	 * \return		Nombre de caractres copis, sans tenir compte du caractre '\\0'
	 **/
	DWORD load_str(LPCTSTR lpKeyName, LPTSTR lpValue, DWORD bufSize, LPCTSTR lpDefault=TEXT(""));

	/**
	 * Version "template" de \p load_str
	 **/
	template <size_t S>
		DWORD load_str(LPCTSTR lpKeyName, TCHAR(&lpValue)[S], LPCTSTR lpDefault=TEXT(""))
	{
		return load_str(lpKeyName, lpValue, S, lpDefault);
	}

	/**
	 * Cherche une valeur numrique dans le fichier de configuration
	 *
	 * \param[in]  lpKeyName  nom de l'option  rechercher
	 * \param[in]  nDefault   valeur par dfaut, si on n'a rien trouv
	 * \return                valeur recherche (trouve, ou par dfaut)
	 **/
	INT32 load_int(LPCTSTR lpKeyName, INT nDefault);

	/**
	 * Cherche une valeur numrique, en convertissant la valeur textuelle contenue dans le fichier de configuration
	 *
	 * \param[in]  lpKeyName   nom de l'option  rechercher
	 * \param[in]  assocTable  tableau associatif contenant les faleurs  rechercher
	 * \param[in]  bydefault   valeur par dfaut, si on n'a rien trouv
	 * \return                 valeur recherche (trouve, ou par dfaut)
	 **/
	DWORD load_assoc(LPCTSTR lpKeyName, const AssocElement * assocTable, DWORD bydefault);

	/**
	 * Charge une valeur binaire dans le fichier de configuration, en convertissant les valeurs textuelles
	 *
	 * \param[in]  lpKeyName  nom de l'option  rechercher
	 * \param[in]  bydefault  valeur par dfaut, si on n'a rien trouv
	 * \return                valeur recherche (trouve, ou par dfaut)
	 **/
	bool load_bool(LPCTSTR lpKeyName, bool bydefault=false);
};

/**
 * Fonction retournant une table contenant une liste des noms de sections existants.
 * \param[out]  vstrSections	liste des sections
 **/
void CIniFile::GetSectionNames(vector<tstring> & vstrSections)
{
	// Nombre de caractres  ajouter pour toutes les contenir en une fois
	const int	delta = 0x1000;

	// nombre de caractres pour contenir les sections
	UINT32		longueur = delta;

	tstring	strSections;

	strSections.resize(longueur);

	// Dtermine la taille de la chane devant contenir toutes les sections
	// car sinon, si le fichier INI contient trop de sections, on ne le lit pas les dernires
	while (GetPrivateProfileSectionNames(&strSections[0], longueur, sIniFilePathName) == longueur-2) {
		// la taille du tableau est trop petite, on l'agrandit
		strSections.resize(longueur += delta);
	}

	LPCTSTR p = strSections.c_str();

	vstrSections.clear();
	while (*p) {
		vstrSections.push_back(tstring(p));

		// Avance au prochain nom
		p += strlen_T(p)+1;
	}
}

/**
 * Sauve la chane de caractres passe en paramtre
 *
 * \param[in]  lpKeyName   nom de la cl devant tre utilise
 * \param[in]  lpString    chane de caractres  enregistrer
 * \retval  "une valeur non nulle" en cas de succs
 * \retval  0  en cas d'erreur
 **/
BOOL CIniFile::save_str(LPCTSTR lpKeyName, LPCTSTR lpString)
{
	if (dwError)
		return FALSE;

	BOOL bRes =  WritePrivateProfileString(szSectionName, lpKeyName, lpString, sIniFilePathName);

	if (!bRes) {
		// Une erreur a toutes les chances de se rpter pour toutes les critures dans le mme
		// fichier, donc on ne retente pas tant que le fichier n'a pas chang.
		dwError = GetLastError();

		myprintf(ERRX(TEXT("Erreur criture dans :") EOL TEXT("%s :") EOL TEXT("%s")),
			(LPCTSTR)sIniFilePathName,  SystemErrorToString(dwError).c_str());
	}
	return bRes;
}

/**
 * crit une valeur numrique
 *
 * \param[in]  lpKeyName   nom de la cl devant tre utilise
 * \param[in]  value       valeur numrique  enregistrer
 * \retval  "une valeur non nulle" en cas de succs
 * \retval  0  en cas d'erreur
 **/
BOOL CIniFile::save_int(LPCTSTR lpKeyName, INT32 value)
{
	TCHAR valeur[64];

	_itot_s(value, valeur, _countof(valeur), 10);
	return save_str(lpKeyName, valeur);
}

/**
 * crit une valeur numrique au format hexadcimal 32 bits
 *
 * \param[in]  lpKeyName   nom de la cl devant tre utilise
 * \param[in]  value       valeur numrique  enregistrer
 * \retval  "une valeur non nulle" en cas de succs
 * \retval  0  en cas d'erreur
 **/
BOOL CIniFile::save_hex32(LPCTSTR lpKeyName, UINT32 value)
{
	TCHAR valeur[64];

	_stprintf_s(valeur, TEXT("0x%08x"), value);
	return save_str(lpKeyName, valeur);
}

/**
 * criture de la chane de caractres associe  une valeur fournie
 *
 * \param[in]  lpKeyName   nom de la cl devant tre utilise
 * \param[in]  assocTable  \p AssocElement contenant le tableau  traduire
 * \param[in]  value       valeur numrique devant tre convertie en chaine de caractres
 * \retval  "une valeur non nulle" en cas de succs
 * \retval  0  en cas d'erreur
 **/
BOOL CIniFile::save_assoc(LPCTSTR lpKeyName, const AssocElement * assocTable, DWORD value)
{
	return save_str(lpKeyName, dword2str(assocTable, value));
}

/**
 * Sauvegarde un boolen
 *
 * \param[in]  lpKeyName  nom de la cl devant tre utilise
 * \param[in]  value      valeur  enregistrer
 * \retval  "une valeur non nulle" en cas de succs
 * \retval  0  en cas d'erreur
 **/
BOOL CIniFile::save_bool(LPCTSTR lpKeyName, bool value)
{
	return save_assoc(lpKeyName, aBoolTable, value);
}

/**
 * Cherche une chane de caractres dans le fichier de configuration
 *
 * \param[in]	lpKeyName	nom de l'option  rechercher
 * \param[out]	lpValue		valeur recherche
 * \param[in]	bufSize		nombre de caractres dans la chane de retours
 * \param[in]	lpDefault	valeur par dfaut, si on n'a rien trouv
 * \return		Nombre de caractres copis, sans tenir compte du caractre '\\0'
 **/
DWORD CIniFile::load_str(LPCTSTR lpKeyName, LPTSTR lpValue, DWORD bufSize, LPCTSTR lpDefault)
{
	return GetPrivateProfileString(szSectionName, lpKeyName, lpDefault, lpValue, bufSize, sIniFilePathName);
}

/**
 * Cherche une valeur numrique dans le fichier de configuration
 *
 * \param[in]  lpKeyName  nom de l'option  rechercher
 * \param[in]  nDefault   valeur par dfaut, si on n'a rien trouv
 * \return                valeur recherche (trouve, ou par dfaut)
 **/
INT32 CIniFile::load_int(LPCTSTR lpKeyName, INT nDefault)
{
	return (INT)GetPrivateProfileInt(szSectionName, lpKeyName, nDefault, sIniFilePathName);
}

/**
 * Cherche une valeur numrique, en convertissant la valeur textuelle contenue dans le fichier de configuration
 *
 * \param[in]  lpKeyName   nom de l'option  rechercher
 * \param[in]  assocTable  tableau associatif contenant les faleurs  rechercher
 * \param[in]  bydefault   valeur par dfaut, si on n'a rien trouv
 * \return                 valeur recherche (trouve, ou par dfaut)
 **/
DWORD CIniFile::load_assoc(LPCTSTR lpKeyName, const AssocElement * assocTable, DWORD bydefault)
{
	TCHAR str[32];

	load_str(lpKeyName, str);
	return str2dword(assocTable, str, bydefault);
}

/**
 * Charge une valeur binaire dans le fichier de configuration, en convertissant les valeurs textuelles
 *
 * \param[in]  lpKeyName  nom de l'option  rechercher
 * \param[in]  bydefault  valeur par dfaut, si on n'a rien trouv
 * \return                valeur recherche (trouve, ou par dfaut)
 **/
bool CIniFile::load_bool(LPCTSTR lpKeyName, bool bydefault)
{
	return load_assoc(lpKeyName, aBoolTable, bydefault)!=0;
}

// ====================================================================================
/// Classe d'accs au fichier de configuration gnrale
// ====================================================================================

class CConfigFile : public CIniFile
{
public:

	/// Constructeur
	CConfigFile(LPCTSTR pszDir=pouchindir_conf) :
		CIniFile(config_file, true, pszDir)
	{
		NewSection(ini_config_section);		// Section initiale, par dfaut
	}

	/**
	 * Ajustement d'un nom de cl selon la plateforme utilise
	 * \param[in]	pszKey		Nom de la cl devant tre ajuste
	 * \param[in]	nBufSize	Taille du tampon qui la contient
	 **/
	void PlatformAdjust(LPTSTR pszKey, UINT32 nBufSize);

	/// Slection d'une nouvelle section base sur un nom de tuner
	CConfigFile & NewTunerSection(LPCTSTR pszTunerName, bool bMigrating = false);

	/**
	 * Sauve un nom de pilote pass en paramtre.
	 * Gre le cas de la prsence d'un debugger.
	 *
	 * \param[in]  lpKeyName   nom de la cl devant tre utilise
	 * \param[in]  lpString    chane de caractres  enregistrer
	 * \retval  "une valeur non nulle" en cas de succs
	 * \retval  0  en cas d'erreur
	 **/
	BOOL save_driver_str(LPCTSTR lpKeyName, LPCTSTR lpString);

	/**
	 * Sauvegarde d'une position normale de fentre
	 *
	 * \param[in]  lpKeyName  nom de la cl devant tre utilise pour enregistrer la
	 *                        position/taille de la fentre
	 * \param[in]  hWnd       handle de la fentre sui doit tre sauvegarde
	 * \retval                0 en cas d'chec
	 * \retval                "toute autre valeur" en cas de succs
	 **/
	BOOL save_winpos(LPCTSTR lpKeyName, HWND hWnd);

	/**
	 * \brief Charge les donnes spcifiques  un tuner
	 * \param[in]	pszNomTuner	Nom du tuner dont les donnes sont  charger
	 * \param[in]	bMigrating	Mettre \p true pour rcuprer les donnes d'une ancienne version
	 **/
	void load_tuner_config(LPCTSTR pszNomTuner, bool bMigrating);

	/**
	 * Sauvegarde d'un raccourci
	 *
	 * \param[in]  lpKeyName  nom de la cl devant tre utilise pour enregistrer le raccourci
	 * \param[in]  sShtCut    rfrence du raccourci  sauvegarder
	 * \param[in]  sShtDef    rfrence du raccourci par dfaut correspondant (si les deux
	 *                        sont les mmes, la section est efface)
	 * \retval                0 en cas d'chec
	 * \retval                "toute autre valeur" en cas de succs
	 **/
	BOOL save_shortcut(LPCTSTR lpKeyName, const Raccourci & sShtCut, const Raccourci & sShtDef);

	/**
	 * Sauvegarde d'un item dcrit par structure
	 *
	 * \param[in]  sItem	description de l'item  sauvegarder
	 **/
	void save_item(const ConfigItem & sItem);

	/**
	 * Chargement d'une liste d'items
	 **/
	template <size_t S>
		void save_items(const ConfigItem(&aItDescr)[S])
	{
		for (int nInx=0; nInx < S; ++nInx)
			save_item(aItDescr[nInx]);
	}

	/**
	 * Sauvegarde des couleurs personnalises
	 **/
	void save_colors();

	/**
	 * Sauvegarde tous les raccourcis
	 **/
	void save_raccourcis();

	/**
	 * \brief Sauvegarde les donnes spcifiques  un tuner
	 * \param[in]	pszNomTuner	Nom du tuner dont les donnes sont  sauvegarder
	 **/
	void save_tuner_config(LPCTSTR pszNomTuner);

	/**
	 * Cherche un nom de pilote dans le fichier de configuration.
	 * Gre le cas de la prsence d'un debugger.
	 *
	 * \param[in]	lpKeyName	nom de l'option  rechercher
	 * \param[out]	lpValue		valeur recherche
	 * \param[in]	bufSize		nombre de caractres dans la chane de retours
	 * \return		Nombre de caractres copis, sans tenir compte du caractre '\\0'
	 **/
	DWORD load_driver_str(LPCTSTR lpKeyName, LPTSTR lpValue, DWORD bufSize);

	/**
	 * Version "template" de \p load_driver_str
	 **/
	template <size_t S>
		DWORD load_driver_str(LPCTSTR lpKeyName, TCHAR(&lpValue)[S])
	{
		return load_driver_str(lpKeyName, lpValue, S);
	}

	/**
	 * Charge la taille/position de la fentre dans le fichier de configuration
	 *
	 * \param[in]  lpKeyName  nom de l'option  rechercher
	 * \param[out] wpos       structure retournant la taille trouve, ou initialise avec les valeurs par dfaut
	 * \param[in]  defL       largeur par dfaut, si pas trouv
	 * \param[in]  defH       hauteur par dfaut, si pas trouv
	 **/
	void load_winpos(LPCTSTR lpKeyName, WindowPos & wpos, long defL, long defH);

	/**
	 * Cherche un paramtre dans le fichier de configuration \c "config.ini"
	 *
	 * \param[in]	lpKeyName		Cl dans le fichier de configuration recherche
	 * \param[out]	lpValue			Valeur recherche, devant dj tre initialis
	 * \param[in]	bufSize			Nombre de caractres pouvant tre retourn dans \a lpValue
	 * \param[in]	nDefaultFolder	Dossier par dfaut  retourner, si on n'a rien trouv
	 *								(valeur CSIDL obligatoire: http://msdn.microsoft.com/en-us/library/bb762494(VS.85).aspx)
	 * \return		Nombre de caractres copis, sans tenir compte du caractre '\\0'
	 * \warning		Si le dossier dsir demand par \a nDefaultFolder n'existe pas (parce que la version est trop vieille, ou autre),
	 *				celui-ci est remplac par le dossier \c "Mes Documents"
	 **/
	DWORD load_path(LPCTSTR lpKeyName, LPTSTR lpValue, DWORD bufSize, int nDefaultFolder);

	/**
	 * Chargement d'un raccourci
	 *
	 * \param[in]  lpKeyName  nom de la cl devant tre utilise pour enregistrer le raccourci
	 * \param[out] sShtCut    rfrence sur le raccourci  charger
	 * \param[in]  reset	  \p true si le raccourci doit tre rinitialis  sa valeur par dfaut
	 * \retval                0 en cas d'chec
	 * \retval                "toute autre valeur" en cas de succs
	 **/
	void load_shortcut(LPCTSTR lpKeyName, Raccourci & sShtCut, bool reset);

	/**
	 * Chargement d'un item dcrit par structure
	 *
	 * \param[in]  sItem	description de l'item  charger
	 * \return     (pour les chanes seulement) nombre de caractres copis,
	 *				sans tenir compte du caractre '\\0'
	 **/
	DWORD load_item(const ConfigItem & sItem);

	/**
	 * Chargement d'une liste d'items
	 **/
	template <size_t S>
		void load_items(const ConfigItem(&aItDescr)[S])
	{
		for (int nInx=0; nInx < S; ++nInx)
			load_item(aItDescr[nInx]);
	}

	/**
	 * Chargement des couleurs personnalises
	 **/
	void load_colors();

	/**
	 * Charge les raccourcis
	 *
	 * \param[in]  reset  si \a true , on charge les raccourcis par dfaut
	 **/
	void load_raccourcis(bool reset=false);
};

// ====================================================================================
// Recherche de chanes
// ====================================================================================

/**
 * Vrifie l'existence du fichier "canaux.ini", et le cre  partir des ressources
 * s'il n'existe pas.
 **/
void cree_fichier_scan()
{
	// Extrait le fichier canaux.ini des resources
	RawResource	vCompressed;
	ByteVector	vUncompressed;

	if (!vCompressed.Extraction(hAppInstance, IDR_CANAUX))
		return;	// -> Ressource inexistante

	if (!DecompressLzma(vUncompressed, vCompressed))
		return;	// -> Dcompression impossible

	// Tout s'est bien pass : on peut utiliser les canaux.ini
	CIniFile	cPrevScan(scan_ini, false);
	TCHAR		szBuffer[32] = {0};

	cPrevScan.NewSection(TEXT("Gnral"));

	if (cPrevScan.load_str(TEXT("revision"), szBuffer) > 0) {
		int		vers_ini, vers_res;	// les diffrentes valeurs des rvisions

		// Extraction de la version du fichier local existant
		if (_stscanf_s(szBuffer, TEXT("$Rev: %u $"), &vers_ini) == 1) {
			// Extraction de la version du fichier intgr dans la ressource
			if (sscanf_s(vUncompressed, "%*[^$]$Rev: %u $", &vers_res) == 1) {
				// on compare les versions
				if (vers_res <= vers_ini)
					return;	// -> On a dj la dernire version
			}
		}
	}

	DWORD dwRes = vUncompressed.SaveAsFile(cPrevScan.getFileName());

#if USE_CONSOLE
	if (dwRes != 0)
		myprintf(ERRX(TEXT("Fichier %s => Erreur criture : %s")), scan_ini, SystemErrorToString(dwRes));
	else
		myprintf(TEXT("Fichier %s => Cration") EOL, scan_ini);
#endif
}

/// Ajout de tous les canaux possibles  la liste
void ListeCanaux::add_fullscan()
{
	for (BYTE canal_no=21; canal_no<=69; ++canal_no)
		add(canal_no);
}

ListeCanaux::ParseListRes ListeCanaux::get_num(LPCTSTR & pszStr, BYTE & nChan)
{
	LPTSTR	pszEnd;
	long	nVal;

	while (*pszStr == TCHAR(' '))
		++pszStr;
	nVal = _tcstol(pszStr, &pszEnd, 10);
	if (pszStr==pszEnd)
		return plr_syntaxerror;
	while (*pszEnd == TCHAR(' '))
		++pszEnd;
	pszStr = pszEnd;
	if (nVal<21 || nVal>69)
		return plr_outofrange;
	nChan = BYTE(nVal);
	return plr_noerror;
}

/// Convertit directement un numro de canal en frquence, en tenant compte de l'offset
/// ventuellement ajout
UINT ListeCanaux::canal2khz(BYTE nNoCanal) const
{
	static INT32 aOffsets[] = {-167, 0, 167, 333, 500};

	UINT nKHz = nNoCanal * (DVBT_BANDWIDTH*1000);	// largeur de bande

	nKHz += nOffset;				// Offset par site (dsactiv)

	// Application de l'offset
	if (use_offsets>=0 && use_offsets<_countof(aOffsets))
		nKHz += aOffsets[use_offsets];

	return nKHz;
}

/// Ajout d'une liste saisie
ListeCanaux::ParseListRes ListeCanaux::add_list(LPCTSTR pszList)
{
	BYTE			nBeg, nEnd;
	ParseListRes	eRes;

	while (*pszList) {
		// Lecture d'une valeur ou d'un intervalle
		if ((eRes=get_num(pszList, nBeg))!=plr_noerror)
			return eRes;						// Nombre incorrect ou pas de nombre
		if (*pszList==TCHAR('-')) {
			++pszList;
			// Lecture fin d'intervalle
			if ((eRes=get_num(pszList, nEnd))!=plr_noerror)
				return eRes;					// Nombre incorrect ou pas de nombre
		} else
			nEnd = nBeg;

		// Insrer dans la table
		if (nEnd>=nBeg) {
			do {
				if (!add(nBeg++))
					return plr_toomanyvals;		// Espace insuffisant
			} while (nBeg<=nEnd);
		} else {
			while (nBeg>=nEnd)
				if (!add(nBeg--))
					return plr_toomanyvals;		// Espace insuffisant
		}

		// Sauter la virgule ventuelle
		if (*pszList==TCHAR(','))
			++pszList;
	}

	return plr_noerror;
}

/**
 * Lecture du fichier "canaux.ini", qui dcrit les listes disponibles de canaux  explorer
 *
 * \param[in]	pszCanaux	nom de la section contenant les canaux  explorer
 * \note	Ce fichier contenait en plus des listes de canaux par metteurs (noms de villes)
 *			dans les versions prcdentes, mais suite  la multiplication des metteurs et 
 *			la frquente modification de ceux-ci, ceci a t supprim.
 **/
void ListeCanaux::lit_fichier_scan(LPCTSTR pszCanaux)
{
	CIniFile	cChanFile(scan_ini, false);

	clear();
	cChanFile.NewSection(pszCanaux);

	// Dtermine si la liste de canaux choisie est dfinie. Si elle ne l'est pas, explore
	// toutes les frquences, entre les canaux 21 et 69.
	BYTE canal_no = (BYTE)cChanFile.load_int(TEXT("R1"), 0);

	if (canal_no == 0)
		add_fullscan();
	else {
		// On rcupre les canaux dans le fichier "canaux.ini"
		TCHAR	id_multiplex[4];
		UINT	nr_multiplex = 1;
		do {
			_stprintf_s(id_multiplex, _countof(id_multiplex), TEXT("R%d"), nr_multiplex);
			canal_no = (BYTE)cChanFile.load_int(id_multiplex, 0xff);
			if (canal_no != 0 && canal_no != 0xff)
				add(canal_no);
			nr_multiplex++;
		} while (canal_no != 0xff && nr_multiplex <= 99);
	}
}

// ====================================================================================
// Programmations d'enregistrements
// ====================================================================================

/**
 * Lecture des programmations d'enregistrement depuis un fichier.
 * \todo Retirer dans quelques versions (car plus utile) l'ancien systme de sauvegarde des
 * enregistrements programms, exploitant les tableaux \p Xstn_S et \p Xstn_E, ainsi que le
 * recalcul du SID avec \p indexToSID ; attention cependant aux personnes qui migreraient
 * tardivement depuis une trs ancienne version du programme.
 **/
HRESULT lit_programmes()
{
	TiXmlDocument	cTiXmlDoc;

	if (cTiXmlDoc.LoadFile(NomFichierConf<CHAR, CHAR>(FILENAME_PROGRAMS), TIXML_ENCODING_UTF8)) {
		if (Programmes.SetFromXMLElement(cTiXmlDoc.RootElement()))
			return S_OK;
	}
	return E_FAIL;
}

/**
 * Sauvegarde des programmations d'enregistrements dans un fichier.
 **/
void sauve_programmes()
{
	TiXmlDocument		xmlDoc(NomFichierConf<CHAR, CHAR>(FILENAME_PROGRAMS));
	TiXmlDeclaration *	pXmlDecl(new TiXmlDeclaration("1.0", "UTF-8", "yes"));

	if (pXmlDecl) {
		xmlDoc.LinkEndChild(pXmlDecl);

		TiXmlElement *	pXmlPrLst(Programmes.GetAsXMLElement());
		if (pXmlPrLst) {
			xmlDoc.LinkEndChild(pXmlPrLst);
			xmlDoc.SaveFile();
		}
	}
}

// ====================================================================================
// Liste des chanes dfinies
// ====================================================================================

/**
 * Chargement du fichier des chanes "chaines.xml".
 *
 * \retval S_OK		en cas de succs
 * \retval S_FAILED (ou autre) si erreur
 **/
HRESULT lit_chaines()
{
	TiXmlDocument	cTiXmlDoc;

	if (cTiXmlDoc.LoadFile(NomFichierConf<CHAR, CHAR>(FILENAME_CHANNELS), TIXML_ENCODING_UTF8)) {
		if (Canaux.SetFromXMLElement(cTiXmlDoc.RootElement()))
			return S_OK;
	}
	return E_FAIL;
}

/**
 * Sauvegarde de la liste des chanes dans "chaines.xml".
 **/
void sauve_chaines()
{
	TiXmlDocument		xmlDoc(NomFichierConf<CHAR, CHAR>(FILENAME_CHANNELS));
	TiXmlDeclaration *	pXmlDecl(new TiXmlDeclaration("1.0", "UTF-8", "yes"));

	if (pXmlDecl) {
		xmlDoc.LinkEndChild(pXmlDecl);

		TiXmlElement *	pXmlChLst(Canaux.GetAsXMLElement());
		if (pXmlChLst) {
			xmlDoc.LinkEndChild(pXmlChLst);

			xmlDoc.SaveFile();
		}
	}
}

// ====================================================================================
// Membres de la classe d'accs aux fichiers d'initialisation (.ini) - sauvegarde
// ====================================================================================

/**
 * Ajustement d'un nom de cl selon la plateforme utilise
 * \param[in]  pszKey	Nom de la cl devant tre ajuste
 * \param[in]  nBufSize	Taille du tampon qui la contient
 **/
void CConfigFile::PlatformAdjust(LPTSTR pszKey, UINT32 nBufSize)
{
#ifdef _WIN64
	// Utilisation d'appellations distinctes en OS 64 bits : les deux versions peuvent
	// y exister concurremment, et les pilotes ou codecs vus par chacune sont diffrents.
	strcat_T(pszKey, nBufSize, TEXT(" (x64)"));
#endif
	if (TestIfDebuggerPresent())
		strcat_T(pszKey, nBufSize, TEXT(" (debug)"));
}

CConfigFile & CConfigFile::NewTunerSection(LPCTSTR pszTunerName, bool bMigrating)
{
	TCHAR szWrk[256] = TEXT("Tuner");

	if (!bMigrating)
		PlatformAdjust(szWrk, _countof(szWrk));
	strcat_T(szWrk, TEXT(" : "));
	strcat_T(szWrk, pszTunerName);
	NewSection(szWrk);
	return *this;
}

/**
 * Sauve un nom de pilote pass en paramtre.
 * Gre le cas de la prsence d'un debugger.
 *
 * \param[in]  lpKeyName   nom de la cl devant tre utilise
 * \param[in]  lpString    chane de caractres  enregistrer
 * \retval  "une valeur non nulle" en cas de succs
 * \retval  0  en cas d'erreur
 **/
BOOL CConfigFile::save_driver_str(LPCTSTR lpKeyName, LPCTSTR lpString)
{
	TCHAR	szActualName[64];

	strcpy_T(szActualName, lpKeyName);
	PlatformAdjust(szActualName, _countof(szActualName));
	return save_str(szActualName, lpString);
}

/**
 * Sauvegarde d'une position normale de fentre
 *
 * \param[in]	lpKeyName	nom de la cl devant tre utilise pour enregistrer la
 *							position/taille de la fentre
 * \param[in]	hWnd		handle de la fentre qui doit tre sauvegarde
 * \retval					0 en cas d'chec
 * \retval					"toute autre valeur" en cas de succs
 **/
 BOOL CConfigFile::save_winpos(LPCTSTR lpKeyName, HWND hWnd)
{
	WINDOWPLACEMENT wpl;
	TCHAR position[32];

	GetWindowPlacement(hWnd, &wpl);
	_stprintf_s(position, _countof(position), TEXT("%i,%i,%i,%i"),
		wpl.rcNormalPosition.left,
		wpl.rcNormalPosition.top,
		wpl.rcNormalPosition.right - wpl.rcNormalPosition.left,
		wpl.rcNormalPosition.bottom - wpl.rcNormalPosition.top);
	return save_str(lpKeyName, position);
}

/**
 * Sauvegarde d'un raccourci
 *
 * \param[in]  lpKeyName  nom de la cl devant tre utilise pour enregistrer le raccourci
 * \param[in]  sShtCut    rfrence du raccourci  sauvegarder
 * \param[in]  sShtDef    rfrence du raccourci par dfaut correspondant (si les deux
 *                        sont les mmes, la section est efface)
 * \retval                0 en cas d'chec
 * \retval                "toute autre valeur" en cas de succs
 **/
BOOL CConfigFile::save_shortcut(LPCTSTR lpKeyName, const Raccourci & sShtCut, const Raccourci & sShtDef)
{
	TCHAR	valeur[128];

	if (sShtDef != sShtCut && sShtCut.isSet()) {
		_stprintf_s(valeur, _countof(valeur), TEXT("%i,%i,%i,%i"),
			sShtCut.maj,
			sShtCut.ctrl,
			sShtCut.alt,
			sShtCut.touche);
		return save_str(lpKeyName, valeur);
	} else
		return save_str(lpKeyName, NULL);
}

/**
 * Sauvegarde des couleurs personnalises
 **/
void CConfigFile::save_colors()
{
	for (int nInx=0; nInx<_countof(aCustomColors); ++nInx) {
		COLORREF	col = aCustomColors[nInx];

		if (col != RGB(192, 192, 192)) {
			TCHAR		szKey[64];

			_stprintf_s(szKey, ini_color_num, nInx);
			save_hex32(szKey, col);
		}
	}
}

/**
 * Sauvegarde des raccourcis dans leur section
 **/
void CConfigFile::save_raccourcis()
{
	NewSection(ini_shortcut_section);

	for (size_t i=0; i<_countof(shortcuts); i++) {
		const ShortcutItem &	shortcut = shortcuts[i];
		TCHAR					cle[32];

		if (shortcut.nom_option == NULL)
			continue;

		// cl de configuration, raccourci normal
		strcpy_T(cle, shortcut.nom_option);
		save_shortcut(cle, shortcut.raccourci, shortcut.defaut);

		// cl de configuration, raccourci alternatif
		strcat_T(cle, TEXT("_alter"));
		save_shortcut(cle, shortcut.alternatif, shortcut.defaut);
	}
}

/**
 * Sauvegarde d'un item dcrit par structure
 *
 * \param[in]  sItem	description de l'item  sauvegarder
 **/
void CConfigFile::save_item(const ConfigItem & sItem)
{
	INT32	nInt;
	LPCTSTR	pStr;
	DWORD	dwVal;

	switch (sItem.eType) {

	case cit_INT:
		nInt = *reinterpret_cast<const INT32 *>(sItem.pVar);
		if (nInt != static_cast<INT32>(sItem.nDef)) {
			save_int(sItem.pszName, nInt);
			return;
		}
		break;

	case cit_HEX:
		nInt = *reinterpret_cast<const INT32 *>(sItem.pVar);
		if (nInt != static_cast<INT32>(sItem.nDef)) {
			save_hex32(sItem.pszName, nInt);
			return;
		}
		break;

	case cit_BOOL:
		nInt = *reinterpret_cast<const bool *>(sItem.pVar);
		if (nInt != (static_cast<INT32>(sItem.nDef)!=0)) {
			save_bool(sItem.pszName, nInt!=0);
			return;
		}
		break;

	case cit_STR:
		pStr = reinterpret_cast<LPCTSTR>(sItem.pVar);
		if (sItem.nDef!=0 && _tcscmp(pStr, reinterpret_cast<LPCTSTR>(sItem.nDef)) == 0)
			break;
		// ... (passage  travers)
	case cit_PATH:
		save_str(sItem.pszName, reinterpret_cast<LPCTSTR>(sItem.pVar));
		return;

	case cit_DRV:
		pStr = reinterpret_cast<LPCTSTR>(sItem.pVar);
		if (sItem.nDef==0 || _tcscmp(pStr, reinterpret_cast<LPCTSTR>(sItem.nDef)) != 0) {
			save_driver_str(sItem.pszName, pStr);
			return;
		}
		break;

	case cit_ASSOC:
		dwVal = *reinterpret_cast<const DWORD *>(sItem.pVar);
		if (dwVal != static_cast<DWORD>(sItem.nDef)) {
			save_assoc(sItem.pszName, reinterpret_cast<const AssocElement *>(sItem.nExtra), dwVal);
			return;
		}
		break;

	case cit_WPOS:
		save_winpos(sItem.pszName, *reinterpret_cast<HWND *>(sItem.nExtra));
		return;
	}

	// Valeur gale  la valeur par dfaut : suppression
	save_str(sItem.pszName, NULL);
}

// ====================================================================================
// Dtection des plantages
// ====================================================================================

/// Enregistrement de l'tat d'excution de l'application
void set_running_state(bool bRunning)
{
	CConfigFile().save_bool(ini_running, bRunning);
}
/// Obtenir l'tat d'excution de l'application
bool get_running_state()
{
	return CConfigFile().load_bool(ini_running, false);
}

// ====================================================================================
// Sauvegarde de la configuration gnrale
// ====================================================================================

/**
 * Sauvegarde les raccourcis
 **/
void save_config_raccourcis()
{
	CConfigFile().save_raccourcis();
}

/**
 * \brief Sauvegarde les donnes spcifiques  un tuner
 * \param[in]	pszNomTuner	Nom du tuner dont les donnes sont  sauvegarder
 **/
void CConfigFile::save_tuner_config(LPCTSTR pszNomTuner)
{
	if (IsNullName(pszNomTuner))
		return;

	NewTunerSection(pszNomTuner);

	save_items(ini_tuner_items);
}

/**
 * Sauvegarde tous les paramtres de configuration
 **/
void save_config()
{
	myprintf(TEXT("Enregistrement de la configuration dans %s") EOL, pouchindir_conf);

	// Actualiser 'cleChaineConfig' au cas o on relancerait immdiatement l'application
	cleChaineConfig = indexToCle(ixChaineCourante);

	// Actualiser la priorit
	configPriority = get_priority();

	// tat de la fentre
	configState = etat_fenetre(hMainWnd);

	CConfigFile cConfig;

	cConfig.save_item(ini_tuner);

	cConfig.save_items(ini_general_items);

	// Couleurs perso, s'il y a lieu
	cConfig.save_colors();

	// Section spcifique au tuner
	cConfig.save_tuner_config(nom_tuner);

	// Section des raccourcis
	cConfig.save_raccourcis();
}

/**
 * \brief Sauvegarde les donnes spcifiques  un tuner
 * \param[in]	pszNomTuner	Nom du tuner dont les donnes sont  sauvegarder
 **/
void save_config_tuner(LPCTSTR pszNomTuner)
{
	CConfigFile().save_tuner_config(pszNomTuner);
}

/**
 * Dfinition des chemins de rpertoires de base de l'application
 * (variables #pouchindir_prog et #pouchindir_conf), qui seront ensuite
 * utiliss pour localiser tous les autres fichiers
 **/
void ini_app_paths()
{
	DWORD nRes = 0;

	// Rcupre les dossiers de Pouchin TV Mod
	nRes = GetModuleFileName(NULL, pouchindir_prog, _countof(pouchindir_prog));
	for (size_t i = nRes - 1; TCHAR('\\')!=pouchindir_prog[i]; i--)
		pouchindir_prog[i] = TCHAR('\0');

	// CSIDL_COMMON_APPDATA : ApplicationData partag par tous
	// CSIDL_APPDATA : ApplicationData de l'utilisateur
	SHGetFolderPath(NULL, CSIDL_APPDATA, NULL, SHGFP_TYPE_CURRENT, pouchindir_conf);
	_tcscat_s(pouchindir_conf, TEXT("\\Pouchin TV Mod\\"));

	nRes = GetCurrentDirectory(_countof(pouchindir_curr), pouchindir_curr);
	if (nRes > 0 && pouchindir_curr[nRes-1] != TCHAR('\\') && nRes < _countof(pouchindir_curr)-1) {
		pouchindir_curr[nRes++] = TCHAR('\\');
		pouchindir_curr[nRes++] = 0;
	}

	// Place le rpertoire courant sur le rpertoire de Pouchin TV Mod
	// SetCurrentDirectory(pouchindir_prog);

	// Cre le rpertoire. S'il existe, ne pose pas de problme
	CreateDirectory(pouchindir_conf, NULL);
}


/**
 * Recherche (optionnellement rcursive) d'un fichier correspondant au nom pass en paramtre,
 * dans les rpertoires de l'application.
 *
 * \param[in]	pszSrchDir	rpertoire initial pour la recherche
 * \param[in]	pszSubDir	sous-rpertoire initial pour la recherche
 * \param[in]	pszFile		fichier, ou motif de recherche
 * \param[in]	dwFlags		Options de recherche (FF_RECURSIVE, FF_DIRECTORY)
 * \return		Nom du premier fichier trouv avec le chemin (ou chane vide si pas trouv)
 **/
static tstring FindAppFileBase(LPCTSTR pszSrchDir, LPCTSTR pszSubDir, LPCTSTR pszFile, DWORD dwFlags)
{
	SZPathName	pathName;

	// Recherche dans le rpertoire de configuration (%AppData%) :
	strcpy_T(pathName, pszSrchDir);
	if (pszSubDir)
		strcat_T(pathName, pszSubDir);
	return FindFile(pathName, pszFile, dwFlags);
}

/**
 * Recherche (optionnellement rcursive) d'un fichier correspondant au nom pass en paramtre,
 * dans les rpertoires de l'application.
 *
 * \param[in]	pszSubDir	sous-rpertoire initial pour la recherche
 * \param[in]	pszFile		fichier, ou motif de recherche
 * \param[out]	strResult	Nom du premier fichier trouv avec le chemin
 * \param[in]	dwFlags		Options de recherche (FF_RECURSIVE, FF_DIRECTORY)
 * \return	Valeur indiquant si le fichier a t trouv ou pas, et dans quel rpertoire
 **/
FapfResult FindAppFile(LPCTSTR pszSubDir, LPCTSTR pszFile, tstring & strResult, DWORD dwFlags)
{
	// Recherche dans le rpertoire de configuration (%AppData%) :
	if (!(strResult = FindAppFileBase(pouchindir_conf, pszSubDir, pszFile, dwFlags)).empty())
		return pd_conf;

	if (_tcscmp(pouchindir_conf, pouchindir_prog)!=0) {
		// Recherche dans le rpertoire de l'application (si ce n'est pas le mme) :
		if (!(strResult = FindAppFileBase(pouchindir_prog, pszSubDir, pszFile, dwFlags)).empty())
			return pd_prog;

		if (
			_tcscmp(pouchindir_conf, pouchindir_curr)!=0 &&
			_tcscmp(pouchindir_prog, pouchindir_curr)!=0
		) {
			if (!(strResult = FindAppFileBase(pouchindir_curr, pszSubDir, pszFile, dwFlags)).empty())
				return pd_curr;
		}
	}
	return pd_notfound;
}

// ====================================================================================
// Membres de la classe d'accs aux fichiers d'initialisation (.ini) - chargement
// ====================================================================================

/**
 * Cherche un nom de pilote dans le fichier de configuration.
 * Gre le cas de la prsence d'un debugger.
 *
 * \param[in]	lpKeyName	nom de l'option  rechercher
 * \param[out]	lpValue		valeur recherche
 * \param[in]	bufSize		nombre de caractres dans la chane de retour
 * \return		Nombre de caractres copis, sans tenir compte du caractre '\\0'
 **/
DWORD CConfigFile::load_driver_str(LPCTSTR lpKeyName, LPTSTR lpValue, DWORD bufSize)
{
	TCHAR	szActualName[64];

#ifdef _WIN64
	// Utilisation d'appellations distinctes en OS 64 bits : les deux versions peuvent
	// y exister concurremment, et les pilotes ou codecs vus par chacune sont diffrents.
	strcpy_T(szActualName, lpKeyName);
	PlatformAdjust(szActualName, _countof(szActualName));

	DWORD	nLen = load_str(szActualName, lpValue, bufSize);

	if (nLen)
		return nLen;

	// En cas d'chec, il se peut que la configuration ait t faite avec une ancienne
	// version 64 bits. Au cas o, on ressaie avec l'ancien nom :
#endif
	strcpy_T(szActualName, lpKeyName);
	if (TestIfDebuggerPresent())
		strcat_T(szActualName, TEXT(" (debug)"));
	return load_str(szActualName, lpValue, bufSize);
}

/**
 * Charge la taille/position de la fentre dans le fichier de configuration
 *
 * \param[in]  lpKeyName  nom de l'option  rechercher
 * \param[out] wpos       structure retournant la taille trouve, ou initialise avec les valeurs par dfaut
 * \param[in]  defL       largeur par dfaut, si pas trouv
 * \param[in]  defH       hauteur par dfaut, si pas trouv
 **/
void CConfigFile::load_winpos(LPCTSTR lpKeyName, WindowPos & wpos, long defL, long defH)
{
	TCHAR position[32];

	if (load_str(lpKeyName, position) < 7
		|| _stscanf_s(position, TEXT("%i,%i,%i,%i"), &wpos.X, &wpos.Y, &wpos.L, &wpos.H)!=4
		|| !wpos.verifie())
	{
		wpos.X = CW_USEDEFAULT;
		wpos.Y = CW_USEDEFAULT;
		wpos.L = defL;
		wpos.H = defH;
	}
}

/**
 * Cherche un paramtre dans le fichier de configuration \c "config.ini"
 *
 * \param[in]	lpKeyName		Cl dans le fichier de configuration recherche
 * \param[out]	lpValue			Valeur recherche, devant dj tre initialis
 * \param[in]	bufSize			Nombre de caractres pouvant tre retourn dans \a lpValue
 * \param[in]	nDefaultFolder	Dossier par dfaut  retourner, si on n'a rien trouv
 *								(valeur CSIDL obligatoire: http://msdn.microsoft.com/en-us/library/bb762494(VS.85).aspx)
 * \return		Nombre de caractres copis, sans tenir compte du caractre '\\0'
 * \warning		Si le dossier dsir demand par \a nDefaultFolder n'existe pas (parce que la version est trop vieille, ou autre),
 *				celui-ci est remplac par le dossier \c "Mes Documents"
 **/
DWORD CConfigFile::load_path(LPCTSTR lpKeyName, LPTSTR lpValue, DWORD bufSize, int nDefaultFolder)
{
	SZPathName default_dir;

	SHGetFolderPath(NULL, nDefaultFolder, NULL, SHGFP_TYPE_CURRENT, default_dir);
	if (default_dir[0] == 0)
		SHGetFolderPath(NULL, CSIDL_PERSONAL, NULL, SHGFP_TYPE_CURRENT, default_dir);
	return load_str(lpKeyName, lpValue, bufSize, default_dir);
}

/**
 * Chargement d'un raccourci
 *
 * \param[in]  lpKeyName  nom de la cl devant tre utilise pour enregistrer le raccourci
 * \param[out] sShtCut    rfrence sur le raccourci  charger
 * \param[in]  reset	  \p true si le raccourci doit tre rinitialis  sa valeur par dfaut
 * \retval                0 en cas d'chec
 * \retval                "toute autre valeur" en cas de succs
 **/
void CConfigFile::load_shortcut(LPCTSTR lpKeyName, Raccourci & sShtCut, bool reset)
{
	TCHAR buffer[128];

	if (!reset) {
		load_str(lpKeyName, buffer);
		_stscanf_s(buffer, TEXT("%i,%i,%i,%i"),
			&sShtCut.maj, &sShtCut.ctrl, &sShtCut.alt, &sShtCut.touche);
		if (sShtCut.touche > 0xFF || sShtCut.touche < 0x01)
			sShtCut.clear();
	} else
		sShtCut.clear();
}

/**
 * Charge les raccourcis
 *
 * \param[in]  reset  si \a true , on charge les raccourcis par dfaut
 **/
void CConfigFile::load_raccourcis(bool reset)
{
	NewSection(ini_shortcut_section);

	// Pour les acclrateurs
	ACCEL accel[_countof(shortcuts)*2];
	ZeroMemory(accel, sizeof(ACCEL)*_countof(shortcuts)*2);

	UINT j=0;				// index pour les acclrateurs

	for (UINT i=0; i<_countof(shortcuts); i++) {
		ShortcutItem	& item = shortcuts[i];	// reprsente la ligne actuelle des valeurs par dfaut
		TCHAR			cle[32];				// Cl  questionner dans le fichier INI

		// Cl pour les options
		if (item.nom_option == NULL)
			cle[0] = 0;
		else
			strcpy_T(cle, item.nom_option);

		// Rcupre le raccourci prcdent enregistr pour cette action
		load_shortcut(cle, item.raccourci, reset);

		// Si pas dfini, mettre la valeur par dfaut
		if (!item.raccourci.isSet())
			item.raccourci = item.defaut;

		// Raccourci alternatif
		_tcscat_s(cle, _countof(cle), TEXT("_alter"));
		load_shortcut(cle, item.alternatif, reset);

		// Remplissage du tableau des acclrateurs

		// Raccourci standard
		accel[j++] = item.raccourci.MakeAccel(item.action);

		// Raccourci alternatif (si il existe)
		if (item.alternatif.isSet())
			accel[j++] = item.alternatif.MakeAccel(item.action);
	}

	DestroyAcceleratorTable(hMainAccel);
	// Produit l'acclrateur  introduire dans la boucle du programme
	hMainAccel = CreateAcceleratorTable(accel, j+1);
}

/**
 * Chargement des couleurs personnalises
 **/
void CConfigFile::load_colors()
{
	for (int nInx=0; nInx<_countof(aCustomColors); ++nInx) {
		COLORREF &	col = aCustomColors[nInx];
		TCHAR		szKey[64];

		_stprintf_s(szKey, ini_color_num, nInx);
		col = (COLORREF)load_int(szKey, RGB(192, 192, 192));
	}
}

/**
 * Chargement d'un item dcrit par structure
 *
 * \param[in]  sItem	description de l'item  charger
 **/
DWORD CConfigFile::load_item(const ConfigItem & sItem)
{
	switch (sItem.eType) {

	case cit_INT:
	case cit_HEX:
		*reinterpret_cast<INT32 *>(sItem.pVar) = load_int(sItem.pszName, (INT)sItem.nDef);
		break;

	case cit_BOOL:
		*reinterpret_cast<bool *>(sItem.pVar) = load_bool(sItem.pszName, sItem.nDef!=0);
		break;

	case cit_STR:
		return load_str(sItem.pszName, reinterpret_cast<LPTSTR>(sItem.pVar),
			(DWORD)sItem.nExtra, reinterpret_cast<LPCTSTR>(sItem.nDef));

	case cit_DRV:
		return load_driver_str(sItem.pszName,
			reinterpret_cast<LPTSTR>(sItem.pVar), (DWORD)sItem.nExtra);

	case cit_PATH:
		return load_path(sItem.pszName, reinterpret_cast<LPTSTR>(sItem.pVar), (DWORD)sItem.nExtra, (int)sItem.nDef);

	case cit_WPOS:
		load_winpos(sItem.pszName, *reinterpret_cast<WindowPos *>(sItem.pVar), LOWORD(sItem.nDef), HIWORD(sItem.nDef));
		break;

	case cit_ASSOC:
		*reinterpret_cast<int *>(sItem.pVar) = load_assoc(sItem.pszName,
			reinterpret_cast<const AssocElement *>(sItem.nExtra), (DWORD)sItem.nDef);
	}
	return 0;
}

// ====================================================================================
// Chargement de la configuration gnrale
// ====================================================================================

/**
 * \brief Charge les donnes spcifiques  un tuner
 * \param[in]	pszNomTuner	Nom du tuner dont les donnes sont  charger
 * \param[in]	bMigrating	Mettre \p true pour rcuprer les donnes d'une ancienne version
 **/
void load_config_tuner(LPCTSTR pszNomTuner, bool bMigrating)
{
	CConfigFile().load_tuner_config(pszNomTuner, bMigrating);
}

/**
 * Obtenir le nom de "receiver" associ au nom de tuner fourni
 * \param[in]		pszNomTuner		Nom du tuner slectionn
 * \param[in,out]	pszReceiver		Pointeur sur un tampon qui recevra le nom du "receiver"
 * \param[in]		bufReceiverSize	Longueur de ce tampon
 **/
void get_matching_receiver(LPCTSTR pszNomTuner, LPTSTR pszReceiver, DWORD bufReceiverSize)
{
	if (IsNullName(pszNomTuner)) {
		strcpy_T(pszReceiver, bufReceiverSize, NULL_NAME);
		return;
	}

	CConfigFile cConfig;

	cConfig.NewTunerSection(pszNomTuner);
	cConfig.load_driver_str(ini_tuner_items[0].pszName, pszReceiver, bufReceiverSize);
}

/**
 * Charge les raccourcis
 *
 * \param[in]  reset  si \a true , on charge les raccourcis par dfaut
 **/
void load_config_raccourcis(bool reset)
{
	CConfigFile().load_raccourcis(reset);
}

/**
 * \brief Charge les donnes spcifiques  un tuner
 * \param[in]	pszNomTuner	Nom du tuner dont les donnes sont  charger
 * \param[in]	bMigrating	Mettre \p true pour rcuprer les donnes d'une ancienne version
 **/
void CConfigFile::load_tuner_config(LPCTSTR pszNomTuner, bool bMigrating)
{
	if (IsNullName(pszNomTuner))
		return;

	NewTunerSection(pszNomTuner, bMigrating);

	load_items(ini_tuner_items);
}

/**

 * Lecture de la configuration depuis le fichier de configuration
 * se trouvant dans \c "%AppData%\Pouchin TV Mod", ou dans le dossier actuel
 **/
void load_config()
{
	CConfigFile cConfig;

	// Recherche si on a un fichier de config dans AppData
	if (cConfig.load_item(ini_tuner) == 0) {
		// Pas de tuner trouv. On recherche dans le dossier courant
		CConfigFile cConfig2(pouchindir_prog);

		// Vrifie si on a une rponse
		if (cConfig2.load_item(ini_tuner) != 0) {
			// la config du tuner a t trouve dans le rpertoire de PTVM
			// On change le rpertoire de config vers celui du programme
			swap(cConfig, cConfig2);
			// ... et on dcide que le rpertoire de configuration sera celui du
			// programme .
			/// \todo Faire en sorte que \c "%AppData%" devienne la seule option
			strcpy_T(pouchindir_conf, pouchindir_prog);
		}
	}

	myprintf(TEXT("Chargement de la configuration depuis %s") EOL, pouchindir_conf);

	// Ajustement de la valeur par dfaut du rendu vido selon la version de Windows
	if (eWinVers < wv_Vista)
		ini_general_items[0].nDef = rdt_VMR9_windowless;

	cConfig.load_items(ini_general_items);

	cConfig.load_colors();

	// Section spcifique au tuner
	cConfig.load_tuner_config(nom_tuner, false);

	// Section des raccourcis
	cConfig.load_raccourcis();

	// Post-ajustements :
	if (methode_recherche == sm_full)						// On ne restaure pas l'tat
		methode_recherche = sm_prefs;						// "tout rinitialiser"

	if (epg_typerecvideo == meth_multiplex)					// On ne restaure pas le mode
		epg_typerecvideo = meth_TS;							// "enregistrement du multiplex"
}

// ====================================================================================
// Fonctions
// ====================================================================================

/**
 * Chargement de la combo box 'hItm'  partir du fichier des noms de listes de canaux.
 *
 * \param[in] hItm  handle du combo devant tre remplie
 **/
void remplit_canaux(HWND hItm)
{
	CIniFile		cChanFile(scan_ini, false);
	vector<tstring>	vstrSections;

	cChanFile.GetSectionNames(vstrSections);

	for (ITERATE_CONST_VECTOR(vstrSections, tstring, it)) {
		// On ignore la catgorie Gnral car elle permet de dfinir des informations
		// dans le fichier des canaux.ini
		if (_tcscmp(TEXT("Gnral"), it->c_str())!=0)
			// Ajoute l'entre  la liste droulante
			SendMessage(hItm, CB_ADDSTRING, 0, (LPARAM)it->c_str());
	}

	// Slectionner le premier item (sera sans doute chang juste aprs)
	SendMessage(hItm, CB_SETCURSEL, 0, NULL);
}

/**
 * Vrification de la validit d'un device path.
 * \param[in]	pszDevPath	device path  vrifier.
 **/
bool check_path(LPCTSTR pszDevPath)
{
	HRESULT hr;
	MonikerBindings cMnkb(pszDevPath, &hr);

	if (!cMnkb.isOk() || FAILED(hr))
		return false;

	// S'assurer qu'on peut rellement crer le filtre (ce qui ne sera pas le cas
	// si le tuner a t retir) :
	hr = cMnkb.BindToFilter();
	if (FAILED(hr))
		return false;
	return true;
}

/**
 * Vrification de la validit de la configuration du tuner TNT.
 *
 * \retval true si le tuner configur est valide
 * \retval false si le tuner n'est pas valide
 **/
bool check_config()
{
	if (!IsNullName(sTunProps.szPathTuner)) {
		// Au moins un chemin tuner a t spcifi.
		// Vrifier qu'il correspond bien  un tuner valide et prsent
		if (!check_path(sTunProps.szPathTuner))
			return false;

		// S'il y a un filtre dmodulateur, il doit tre valide
		if (!IsNullName(sTunProps.szPathDemod)) {
			if (!check_path(sTunProps.szPathDemod))
				return false;
		}

		// S'il y a un filtre de capture, il doit tre valide
		if (!IsNullName(sTunProps.szPathCapture)) {
			if (!check_path(sTunProps.szPathCapture))
				return false;
		}

		return true;	// La configuration est valide
	}

	if (IsNullName(nom_tuner))
		return false;	// La configuration n'est pas valide : on n'a mme pas de nom de tuner

	// La configuration n'est pas valide, mais il existait sans doute une configuration valide
	// en relation une ancienne version du logiciel, ce qu'on va vrifier, et le cas chant,
	// essayer de convertir silencieusement,  partir du seul nom de tuner.

	DeviceInfoList	vTunerList;

	CSearchTuner(vTunerList, autre_graphe).Do();

	// Recherche d'une configuration compatible avec le prcdent nom de tuner sauvegard
	for (ITERATE_CONST_VECTOR(vTunerList, DeviceInfo, it)) {
		if (it->TunerDisplayName()==nom_tuner) {
			// Dans certaines versions, le nom de la section Tuner du fichier config.ini peut
			// avoir chang (chane(s)  (x64) et/ou  (debug) ajoute(s)).
			// On rcupre donc ici le paramtrage de la section sous son ancien nom afin de le
			// sauvegarder dans la section utilisant le nouveau nom.
			load_config_tuner(nom_tuner, true);

			// Sauvegarde des nouveaux paramtres de configuration automatiquement dtermins :
			strcpy_T(sTunProps.nom_receiver, it->ReceiverDisplayName().c_str());
			strcpy_T(sTunProps.szPathTuner, it->asDev[dt_Tuner].strPath.c_str());
			strcpy_T(sTunProps.szPathDemod, it->asDev[dt_Demod].strPath.c_str());
			strcpy_T(sTunProps.szPathCapture, it->asDev[dt_Capture].strPath.c_str());

			// Sauvegarde de la configuration ajuste
			save_config();
			return true;
		}
	}

	// Migration impossible : l'utilisateur devra refaire sa configuration
	return false;
}

/**
 * Reconstituer l'options "--prefix" ventuelle d'instance secondaire de l'application
 **/
tstring GetInstancePrefix()
{
	TCHAR szTmp[256] = TEXT("");

	if (*prefix_conf)
		_stprintf_s(szTmp, TEXT(" --prefix %s"), prefix_conf);
	return szTmp;
}

/// Ajout d'une option "Ne plus afficher ce message" aux dialogues de messages
struct MessageBoxHookData
{
	/// Liste simplifie de types d'items
	enum ClassTypes {
		ct_other,
		ct_button,	//!< Bouton
		ct_static	//!< Item statique avec texte ou icne
	};

	/// Identification simplifie du type d'un item prsent dans la bote de dialogue
	static ClassTypes GetClassType(HWND hCtl)
	{
		TCHAR		szClass[64];

		GetClassName(hCtl, szClass, _countof(szClass));
		if (_tcsicmp(szClass, TEXT("Button"))==0)
			return ct_button;
		if (
			_tcsicmp(szClass, TEXT("Static"))==0 &&
			(GetWindowTextLength(hCtl)>0 || (GetWindowLongPtr(hCtl, GWL_STYLE)&SS_ICON)!=0)
		)
			return ct_static;
		return ct_other;
	}

	/// Rcupration des informations concernant les items prsents dans la bote de dialogue
	struct ParseItemsHelper
	{
		RECT	sAllItemsRect;	//!< Union des rectangles de tous les items (sauf statiques vides)
		RECT	sAllButtsRect;	//!< Union des rectangles de tous les boutons
		HWND	hButton;		//!< Handle du premier bouton rencontr

		/// Fonction "callback" non statique appele pour chaque item
		BOOL CALLBACK ItemProc(HWND hCtl)
		{
			RECT		sRect;

			GetChildWindowRect(hCtl, sRect);
			switch (GetClassType(hCtl)) {
			case ct_button:
				UnionRect(&sAllButtsRect, &sAllButtsRect, &sRect);
				if (hButton==NULL)
					hButton = hCtl;
				// ... (passage  travers)
			case ct_static:
				UnionRect(&sAllItemsRect, &sAllItemsRect, &sRect);
			}
			return TRUE;
		}

		/// Fonction "callback" statique appele pour chaque item
		static BOOL CALLBACK ItemProc(HWND hCtl, LPARAM lParam) {
			return reinterpret_cast<ParseItemsHelper *>(lParam)->ItemProc(hCtl); }

		/**
		 * Constructeur, numrant les items afin d'obtenir :
		 * - L'union des rectangles de tous les items
		 * - L'union des rectangles de tous les boutons
		 * - La "handle" d'un bouton quelconque pour en extraire la configuration du nouvel item
		 * \param[in]	hDlg	"Handle" de la bote de dialogue dont les items sont  numrer
		 * \note	Normalement, les items d'une "MessageBox" sont l'item de texte, l'item d'icne
		 * (optionnel) et un item pour chaque bouton. Sous Windows Vista (mais pas sous Windows 7,
		 * o cette fonctionnalit semble tre assure autrement) il y a un item statique qui dfinit
		 * un bandeau gris sous les boutons. Cet item est ici ignor s'il est rencontr.
		 **/
		ParseItemsHelper(HWND hDlg) :
			sAllItemsRect(RECT()),
			sAllButtsRect(RECT()),
			hButton(NULL)
		{
			EnumChildWindows(hDlg, ItemProc, (LPARAM)this);
		}
	};

	/// Fonction "callback" statique appele pour aligner les boutons  droite
	static BOOL CALLBACK AlignRightProc(HWND hCtl, LPARAM lParam)
	{
		if (GetClassType(hCtl)==ct_button) {
			RECT	sRect;

			GetChildWindowRect(hCtl, sRect);
			OffsetRect(&sRect, (int)lParam, 0);
			SetWindowPos(hCtl, 0, sRect.left, sRect.top, 0, 0,
				SWP_NOSIZE|SWP_NOZORDER|SWP_NOREPOSITION);
		}
		return TRUE;
	}

	static WNDPROC	procMsgBox;	//!< Sauvegarde de la procdure d'origine de la bote de dialogue
	static HHOOK	hHook;		//!< "Handle" du "hook" cr
	static LPCTSTR	pszMbcKey;	//!< Cl d'identification de la bote de dialogue
	static LPCTSTR	pszMbTitle;	//!< Titre de la bote de message (pour vrifier, par scurit)

	static void UnsetHook()
	{
		if (hHook) {
			UnhookWindowsHookEx(hHook);
			hHook = NULL;
		}
	}

	/// Fonction "callback" de surclassement de la fonction de gestion de la bote de dialogue
	static LRESULT CALLBACK MbWindowProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
	{
		LRESULT rc = CallWindowProc(procMsgBox, hDlg, uMsg, wParam, lParam );

		switch (uMsg) {

		case WM_INITDIALOG: {
			LPCTSTR				pszCheckBoxText = TEXT("   ");
			ParseItemsHelper	sChEnDat(hDlg);
			SIZE				sTxtSiz = {0, 0};	// Rectangle minimal du texte
			SIZE				sCtlSiz = {0, RHgt(sChEnDat.sAllButtsRect)};	// Rectangle de la "check box"
			HFONT				hTxtFont = NULL;	// Police de caractres extraite d'un bouton

			// Distance entre le bouton le plus  gauche et l'item le plus  gauche (diminu de 4 pixels) :
			LONG	nSpaceOnLeft	= sChEnDat.sAllButtsRect.left - sChEnDat.sAllItemsRect.left - 6;

			// Calculer le rectangle exact dont a besoin la case  cocher pour s'afficher correctement,
			// avec son texte
			if (sChEnDat.hButton) {
				hTxtFont = (HFONT)SendMessage(sChEnDat.hButton, WM_GETFONT, 0, NULL);
				// Obtenir la taille requise pour le texte :
				GetTextExtentPoint32(AutoSelect<HFONT>(AutoDC(sChEnDat.hButton), hTxtFont),
					pszCheckBoxText, strlen_T(pszCheckBoxText), &sTxtSiz);
				sCtlSiz.cx = sTxtSiz.cx+16+2;	// Ajouter la largeur ncessaire pour la case elle-mme
			}

			if (nSpaceOnLeft < sCtlSiz.cx) {
				// Espace insuffisant  gauche des boutons : aligner tous les boutons  droite.

				// Distance entre le bouton le plus  droite et l'item le plus  droite :
				LONG	nSpaceOnRight	= sChEnDat.sAllItemsRect.right - sChEnDat.sAllButtsRect.right;

				// Note : sous Windows Vista et Windows 7, les boutons sont dj aligns  droite. Ceci ne
				// devrait donc avoir d'effet que sous Windows XP.
				if (nSpaceOnRight>=0) {
					// Aligner tous les boutons  droite de la bote de dialogue :
					EnumChildWindows(hDlg, AlignRightProc, (LPARAM)nSpaceOnRight);
					OffsetRect(&sChEnDat.sAllButtsRect, nSpaceOnRight, 0);
					nSpaceOnLeft += nSpaceOnRight;
				}
			}

			if (nSpaceOnLeft < sCtlSiz.cx) {
				// Espace toujours insuffisant : tenter d'afficher sur deux lignes
				sCtlSiz.cx = nSpaceOnLeft;
				sCtlSiz.cy = sTxtSiz.cy*2+2;
			}

			HWND hWndCheck =
				CreateWindowEx(0, TEXT("Button"), pszCheckBoxText,
					WS_CHILD|WS_VISIBLE|BS_AUTOCHECKBOX|BS_CHECKBOX|BS_MULTILINE ,
					sChEnDat.sAllItemsRect.left, sChEnDat.sAllButtsRect.bottom-sCtlSiz.cy,
					sCtlSiz.cx, sCtlSiz.cy,
					hDlg, (HMENU)IDC_DONTSHOWAGAIN, hAppInstance, NULL);

			if (hWndCheck && hTxtFont)
				SendMessage(hWndCheck, WM_SETFONT, (WPARAM)hTxtFont, FALSE);
			break; }

		case WM_DESTROY:
			// Fermeture de la bote de dialogue : on teste l'tat de la case  cocher,
			// et si elle est coche, on l'indique dans le fichier de configuration.
			if (pszMbcKey && IsDlgButtonChecked(hDlg, IDC_DONTSHOWAGAIN)==BST_CHECKED)
				CConfigFile().NewSection(ini_dontshow_section).save_bool(pszMbcKey, true);
		}
		return rc;
	}

	/// Fonction "hook" dtectant l'initialisation de la bote de dialogue "MessageBox", afin
	/// d'en surclasser la procdure de gestion
	static LRESULT CALLBACK CallWndProc(int nCode, WPARAM wParam, LPARAM lParam)
	{
		LRESULT lResult = CallNextHookEx(hHook, nCode, wParam, lParam);

		if (nCode==HC_ACTION) {	//Checks if an action is beeing active
			const	CWPSTRUCT & pwp = *reinterpret_cast<const CWPSTRUCT*>(lParam);

			if (pwp.message==WM_INITDIALOG) {
				TCHAR	szTitle[128];

				GetWindowText(pwp.hwnd, szTitle, _countof(szTitle));
				if (_tcscmp(pszMbTitle, szTitle)==0) {	// Vrifier le titre, par prcaution
					procMsgBox=(WNDPROC)SetWindowLongPtr(pwp.hwnd, GWLP_WNDPROC, (LONG_PTR)MbWindowProc);

					// Annulation du "hook", qui n'est plus ncessaire
					UnsetHook();
				}
			}
		}

		return lResult;
	}

	/// Constructeur
	MessageBoxHookData(LPCTSTR pszMbcKy, LPCTSTR pszTitle)
	{
		if (hHook == NULL) {
			hHook = SetWindowsHookEx(WH_CALLWNDPROC, CallWndProc, NULL, GetCurrentThreadId());
			pszMbcKey = pszMbcKy;
			pszMbTitle = pszTitle;
		}
	}

	/// Destructeur
	~MessageBoxHookData()
	{
		UnsetHook();	// Probablement redondant, mais prcaution indispensable
		pszMbcKey = NULL;
		pszMbTitle = NULL;
	}
};

WNDPROC	MessageBoxHookData::procMsgBox	= NULL;
HHOOK	MessageBoxHookData::hHook		= NULL;
LPCTSTR	MessageBoxHookData::pszMbcKey	= NULL;
LPCTSTR	MessageBoxHookData::pszMbTitle	= NULL;

/**
 * Version de \p MessageBox avec une case ajoute "Ne plus afficher ce message".
 * Si l'utilisateur coche cette case (dont l'tat sera sauvegard dans le fichier de configuration)
 * avant de fermer la bote de dialogue, alors tous les appels subsquents seront ignors,
 * et la valeur par dfaut sera retourne.
 * \param[in]		hWnd		"Handle" de la fentre parente
 * \param[in]		pszText		Message  afficher
 * \param[in]		pszTitle	Titre de la fentre
 * \param[in]		uType		Flags spcifiant le contenu et le comportement de la bote de message
 * \param[in]		iDefault	Valeur par dfaut retourne si l'affichage est ignor
 * \param[in]		pszMbcKey	Identifiant unique pour ce message
 * \param[in,out]	pSessUsed	Pointeur optionnel sur une variable boolenne qui doit avoir t
 *								initialise  \p false, et qui sera mise  \p true aprs le premier appel,
 *								vitant l'affichage multiple au cours d'un mme contexte d'application.
 * \note	La fonction cherche  ajouter la case "Ne plus afficher ce message"  gauche des
 * boutons de la bote de dialogue, dcalant ceux-ci  droite si ncessaire et possible, et tentant
 * l'affichage sur deux lignes si l'espace est ensuite toujours insuffisant. Nanmoins, il n'est pas
 * garanti que cette case puisse s'afficher si la bote de message a trs peu de texte et beaucoup
 * de boutons. Il est de la responsabilit du programmeur d'viter cette situation.
 **/
int MessageBoxCheck(HWND hWnd, LPCTSTR pszText, LPCTSTR pszTitle,
					UINT uType, int iDefault, LPCTSTR pszMbcKey, bool * pSessUsed)
{
	if (pSessUsed) {
		if (*pSessUsed)
			return iDefault;
		*pSessUsed = true;
	}

	if (CConfigFile().NewSection(ini_dontshow_section).load_bool(pszMbcKey))
		return iDefault;	// Si ce message est dsactiv

	MessageBoxHookData	sHookData(pszMbcKey, pszTitle);	// La simple prsence de cet objet installe
														// et gre le "hook" pour la bote de message.

	return MessageBox(hWnd, pszText, pszTitle, uType);
}
