/*
 * $Id: ini.h 276 2011-03-06 10:17:01Z gingko $
 * Copyright (C) 2006 Pouchin <pouchinteve@free.fr>
 *
 * This file is part of Pouchin TV Mod, a free DVB-T viewer.
 * See http://www.pouchintv.fr/ for updates.
 *
 * Pouchin TV Mod is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Pouchin TV Mod is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 *
 *
 * Most files of this project have been changed from the Pouchin TV
 * project (Copyright (C) 2006 Pouchin <pouchinteve@free.fr>),
 * to use with a modified version of this software.
 * See http://pouchinteve.free.fr/ for the original project.
 */

/** \file
 *	\brief Dfinition pour l'accs aux fichiers de configuration
 *
 *	Ce module dfinit les fonctions permettant de sauvegarder et de rcuprer
 *	la configuration, les chanes reues, les enregistrements programms, etc.
 **/

#pragma once

#include "channels.h"
#include "epg.h"

/// Nom du fichier des chaines reues
#define FILENAME_CHANNELS	"chaines.xml"

/// Nom du fichier de sauvegarde des programmations
#define FILENAME_PROGRAMS	"programmes.xml"

/// Rpertoire de Pouchin TV Mod
extern SZPathName	pouchindir_prog;

/// Rpertoire pour la sauvegarde des paramtres (Dossier App Data)
extern SZPathName	pouchindir_conf;

/// Rpertoire initial au lancement du programme
extern SZPathName	pouchindir_curr;

/// Prfixe optionnel aux noms de fichiers de configuration
extern TCHAR		prefix_conf[32];

/// Nom du fichier de configuration
extern TCHAR		config_file[];

/**
 * Listes des types de rendu disponibles
 **/
enum RdrTypes {
	rdt_null,					//!< Aucun rendu dfini
	rdt_VMR7_windowless,		//!< VMR version 7 windowless
	rdt_VMR7_renderless,		//!< VMR version 7 renderless
	rdt_VMR9_windowless,		//!< VMR version 9 windowless (ancien)
#if NEW_VMR9_OUTPUT
	rdt_VMR9_windowless_new,	//!< VMR version 9 windowless (nouveau)
#endif
	rdt_EVR,					//!< Enhanced Video Renderer (ancien)
	rdt_EVR_new,				//!< Enhanced Video Renderer (nouveau)
	rdt_VMR9_renderless,		//!< \todo VMR version 9 renderless (en attente d'implmentation)
	rdt_DirectSound				//!< Rendu audio
};

/// Macro : chane de configuration gnrique pour "Non dfini"
#define NULL_NAME	TEXT("[null]")

/// Macro : chane de configuration gnrique pour "Liste" (de frquences)
#define LIST_NAME	TEXT("[list]")

/**
 * Listes des modes panoramiques
 **/
enum PanModes
{
	pm_Normal,				//!< Normal : on affiche aussi grand qu'on peut
	pm_Stretch,				//!< tirer la vido
	pm_AllWidth,			//!< Utiliser toute la largeur
	pm_AllHeight			//!< Utiliser toute la hauteur
};

/**
 * Listes des modes de ratio d'aspect
 **/
enum ArModes
{
	ar_Auto,				//!< Automatique
	ar_1x1,					//!< 1:1
	ar_5x4,					//!< 5:4
	ar_4x3,					//!< 4:3
	ar_16x10,				//!< 16:10
	ar_16x9					//!< 16:9
};

/// Dfinit le type d'affichage de la fentre de PTVM
enum EtatsFenetre {
	etf_normal,				///< normal avec bordures
	etf_noborders,			///< normal sans bordures
	etf_maximized,			///< maximis (avec bordures)
	etf_fullscreen,			///< plein cran (sans bordures)
	etf_minimized			///< minimise (en icne ou pas)
};

/// Spcification de la mthode d'enregistrement
enum MethodeEnregistrement {
	meth_TS,				//!< 0: L'enregistrement s'effectuera en mode TS
	meth_PS,				//!< 1: L'enregistrement s'effectuera en mode PS
	meth_multiplex			//!< 2: Le multiplex entier sera enregistr
};

/// Mthode de recherche des nouvelles chanes
enum SearchMethod {
	sm_undef,				//!< Indfini
	sm_incremental,			//!< Recherche incrmentale (ajout de chanes)
	sm_prefs,				//!< Recherche en conservant les prfrences
	sm_full					//!< Recherche totalement rinitialise
};

/// Liste d'offsets  utiliser
/// \todo Permettre des offsets multiples pour chaque canal explor (par exemple en
/// spcifiant des items tels \p uo_m167to333 pour "-167, 0, +167, +333" ...)
enum OffsetsToUse {
	uo_m167,				//!< -167
	uo_0,					//!< 0
	uo_167,					//!< +167
	uo_333,					//!< +333
	uo_500,					//!< +500
};

/// Usages possibles de la molette de la souris
enum MouseWheelUsage {
	mwu_None,				//!< Aucun effet
	mwu_Channels,			//!< Changer de chane
	mwu_Channels_MX,		//!< Changer de chane (mme multiplex)
	mvu_Soundtrack,			//!< Changer de piste audio
	mwu_Volume,				//!< Rgler le volume
	mvu_Zoom				//!< Rgler le zoom
};

/// Rglage vido indfini
#define VIDEO_UNDEF 0x80000000

/// lments de rglage vido
enum ProcAmpType {
	pat_Brightness,
	pat_Contrast,
	pat_Hue,
	pat_Saturation,
	pat_MAX
};

/// Largeur de bande utilise (MHz)
#define DVBT_BANDWIDTH	8

/// Base de calcul des canaux (frquence du canal - thorique) zro
#define DVBT_CHANBASE	306000

// ====================================================================================
// Variables de configuration
// ====================================================================================

/** @defgroup configVars Variables de configuration
 *  Ces variables contiennent les donnes qui sont sauvegardes entre les sessions \p PouchinTVMod.
 *  elles sont charges au dmarrage de l'application, et sont sauvegardes chaque fois qu'une
 *  modification intervient dans l'une de celles-ci.
 *  @{
 **/

typedef	TCHAR SZFilterName[256];			//!< Chane de longueur adapte pour filtre ou codec
typedef	TCHAR SZDvPathName[256];			//!< Chane de longueur adapte pour "device path"

extern SZFilterName	filtreMPEG2;			//!< Nom du codec Audio MPEG2 choisi
extern SZFilterName	filtreH264;				//!< Nom du codec H264, utilis pour la HD
extern SZFilterName	filtreAudio;			//!< Nom du codec Audio MPEG2 choisi
extern SZFilterName	filtreAc3;				//!< Nom du codec AC3 choisi
extern SZFilterName	filtreEac3;				//!< Nom du codec E-AC3 choisi
extern TCHAR		choixCanaux[256];		//!< Jeu (standard) de canaux  explorer

extern TCHAR		liste_canaux[256];		//!< Liste de canaux  spcifier manuellement

extern bool			exact_match;			//!< Indique si une correspondance exacte est souhaite
											//!< dans la recherche de filtres

extern SZPathName	screenshots_dir;		//!< Rpertoire pour les captures d'cran
extern SZPathName	video_dir;				//!< Rpertoire pour les enregistrements vido

extern UINT32		cleChaineConfig;		//!< Cl chane courante
extern INT32		avance_enreg;			//!< Nombre de minutes d'anticipation du dbut enregistrements EPG
extern INT32		retard_enreg;			//!< Nombre de minutes de retard sur la fin des enregistrements EPG
extern SearchMethod	methode_recherche;		//!< Mthode de recherche des chanes
extern OffsetsToUse	use_offsets;			//!< Offsets  utiliser

extern INT32		autonum_undef;			//!< Numro de dpart pour les chanes non numrotes
extern INT32		autonum_duplicate;		//!< Dcalage de numrotation pour les chanes dupliques

extern bool			suspend_minimized;		//!< Indique que le son doit tre coup en mode minimis
extern bool			minimize_system_tray;	//!< Indique que la minimisation doit se faire dans la zone de notification
extern bool			start_with_windows;		//!< Dmarrer en mode minimis avec Windows
extern bool			record_protect;			//!< Rduire et se mettre en attente au lieu de fermer
extern bool			allow_wakeup;			//!< Permettre la sortie de mise en veille
extern INT32		wakeup_delay;			//!< Nombre de minutes entre sortie de mise en veille et enregistrement

/**
 * \brief Dlai de grce pour l'arrt aprs enregistrement (en minutes)
 *
 * Si,  l'arrt d'un enregistrement, un autre enregistrement est programm dans un
 * dlai plus court que le dlai spcifi, alors les options d'arrt du programme,
 * d'extinction de l'ordinateur ou de mise en veille sont ignores.
 * \todo Rendre ceci configurable par l'utilisateur.
 **/
extern INT32		duree_grace_quit;

extern INT32		time_before_shutdown;	//!< Temps d'attente (avec message) avant arrt ordinateur (en secondes)

#if USE_VMR_DEINTERLACE
extern bool			use_vmr_deinterlace;	//!< \test Dsentrelacement ; n'a sans doute jamais fonctionn
#endif

extern bool			allow_stream_record;	//!< Indique que l'enregistrement du multiplex entier est autoris
extern bool			use_msn;				//!< Indique que l'affichage dans MSN est actif
extern TCHAR		titre_fenetre[128];		//!< Nom affich dans la barre de titre de la fentre
extern bool			on_top;					//!< Indique que l'affichage doit se faire devant toutes les autres fentres
extern AudioMode	audio_pref;				//!< Indique la prfrence audio pour l'affichage
extern bool			use_osd;				//!< \p true si l'OSD doit tre utilis
extern INT32		osd_transparency;		//!< Transparence OSD \todo implmenter configuration quand il y aura une place
extern COLORREF		osd_color;				//!< Couleur de l'OSD
extern COLORREF		aCustomColors[16];		//!< Sauvegarde des couleurs personnalises
extern RdrTypes		vmr_mode;				//!< Indique le mode VMR  utiliser

extern PanModes		video_pan_mode;			//!< Mode panoramique de la vido
											//!< \todo Est-il vraiment opportun que ce mode soit sauvegard ?
											//!< (si oui, attention  la rinitialisation implicite quand on
											//!< remet le zoom par dfaut, ce qui nuit  la cohrence)
extern ArModes		aspect_ratio;			//!< Ratio d'aspect
extern INT32		zoom_ratio;				//!< Facteur de zoom  appliquer  la vido
											//!< \todo Est-il vraiment opportun que le zoom soit sauvegard ?

extern INT32		video_ctl[pat_MAX];		//!< Vido : luminosit, constraste, teinte, saturation
extern INT32		video_sav[pat_MAX];		//!< Sauvegarde rglages temporaires pour restauration aprs redmarrage du graphe

extern INT32		volumeCourant;			//!< Niveau de volume sonore courant

extern bool			use_multimedia;			//!< Utiliser les commandes multimdia
extern MouseWheelUsage
					role_molette;			//!< Rle de la molette de la souris
extern bool			inversion_molette;		//!< Inversion du sens de la molette de la souris
extern INT32		duree_zapping;			//!< Temps de latence avant changement de chane par dfilement

/**
 * Motif pour le nom des fichiers pour les enregistrements
 <pre>
 *    %c = chane
 *    %n = numro de chane
 *    %e = nom de l'enregistrement
 *    %a = anne
 *    %m = mois
 *    %j = jour
 *    %h = heure
 *    %M = minute
 *    %s = secondes</pre>
 **/
extern SZPathName	motif_noms_enreg;

extern MethodeEnregistrement
					epg_typerecvideo;		//!< Type d'enregistrement prcdement utilis dans la
											//!< fentre EPG pour la vido
extern AudioMode	epg_typerecaudio;		//!< Type d'enregistrement prcdement utilis dans la
											//!< fentre EPG pour l'audio
extern EpgTimeLimit	epg_timelimit;			//!< Limite d'exploration de l'EPG

extern SZFilterName	nom_tuner;				//!< Nom du tuner  utiliser
extern bool			autre_graphe;			//!< Utilisation du graphe des versions prcdentes

#ifdef _DIAGNOSTIC
extern INT32		mpx_frequence_diag;		//!< Frquence du multiplex enregistr (diagnostic slt.)
#endif

/// Valeurs sauvegardes sparment pour chaque type de tuner
struct TunerProps
{
	SZFilterName	nom_receiver;			//!< Nom du rcepteur  utiliser
	SZDvPathName	szPathTuner;			//!< "Device Path" du tuner
	SZDvPathName	szPathDemod;			//!< "Device Path" du filtre dmodulateur
	SZDvPathName	szPathCapture;			//!< "Device Path" du filtre de capture
	int				offset_tuner;			//!< Offset  appliquer  toutes les frquences recherches
	bool			try_adding_5khz;		//!< Essayer d'ajouter 5 kHz si aucun signal trouv
	int				delai_tuner1;			//!< Dlai de verrouillage  appliquer au tuner pendant la recherche
	int				delai_tuner2;			//!< Dlai de stabilisation  appliquer au tuner pendant la recherche
	bool			scan_ignore_presence;	//!< Ignorer indicateur de prsence du signal prendant la recherche
	bool			scan_ignore_lock;		//!< Ignorer indicateur de verrou du signal prendant la recherche
	bool			scan_ignore_quality;	//!< Ignorer indicateur de qualit du signal prendant la recherche
	int				force_signal_zero;		//!< Valeur intensit de signal nulle releve pendant la recherche
	int				force_signal_max;		//!< Valeur intensit de signal maximale releve pendant la recherche
	bool			force_signal_aff_pct;	//!< Affichage de la force du signal en pourcentage
	bool			auto_resyntonisation;	//!< Resyntonisation automatique en cas de problme
};

extern TunerProps sTunProps;

/** @} */ // fin de configVars

/** @defgroup configVarsDefault Valeurs de configuration par dfaut
 *  Macros dfinissant les valeurs par dfaut de certains paramtres de configuration
 *  @{
 **/

#define	INI_DELAI_TUNER1_DEFAULT	1000		//!< Dlai de verrouillage  appliquer au tuner
#define	INI_DELAI_TUNER2_DEFAULT	600			//!< Dlai de stabilisation  appliquer au tuner
#define	INI_AUTONUM_UNDEF_DEFAULT	60			//!< Numro de dpart pour les chanes non numrotes
#define	INI_AUTONUM_DUP_DEFAULT		100			//!< Dcalage de numrotation pour les chanes dupliques
#define INI_USE_OFFSETS_DEFAULT		uo_167		//!< Offsets  utiliser
#define INI_METHODE_RECH_DEFAULT	sm_prefs	//!< Mthode de recherche des chanes
#define INI_TRY_ADDING_5KHZ_DEFAULT	true		//!< Essayer d'ajouter 5 kHz si aucun signal trouv

/// Couleur utilise pour l'OSD
#define INI_OSD_COLOR				RGB(0, 255, 0)	// vert

/// Transparence de l'OSD (en mode VMR9 seulement), de 0  100
#define INI_OSD_TRANSPARENCY		100				// 100 %

/// Temps de latence par dfaut avant changement de chane par dfilement
#define INI_DUREE_ZAPPING			2000			// 2000 mS

/// Motif par dfaut pour nommer les enregistrements
#define MOTIF_NOMS_ENREG_DEFAULT	"%e - ?%c %a-%m-%j %h-%M-%s"

/** @} */ // fin de configVarsDefault

// ====================================================================================
// Fin des variables de configuration
// ====================================================================================

/**
 * Dfinition des chemins de rpertoires de base de l'application
 * (variables #pouchindir_prog et #pouchindir_conf), qui seront ensuite
 * utiliss pour localiser tous les autres fichiers
 **/
void ini_app_paths();

/// Rsultat de recherche dans les rpertoires
enum FapfResult
{
	pd_notfound,	//!< Pas trouv
	pd_prog,		//!< Trouv dans le rpertoire de configuration
	pd_conf,		//!< Trouv dans le rpertoire du programme
	pd_curr			//!< Trouv dans le rpertoire initial (courant)
};

/**
 * Recherche (optionnellement rcursive) d'un fichier correspondant au nom pass en paramtre,
 * dans les rpertoires de l'application.
 *
 * \param[in]	pszSubDir	sous-rpertoire initial pour la recherche
 * \param[in]	pszFile		fichier, ou motif de recherche
 * \param[out]	strResult	Nom du premier fichier trouv avec le chemin
 * \param[in]	dwFlags		Options de recherche (FF_RECURSIVE, FF_DIRECTORY)
 * \return	Valeur indiquant si le fichier a t trouv ou pas, et dans quel rpertoire
 **/
FapfResult FindAppFile(LPCTSTR pszSubDir, LPCTSTR pszFile, tstring & strResult, DWORD dwFlags=FF_RECURSIVE);

/// Nom de fichier situ avec les donnes de l'application
template <typename T=TCHAR, typename U=TCHAR>
	struct NomFichierConf
{
	T str[MAX_PATH];

	NomFichierConf(const U * pszBaseName, bool bUsePrefix=true, LPCTSTR pszDir=pouchindir_conf)
	{
		strcpy_T(str, pszDir);
		if (bUsePrefix)
			strcat_T(str, prefix_conf);
		strcat_T(str, pszBaseName);
	}
	operator const T * () const {
		return str;}
};

/**
 * Dtermine si une dfinition de configuration est dfinie comme "Aucun" ("[Null]") ou
 * "non dfini" (chane vide).
 *
 * \param[in]  pszName  Nom du codec
 * \retval	\p true si aucun
 * \retval	\p false si dfini
 **/
bool IsNullName(LPCTSTR pszName);

/**
 * Vrifie l'existence du fichier "canaux.ini", et le cre  partir des ressources
 * s'il n'existe pas.
 **/
void cree_fichier_scan();

/// Liste de canaux destins  faire l'objet d'une recherche en vue d'y trouver
/// un multiplex contenant des chanes :
struct ListeCanaux
{
	BYTE	nCount;			//!< Nombre de canaux dans la liste
	BYTE	aNoCan[59];		//!< Liste des canaux (normalement, valeurs de 21  69)
	INT32	nOffset;		//!< Valeur de base  appliquer pour le calcul de frquence
	bool	bScanLoop;		//!< Option de recherche en boucle

	/// Rsultat de l'analyse de la liste
	enum ParseListRes {
		plr_noerror,
		plr_syntaxerror,
		plr_outofrange,
		plr_toomanyvals
	};

	ListeCanaux() :
		nCount(0),
		nOffset(DVBT_CHANBASE),	// Canal (thorique) zro
		bScanLoop(false)
	{
	}

	/// Rinitialisation
	void clear() {
		nCount = 0; nOffset = DVBT_CHANBASE; }

	/**
	 * Lecture du fichier "canaux.ini", qui dcrit les listes disponibles de canaux  explorer
	 *
	 * \param[in]	pszCanaux	nom de la section contenant les canaux  explorer
	 * \note	Ce fichier contenait en plus des listes de canaux par metteurs (noms de villes)
	 *			dans les versions prcdentes, mais suite  la multiplication des metteurs et 
	 *			la frquente modification de ceux-ci, ceci a t supprim.
	 **/
	void	lit_fichier_scan(LPCTSTR pszCanaux);

	/// Convertit directement un numro de canal en frquence, en tenant compte de l'offset
	/// ventuellement ajout
	UINT	canal2khz(BYTE nNoCanal) const;

	/// Ajout d'un canal  la liste
	bool	add(BYTE nNoCanal) {
				if (nCount<_countof(aNoCan)) {
					aNoCan[nCount++] = nNoCanal; return true;}
				return false;}

	static ParseListRes
			get_num(LPCTSTR & pszStr, BYTE & nChan);

	/// Ajout d'une liste saisie
	ParseListRes
			add_list(LPCTSTR pszList);

	/// Ajout de tous les canaux possibles  la liste
	void	add_fullscan();
};

/**
 * Chargement du fichier des chanes "chaines.xml".
 *
 * \retval S_OK		en cas de succs
 * \retval S_FAILED (ou autre) si erreur
 **/
HRESULT lit_chaines();

/**
 * Sauvegarde de la liste des chanes dans "chaines.xml".
 **/
void sauve_chaines();

/**
 * Lecture des programmations d'enregistrement depuis un fichier.
 **/
HRESULT lit_programmes();

/**
 * Sauvegarde des programmations d'enregistrements dans un fichier.
 **/
void sauve_programmes();

/**
 * Sauvegarde tous les paramtres de configuration
 **/
void save_config();

/**
 * Lecture de la configuration depuis le fichier de configuration
 * se trouvant dans \c "%AppData%\Pouchin TV Mod", ou dans le dossier actuel
 **/
void load_config();

/**
 * Chargement de la combo box 'hItm'  partir du fichier des noms de listes de canaux.
 *
 * \param[in] hItm  handle du combo devant tre remplie
 **/
void remplit_canaux(HWND hItm);

/// Enregistrement de l'tat d'excution de l'application
void set_running_state(bool bRunning);

/// Obtenir l'tat d'excution de l'application
bool get_running_state();

/**
 * Vrification de la validit de la configuration du tuner TNT.
 *
 * \retval true si le tuner configur est valide
 * \retval false si le tuner n'est pas valide
 **/
bool check_config();

/**
 * \brief Sauvegarde les donnes spcifiques  un tuner
 * \param[in]	pszNomTuner	Nom du tuner dont les donnes sont  sauvegarder
 **/
void save_config_tuner(LPCTSTR pszNomTuner);

/**
 * Sauvegarde les raccourcis
 **/
void save_config_raccourcis();

/**
 * \brief Charge les donnes spcifiques  un tuner
 * \param[in]	pszNomTuner	Nom du tuner dont les donnes sont  charger
 * \param[in]	bMigrating	Mettre \p true pour rcuprer les donnes d'une ancienne version
 **/
void load_config_tuner(LPCTSTR pszNomTuner, bool bMigrating = false);

/**
 * Obtenir le nom de "receiver" associ au nom de tuner fourni
 * \param[in]		pszNomTuner		Nom du tuner slectionn
 * \param[in,out]	pszReceiver		Pointeur sur un tampon qui recevra le nom du "receiver"
 * \param[in]		bufReceiverSize	Longueur de ce tampon
 **/
void get_matching_receiver(LPCTSTR pszNomTuner, LPTSTR lpValue, DWORD bufSize);

/**
 * Charge les raccourcis depuis les paramtres
 *
 * \param[in]  reset  si \a true , on charge les raccourcis par dfaut
 */
void load_config_raccourcis(bool reset=false);

/**
 * Reconstituer l'options "--prefix" ventuelle d'instance secondaire de l'application
 **/
tstring GetInstancePrefix();

/// Position de la fentre (enregistr) de PTVM
extern WindowPos	winPos;

#if USE_CONSOLE==1
extern WindowPos	consolePos;
#endif

/// Priorit du processus courant selon la configuration
extern DWORD configPriority;

/// tat de la fentre principale selon la configuration
extern EtatsFenetre configState;

/// Nom de base du fichier dfinissant les listes de canaux  explorer
extern TCHAR scan_ini[];

/**
 * Structure avec constructeur, chargeant une chane de caractres et la protgeant
 * en substituant les caractres non acceptables dans des noms de fichiers par le
 * caractre fourni dans le paramtre "remplacement".
 * Cette structure est rige en "template" pour permettre de choisir le type de
 * chane de caractres  grer, car sinon, son usage combin avec des chanes qui
 * ne peuvent pas ncessairement tre du mme type devient inextricable.
 **/
template <class T, int S=64>
	struct NomProtege
{
	/// Nom de fichier protg
	T nom[S];

	/// Recherche de la substitution, version CHAR
	static inline LPSTR find_invalid(LPSTR str) {
		return strpbrk(str, "\"*/:<>?\\|");}
	/// Recherche de la substitution, version WCHAR
	static inline LPWSTR find_invalid(LPWSTR str) {
		return wcspbrk(str, L"\"*/:<>?\\|");}

	/**
	 * Substitution, dans \p nom, des caractres invalides
	 * pour des noms de fichiers par un caractre valide
	 * \param[in]  remplacement    caractre de remplacement en cas de caractre invalide
	 **/
	void Protege(T remplacement) {
		T *			pcar;

		while ((pcar = find_invalid(nom)) != NULL)
			*pcar = remplacement;
	}


	/**
	 * Constructeur permettant de remplacer les caractres invalides
	 * pour des noms de fichiers par un caractre valide
	 *
	 * \param[in]  src             nom de fichier  convertir
	 * \param[in]  remplacement    caractre de remplacement en cas de caractre invalide
	 **/
	template <class TS>
		NomProtege(const TS * src, T remplacement)
	{

		if (src) {
			strcpy_T(nom, src);
			Protege(remplacement);
		} else
			nom[0] = 0;
	}

	/**
	 * Extraction d'une copie
	 *
	 * \param[out] pDst		chane rsultante
	 * \param[in]  nSizDst	nombre de caractres pouvant contenir \p pDst
	 **/
	template <class TD>
		void Extract(TD * pDst, int nSizDst) const
	{
		strcpy_T(pDst, nSizDst, nom);
	}
};

/**
 * \brief Gnration d'un nom de fichier sur la base du motif configur
 *
 * Structure avec constructeur, gnrant un nom de fichier complet incluant la date et l'heure
 * courantes,  partir d'une chane de caractre prfixe et d'une extension
 **/
struct NomFichierAvecMotif
{
	/// Nom complet du fichier gnr
	SZPathName	nom;

	/**
	 * Constructeur permettant de gnrer le nom du fichier d'enregistrement
	 *
	 * \param[in]	dir			dossier pour l'enregistrement du fichier
	 * \param[in]	pszPrgName	nom de l'enregistrement (seulement en cas de programmation d'enregistrement)
	 * \param[in]	pszChanName	nom de la chane enregistre
	 * \param[in]	nChanNo		numro de la chane enregistre
	 * \param[in]	pszExt		extension du fichier (permet de dfinir le type du fichier (BMP, MPG, TS, ...)
	 **/
	NomFichierAvecMotif(LPCTSTR dir, LPCTSTR pszPrgName, LPCSTR pszChanName, int nChanNo, LPCTSTR pszExt);

	/**
	 * Ouvre le fichier (d'enregistrement) en criture
	 *
	 * \return  le handle d'ouverture du fichier
	 **/
	HANDLE Create() const;
};

/**
 * Table d'associations pour nommer les tats possible des chanes.
 **/
extern const AssocElement aChStateTable[];

/**
 * Tableau associatif dcrivant les diffrents types de flux vido reconnus
 **/
extern const AssocElement aVideoTypeTable[];

/**
 * Variable  mettre  'true' toutes les fois que quelque chose a t modifi
 * dans les enregistrements programms, ncessitant une sauvegarde.
 **/
extern bool programmation_modifiee;

/**
 * Dcrit un raccourci
 **/
struct Raccourci
{
	bool	maj;	///< Indique si la touche majuscule doit tre enfonce aussi
	bool	ctrl;	///< Indique si la touche Ctrl doit tre enfonce
	bool	alt;	///< Touche Alt enfonce ?
	DWORD	touche;	///< Code de la touche  enfoncer

	bool		operator == (const Raccourci &r) const;
	bool		operator != (const Raccourci &r) const;
	Raccourci &	operator =(const Raccourci &r);

	ACCEL		MakeAccel(WORD action) const;

	/**
	 * Gnre une chane de caractres correspondant au raccourci
	 *
	 * \param[out] buffer    chane de caractres gnre
	 * \param[in]  taille    nombre de caractres dans le buffer
	 **/
	void		ToString(LPTSTR buffer, UINT taille) const;

	/// Effacement du raccourci
	void		clear() {
					touche = 0; maj = ctrl = alt = false; }

	/// Vrifier s'il y a une affectation sur ce raccourci
	bool		isSet() const {
					return touche != 0; }

	/// Version template de \p ToString.
	template <size_t S>
		void ToString(TCHAR(&buffer)[S]) const {
			ToString(buffer, S);}
};

/**
 * Structure dcrivant un raccourci
 **/
struct ShortcutItem
{
	WORD		action;			///< Identifiant de l'action
	LPTSTR		nom;			///< Nom de l'action pour afficher dans la fentre des options
	LPTSTR		nom_option;		///< Nom de l'option permettant de l'associer dans le fichier de configuration
								///< Elle sera moins modifie que la valeur numrique #action
	Raccourci	defaut;			///< Raccourci par dfaut
	Raccourci	raccourci;		///< Raccourci principal
	Raccourci	alternatif;		///< Raccourci alternatif

	ShortcutItem &	operator =(const ShortcutItem &s);

	/// Effacement du raccourci alternatif (si \p bAlt = \p true) ou normal
	void			clear(bool bAlt) {
						if (!bAlt)
							raccourci = alternatif;
						alternatif.clear(); }
};

/**
 * Liste des diffrents raccourcis
 **/
extern ShortcutItem shortcuts[];

/**
 * Obtenir le nombre de raccourcis dfini.
 **/
int get_shortcuts_count();


/**
 * Version de \p MessageBox avec une case ajoute "Ne plus afficher ce message".
 * Si l'utilisateur coche cette case (dont l'tat sera sauvegard dans le fichier de configuration)
 * avant de fermer la bote de dialogue, alors tous les appels subsquents seront ignors,
 * et la valeur par dfaut sera retourne.
 * \param[in]		hWnd		"Handle" de la fentre parente
 * \param[in]		pszText		Message  afficher
 * \param[in]		pszTitle	Titre de la fentre
 * \param[in]		uType		Flags spcifiant le contenu et le comportement de la bote de message
 * \param[in]		iDefault	Valeur par dfaut retourne si l'affichage est ignor
 * \param[in]		pszMbcKey	Identifiant unique pour ce message
 * \param[in,out]	pSessUsed	Pointeur optionnel sur une variable boolenne qui doit avoir t
 *								initialise  \p false, et qui sera mise  \p true aprs le premier appel,
 *								vitant l'affichage multiple au cours d'un mme contexte d'application.
 **/
int MessageBoxCheck(HWND hWnd, LPCTSTR pszText, LPCTSTR pszTitle,
					UINT uType, int iDefault, LPCTSTR pszMbcKey, bool * pSessUsed = NULL);

