/*
 * $Id: main.cpp 277 2011-03-13 10:24:41Z gingko $
 * Copyright (C) 2006 Pouchin <pouchinteve@free.fr>
 *
 * This file is part of Pouchin TV Mod, a free DVB-T viewer.
 * See http://www.pouchintv.fr/ for updates.
 *
 * Pouchin TV Mod is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Pouchin TV Mod is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 *
 *
 * Most files of this project have been changed from the Pouchin TV
 * project (Copyright (C) 2006 Pouchin <pouchinteve@free.fr>),
 * to use with a modified version of this software.
 * See http://pouchinteve.free.fr/ for the original project.
 */

/** \file
 *	\brief Interface utilisateur principale de l'application
 *
 *	Ce fichier est le fichier principal de l'application. Il dfinit le
 *	point d'entre, implmente la cration de la fentre principale, et
 *	prend en charge les commandes donnes par l'utilisateur.
 **/

#include "stdafx.h"
#include "resource.h"
#include "dlgutils.h"
#include "settings.h"
#include "search.h"
#include "trayicon.h"
#include "msn_notify.h"
#include "epg.h"

#include "record.h"
#include "recprog.h"

#include "Hyperlinks.h"
#include "HtmlHelp.h"
#include "internet.h"
#include "update.h"
#include "version.h"

#include <PowrProf.h>

#if USE_LOGITECH_LCD
#include "LCD.h"
#endif // #if USE_LOGITECH_LCD

// Voir http://blogs.msdn.com/b/oldnewthing/archive/2007/05/31/2995284.aspx
#if defined(_M_IX86)
	#define MANIFEST_PROCESSORARCHITECTURE "x86"
#elif defined(_M_X64) || defined(_M_AMD64)
	#define MANIFEST_PROCESSORARCHITECTURE "amd64"
#elif defined(_M_IA64)
	#define MANIFEST_PROCESSORARCHITECTURE "ia64"	// Peu vraisemblable, mais bon
#else
	// #error Unknown processor architecture.
	#define MANIFEST_PROCESSORARCHITECTURE "*"
#endif

#pragma comment(linker, \
	"\"/manifestdependency:type='win32' "\
	"name='Microsoft.Windows.Common-Controls' "\
	"version='6.0.0.0' "\
	"processorArchitecture='" MANIFEST_PROCESSORARCHITECTURE "' "\
	"publicKeyToken='6595b64144ccf1df' "\
	"language='*'\"")

/**
 * Nom de l'application
 **/
TCHAR	szAppName[]		= APPLICATION_NAME;

/**
 * Nom du fichier d'aide de l'application
 **/
TCHAR	szChmFileName[]	= TEXT("MMzinTV.chm");

/**
 * Texte de version de l'application
 **/
TCHAR	szAppVersion[]	=
	TEXT(PRODUCT_VERSION_STR) TEXT(" (")
	#ifdef _WIN64
		TEXT("x64")
	#else
		TEXT("W32")
	#endif
	#ifdef _UNICODE
		TEXT(" Unicode")
	#else
		TEXT(" SBCS")
	#endif
	#ifdef _DEBUG
		TEXT(" Debug")
	#elif _DIAGNOSTIC
		TEXT(" Diag")
	#endif
		TEXT(")");

/// Champs de la barre d'tat
enum StatusBarFields
{
	sbf_ChanName,		//!< Nom de la chane
	sbf_ActualProgTime,	//!< Heure de l'mission en cours
	sbf_ActualProgName,	//!< Nom de l'mission en cours
	sbf_NextProgTime,	//!< Heure de l'mission suivante
	sbf_NextProgName,	//!< Nom de l'mission suivante
	sbf_MutedState		//!< tat "muet" (seulement si pas d'OSD)
};

/// Nom du binaire de PTVM
SZPathName		szAppFileName;

HINSTANCE		hAppInstance;		//!< Handle d'instance de l'application
HMENU			hMainMenu	= NULL;	//!< Handle de la barre de menus de l'application
HWND 			hMainWnd	= NULL;	//!< Handle de la fentre principale de l'application
HWND 			hMainStatus	= NULL;	//!< Handle de la barre d'tat de l'application
HICON			hCurIcon	= NULL;	//!< Icne courante (normale, ou bien rouge pour enregistrement)
HICON			hPrgIcon	= NULL;	//!< Icne normale du programme
HICON			hRecIcon	= NULL;	//!< Icne utilise en mode enregistrement

WORD			numeroChaineSaisi = 0;

static bool		muted = false;		//!< Mode "muet"
static CMSNMsg	cMsnData;			//!< Donnes concernant les notifications MSN

/// Nombre maximal de chanes
#define			NB_MAX_CHAINES		999

/// Dlai de persistance du curseur (pointeur souris) en mode plein cran :
#define			DUREE_CURSEUR		3000		// Valeur en millisecondes

/// Temps avant qu'une suite de numros se traduise par un changement de chane
#define			DUREE_SAISIE		1000

/// Plafond de temps avant qu'un zapping par rotation (clavier ou molette de la souris)
/// se traduise par un changement de chane, si pas d'OSD ni de barre d'tat.
#define			DUREE_MAX_ZAPNOAFF	500

/// Temps avant application d'un changement de zoom
#define			DUREE_ZOOM			500

/// Dlai d'attente avant affichage des informations OSD de la chane, une fois qu'on
/// a chang de chane
#define			DUREE_SHOW_PROGRAMME 2500

/// Temps d'attente aprs sortie de l'ordinateur du mode veille
#define			DUREE_WAKEUP		7000

/// Temps de stabilisation avant mise  jour des coordonnes OSD
#define			DUREE_UPDATE_OSD	50

/**
 * \brief	Dure de protection des enregistrement
 * Si un enregistrement programm est prvu dans ce dlai, l'application peut
 * (optionnellement) viter de quitter. Exprim en millisecondes.
 **/
#define			RECORD_PROTECT_TIME	(24 * 60 * 60 * 1000)	// 24h

// Positions des sous-menus du menu principal :
#define			MENU_FICHIER_POSITION		0
#define			MENU_CHAINES_POSITION		1
#define			MENU_VIDEO_POSITION			2
#define			MENU_AUDIO_POSITION			3
#define				SMENU_PISTES_POSITION	4	// base pistes dans menu "Audio"
#define			MENU_INFORMATION_POSITION	4
#define			MENU_ENREGISTRER_POSITION	5
#define			MENU_OPTIONS_POSITION		6
#define				SMENU_PRIORITE_POSITION	3	// dans menu "Options"

WinVersion		eWinVers = wv_Unknown;			//!< Version de Windows (dcodage basique)

/// Diverses informations sur la version de Windows
OSVERSIONINFOEX	windows_version;

HACCEL			hMainAccel			= NULL;

ExitMode		eExitMode			= em_Quit;	//!< Indique si (et comment) l'application doit tre relance

static bool		bCrashDetected		= false;	//!< Dtection de plantages

#if USE_LOGITECH_LCD
/// LCD pour ceux qui en possdent un
CLcd 			lcd;
#endif // #if USE_LOGITECH_LCD

/**
 * Identifiant du message indiquant une recration de la barre des tches
 * (envoy par exemple aprs un plantage de l'explorateur Windows)
 **/
static UINT		nWM_TASKBARCREATED	= 0;

/**
 * Identifiant du message utilis pour changer entre des requtes entre instances de la
 * mme application. Les requtes elles-mmes sont places dans \p wParam, tandis que
 * \p lParam contiendra la handle de la fentre principale de l'instance qui envoie.
 **/
UINT			nWM_APP_INSTANCES	= 0;

/// Mis  \p true si une autre instance a refus l'arrt ou la mise en veille
bool			bPowerStateChangeDenied = false;

CShutdownBlockReason
				cShutBlockReason;

/// Structure regroupant les tats dcods des options de ligne de commande
struct CmdOptions
{
	bool	bMinimize;
	bool	bFs;
	bool	bMax;
	bool	bNoborder;
	bool	bNocrashwarn;
	bool	bNormal;
	bool	bRecPS;
	bool	bRecTS;
	bool	bHelp;
	bool	bMuted;
	INT		nChaine;
	tstring	strPrefix;

	CmdOptions() :
		bMinimize(false),
		bFs(false),
		bMax(false),
		bNoborder(false),
		bNocrashwarn(false),
		bNormal(false),
		bRecPS(false),
		bRecTS(false),
		nChaine(0),
		bHelp(false),
		bMuted(false)
	{
	}
};

/// Pointeur sur la structure contenant les options de ligne de commande
static auto_ptr<CmdOptions> cmdOptions;

// Flag mis  TRUE si on est en mode plein cran,
// ET que le curseur de la souris est temporairement visible.
static enum EtatCurseur {
	etc_normal,				//!< Mode normal, allum
	etc_fullscreen_on,		//!< Mode plein cran, allum
	etc_fullscreen_off		//!< Mode plein cran, teint
} etat_curseur;

/**
 * Fonction retournant le nom de la plate-forme sous laquelle tourne Windows.
 **/
static LPCTSTR GetRunningPlatform()
{
  #ifdef _WIN64
	// Si l'application elle-mme est en 64 bits, il n'y a rien  tester
	return TEXT("x64");
  #else
	// Sinon, il faut appeler la fonction 'IsWow64Process', pour en tester le
	// rsultat. Comme cette fonction n'existe pas sous toutes les versions de
	// Windows, il faut en plus d'abord tester si elle existe.

	typedef BOOL (WINAPI *LPFN_ISWOW64PROCESS) (HANDLE, PBOOL);

	static LPFN_ISWOW64PROCESS  fnIsWow64Process =
									(LPFN_ISWOW64PROCESS)GetProcAddress(
										GetModuleHandle(TEXT("kernel32")), "IsWow64Process");

	if (fnIsWow64Process != NULL) {
		BOOL	bIsWow64 = FALSE;

		if (fnIsWow64Process(GetCurrentProcess(), &bIsWow64)==0)
			return TEXT("[platform error]");
		if (bIsWow64)
			return TEXT("x64");

	}
	return TEXT("x86");
  #endif
}

/**
 * Gnration d'une chane de caractres reprsentant la version actuelle de Windows
 * \param[in]	sWv	Structure d'informations pralablement charge
 **/
tstring GetWindowsVersionStr(const OSVERSIONINFOEX & sWv)
{
	return
		tstr_printf(
			TEXT("%u.%u (%s) SP %u.%u build %u (%") A2t TEXT(")"),
			sWv.dwMajorVersion,
			sWv.dwMinorVersion,
			GetRunningPlatform(),
			sWv.wServicePackMajor,
			sWv.wServicePackMinor,
			sWv.dwBuildNumber,
			EnumToString(sWv.wProductType,
				"\1\0Workstation\0Server\0Domain controller\0", "Unknown"));
}

/**
 * Change l'tat du curseur (l'affiche, le masque, etc.)
 *
 * nouvel_etat		Futur tat du curseur
 **/
static void change_etat_curseur(EtatCurseur nouvel_etat)
{
	if (nouvel_etat != etat_curseur) {
		if (etat_curseur!=etc_fullscreen_off && nouvel_etat==etc_fullscreen_off)
			ShowCursor(FALSE);
		else if (etat_curseur==etc_fullscreen_off && nouvel_etat!=etc_fullscreen_off)
			ShowCursor(TRUE);
		etat_curseur = nouvel_etat;
	}
}

/**
 * Met  jour le, ou les menu(s) passs en paramtres (via les fonctions idoines)
 *
 * \param[in]  hWnd   handle de la fentre dont le menu doir tre mis  jour (gnralement hMainWnd)
 * \param[in]  count  nombre de fonctions  appeler pour mettre  jour
 * \param[in]  ...    liste des fonctions  appeler pour mettre  jour les menus
 **/
static void update_menus(HWND hWnd, int count, ...)
{
	typedef void (* update_menu_proc) (HMENU hMenu);

	HMENU hMenu = GetMenu(hWnd);

	if (hMenu != NULL) {
		va_list	argptr;

		va_start(argptr, count);
		for (int i=0; i<count; i++)
			va_arg(argptr, update_menu_proc)(hMenu);
		va_end(argptr);
		DrawMenuBar(hWnd);
	}
}

/**
 * Obtenir la priorit d'excution courante de l'application
 * \return	prio	Priorit actuelle. Voir fonction systme \p GetPriorityClass.
 **/
DWORD get_priority()
{
	return GetPriorityClass(GetCurrentProcess());
}

/**
 * Modification de la priorit d'excution de l'application
 * \param[in]	prio	Priorit  spcifier. Voir fonction systme \p SetPriorityClass.
 * \todo La priorit "temps rel" semble ne pas tre possible sous Windows Vista.
 * Si on essaie, la fonction \p SetPriorityClass (de Windows) ne retourne cependant
 * aucune erreur, mais ne change rien  la priorit existante.
 * Voir si a peut se contourner, ou bien faire en sorte que cette slection ne
 * soit pas possible en cas d'excution sous Windows Vista.
 **/
static void set_priority(DWORD prio)
{
	SetPriorityClass(GetCurrentProcess(), prio);
}

static BOOL set_menu_check_state(HMENU hMenu, UINT uItem, bool checked)
{
	return SetMenuItemState(hMenu, uItem,
		MFS_CHECKED, checked ? MFS_CHECKED : MFS_UNCHECKED);
}

static BOOL set_menu_enable_state(HMENU hMenu, UINT uItem, bool enabled)
{
	return SetMenuItemState(hMenu, uItem,
		MFS_DISABLED, enabled ? MFS_ENABLED : MFS_DISABLED);
}

static void update_priority_menu(HMENU hMenu)
{
	DWORD prio = get_priority();

	set_menu_check_state(hMenu, IDM_BELOW_NORMAL_PRIORITY,
		prio == BELOW_NORMAL_PRIORITY_CLASS);
	set_menu_check_state(hMenu, IDM_NORMAL_PRIORITY,
		prio == NORMAL_PRIORITY_CLASS);
	set_menu_check_state(hMenu, IDM_ABOVE_NORMAL_PRIORITY,
		prio == ABOVE_NORMAL_PRIORITY_CLASS);
	set_menu_check_state(hMenu, IDM_HIGH_PRIORITY,
		prio == HIGH_PRIORITY_CLASS);
	set_menu_check_state(hMenu, IDM_REALTIME_PRIORITY,
		prio == REALTIME_PRIORITY_CLASS);

	// L'tat IDLE_PRIORITY_CLASS est prserv (modif possible par le gestionnaire
	// des tches ou par une application externe), mais pas slectionnable par menu
	//
	// Cependant, si on lance Pouchin TV Mod, et que celui-ci dmarre en priorit Basse,
	// on rajoute une entre nomme "Basse"  la fin du menu, qui est retire ds que
	// l'on change la priorit
	MENUITEMINFO info = {sizeof(info)};
	BOOL mi_exists = GetMenuItemInfo(hMenu, IDM_IDLE_PRIORITY, FALSE, &info);

	if (prio == IDLE_PRIORITY_CLASS) {
		// On vrifie que l'entre n'existe pas
		if (!mi_exists) {
			// Slectionne le menu "Priorits"
			HMENU hMenuPriorite = GetSubMenu(GetSubMenu(hMenu, MENU_OPTIONS_POSITION), SMENU_PRIORITE_POSITION);

			// Ajoute l'entre dans le menu
			AppendMenu(hMenuPriorite, MF_CHECKED, IDM_IDLE_PRIORITY, TEXT(""));
		}
	} else {
		// On cherche si l'entre "Priorit basse" existe
		if (mi_exists) {
			// On la vire donc
			DeleteMenu(hMenu, IDM_IDLE_PRIORITY, MF_BYCOMMAND);
		}
	}

	configPriority = prio;	// Sauvegarde pour test WM_MENUSELECT
}

static void change_priority(DWORD prio)
{
	set_priority(prio);
	update_menus(hMainWnd, 1, update_priority_menu);
	save_config();
}

// Obtenir l'tat courant de la fentre ( partir de la valeur dj obtenue par GetWindowLongPtr) :
inline EtatsFenetre etat_fenetre(LONG_PTR ws)
{
	return
		ws & WS_MINIMIZE ? etf_minimized :
		ws & WS_MAXIMIZE ?
			(ws & WS_CAPTION ? etf_maximized : etf_fullscreen) :
			(ws & WS_CAPTION ? etf_normal : etf_noborders);
}

// Obtenir l'tat courant de la fentre ( partir de la handle de la celle-ci) :
EtatsFenetre etat_fenetre(HWND hWnd)
{
	return etat_fenetre(GetWindowLongPtr(hWnd, GWL_STYLE));
}

static void update_record_icon()
{
	HICON hNewIcon = enregistrements_actuels.recording()!=0 ? hRecIcon : hPrgIcon;

	if (hNewIcon != hCurIcon) {
		hCurIcon = hNewIcon;

		// Change l'icone indiquant si oui ou non on enregistre en ce moment
		niData.update_icon();

		// On change l'icne de la fentre de Pouchin TV Mod
		// (usage de SetClassLongPtr au lieu de SetClassLong pour compatibilit 64 bits)
		SetClassLongPtr(hMainWnd, GCLP_HICON, (LONG_PTR)hCurIcon);
	}
}

static void formate_menu_chaine_nom(int ixRecDescr, LPCTSTR prefixe, LPTSTR pstr, size_t bufSize)
{
	LPCSTR nom = enregistrements_actuels[ixRecDescr].channelName();

	if (*nom=='*')
		_stprintf_s(pstr,  bufSize, TEXT("%s du multiplex"), prefixe);
	else
		_stprintf_s(pstr, bufSize, TEXT("%s de %") A2t, prefixe, nom);
}

static void update_menu_recording(HMENU hMenu)
{
#ifdef _DIAGNOSTIC
	static INT	nRecOtherPos		= -1;
	HMENU		hRecMenu			= GetMenuParentAndPos(hMenu, IDM_RECORD_OTHER, &nRecOtherPos);
#else
	HMENU		hRecMenu			= GetMenuParentAndPos(hMenu, IDM_RECORD_OTHER);
#endif
	int			ixRecDescr			= enregistrements_actuels.getRecordingDescr(ixChaineCourante);
	bool		curChannelRecording = ixRecDescr != -1;
	bool		bFreeRecAvailable	= enregistrements_actuels.getFreeDescriptor() != -1;
	bool		bCanRecordPs		= ixChaine_ok(ixChaineCourante) && Canaux[ixChaineCourante].isPsRecordAvailable();

	// Items d'enregistrement de la chane courante : ces items sont actifs
	// si la chane courante n'est pas dj en cours d'enregistrement, et si
	// le nombre maximal de chanes enregistres n'est pas atteint.
	bool		active				= bFreeRecAvailable && !curChannelRecording;

	set_menu_enable_state(hRecMenu, IDM_RECORD_CHANNEL_PS, active && bCanRecordPs);
	set_menu_enable_state(hRecMenu, IDM_RECORD_CHANNEL_TS, active);
	set_menu_enable_state(hRecMenu, IDM_RECORD_OTHER, bFreeRecAvailable);

#ifdef _DIAGNOSTIC
	// Enregistrement d'un multiplex inconnu (commande de diagnostic) :
	// cette fonctionnalit doit tre active dans la configuration.
	// L'item est cr ici ou bien dtruit selon cette configuration.
	// S'il existe, il est actif aucun enregistrement n'est actuellement en cours,
	// et si le nombre maximal de chanes enregistres n'est pas atteint.
	if (allow_stream_record) {
		if (GetMenuState(hRecMenu, IDM_RECORD_UNDEFSTREAM, MF_BYCOMMAND)==-1) {
			InsertMenu(hRecMenu, nRecOtherPos+1, MF_BYPOSITION, IDM_RECORD_UNDEFSTREAM,
				TEXT("   "));
		}
		set_menu_enable_state(hRecMenu, IDM_RECORD_UNDEFSTREAM,
			bFreeRecAvailable && !enregistrements_actuels.recording());
	} else
		DeleteMenu(hRecMenu, IDM_RECORD_UNDEFSTREAM, MF_BYCOMMAND);
#endif

	// Arrt de l'enregistrement : cet item est actif si l'enregistement de la chane
	// courante ( l'exclusion de tout autre) est actif.
	set_menu_enable_state(hRecMenu, IDM_STOP_RECORD, curChannelRecording);

	// Pour chaque **autre** enregistrement existant, multiplex inclus, un item d'arrt
	// spcifique est ajout et activ. Cet item est dtruit si l'enregistrement
	// n'est plus actif.
	active = false;	// -> pour se souvenir s'il y a des enregistrements en cours
	for (int i=0; i < enregistrements_actuels.count(); i++) {
		const Enregistrement & enreg = enregistrements_actuels[i];

		// Dtruire l'ancien item pour ce poste :
		DeleteMenu(hRecMenu, IDM_STOP_RECORD_BASE+i, MF_BYCOMMAND);

		if (enreg.recording()) {
			active = true;	// Au moins en enregistrement en cours
			// Ce poste correspond  une chane en cours d'enregistrement.
			if (i == ixRecDescr)
				continue;	// ... mais il s'agit de la chane courante

			// Enregistrement sur une autre chane que la chane en cours :
			TCHAR	buf[128];

			formate_menu_chaine_nom(i, TEXT(" "), buf, _countof(buf));
			InsertMenu(hRecMenu, IDM_DELAYED_STOP, MF_BYCOMMAND, IDM_STOP_RECORD_BASE+i, buf);
		}
	}

	// Arrt d'enregistrement retard : cet item est actif si au moins un enregistrement
	// est en cours, quel qu'il soit :
	set_menu_enable_state(hRecMenu, IDM_DELAYED_STOP, active);

	// Rafrachissement de la liste EPG si ouverte (pour changement de couleurs)
	CEpgDialog::PostDlgMessage(WM_APP_EPG_EVENT);
}

static void update_pistes_menu(HMENU hMenu)
{
	HMENU			hTrack = GetSubMenu(hMenu, MENU_AUDIO_POSITION);

	// Dtruire les items de pistes audio antrieurs, s'ils existent
	while (DeleteMenu(hTrack, SMENU_PISTES_POSITION, MF_BYPOSITION));

	if (!ixChaine_ok(ixChaineCourante))
		return;

	const Chaine &	canal_courant = Canaux[ixChaineCourante];
	TCHAR			szMenuName[64];
	UINT			nFlags;
	int				i;

	// Insrer les nouveaux items de pistes audio
	for (i=0; i < canal_courant.nbr; i++) {
		nFlags = i==ixSonCourant ? MF_CHECKED : MF_UNCHECKED;

		canal_courant.toTrackMenuString(i, szMenuName, _countof(szMenuName), i+1);
		AppendMenu(hTrack, nFlags, IDM_PISTES_BASE+i, szMenuName);
	}
}

/**
 * Met  jour la liste des chanes
 **/
static void update_chaines_menu(HMENU hMenu)
{
	static INT		nPos		= -1;	// Position du menu (indfinie au premier passage)
	HMENU			hSubMenu	= CreatePopupMenuFromItem(hMenu, nPos, IDM_CHAINES);
	bool			bRecording	= enregistrements_actuels.recording()!=0;
	const Chaine *	pCurChan	= ixChaine_ok(ixChaineCourante) ? & Canaux[ixChaineCourante] : NULL;

	for (ITERATE_CONST_VECTOR(Canaux, Chaine, it)) {
		const Chaine &	canal = *it;
		TCHAR			szMenuName[64];

		if (canal.etat == ec_inactive)
			continue;

		UINT		uFlags		= (pCurChan == &canal) ? MFS_CHECKED : MFS_UNCHECKED;
		int			nMenuID		= IDM_CHAINES_BASE + Canaux.iterToIndex(it);

		canal.toMenuString(szMenuName, _countof(szMenuName));
		if (bRecording && (!pCurChan || !pCurChan->sameMultiplexAndVideoType(canal)))
			uFlags = MFS_GRAYED;

		AppendMenuItemExt(hSubMenu, nMenuID, szMenuName, uFlags, canal.GetIconAsBmp());
	}
}

static void update_aspect_menus(HMENU hMenu)
{
	bool	bHaveVmr = vmr_mode != rdt_null && pVMR;
	int		nInx;

	set_menu_enable_state(hMenu, IDM_ZOOM,		bHaveVmr);
	set_menu_enable_state(hMenu, IDM_DEZOOM,	bHaveVmr);
	set_menu_enable_state(hMenu, IDM_ALLWDT,	bHaveVmr);
	set_menu_enable_state(hMenu, IDM_ALLHGT,	bHaveVmr);
	set_menu_enable_state(hMenu, IDM_ETIRER,	bHaveVmr);
	set_menu_enable_state(hMenu, IDM_DEFAULT_ZOOM, bHaveVmr);

	for (nInx = ar_Auto; nInx <= ar_16x9; ++ nInx)
		set_menu_enable_state(hMenu, IDM_AR_AUTO+nInx,	bHaveVmr);

	if (bHaveVmr) {
		bool bStretch = video_pan_mode==pm_Stretch;

		set_menu_check_state(hMenu, IDM_ALLWDT, video_pan_mode==pm_AllWidth);
		set_menu_check_state(hMenu, IDM_ALLHGT, video_pan_mode==pm_AllHeight);
		set_menu_check_state(hMenu, IDM_ETIRER, bStretch);
		set_menu_enable_state(hMenu, IDM_DEFAULT_ZOOM, zoom_ratio!=100);
		set_menu_enable_state(hMenu, IDM_VIDEOCONTROL, pVMR->isProcAmpAvailable()!=paa_MissingFromMode);

		for (nInx = ar_Auto; nInx <= ar_16x9; ++ nInx)
			set_menu_check_state(hMenu, IDM_AR_AUTO+nInx,	!bStretch && aspect_ratio == nInx);
	}
}

static void update_ontop_menu(HMENU hMenu)
{
	set_menu_check_state(hMenu, IDM_ALWAYS_ON_TOP, on_top);
}

static void update_fullscreen_menu(HMENU hMenu)
{
	EtatsFenetre eEtat = etat_fenetre(hMainWnd);

	set_menu_check_state(hMenu, IDM_FULLSCREEN, eEtat==etf_fullscreen);
	set_menu_check_state(hMenu, IDM_BORDERS, eEtat==etf_noborders);
	set_menu_enable_state(hMenu, IDM_BORDERS, eEtat!=etf_fullscreen);
}

static void update_mute_menu(HMENU hMenu)
{
	set_menu_check_state(hMenu, IDM_MUTE, muted);
}

static void update_record_menus(HWND hWnd)
{
	// Mise  jour au passage de la liste des enregistrements
	// programms dans le dialogue correspondant s'il est ouvert :
	record_list_updated();

	update_menus(hMainWnd, 2,
		update_menu_recording,
		update_chaines_menu);

	update_record_icon();

	// Notifier le dialogue de qualit du signal
	PostMessage(hMainWnd, WM_APP_RECORD_EVENT, 0, NULL);

	// Mise  jour du blocage de l'extinction
	cShutBlockReason.Update();
}

/**
 * Retourne l'index du raccourci correspondant  l'action passe en paramtre
 *
 * \param[in]	id  action recherche
 * \retval		NULL  si on n'a rien trouv
 * \return		Un pointeur sur la description des raccourcis pour cet item
 **/
static const ShortcutItem * trouve_shortcut_item(UINT id)
{
	int	count = get_shortcuts_count();

	for (int i=0; i<count; i++) {
		const ShortcutItem * pShortcut = &shortcuts[i];

		if (id == pShortcut->action)
			return pShortcut;
	}
	return NULL;
}

static void update_shortcut_menu_item(HMENU hMenu, UINT uIDItem, BOOL bByPosition)
{
	// Il existe bien un raccourci
	// On change donc le texte du menu pour y mettre  jour le raccourci

	TCHAR			szWrk[64];
	MENUITEMINFO	sMii = {
		sizeof(sMii),			// cbSize
		MIIM_STRING|MIIM_ID,	// fMask
		0,						// fType
		0,						// fState
		0,						// wID
		NULL,					// hSubMenu
		NULL,					// hbmpChecked
		NULL,					// hbmpUnchecked
		NULL,					// dwItemData
		szWrk,					// dwTypeData
		_countof(szWrk)			// cch
		// Le reste  zro (implicite)
	};

	// Rcupre le texte actuel
	GetMenuItemInfo(hMenu, uIDItem, bByPosition, &sMii);

	// Recherche si il existe un raccourci correspondant  ajuster
	const ShortcutItem *
					pShortcut = trouve_shortcut_item(sMii.wID);

	if (pShortcut == NULL)
		return;

	// Rechercher le dlimiteur '\t' ou le terminateur
	LPTSTR	pszEnd = _tcschr(szWrk, '\t');

	if (!pszEnd)
		pszEnd = &szWrk[_tcslen(szWrk)];

	if (pShortcut->raccourci.isSet()) {
		if (pszEnd < & szWrk[_countof(szWrk)]-1)
			*pszEnd++ = TCHAR('\t');
		*pszEnd = 0;
		pShortcut->raccourci.ToString(pszEnd, UINT(_countof(szWrk) - (pszEnd - szWrk)));
	} else
		*pszEnd = 0;	// Couper la chane

	// Met  jour le menu
	SetMenuItemInfo(hMenu, uIDItem, bByPosition, &sMii);
}

/**
 * Met  jour de manire rcursive les menus et sous menus, en y rajoutant les raccourcis dfinis par l'utilisateur,
 * pour chaque action dfinie dans le tableau des raccourcis.\n
 * Cette fonction est rcursive, car elle se rapelle en cas de sous-menu
 *
 * \param[in]  hMenu  handle du menu devant tre analys
 **/
static void update_shortcut_menu_recur(HMENU hMenu)
{
	int nbMenus = GetMenuItemCount(hMenu);

	for (int i=0; i<nbMenus; i++) {
		HMENU hSubMenu = GetSubMenu(hMenu, i);
		if (hSubMenu) {
			// Appel rcursif pour taiter tous les sous menus
			update_shortcut_menu_recur(hSubMenu);
			continue;
		}

		// Recherche si il existe un raccourci correspondant  ajuster
		update_shortcut_menu_item(hMenu, i, TRUE);
	}
}

/**
 * Met  jour l'tat d'activation des menus de filtres et d'affichage
 * selon la prsence ou l'absence de certains filtres.
 **/
static void update_filtres_menus(HMENU hMenu)
{
	// Ajustement des menus
	set_menu_enable_state(hMenu, IDM_MP2V,		check_property_dialog(ft_Video_Mp2v));
	set_menu_enable_state(hMenu, IDM_H264,		check_property_dialog(ft_Video_H264));
	set_menu_enable_state(hMenu, IDM_MP2A,		check_property_dialog(ft_Audio_Mp2a));
	set_menu_enable_state(hMenu, IDM_AC3,		check_property_dialog(ft_Audio_Ac3));
	set_menu_enable_state(hMenu, IDM_EAC3,		check_property_dialog(ft_Audio_Eac3));

	// Note : toutes les instances de rendu audio tant identiques, on se contente de n'en tester qu'une seule
	set_menu_enable_state(hMenu, IDM_DSOUND,	check_property_dialog(ft_AudioRenderer));

	set_menu_enable_state(hMenu, IDM_VMR,		vmr_mode != rdt_null && check_property_dialog(ft_VideoRenderer));
}

/**
 * Met  jour les raccourcis dans les menus.
 **/
void update_shortcuts_menus()
{
	// Parcourt le menu rcursivement, et recherche si un vnement est dfini
	// si c'est le cas, on met  jour le raccourci
	update_shortcut_menu_recur(hMainMenu);
}

static void update_all_menus(HWND hWnd)
{
	update_menus(hWnd, 8,
		update_pistes_menu,
		update_chaines_menu,
		update_aspect_menus,
		update_ontop_menu,
		update_fullscreen_menu,
		update_menu_recording,
		update_priority_menu,
		update_mute_menu);

	update_record_icon();
}

static void update_all_menus_popup(HMENU hMenu)
{
	if (hMenu != NULL) {
		update_pistes_menu(hMenu);
		update_chaines_menu(hMenu);
		update_aspect_menus(hMenu);
		update_ontop_menu(hMenu);
		update_fullscreen_menu(hMenu);
		update_menu_recording(hMenu);
		update_mute_menu(hMenu);
	}
}

static DWORD time_context_quit;

/**
 * Copie un menu, rcursivement
 **/
static HMENU copie_menu(const HMENU origMenu)
{
	HMENU copie = CreatePopupMenu();

	int count = GetMenuItemCount(origMenu);

	for (int i=0; i<count; i++) {
		TCHAR			szMenuName[256];
		MENUITEMINFO	mnu = {
			sizeof(mnu),			// cbSize
			MIIM_BITMAP|MIIM_ID|MIIM_STATE|MIIM_SUBMENU|MIIM_STRING,	// fMask
			0, 0, 0,				// fType, fState, wID
			NULL, NULL, NULL,		// hSubMenu, hbmpChecked, hbmpUnchecked
			0,						// dwItemData
			szMenuName,				// dwTypeData
			_countof(szMenuName)	// cch
			// le reste  zro (implicite)
		};

		GetMenuItemInfo(origMenu, (UINT)i, TRUE, &mnu);
		if (mnu.hSubMenu != NULL)
			mnu.hSubMenu = copie_menu(mnu.hSubMenu);

		if(_tcscmp(szMenuName, TEXT("")) != 0) {
			AppendMenu(copie, mnu.fState, mnu.wID, szMenuName);
			SetMenuItemInfo(copie, (UINT)i, TRUE, &mnu);
		} else {
			// Sparateur
			AppendMenu(copie, MF_SEPARATOR, NULL, NULL);
		}
	}

	return copie;

}

static void popup_menu(bool from_tray_icon)
{
	HMENU	hPopMenu		= CreatePopupMenu();
	POINT	pos;

	GetCursorPos(&pos);

	// Copie des menus 0=Fichier, 1=Chanes, 2=Vido,
	// 3=Audio, 4=Information, 5=Enregistrer, 6=Options.
	for (int i=MENU_FICHIER_POSITION; i<=MENU_OPTIONS_POSITION; i++) {
		TCHAR nom_menu[64];
		HMENU contenu_copie = copie_menu(GetSubMenu(hMainMenu, i));

		GetMenuString(hMainMenu, i, nom_menu, _countof(nom_menu), MF_BYPOSITION);
		AppendMenu(hPopMenu, MF_STRING | MF_POPUP, (UINT_PTR)contenu_copie, nom_menu);
	}

	// Met  jour les menus (chanes + pistes)
	update_all_menus_popup(hPopMenu);

	if (from_tray_icon) {
		// Ajoute d'autres infos si venant de la zone de notification
		// (effectu  la fin, pour que les oprations prcdentes n'aient
		// pas besoin de s'effectuer avec des positions de menus dcales)
		MENUITEMINFO mnu = {
			sizeof(mnu),					// cbSize
			MIIM_BITMAP|MIIM_ID|MIIM_STRING	// fMask
			// le reste  zro (implicite)
		};

		if (!IsMinimized(hMainWnd)) {
			mnu.wID			= IDM_MINIMIZE;
			mnu.hbmpItem	= HBMMENU_POPUP_MINIMIZE;
			mnu.dwTypeData	= TEXT("");
		} else {
			mnu.wID			= IDM_RESTORE;
			mnu.hbmpItem	= HBMMENU_POPUP_RESTORE;
			mnu.dwTypeData	= TEXT("");
		}
		InsertMenuItem(hPopMenu, 0, TRUE, &mnu);
		InsertMenu(hPopMenu, 1, MF_BYPOSITION|MF_SEPARATOR, NULL, NULL);

		// Rplication de l'item "Quitter"  la fin du menu dans la zone de notification
		AppendMenu(hPopMenu, MF_SEPARATOR, NULL, NULL);
		AppendMenu(hPopMenu, MF_STRING, IDM_QUIT, TEXT(""));

		// Vrifier s'il n'y a pas un raccourci  associer  cette commande. On recherche par position,
		// car cet item peut exister plusieurs fois dans l'arborescence.
		update_shortcut_menu_item(hPopMenu, GetMenuItemCount(hPopMenu)-1, TRUE);
	}

	SetForegroundWindow(hMainWnd);

	ShowCursor(TRUE);

	TrackPopupMenuEx(hPopMenu, TPM_LEFTALIGN | TPM_LEFTBUTTON, pos.x, pos.y, hMainWnd, NULL);

	ShowCursor(FALSE);

	time_context_quit = GetTickCount();

	DestroyMenu(hPopMenu);
}

static void update_status_size()
{
	RECT			rect;

	GetWindowRect(hMainStatus, &rect);

	INT				width		= RWdt(rect);
	StatusBarFields	eLast		= sbf_NextProgName;
	INT				tab[]		= {100, 160, 290, 350, -1, -1};	// 100  60  130  60  130
	INT				minwidth	= 480;

	if (!use_osd) {
		eLast = sbf_MutedState;
		tab[sbf_MutedState-1] = width-100;
		// Faire disparatre les champs "next" qui deviennent trop petits
		if (tab[sbf_MutedState-1] < tab[sbf_NextProgTime-1]+30)
			tab[sbf_MutedState-1] = tab[sbf_NextProgName-1] = tab[sbf_NextProgTime-1];
		else if (tab[sbf_MutedState-1] < tab[sbf_NextProgName-1]+40)
			tab[sbf_MutedState-1] = tab[sbf_NextProgName-1];
		width = tab[sbf_MutedState-1];
	}

	// on a de la place en plus
	if (width > minwidth) {
		width = (width-minwidth)/2;

		tab[sbf_NextProgTime-1] += width;
		tab[sbf_NextProgName-1] += width;
	}

	SendMessage(hMainStatus, SB_SETPARTS, eLast+1, (LPARAM)tab);
}

static void update_systray_state()
{
	LONG_PTR ex_style = GetWindowLongPtr(hMainWnd, GWL_EXSTYLE);
	LONG_PTR ex_style_save = ex_style;

	if (minimize_system_tray) {
		// Si on minimise dans la zone systme, alors on n'affiche
		// pas la fentre dans la barre des tches
		ex_style &= ~WS_EX_APPWINDOW;
	} else
		ex_style |= WS_EX_APPWINDOW;

	if (ex_style != ex_style_save) {
		SetWindowLongPtr(hMainWnd, GWL_EXSTYLE, ex_style);
		niData.update_state();
	}
}

/**
 * Mise  jour du message sous MSN et de l'icne de notification
 **/
static void update_msn_and_tip()
{
	cMsnData.set();
	niData.update_tip();

	if (ixChaine_ok(ixChaineCourante)) {
		long nEpgRemainingTime = Canaux[ixChaineCourante].timeBeforeNextEvent();

		if (nEpgRemainingTime != DUREE_INFINIE)
			SetTimer(hMainWnd, TIMER_NOTIFY_UPDATE, nEpgRemainingTime, NULL);
	}
}

// Mise  jour de l'item de lancement de la base de registres pour le dmarrage avec Windows
static void update_start_with_windows()
{
	CRegKey_tstr	regWinStart;
	tstring			strRegRead;
	TCHAR			szValueName[MAX_PATH];

	if (*prefix_conf)
		_stprintf_s(szValueName, TEXT("%s (%s)"), szAppName, prefix_conf);
	else
		strcpy_T(szValueName, szAppName);

	regWinStart.Open(HKEY_CURRENT_USER, TEXT("Software\\Microsoft\\Windows\\CurrentVersion\\Run"),
		KEY_READ|KEY_SET_VALUE);
	regWinStart.QueryStringValue(szValueName, strRegRead);
	if (start_with_windows) {
		TCHAR		szRegWrite[MAX_PATH];

		_stprintf_s(szRegWrite, TEXT("\"%s\" --minimize%s"), szAppFileName, GetInstancePrefix().c_str());

		if (strRegRead != szRegWrite) {
			regWinStart.SetStringValue(szValueName, szRegWrite);
		}
	} else
		regWinStart.DeleteValue(szValueName);
}

static bool can_quit(LONG nTimeBeforeNextRecording)
{
	if (!record_protect)
		return true;

	if (enregistrements_actuels.recording())
		return false;

	return nTimeBeforeNextRecording >= RECORD_PROTECT_TIME;
}

/// Met  jour la commande de menu "Quitter" selon le temps restant avant enregistrement
void update_record_protect(LONG nTimeBeforeNextRecording)
{
	UINT	nId = 0;

	// Mise  jour du dernier item du menu "Fichier" :
	if (GetMenuState(hMainMenu, IDM_QUIT, MF_BYCOMMAND) != -1) {
		if (!can_quit(nTimeBeforeNextRecording)) {
			ModifyMenu(hMainMenu, IDM_QUIT, MF_BYCOMMAND | MF_STRING, nId = IDM_CLOSE, TEXT(""));
			update_shortcuts_menus();
		}
	} else {
		if (can_quit(nTimeBeforeNextRecording)) {
			ModifyMenu(hMainMenu, IDM_CLOSE, MF_BYCOMMAND | MF_STRING, nId = IDM_QUIT, TEXT(""));
			update_shortcuts_menus();
		}
	}

	if (nId) {
		// Vrifier s'il n'y a pas un raccourci  rassocier  la commande modifie.
		update_shortcut_menu_item(hMainMenu, nId, FALSE);
	}
}

/// Copie les informations d'une chane dans la barre d'tat, si elle existe
static void set_chaine_status(const Chaine & canal)
{
	if (hMainStatus != NULL) {
		TCHAR				szWrk[40];

		strcpy_T(szWrk, canal.nom);
		SendMessage(hMainStatus, SB_SETTEXT, sbf_ChanName, (LPARAM)szWrk);

		ActAndNxtProg		sCurrProg = canal.getCurrentAndNext();

		for (int nEmis=0; nEmis < 2; nEmis++) {
			tstring				strName;
			const Emission *	pEmis	= sCurrProg.get(nEmis);
			int					nSbfOfs = nEmis*(sbf_NextProgTime - sbf_ActualProgTime);

			szWrk[0] = 0;
			if (pEmis) {
				TimeToStr(pEmis->debut, szWrk, _countof(szWrk));
				strName = pEmis->getName();
			}
			SendMessage(hMainStatus, SB_SETTEXT, sbf_ActualProgTime+nSbfOfs, (LPARAM)szWrk);
			SendMessage(hMainStatus, SB_SETTEXT, sbf_ActualProgName+nSbfOfs, (LPARAM)strName.c_str());
		}
	}
}

static void update_status_bar()
{
	if (!ixChaine_ok(ixChaineCourante))
		return;

	if (use_osd)
		RedrawWindow(hMainWnd, NULL, NULL, RDW_INVALIDATE | RDW_ERASE);

	const Chaine & canal_courant = Canaux[ixChaineCourante];

	set_chaine_status(canal_courant);

	if (hMainStatus != NULL && !use_osd) {
		TCHAR str[16];

		_stprintf_s(str, muted ? TEXT("Muet (%i)") : TEXT("Volume %i"), volumeCourant);
		SendMessage(hMainStatus, SB_SETTEXT, sbf_MutedState, (LPARAM)str);
	}

#if USE_LOGITECH_LCD
	// Ne pas placer dans "if (hMainStatus != NULL)", car sinon
	// pas de mise  jour du LCD si la barre d'tat n'est pas affiche

	lcd.SetCanal(canal_courant);
	lcd.SetVolume(volumeCourant);
	lcd.ActualiseLcd();
#endif
}

/**
 * Changement de chane, slection par index dans la table.
 * Met galement  jour les menus et la barre d'tat.
 * \param[in]	ixChaine	Index dans la table #Canaux.
 *							(mettre -1 pour dsactiver la slection de chane)
 * \return		Code d'erreur DirectShow
 **/
HRESULT change_chaine_update(int ixChaine)
{
	HRESULT hr = change_chaine(ixChaine);

	if (hr==S_OK) {
		update_all_menus(hMainWnd);
		update_status_bar();
		update_msn_and_tip();
	}
	return hr;
}

/**
 * \brief Demande d'affichage des informations de la chane.
 *
 * Programmer un timer pour effectuer certaines mises  jour (recalcul de
 * positionnement et affichage du nom de la chane et du programme en cours)
 * une fois que tout sera bien stabilis (donc aprs un certain dlai).
 **/
static void demande_affichage_infos(UINT nDelai = DUREE_SHOW_PROGRAMME)
{
	KillTimer(hMainWnd, TIMER_ZAPPING);
	KillTimer(hMainWnd, MAKELONG(TIMER_ZAPPING, 1));
	myprint(COLR(LGREEN, TEXT("SetTimer, TIMER_ZAPPING")) EOL);
	SetTimer(hMainWnd, TIMER_ZAPPING, nDelai, NULL);
}

/**
 * Changement de chane, slection par index dans la table.
 * Met galement  jour les menus, la barre d'tat et l'affichage OSD.
 * \param[in]	ixChaine	Index dans la table #Canaux.
 *							(mettre -1 pour dsactiver la slection de chane)
 * \return		Code d'erreur DirectShow
 **/
HRESULT zappe_index(int ixChaine)
{
	myprintf(BOLD(TEXT("################# (dbut slection chane) #####################")) EOL);
	if (pOSD)
		pOSD->Clear();	// Effacer tous les objets OSD courants

	HRESULT hr = change_chaine_update(ixChaine);

	if (hr==S_OK)
		demande_affichage_infos();
	return hr;
}

static void switch_etat_fenetre(EtatsFenetre nouvel_etat);	// forward

static void switch_on_top()
{
	// pas de sens en plein cran, et puis menu pas visible...
	if (etat_fenetre(hMainWnd)==etf_fullscreen)
		return;

	on_top = !on_top;
	switch_etat_fenetre(etat_fenetre(hMainWnd));
	update_menus(hMainWnd, 1, update_ontop_menu);
}

// ====================================================================================
// Dialogue " propos"
// ====================================================================================

class CAboutDialog : public CModalDialogBase
{
	/// Traitement des messages de la bote de dialogue
	virtual INT_PTR DialogProc(UINT uMsg, WPARAM wParam, LPARAM lParam);

public:
	/// Constructeur :
	CAboutDialog() :
		CModalDialogBase(IDD_ABOUT)
	{
	}
};

/// Traitement des messages de la bote de dialogue
INT_PTR CAboutDialog::DialogProc(UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch (uMsg) {

	case WM_INITDIALOG: {
		// Lance la conversion des champs pour insrer les URLs
		ConvertStaticToHyperlink(hDlg, IDC_POUCHINTV_URL);
		ConvertStaticToHyperlink(hDlg, IDC_POUCHINTVMOD_URL);
		ConvertStaticToHyperlink(hDlg, IDC_POUCHINTVMOD_LOL_URL);

		// Ajoute les diffrents contributeurs
		RawResource resource;

		if (resource.Extraction(hAppInstance, IDR_AUTHORS)) {
			resource.AddStrZ();	// -> Ajout d'un zro terminateur
			// Tout s'est bien pass: on peut utiliser la liste des contributeurs
			SetDlgItemTextA(hDlg, IDC_AUTHORS, (LPCSTR)resource);
		}

		// Ajoute la version dans la fentre
		TCHAR		szVersion[128];
		TCHAR		szCompTime[32];

		ReformatCompileTime(szCompTime, _countof(szCompTime), TEXT("  "), __DATE__ ", " __TIME__);
		_stprintf_s(szVersion, TEXT("%s ver. %s") EOL TEXT("Compil %s"),
			szAppName, szAppVersion, szCompTime);
		if (*prefix_conf) {
			strcat_T(szVersion, EOL TEXT("instance "));
			strcat_T(szVersion, prefix_conf);
			strcat_T(szVersion, TEXT(""));
		}
		SetDlgItemText(hDlg, IDC_VERSION, szVersion);

		// Ajoute l'icone de Pouchin TV Mod

		// Charge l'image
		HANDLE icone_pouchin =
			LoadImage(hAppInstance, MAKEINTRESOURCE(IDI_PROG_ICON), IMAGE_ICON, 0, 0, 0);
		// Envoie l'image  l'lment dans la fentre
		SendDlgItemMessage(hDlg, IDC_ABOUT_ICON, STM_SETIMAGE,
			(WPARAM)IMAGE_ICON, (LPARAM) icone_pouchin);
		// Libre (fuites mmoires)
		FreeResource(icone_pouchin);
		return TRUE; }

	case WM_COMMAND:
		switch(LOWORD(wParam)) {

		case IDOK:
		case IDCANCEL:
			EndDialog(hDlg, 0);
			return TRUE;

		case IDC_POUCHINTV_URL:
			ShellOpen(hMainWnd, TEXT("http://mmzin.com"));
			return TRUE;

		case IDC_POUCHINTVMOD_URL:
			ShellOpen(hMainWnd, TEXT("http://www.etud.insa-toulouse.fr/~mvelten/pouchintvmod/"));
			return TRUE;

		case IDC_POUCHINTVMOD_LOL_URL:
			ShellOpen(hMainWnd, TEXT("http://www.pouchintv.fr/"));
			return TRUE;
		}
	}
	return FALSE;
}

// ====================================================================================
// Dialogue de confirmation de d'arrt (car enregistrements en cours)
// ====================================================================================

/**
 * \brief Construit une message pour indiquer pourquoi on ne peut pas quitter
 * \param[in]	nReason			Si > 0, nombre d'enregistrements en cours
 *								Si < 0, nombre de millisecondes (ngatif) avant prochain enregistrement
 * \param[in]	bNoSchedule		Si \p true, indique qu'aucune tche planifie n'existe pour relancer
 **/
static tstring ReasonAsStr(LONG nReason, bool bNoSchedule)
{
	TCHAR szWrk[256] = TEXT("Aucune raison");

	// Remplacement du message par dfaut "Un enregistrement est en cours." si besoin est.
	if (nReason > 0) {
		if (nReason > 1)
			_stprintf_s(szWrk, TEXT("%i enregistrements sont en cours"), nReason);
		else
			strcpy_T(szWrk, TEXT("Un enregistrement est en cours"));
	} else if (nReason < 0) {
		LONG	nTime = -nReason;

		if (nTime <= DUREE_1MN * 2)
			strcpy_T(szWrk, TEXT("    -   ."));
		else {
			if (nTime <= DUREE_1H *2)
				_stprintf_s(szWrk, TEXT("    %i"), nTime / DUREE_1MN);
			else if (nTime <= DUREE_24H)
				_stprintf_s(szWrk, TEXT("    %i"), nTime / DUREE_1H);
			if (bNoSchedule)
				strcat_T(szWrk, ",    Scheduled task,      .");
		}
	}
	return szWrk;
}

/**
 * \brief Vrifier si on peut quitter
 * \param[in]	bProtect		Si \p true, protection tendue (24h)
 * \param[in]	bCheckSchedule	Si \p true, vrifie si une tche programme existe pour relancer (protection restreinte)
 **/
static LONG CheckIfWeCanQuit(bool bProtect, bool bCheckSchedule)
{
	// Compter les enregistrements en cours :
	LONG		nNbrRec = enregistrements_actuels.count_recording();

	if (nNbrRec > 0) {
		return nNbrRec;	// On a appuy sur Annuler, donc, on ne quitte pas
	}

	Programme *	pProg = NULL;
	LONG		nRemain = temps_prochain_enregistrement(NULL, &pProg);

	if (nRemain != DUREE_INFINIE) {
		if (bCheckSchedule && pProg && pProg->tache == planif_avec && nRemain > DUREE_PLANIF + DUREE_1SEC*10)
			return 0;

		if (bProtect) {
			// Dure plus longue si la protection des enregistrements est active
			if (nRemain > RECORD_PROTECT_TIME)
				return 0;
		} else {
			// Sinon, on ne protge que pour la dure "duree_grace_quit"
			if (nRemain > duree_grace_quit * DUREE_1MN)
				return 0;
		}

		// Si un enregistrement est prvu avant la dure spcifie, on ne peut pas quitter
		return -nRemain;	// On a appuy sur Annuler, donc, on ne quitte pas
	}

	return 0;
}

/**
 * Classe de gestion de la fentre de confirmation d'arrt
 **/
class CConfirmCloseDialog : public CModalDialogBase
{
	LONG nReason;	//!< si > 0 : nombre d'enregistrements ; si < 0, temps avant enregistrement

	/// Traitement des messages de la bote de dialogue
	virtual INT_PTR DialogProc(UINT uMsg, WPARAM wParam, LPARAM lParam);

	/// Constructeur :
	CConfirmCloseDialog(LONG nReasn) :
		CModalDialogBase(IDD_CLOSE_CONFIRM),
		nReason(nReasn)
	{
	}

public:
	/** \brief Vrifier si le fait de quitter compromettrait des enregistrements
	 *
	 *	Le cas chant, prsente un dialogue  l'utilisateur pour lui demander confirmation
	 *	\param[in]	hWnd	Handle de la fentre parente
	 *	\return		\p true si on peut quitter, false sinon
	 **/
	static bool Check(HWND hWnd);
};

/// Traitement des messages de la bote de dialogue
INT_PTR CConfirmCloseDialog::DialogProc(UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch (uMsg) {

	case WM_INITDIALOG: {
		// Initialisation de la fentre
		SetItemText(IDC_MESSAGE, ReasonAsStr(nReason, true).c_str());

		// Renvoie True car la fentre a t initialise
		return TRUE; }

	case WM_COMMAND:
		// Commande reue (clic, etc...)
		switch(LOWORD(wParam)) {

		case IDOK:
			// Clic sur le bouton OK
			enregistrements_actuels.stop_all();
			update_record_menus(hMainWnd);
			EndDialog(hDlg, 1);
			return TRUE;

		case IDCANCEL:
			// Clic sur le bouton Annuler
			EndDialog(hDlg, 0);
			return TRUE;
		}
		// Autre commandes, traites par Windows
		return FALSE;

	default:
		// Autre message, trait par Windows
		return FALSE;
	}
}

bool CConfirmCloseDialog::Check(HWND hWnd)	// static
{
	LONG	nReason = CheckIfWeCanQuit(record_protect, true);

	if (nReason == 0)
		return true;	// On peut quitter

	return CConfirmCloseDialog(nReason).Do(hWnd) != 0;
}

// ====================================================================================
// Blocage de l'arrt systme
// ====================================================================================

CShutdownBlockReason::CShutdownBlockReason() :
	eSdm(sdm_NoBlock),
	fnShutdownBlockReasonCreate(NULL),
	fnShutdownBlockReasonDestroy(NULL)
{
	HMODULE		hUser32Module = GetModuleHandle(TEXT("user32"));

	// Les fonction "ShutdownBlockReasonCreate" et "ShutdownBlockReasonDestroy" n'existent
	// qu' partir de Windows Vista. On les charge donc dynamiquement si elles existent.
	if (hUser32Module) {
		fnShutdownBlockReasonCreate		= reinterpret_cast<PFN_SHUTDOWNBLOCKREASONCREATE>(
											GetProcAddress(hUser32Module, "ShutdownBlockReasonCreate"));
		fnShutdownBlockReasonDestroy	= reinterpret_cast<PFN_SHUTDOWNBLOCKREASONDESTROY>(
											GetProcAddress(hUser32Module, "ShutdownBlockReasonDestroy"));
	}
}

void CShutdownBlockReason::Update()
{
	if (!isAvailable())
		return;		// Fonctionnalit pas disponible (Windows XP ...)

	// Compter les enregistrements en cours :
	LONG	nNbrRec		= enregistrements_actuels.count_recording();
	LONG	nRemain		= DUREE_INFINIE;
	WCHAR	szWrk[256]	= L"";
	SdMode	eNewSdm		= sdm_NoBlock;
	BOOL	bRes;

	if (nNbrRec > 0) {
		if (nNbrRec > 1)
			swprintf_s(szWrk, L"%" T2w L" a %i enregistrements en cours", szAppName, nNbrRec);
		else
			swprintf_s(szWrk, L"%" T2w L" a un enregistrement en cours", szAppName);
		eNewSdm = sdm_Recording;
	} else {
		SYSTEMTIME	sCurrTime;

		GetLocalTime(&sCurrTime);
		nRemain = temps_prochain_enregistrement(&sCurrTime);
		if (nRemain <= duree_grace_quit * DUREE_1MN) {
			AddTime(sCurrTime, nRemain);

			swprintf_s(szWrk, L"%" T2w L" est sur le point d'enregistrer une mission TV (dbut  %ih%02i)",
				szAppName, sCurrTime.wHour, sCurrTime.wMinute);
			eNewSdm = sdm_WaitRecording;
		}
	}

	if (eNewSdm != sdm_NoBlock) {
		bRes = fnShutdownBlockReasonCreate(hMainWnd, szWrk);
		myprintf(CMyP ERRX(TEXT("Erreur ShutdownBlockReasonCreate(%s) code 0x%08x")), !bRes, szWrk, GetLastError());
		if (bRes)
			eSdm = eNewSdm;
	} else if (eSdm != sdm_NoBlock) {
		bRes = fnShutdownBlockReasonDestroy(hMainWnd);
		myprintf(CMyP ERRX(TEXT("Erreur ShutdownBlockReasonDestroy code 0x%08x")), !bRes, GetLastError());
		if (bRes)
			eSdm = eNewSdm;
	}
	// myprintf(TEXT("CShutdownBlockReason::Update = %u") EOL, eSdm);
}

void CShutdownBlockReason::Reset()
{
	if (eSdm != sdm_NoBlock && fnShutdownBlockReasonDestroy != NULL)
		fnShutdownBlockReasonDestroy(hMainWnd);
	eSdm = sdm_NoBlock;
}

// ====================================================================================
// Dialogue de qualit du signal
// ====================================================================================

#define SC_DEC 0	//!< Identifiant pour le template de surclassement du bouton de dcrmentation
#define SC_INC 1	//!< Identifiant pour le template de surclassement du bouton d'incrmentation

class CSignalDialog : public WinSubClass<SC_DEC>, public WinSubClass<SC_INC>, public CModelessDialog<CSignalDialog>
{
	enum MouseState
	{
		ms_Idle,
		ms_DownOutsideButton,
		ms_DownInsideButton
	};

	int			nCurChan;		//!< Canal courant
	int			nCurFreq;		//!< Frquence courante
	int			nAdjFreq;		//!< Frquence ajuste
	int			nSavFreq;		//!< Sauvegarde temporaire de frquence
	int			nIncSign;		//!< +1 ou -1 selon incrment ou dcrment
	MouseState	eMState;		//!< tat de la souris par rapport  un bouton
	DWORD		nClickTime;		//!< Temps de dbut du clic de modification
	int			last_force_max;	//!< Utilis en cas de changement de calibration de la force du signal

	/// Actualisation des informations de rception du signal
	void Update();

	/// Activation ou dsactivation des boutons de modification
	void UpdateButtonsState();

	/// Dfinition et affichage d'un numro de canal
	void SetChan(int nNewChan);

	/// Dfinition et affichage d'une frquence
	void SetFreq(int nNewFreq);

	/// Application d'une frquence au tuner
	void ApplyFreq();

	/// Traitement commun aux boutons d'incrmentation et de dcrmentation
	void ProcessButtons(HWND hCtl, UINT uMsg, WPARAM wParam, LPARAM lParam, int nSign);

	/// Vrification et confirmation des changements
	void ChkSave();

	/// Traitement des messages du bouton de dcrmentation surclass
	virtual LRESULT SubProc(WinSubClass<SC_DEC> & sc, UINT uMsg, WPARAM wParam, LPARAM lParam) {
						ProcessButtons(sc.hSWnd, uMsg, wParam, lParam, -1);
						return sc.CallWindowProc(uMsg, wParam, lParam); }

	/// Traitement des messages du bouton d'incrmentation surclass
	virtual LRESULT SubProc(WinSubClass<SC_INC> & sc, UINT uMsg, WPARAM wParam, LPARAM lParam) {
						ProcessButtons(sc.hSWnd, uMsg, wParam, lParam, 1);
						return sc.CallWindowProc(uMsg, wParam, lParam); }

	/// Traitement des messages de la bote de dialogue
	virtual INT_PTR DialogProc(UINT uMsg, WPARAM wParam, LPARAM lParam);

public:
	/// Constructeur
	CSignalDialog();
};

/// Constructeur
CSignalDialog::CSignalDialog() :
	CModelessDialog(IDD_SIGNAL),
	nCurChan(-1),
	nCurFreq(-1),
	nAdjFreq(-1),
	nSavFreq(-1),
	nIncSign(0),
	eMState(ms_Idle),
	last_force_max(0)
{
}

/// Dfinition et affichage d'un numro de canal
void CSignalDialog::SetChan(int nNewChan)
{
	TCHAR	szChan[16] = TEXT(" --");

	if ((nCurChan = nNewChan) > 0)
		_stprintf_s(szChan, TEXT(" %i"), nCurChan);
	SetItemText(IDC_SIGNAL_CHAN, szChan);
}

/// Dfinition et affichage d'une frquence
void CSignalDialog::SetFreq(int nNewFreq)
{
	TCHAR	szFreq[16] = TEXT("------ kHz");

	if (nCurChan > 0) {
		int	nCen = DVBT_CHANBASE + (nCurChan * DVBT_BANDWIDTH*1000);	// Frquence centrale du canal

		// Limitation des valeurs
		nAdjFreq = max<int>(min<int>(nNewFreq, nCen+(DVBT_BANDWIDTH*500)), nCen-(DVBT_BANDWIDTH*500));
		_stprintf_s(szFreq, TEXT("%i kHz"), nAdjFreq);
	}

	SetItemText(IDC_SIGNAL_FREQ, szFreq);
}

/// Application d'une frquence au tuner
void CSignalDialog::ApplyFreq()
{
	pGraph->LowLevelTune(nAdjFreq, DVBT_BANDWIDTH);
}

/// Actualisation des informations de rception du signal
void CSignalDialog::Update()
{
	SignalStatistics	sSStats;
	TCHAR				szWrk[16];

	sSStats.getStats();

	SetDlgItemText(hDlg, IDC_SIGNAL_PRESENT, sSStats.getPresentStr());
	SetDlgItemText(hDlg, IDC_SIGNAL_LOCK, sSStats.getLockedStr());

	sSStats.SetLevelBars(
		GetDlgItem(hDlg, IDC_SIGNAL_STRENGTH_BAR), GetDlgItem(hDlg, IDC_SIGNAL_QUALITY_BAR));

	sSStats.getStrengthStr(sTunProps.force_signal_aff_pct, szWrk, _countof(szWrk), TEXT(" %"));
	SetDlgItemText(hDlg, IDC_SIGNAL_STRENGTH, szWrk);

	sSStats.getQualityStr(szWrk, _countof(szWrk), TEXT(" %"));
	SetDlgItemText(hDlg, IDC_SIGNAL_QUALITY, szWrk);
}

/// Activation ou dsactivation des boutons de modification
void CSignalDialog::UpdateButtonsState()
{	/// \todo Dsactiver aussi pendant la recherche de chanes
	bool bEnable = enregistrements_actuels.recording() == 0;

	EnableDlgItem(IDC_SIGNAL_DEC, bEnable);
	EnableDlgItem(IDC_SIGNAL_INC, bEnable);
}

/// Traitement commun aux boutons d'incrmentation et de dcrmentation
void CSignalDialog::ProcessButtons(HWND hCtl, UINT uMsg, WPARAM wParam, LPARAM lParam, int nSign)
{
	switch (uMsg) {

	case WM_LBUTTONDOWN:
		KillTimer(hDlg, TIMER_SIGNAL_ADJ); // Timer du dialogue principal
		nSavFreq = nAdjFreq;
		SetFreq(nAdjFreq + nSign);
		nIncSign = nSign;
		SetTimer(hCtl, TIMER_SIGNAL_ADJ, 500, NULL);
		nClickTime = GetTickCount();
		eMState = ms_DownInsideButton;
		break;

	case WM_LBUTTONUP:
		if (eMState != ms_Idle) {
			KillTimer(hCtl, TIMER_SIGNAL_ADJ);
			if (eMState != ms_DownInsideButton)
				SetFreq(nSavFreq);
			eMState = ms_Idle;
		}
		break;

	case WM_MOUSEMOVE:
		if (eMState != ms_Idle) {
			POINT	sPt = {GET_X_LPARAM(lParam), GET_Y_LPARAM(lParam)};
			RECT	sRect;

			GetClientRect(hCtl, &sRect);
			if (PtInRect(&sRect, sPt)) {
				if (eMState == ms_DownOutsideButton) {
					SetTimer(hCtl, TIMER_SIGNAL_ADJ, 100, NULL);
					nClickTime = GetTickCount();
					eMState = ms_DownInsideButton;
				}
			} else {
				if (eMState == ms_DownInsideButton) {
					KillTimer(hCtl, TIMER_SIGNAL_ADJ);
					eMState = ms_DownOutsideButton;
				}
			}
		}
		break;

	case WM_TIMER:
		if (wParam == TIMER_SIGNAL_ADJ) {
			if (eMState != ms_Idle) {
				//  plus de 3 secondes, on incrmente par 5
				int	nTim = GetTickCount()-nClickTime;
				int nInc = nTim<2000 ? 1 : nTim<4000 ? 2 : nTim<6000 ? 5 : nTim<8000 ? 10 : nTim<10000 ? 20 : 50;
				int	nTmp = (((nAdjFreq + (nInc-1)) / nInc)+nIncSign) * nInc;

				SetFreq(nTmp);
				SetTimer(hCtl, TIMER_SIGNAL_ADJ, 100, NULL);
			} else
				KillTimer(hCtl, TIMER_SIGNAL_ADJ);
		}
	}
}

/// Vrification et confirmation des changements
void CSignalDialog::ChkSave()
{
	KillTimer(hDlg, TIMER_SIGNAL);
	if (nCurFreq != nAdjFreq) {
		if (MessageBox(hMainWnd,
			TEXT("    .") EOL
			TEXT("      ") EOL
			TEXT("   ?"),
			TEXT("  "), MB_ICONQUESTION | MB_YESNO) == IDYES)
		{
			nCurFreq = nAdjFreq;
			for (ITERATE_VECTOR(Canaux, Chaine, it)) {
				if (it->canal_no == nCurChan)
					it->khz = nCurFreq;
			}
			sauve_chaines();
		} else {
			SetFreq(nCurFreq);	// Restauration
			ApplyFreq();
			MessageBoxCheck(hDlg,
				TEXT("  ."),
				TEXT(" "), MB_OK|MB_ICONINFORMATION,
				MB_OK, TEXT("   "));
		}
	}
}

/// Traitement des messages de la bote de dialogue
INT_PTR CSignalDialog::DialogProc(UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch (uMsg) {

	case WM_INITDIALOG: {
		SendDlgItemMessage(hDlg, IDC_SIGNAL_STRENGTH_BAR, PBM_SETRANGE32, 0, sTunProps.force_signal_max);
		SendDlgItemMessage(hDlg, IDC_SIGNAL_QUALITY_BAR, PBM_SETRANGE32, 0, 100);
		Update();
		SetTimer(hDlg, TIMER_SIGNAL, DUREE_SIGNAL_STATS, NULL);
		WinSubClass<SC_DEC>::SubClass(GetDlgItem(hDlg, IDC_SIGNAL_DEC));
		WinSubClass<SC_INC>::SubClass(GetDlgItem(hDlg, IDC_SIGNAL_INC));
		last_force_max = sTunProps.force_signal_max;
		UpdateButtonsState();

		// Initialisation donnes canal et frquence
		int nChan = 0, nFreq = 0;

		if (ixChaine_ok(ixChaineCourante)) {
			Chaine & sChan = Canaux[ixChaineCourante];

			nChan = sChan.canal_no;
			nCurFreq = nFreq = sChan.khz;
		}

		SetChan(nChan);
		SetFreq(nFreq);
		return TRUE; }

	case WM_COMMAND:
		switch(wParam) {

		case _cmd_(IDC_SIGNAL_DEC, BN_CLICKED):
		case _cmd_(IDC_SIGNAL_INC, BN_CLICKED):
			SetTimer(hDlg, TIMER_SIGNAL_ADJ, 800, NULL);
			return TRUE;

		case _cmd_(IDOK, BN_CLICKED):
			ChkSave();
			DestroyWindow(hDlg);
			return TRUE;
		}
		break;

	case WM_TIMER:
		switch (wParam) {

		case TIMER_SIGNAL_ADJ:
			KillTimer(hDlg, TIMER_SIGNAL_ADJ);
			ApplyFreq();
			break;

		case TIMER_SIGNAL:
			if (sTunProps.force_signal_max != last_force_max) {
				// Un changement de calibration est intervenu
				SendDlgItemMessage(hDlg, IDC_SIGNAL_STRENGTH_BAR, PBM_SETRANGE32, 0, sTunProps.force_signal_max);
				last_force_max = sTunProps.force_signal_max;
			}
			Update();
		}
		return TRUE;

	case WM_APP_NEW_FREQ:
		SetChan((int)wParam);
		SetFreq(nCurFreq = (int)lParam);
		break;

	case WM_APP_RECORD_EVENT:
		UpdateButtonsState();
		// Si dbut d'enregistrement, on rapplique immdiatement la frquence d'origine
		if (nCurFreq != nAdjFreq)
			SetFreq(nCurFreq);
			ApplyFreq();
		break;

	case WM_CLOSE:
		ChkSave();
		DestroyWindow(hDlg);
		return TRUE;

	}
	return FALSE;
}

// ====================================================================================
// Dialogue de rglage de la vido
// ====================================================================================

class CVideoControlDialog : public CModalDialogBase
{
	ProcAmpValues	sProcAmpVals;

	enum ItemOffsets
	{
		io_SLIDER,
		io_MIN,
		io_MAX,
		io_TXT,
		io_VAL
	};

	void SetDlgProcAmpVal(int nBaseID, ProcAmpVal & sVal);

	void InitDlgProcAmp(int nBaseID, ProcAmpVal & sVal);

	/// Traitement des messages de la bote de dialogue
	virtual INT_PTR DialogProc(UINT uMsg, WPARAM wParam, LPARAM lParam);

public:

	/// Constructeur :
	CVideoControlDialog() :
		CModalDialogBase(IDD_VIDEOCONTROL)
	{
	}

	/**
	 * Ouverture et traitement effectifs de la bote de dialogue.
	 * \param[in]	hParentWnd	\p HANDLE de la fentre parente
	 * \return		Valeur passe  la fonction \p EndDialog, ou 0 si erreur
	 **/
	INT_PTR Do(HWND hParentWnd)
	{
		if (pVMR) {
			ProcAmpAvailability eProcAmpAvail = pVMR->isProcAmpAvailable();

			if (eProcAmpAvail != paa_Available) {
				if (eProcAmpAvail == paa_MissingFromGPU)
					MessageBox(hParentWnd,
						TEXT("          .") EOL
						TEXT("          ,")
						TEXT("     ."),
						TEXT("   "), MB_ICONERROR | MB_OK);
				return 0;
			}

			HRESULT hr = pVMR->GetProcAmpValues(sProcAmpVals);

			if (check_hr(hr, "CVideoControlDialog::Do, GetProcAmpValues"))
				return 0;
		}

		return __super::Do(hParentWnd);
	}
};

void CVideoControlDialog::SetDlgProcAmpVal(int nBaseID, ProcAmpVal & sVal)
{
	TCHAR	szWrk[16];

	SendDlgItemMessage(hDlg, nBaseID+io_SLIDER, TBM_SETPOS, TRUE, LPARAM(sVal.fCurrent / sVal.fStep));
	_stprintf_s(szWrk, TEXT("%.*f"), sVal.GetPrecision(), sVal.fCurrent);
	SetItemText(nBaseID+io_VAL, szWrk);
}

void CVideoControlDialog::InitDlgProcAmp(int nBaseID, ProcAmpVal & sVal)
{
	LONG	nMin = LONG(sVal.fMin/sVal.fStep);
	LONG	nMax = LONG(sVal.fMax/sVal.fStep);

	// Ajustement de l'tat d'activation de tous les items
	// (rglage et textes statiques) lis  ce contrle
	for (int nID = io_SLIDER; nID <=io_VAL; ++nID)
		EnableDlgItem(nBaseID+nID, sVal.bValid);
	SendDlgItemMessage(hDlg, nBaseID+io_SLIDER, TBM_SETRANGEMIN, TRUE, nMin);
	SendDlgItemMessage(hDlg, nBaseID+io_SLIDER, TBM_SETRANGEMAX, TRUE, nMax);
	if (sVal.bValid) {
		int			nDec = sVal.GetPrecision();		// Nombre de dcimales  afficher
		TCHAR		szWrk[16];

		// Dterminer l'intervalle de tics sur le "trackbar" par squence de 1, 2, 5, 10 ...
		int			nTFrq=1, nInx=0;
		static int	aFTbl[] = {1, 2, 5};

		while (nTFrq && nTFrq <= (nMax-nMin)/50) {
			++nInx;

			nTFrq = aFTbl[nInx % _countof(aFTbl)] * (int)pow(10., int(nInx/_countof(aFTbl)));
		}

		// myprintf(TEXT("InitDlgProcAmp : id=%i, min=%i, max=%i, step=%i, tfrq=%i") EOL,
		//	nBaseID, nMin, nMax, 1, nTFrq);
		SendDlgItemMessage(hDlg, nBaseID+io_SLIDER, TBM_SETTICFREQ, nTFrq, NULL);
		_stprintf_s(szWrk, TEXT("%.*f"), nDec, sVal.fMin);
		SetItemText(nBaseID+io_MIN, szWrk);
		_stprintf_s(szWrk, TEXT("%.*f"), nDec, sVal.fMax);
		SetItemText(nBaseID+io_MAX, szWrk);
	}

	SetDlgProcAmpVal(nBaseID, sVal);
}

/*
float round_to_num(float fVal, int nDecimals)
{
	float fMul = pow(10.0f, nDecimals);

	fVal *= fMul;
	fVal = (fVal > (floor(fVal)+0.5f)) ? ceil(fVal) : floor(fVal);
	fVal /= fMul;
	return fVal;
}
*/

/// Traitement des messages de la bote de dialogue
INT_PTR CVideoControlDialog::DialogProc(UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	ProcAmpType eTyp;

	switch (uMsg) {

	case WM_INITDIALOG:
		for (eTyp = pat_Brightness; eTyp < pat_MAX; ++eTyp)
			InitDlgProcAmp(IDC_VCONTROL_BASE+eTyp*5, sProcAmpVals.aVal[eTyp]);
		return TRUE;

	case WM_HSCROLL:
		switch (LOWORD(wParam)) {

		case SB_LINELEFT:
		case SB_LINERIGHT:
		case SB_PAGELEFT:
		case SB_PAGERIGHT:
		case SB_THUMBPOSITION:
		case SB_THUMBTRACK: {
			int				nId		= (int)GetWindowLongPtr((HWND)lParam, GWLP_ID);	// ID du contrle concern

			eTyp	= static_cast<ProcAmpType>((nId-IDC_VCONTROL_BASE)/5);

			ProcAmpVal &	sVal	= sProcAmpVals.aVal[eTyp];						// Donnes correspondantes
			LONG			nCur	= (LONG)SendMessage((HWND)lParam, TBM_GETPOS, 0, NULL);

			sVal.fCurrent = nCur*sVal.fStep;
			sVal.bToSet = true;
			SetDlgProcAmpVal(nId, sVal);
			if (pVMR) {
				pVMR->SetProcAmpValues(sProcAmpVals);
				video_sav[eTyp] = sVal.GetIntVal();
			}
			break; }
		}
		return TRUE;

	case WM_COMMAND:
		// Commande reue (clic, etc...)
		switch(wParam) {

		case _cmd_(IDC_VCONTROL_SAVE, BN_CLICKED):
			// Sauvegarde des valeurs courantes pour rcupration au dmarrage suivant
			for (eTyp = pat_Brightness; eTyp < pat_MAX; ++eTyp) {
				ProcAmpVal &	sVal = sProcAmpVals.aVal[eTyp];

				video_ctl[eTyp] = video_sav[eTyp] = sVal.GetIntVal();
			}
			save_config();
			return TRUE;

		case _cmd_(IDC_VCONTROL_RESTORE, BN_CLICKED):
			// Restauration des dernires valeurs sauvegardes
			for (eTyp = pat_Brightness; eTyp < pat_MAX; ++eTyp) {
				ProcAmpVal &	sVal = sProcAmpVals.aVal[eTyp];
				INT32			nRst = video_ctl[eTyp];

				sVal.SetIntVal(nRst);
				SetDlgProcAmpVal(IDC_VCONTROL_BASE+eTyp*5, sVal);
				video_sav[eTyp] = sVal.GetIntVal();
			}
			if (pVMR)
				pVMR->SetProcAmpValues(sProcAmpVals);
			return TRUE;

		case _cmd_(IDC_USE_DEFAULTS, BN_CLICKED):
			// Restauration des valeurs par dfaut
			for (eTyp = pat_Brightness; eTyp < pat_MAX; ++eTyp) {
				ProcAmpVal &	sVal = sProcAmpVals.aVal[eTyp];

				sVal.SetIntVal(VIDEO_UNDEF);
				SetDlgProcAmpVal(IDC_VCONTROL_BASE+eTyp*5, sVal);
				video_sav[eTyp] = sVal.GetIntVal();
			}
			if (pVMR)
				pVMR->SetProcAmpValues(sProcAmpVals);
			return TRUE;

		case _cmd_(IDOK, BN_CLICKED):
			// Clic sur le bouton OK
			EndDialog(hDlg, IDOK);
			return TRUE;

		case _cmd_(IDCANCEL, BN_CLICKED):
			// Clic sur le bouton Annuler (apparemment ncessaire pour grer la case de fermeture)
			EndDialog(hDlg, IDCANCEL);
			return TRUE;
		}
	}

	// Autre message, trait par Windows
	return FALSE;
}

// ====================================================================================
// Dialogue de saisie de frquence de rception (mode diagnostic seulement)
// ====================================================================================

#ifdef _DIAGNOSTIC

class CGetFreqDialog : public CModalDialogBase
{
	/// Traitement des messages de la bote de dialogue
	virtual INT_PTR DialogProc(UINT uMsg, WPARAM wParam, LPARAM lParam)
	{
		switch (uMsg) {

		case WM_INITDIALOG:
			SetDlgItemInt(hDlg, IDC_FREQUENCY, mpx_frequence_diag, FALSE);
			return TRUE;

		case WM_COMMAND:
			// Commande reue (clic, etc...)
			switch(wParam) {

			case _cmd_(IDC_FREQUENCY, EN_CHANGE): {
				int nTmp;

				if (get_int(hDlg, IDC_FREQUENCY, nTmp, FALSE) && nTmp >= 21 && nTmp <= 69) {
					nTmp = ListeCanaux().canal2khz((BYTE)nTmp);
					SetDlgItemInt(hDlg, IDC_FREQUENCY, nTmp, FALSE);
				}
				return TRUE; }

			case _cmd_(IDOK, BN_CLICKED): {
				get_int(hDlg, IDC_FREQUENCY, mpx_frequence_diag, FALSE);
				// Clic sur le bouton OK
				EndDialog(hDlg, IDOK);
				return TRUE;
			}

			case _cmd_(IDCANCEL, BN_CLICKED):
				// Clic sur le bouton Annuler
				EndDialog(hDlg, IDCANCEL);
				return TRUE;
			}
		}

		// Autre message, trait par Windows
		return FALSE;
	}

public:

	/// Constructeur :
	CGetFreqDialog() :
		CModalDialogBase(IDD_GETFREQ)
	{
	}
};

#endif	// #ifdef _DIAGNOSTIC

// ====================================================================================

static void traite_numero(int nb)
{
	numeroChaineSaisi = (numeroChaineSaisi * 10 + (WORD)nb) % 1000;
	// (modulo 1000 au cas o il y aurait trop de chiffres)

	SetTimer(hMainWnd, TIMER_NUMBER, DUREE_SAISIE, NULL);

	if (pOSD)
		pOSD->DisplayChannelNumber(numeroChaineSaisi);
}

static void create_status_bar()
{
	// Create the status bar.
	hMainStatus = CreateWindowEx(
		0,							// no extended styles
		STATUSCLASSNAME,			// name of status bar class
		NULL,						// no text when first created
		WS_CHILD | WS_VISIBLE,		// creates a child window
		0, 0, 0, 0,					// ignores size and position
		hMainWnd,					// handle to parent window
		(HMENU) IDC_STATUS_BAR,		// child window identifier
		hAppInstance,				// handle to application instance
		NULL);						// no window creation data
}

static bool remove_borders()
{
	bool change = false;

	if (hMainStatus!=NULL) {
		DestroyWindow(hMainStatus);
		hMainStatus = NULL;
		change = true;
	}
	if (GetMenu(hMainWnd)!=NULL) {
		SetMenu(hMainWnd, NULL);
		change = true;
	}
	return change;
}

static bool restore_borders()
{
	bool change = false;

	if (hMainStatus==NULL) {
		create_status_bar();
		update_status_size();
		update_status_bar();
		change = true;
	}
	if (GetMenu(hMainWnd)==NULL) {
		SetMenu(hMainWnd, hMainMenu);
		change = true;
	}
	return change;
}

/**
 * \brief Dfinition d'un nouvel tat pour la fentre principale
 * \param[in]	nouvel_etat	Nouvel tat  dfinir
 **/
static void switch_etat_fenetre(EtatsFenetre nouvel_etat)
{
	WINDOWPLACEMENT wpl = {sizeof(wpl)};
	LONG_PTR		style_courant = GetWindowLongPtr(hMainWnd, GWL_STYLE);
	LONG_PTR		nouveau_style = style_courant;
	// EtatsFenetre	etat_courant = etat_fenetre(style_courant);
	UINT			wPosFlags = SWP_NOMOVE | SWP_NOSIZE | SWP_NOZORDER | SWP_FRAMECHANGED;
	HWND			hWndInsertAfter = HWND_TOP;
	bool			change = false;
	RECT			rect = {0, 0, 0, 0};
	EtatCurseur		nouveau_curseur = etc_normal; // si pas chang aprs

	GetWindowPlacement(hMainWnd, &wpl);

	UINT			newShowCmd = wpl.showCmd;

	switch (nouvel_etat) {

	case etf_normal:
		nouveau_style |= WS_OVERLAPPEDWINDOW;
		newShowCmd = SW_SHOWNORMAL;
		if (restore_borders())
			change = true;
		break;

	case etf_noborders:
		nouveau_style &= ~WS_OVERLAPPEDWINDOW;
		nouveau_style |= WS_THICKFRAME;
		newShowCmd = SW_SHOWNORMAL;
		if (remove_borders())
			change = true;
		break;

	case etf_maximized:
		nouveau_style |= WS_OVERLAPPEDWINDOW;
		newShowCmd = SW_SHOWMAXIMIZED;
		if (restore_borders())
			change = true;
		break;

	case etf_fullscreen:
		nouveau_style &= ~WS_OVERLAPPEDWINDOW;
		newShowCmd = SW_SHOWMAXIMIZED;
		wPosFlags &= ~(SWP_NOMOVE | SWP_NOSIZE);
		hWndInsertAfter = HWND_TOP;

		{
			MONITORINFO mi		= {sizeof(MONITORINFO)};
			HMONITOR	hMon	= MonitorFromWindow(hMainWnd, MONITOR_DEFAULTTONEAREST);

			GetMonitorInfo(hMon, &mi);
			rect = mi.rcMonitor;
		}

		if (remove_borders())
			change = true;
		nouveau_curseur = etc_fullscreen_on;
		// Force le lancement du timer demandant la suppression du curseur en plein cran
		SetTimer(hMainWnd, TIMER_CURSOR, DUREE_CURSEUR, NULL);
		break;

	case etf_minimized:
		nouveau_style |= WS_OVERLAPPEDWINDOW;
		newShowCmd = SW_SHOWMINIMIZED;
		if (restore_borders())
			change = true;
	}

	if (nouveau_style != style_courant) {
		SetWindowLongPtr(hMainWnd, GWL_STYLE, nouveau_style);
		change = true;
	}

	if (((GetWindowLongPtr(hMainWnd, GWL_EXSTYLE) & WS_EX_TOPMOST) != 0) != on_top) {
		wPosFlags &= ~SWP_NOZORDER;
		hWndInsertAfter = on_top ? HWND_TOPMOST : HWND_NOTOPMOST;
		change = true;
	}

	if (newShowCmd != wpl.showCmd) {
		wpl.showCmd = newShowCmd;
		SetWindowPlacement(hMainWnd, &wpl);
		//ShowWindow(hMainWnd, newShowCmd);	 // provoque une animation de fentre gnante
		update_menus(hMainWnd, 1, update_fullscreen_menu);
	}

	if (change) {
		SetWindowPos(hMainWnd, hWndInsertAfter,
			rect.left, rect.top, rect.right-rect.left, rect.bottom-rect.top, wPosFlags);
	}

	// Met  jour le menu, car "Chanes" et "Pistes" ne sont pas mis  jours sans les bordures
	if (nouvel_etat == etf_normal || nouvel_etat == etf_maximized)
		update_all_menus(hMainWnd);

	change_etat_curseur(nouveau_curseur);
	configState = etat_fenetre(hMainWnd);
}

/// Basculement de l'affichage des bordures de la fentre
static void switch_borders()
{
	switch (etat_fenetre(hMainWnd)) {

	case etf_normal:
		switch_etat_fenetre(etf_noborders);
		break;

	case etf_noborders:
		switch_etat_fenetre(etf_normal);
		break;

	case etf_maximized:
		switch_etat_fenetre(etf_fullscreen);
/*		break;

	case etf_fullscreen:
		switch_etat_fenetre(etf_maximized); */
	}

	// Le statut "plein cran" peut avoir chang au passage :
	update_menus(hMainWnd, 1, update_fullscreen_menu);
}

void SetMute(bool bMute)
{
	muted = bMute;
#if USE_LOGITECH_LCD
	lcd.Mute(muted);
#endif // #if USE_LOGITECH_LCD

	set_volume(muted ? 0 : volumeCourant);

	if (pOSD)
		pOSD->DisplayMute(muted);
	update_menus(hMainWnd, 1, update_mute_menu);
}

static void update_volume(bool augmente)
{
	volumeCourant += augmente ? 5 : -5;
	volumeCourant = max(min(volumeCourant, 100), 0);
	if (!muted)
		set_volume(volumeCourant);

#if USE_LOGITECH_LCD
	lcd.SetVolume(volumeCourant);
	lcd.ActualiseLcd();
#endif // #if USE_LOGITECH_LCD
	if (pOSD)
		pOSD->DisplayVolume(volumeCourant);
}

/**
 * Changement de chane par rotation
 * \param[in]	pas				Sens de la progression
 * \param[in]	bInMultiplex	Si \p true, on reste dans le mme multiplex
 **/
static void chaine_avance(int pas, bool bInMultiplex=false) // pas = 1 ou -1
{
	if (ixChaine_ok(ixChaineAvance)) {
		int		nbChaines		= (int)Canaux.size();
		INT32	freq_courante	= Canaux[ixChaineAvance].khz;

		// Si enregistrement en cours, on reste dans le mme multiplex
		if (enregistrements_actuels.recording()!=0)
			bInMultiplex = true;

		for (int ixTmp=ixChaineAvance;;) {
			ixChaineAvance = (ixChaineAvance+nbChaines+pas)%nbChaines;
			if (ixTmp==ixChaineAvance)
				break;	// Protection contre boucle infinie

			const Chaine & canal = Canaux[ixChaineAvance];

			if (canal.etat != ec_preferee)
				continue;	// On ne retient que les chanes "prfres"
			if (bInMultiplex && canal.khz != freq_courante) {
				// Si enregistrement, la chane doit avoir la mme frquence
				continue;
			}

			numeroChaineSaisi = canal.nNumeroChaine;

			SetTimer(hMainWnd, TIMER_NUMBER,
				pOSD || hMainStatus ? duree_zapping : min(duree_zapping, DUREE_MAX_ZAPNOAFF), NULL);

			if (pOSD)
				pOSD->DisplayChannelInfo(canal);
			set_chaine_status(canal);
			break;
		}
	}
}

/**
 * Changement de piste son par rotation
 * \param[in]	pas	Sens de la progression
 **/
static void son_avance(int pas)
{
	if (ixChaine_ok(ixChaineCourante)) {
		INT	son = ixSonCourant+pas;
		INT	max = Canaux[ixChaineCourante].nbr;

		if (son < 0)
			son = max-1;
		else if (son >= max)
			son = 0;
		switch_audio(son);
	}
}

/// Appliquer un changement de chane par saisie
static void chaine_saisie()
{
	// trouver l'index de la chane
	int ixChaine = Canaux.trouve_par_no(numeroChaineSaisi);

	numeroChaineSaisi = 0;

	// La nouvelle chane doit videmment tre valide :
	if (!ixChaine_ok(ixChaine))
		return;

	// ... et ne pas tre identique  la chane courante :
	if (ixChaine == ixChaineCourante)
		return;

	// Si enregistrement en cours ...
	if (enregistrements_actuels.recording() !=0) {
		// ... ne changer de chane que si on ne change pas ni multiplex, ni de type vido
		if (ixChaine_ok(ixChaineCourante) && !Canaux[ixChaine].sameMultiplexAndVideoType(Canaux[ixChaineCourante]))
			return;
	}

	zappe_index(ixChaine);
}

static void change_zoom(bool bInc)
{
	#define ZOOM_SWITCH_TIME	100	// Dlai entre deux zooms
	#define ZOOM_STEP			2

	// Dterminer le temps depuis le dernier changement
	static DWORD	dwLastZoomUpdate = 0;
	DWORD			dwTicks = GetTickCount();
	UINT			nDiff	= dwTicks-dwLastZoomUpdate;

	if (nDiff < ZOOM_SWITCH_TIME)
		return;	// Ignorer si zoom prcdent trop rcent
	dwLastZoomUpdate = dwTicks;

	if (bInc) {
		zoom_ratio += (ZOOM_STEP - (zoom_ratio % ZOOM_STEP));

		if (zoom_ratio > 200)
			zoom_ratio = 200;
	} else {
		zoom_ratio -= (ZOOM_STEP + (zoom_ratio % ZOOM_STEP));
		if (zoom_ratio <= 0)
			zoom_ratio = 1;
	}
	update_menus(hMainWnd, 1, update_aspect_menus);
	if (pOSD)
		pOSD->DisplayZoom(zoom_ratio);
	update_coords();
}

/**
 * Dplace la fentre de PTVM (si celle-ci n'est pas en plein cran) quand on maintient enfonc le bouton de la souris
 *
 * \param[in]  wParam  bouton de la souris enfonc
 * \param[in]  lParam  WORD de poids faible correspond aux coordonnes X, et le WORD de poids fort correcpond  Y
 **/
static void mouse_move(WPARAM wParam, LPARAM lParam)
{
	static bool last_valid = false;
	static int last_x, last_y;

	if ((wParam & MK_LBUTTON)!=0 ) {
		RECT rect;
		GetWindowRect(hMainWnd, &rect);

		int x_screen = LOWORD(lParam) + rect.left;
		int y_screen = HIWORD(lParam) + rect.top;

		if (last_valid) {
			int dx = x_screen - last_x;
			int dy = y_screen - last_y;

			MoveWindow(hMainWnd, rect.left+dx, rect.top+dy,
				rect.right-rect.left, rect.bottom-rect.top, TRUE);
		}

		last_x = x_screen;
		last_y = y_screen;
		last_valid = true;
	} else {
		last_valid = false;
	}
}

#ifdef _DIAGNOSTIC
static void display_graph_state()
{
	if (!pGraph)
		return;
	myprintf(TEXT("Demande de l'tat du graphe") EOL);

	MessageBox(
		hMainWnd,
		pGraph->get_graph_state().c_str(),
		TEXT("tat du graphe"),
		MB_ICONINFORMATION);
}
#endif

static void do_screenshot()
{
	CComHeapPtr<BYTE> pDib;

	if (pVMR && ixChaine_ok(ixChaineCourante)) {
		HRESULT hr = pVMR->GetCurrentImage(pDib);

		if (SUCCEEDED(hr) && pDib) {
			const Chaine &			sChan = Canaux[ixChaineCourante];
			NomFichierAvecMotif		nom_fichier(screenshots_dir, NULL, sChan.nom, sChan.nNumeroChaine, TEXT("bmp"));
			HANDLE					file = nom_fichier.Create();

			if (file != INVALID_HANDLE_VALUE) {
				BITMAPINFOHEADER *	pBMIH = (BITMAPINFOHEADER*)(BYTE *)pDib;
				DWORD				bytes;
				DWORD				imageSize = pBMIH->biSize+pBMIH->biSizeImage;
				BITMAPFILEHEADER	bfh={'MB', sizeof(BITMAPFILEHEADER)+imageSize, 0, 0, sizeof(BITMAPFILEHEADER)+pBMIH->biSize};

				WriteFile(file, &bfh, sizeof(BITMAPFILEHEADER), &bytes, NULL);
				WriteFile(file, pDib, imageSize, &bytes, NULL);

				CloseHandle(file);
			}
		}
	}
}

// ====================================================================================
// Mise en route et arrt du graphe, ainsi que des lments d'interface associs
// ====================================================================================

/**
 * Vrification de la configuration, avec, en cas d'chec (de configuration non valide),
 * affichage du dialogue de configuration et nouvelle vrification en boucle.
 * \param[in]	bForce	si \p true, l'ouverture du dialogue de configuration est force au premier passage
 * \return		\p true si la configuration est correcte
 **/
static bool CheckAndAdjustConfig(bool bForce)
{
	bool config_ok = check_config();

	while (!config_ok || bForce) {
		bForce = false;
		if (drivers_dialog()!=IDOK) {
			affiche_erreurs(TEXT("  .") EOL EOL
				TEXT("     ."));
			return false;
		}
		config_ok = check_config();
		if (!config_ok)
			affiche_erreurs(TEXT(" .") EOL EOL
				TEXT("   ."));
	}
	return config_ok;
}

static void StopGraph()
{
	if (!pGraph)	// Ne pas excuter si dj arrt
		return;

	myprintf(BOLD(TEXT("############### (Arrt et destruction du graphe) ###############")) EOL);

	// Je mets ceci (temporairement ... ??)  cause d'un problme que je rencontre parfois,
	// o on voit PouchinTV quitter tout en restant comme "zombie",
	// mais consommant toutes les ressources, dans le systme.
	// Au moins ainsi a devrait viter de bloquer l'ordinateur
	// si le programme tournait en priorit leve.
	//  virer quand ce problme aura t rsolu, donc. - Gingko
	set_priority(IDLE_PRIORITY_CLASS);

	// En cas de redmarrage, laisser dans un tat connu
	suspend();
	debranche_medias();	// Apparemment redondant avec le prcdent, mais en ralit pas tout  fait

	clean_dshow();

	niData.disable();

	// Si on a demand la mise  jour de MSN, on retire le titre modifi
	cMsnData.reset();

	// Mmoriser tat de la fentre au cas o on redmarrerait le graphe ensuite
	configState = etat_fenetre(hMainWnd);
}

/** \brief Construction et dmarrage du graphe
 *
 * Construit et dmarre le graphe en grant tous les lments d'interface utilisateur qui
 * s'associent  ces fonctionnalits.
 * \param[in]	bCheck			Vrifier la configuration en entre, et afficher le dialogue en cas d'chec.
 * \param[in]	bForceConfig	Force l'affichage du dialogue de configuration en entre
 * \return		\p true si succs, \p false si chec
 **/
static bool StartGraph(bool bCheck, bool bForceConfig)
{
	if (pGraph)	// Ne pas excuter si dj en fonctionnement
		return true;

	static LONG bInside = FALSE;

	if (InterlockedExchange(&bInside, TRUE))
		return true;

	// Sous Vista, on n'utilise l'OSD qu'avec le VMR9
	if (eWinVers>=wv_Vista && (vmr_mode==rdt_VMR7_renderless || vmr_mode==rdt_VMR7_windowless))
		use_osd = false;

	myprintf(BOLD(TEXT("############ (Construction et dmarrage du graphe) #############")) EOL);

	// Si on minimise dans la zone systme, n'affiche pas la fentre dans la barre des tches
	update_systray_state();

	update_status_size();
	update_status_bar();

	// Rtablissement de la priorit de traitement sauve dans la configuration :
	set_priority(configPriority);

	HRESULT hr			= S_FALSE;
	LPCTSTR	pszErrTtl	= TEXT("");	// Titre de la fentre d'erreur

	while (hr != S_OK) {
		if (FAILED(hr)) {	// FAILED(hr) est false si 'hr' est positif (ex : hr = S_FALSE)
			if (!bCheck) {
				StopGraph();
				bInside = FALSE;
				return false;
			}

			// Traitement des erreurs avec retour au dpart si erreur au dbut du traitement
			LPCTSTR pszMsg = TEXT("Veuillez corriger la configuration.") EOL;	// message par dfaut

			// Messages d'erreurs spcifiques :
			if (hr == HRESULT_FROM_WIN32(ERROR_GEN_FAILURE))
				pszMsg =	TEXT("     ,")
							TEXT("  -,") EOL
							TEXT(" ,     ")
							TEXT(" .") EOL;
			else if (hr == HRESULT(-2)) {
				pszMsg =	TEXT("    .    ")
							TEXT("    .") EOL;
				hr = S_OK;	// Ne pas rapporter de code d'erreur
			}

			clean_dshow();
			affiche_erreurs(pszMsg, hr, pszErrTtl);
			bForceConfig = true;

			// Appeler une nouvelle fois, car certains filtres trouvent trs malin d'appeler
			// CoUninitialize pour se protger des usages non autoriss :
			CoInitializeEx(NULL, COINIT_MULTITHREADED);
		}

		if (bCheck) {
			if (!CheckAndAdjustConfig(bForceConfig)) {
				bInside = FALSE;
				return false;
			}
			bForceConfig = false;	// Ne plus forcer la configuration  l'essai suivant
		}

		pszErrTtl = TEXT("  .");

		// ###### Ici est **LE** endroit o on construit le graphe !!
		hr = build_graph();

		if (FAILED(hr))
			continue;	// chec  la construction

		if (Canaux.size() == 0) {
			// Aucune chane n'existe : lancer la recherche initiale

			pszErrTtl = TEXT("    ");

			// Dmarrer d'abord le graphe (qui ne l'aurait t autrement que plus tard)
			if (pGraph) {
				hr = pGraph->graph_Run();
				check_hr(hr, "graph_Run");
				if (FAILED(hr))
					continue;
			}

			pszErrTtl = TEXT("    ");

			if (scan_dialog(hMainWnd) != IDOK || Canaux.size() == 0) {
				hr = HRESULT(-2);
				continue;			// La recherche n'a rien donn
			}
		}

		// Mise  jour du menu des filtres (pour dsactiver les items correspondant
		//  des filtres inexistants ou n'ayant pas de pages de proprits) :
		update_menus(hMainWnd, 2, update_filtres_menus, update_aspect_menus);

		// Slectionner la chane
		zappe_index(ixChaineAvance);

		pszErrTtl = TEXT("    ");

		if (!rebranche(true)) {
			hr = E_FAIL;
			continue;
		}
		if (configState == etf_minimized && suspend_minimized) {
			// On a lanc avec la fentre minimise ou le mode suspension si minimis dsactiv
			// (note : on dbranche aprs avoir rebranch plutt que de seulement s'abstenir de
			// rebrancher, car on doit tre sr que le graphe peut tre construit compltement)
			suspend();
		}

		pszErrTtl = TEXT("    ");

		// Lancement du graphe
		if (pGraph) {
			hr = pGraph->graph_Run();
			check_hr(hr, "graph_Run");
		}
	}

	// Utile car il semble que certains accs au rendu vido risquent de produire un "deadlock" si
	// effectus trop rapidement aprs le dmarrage du graphe (vrifi seulement en VMR9 avec debugger
	// prsent, mais autant mettre toutes les chances de notre ct).
	Sleep(300);

	if (pVMR)	// Restauration des rglages de la vido, s'il y a lieu
		hr = pVMR->InitProcAmp(video_sav);

	// Rgler le volume
	set_volume(volumeCourant);

	// On met  jour les raccourcis dfinis par l'utilisateur
	update_shortcuts_menus();

	/*
	 * Rtablit la fentre, en enlevant les bordures si Pouchin TV Mod a t
	 * arrt en pleine taille (sans barre d'tat, ni menu), ainsi qu'en le
	 * commutant en plein cran, le cas chant.
	 */
	switch_etat_fenetre(configState);

	niData.update_state();

	// Dfinir le timer avant l'enregistrement suivant,
	// ventuellement dbutant immdiatement si intervalle en cours :
	set_timer_record();

	if (muted)
		SetMute(true);
	bInside = FALSE;
	return true;
}

static void Suspend(bool bHibernate)
{
	myprintf(TEXT("Entre dans SetSuspendState (hibernate = %u)") EOL, bHibernate);
	if (!SetSuspendState(bHibernate ? TRUE : FALSE, FALSE, FALSE))
		myprintf(ERRX(TEXT("chec SetSuspendState : %s")), SystemErrorToString().c_str());
	else {
		myprintf(TEXT("Sortie de SetSuspendState (hibernate = %u) ; dbut dlai de %u secondes") EOL,
			bHibernate, DUREE_WAKEUP / DUREE_1SEC);
		nWakeupRetryCount = WAKEUP_RETRIES;
		SetTimer(hMainWnd, TIMER_WAKEUP, DUREE_WAKEUP, NULL);
	}
}

// ====================================================================================
// Excution des commandes de la fentre principale
// ====================================================================================

/// Traitement des commandes de la fentre principale
static void do_command(WORD wID, LPARAM lParam)
{
	int ixRecDescr;

	switch (wID) {

	case IDM_QUIT:
		// Note : lParam contient un paramtre spcifique :
		//	- Si HIWORD(lParam) != 0, pas de contrle avant de quitter (dj contrl avant, normalement)
		//	- LOWORD(lParam) indique comment on quitte (juste quitter, teindre l'ordinateur ou relancer l'application)
		// Si command par menu, lParam = NULL, ce qui donne LOWORD(lParam) = em_Quit,
		// et HIWORD(lParam) = 0, donc confirmation requise
		if (HIWORD(lParam) == 0) {
			// Si on enregistre, affichage de la fentre de confirmation
			if (!CConfirmCloseDialog::Check(hMainWnd))
				break;	// On enregistre et on a appuy sur Annuler, donc, on ne quitte pas
		}
		eExitMode = static_cast<ExitMode>(LOWORD(lParam));
		DestroyWindow(hMainWnd);
		break;

	case IDM_CONFIG: {
		bool bNeedRestart = do_config(hMainWnd);

		// Mise  jour du message sous MSN au cas o la configuration correspondante aurait chang :
		if (use_msn)
			cMsnData.set();
		else
			cMsnData.reset();

		// Mise  jour de l'item de lancement de la base de registres pour le dmarrage avec Windows
		update_start_with_windows();

		// Mise  jour de la commande de fermeture
		update_record_protect(temps_prochain_enregistrement());

		// Mise  jour du titre de la fentre au cas o celui-ci aurait chang :
		SetWindowText(hMainWnd, titre_fenetre);

		update_record_menus(hMainWnd);	// Mise  jour des menus "Chanes" et "Programme"
		// Si les changements dans la configuration requirent que l'application soit relance,
		// on quitte (la relance sera automatique)
		if (bNeedRestart) {
			if (enregistrements_actuels.recording()!=0) {
				MessageBox(hMainWnd,
					TEXT("     :") EOL
					TEXT("    ") EOL
					TEXT("       ."),
					TEXT("  "), MB_ICONINFORMATION);
				eExitMode = em_Quit;	// Rtablissement de la valeur par dfaut
			} else
				// Redmarrage (partiel) de l'application
				PostMessage(hMainWnd, WM_COMMAND, IDM_APP_RESTART, NULL);
		}
		break; }

	case IDM_HELPDOC: {
		tstring strHelp;

		FindAppFile(TEXT(""), szChmFileName, strHelp, 0);
		HtmlHelp(hMainWnd, strHelp.c_str(), HH_DISPLAY_TOC, 0);
		// new CShellExThread((tstring(szAppName)+TEXT(".chm")).c_str());
		break; }

	case IDM_ABOUT:
		CAboutDialog().Do(hMainWnd);
		break;

	case IDM_DELAYED_RECORD:
		CRecordDialog::ShowDialog(hMainWnd);
		break;

	case IDM_MP2V:		create_property_dialog(ft_Video_Mp2v);		break;
	case IDM_H264:		create_property_dialog(ft_Video_H264);		break;
	case IDM_MP2A:		create_property_dialog(ft_Audio_Mp2a);		break;
	case IDM_AC3:		create_property_dialog(ft_Audio_Ac3);		break;
	case IDM_EAC3:		create_property_dialog(ft_Audio_Eac3);		break;
	case IDM_DSOUND:	create_property_dialog(ft_AudioRenderer);	break;
	case IDM_VMR:		create_property_dialog(ft_VideoRenderer);	break;

	case IDM_ZOOM:
		if (pVMR)
			change_zoom(true);
		break;

	case IDM_DEZOOM:
		if (pVMR)
			change_zoom(false);
		break;

	case IDM_DEFAULT_ZOOM:
		// Note : la version prcdente annulait aussi le mode "utiliser toute la largeur".
		// avec la multiplication des modes, il conviendrait sans doute de rflchir  qu'est-ce
		// que cette commande devrait rinitialiser d'autre que le seul zoom.
		if (! pVMR)
			break;
		zoom_ratio = 100;
		update_menus(hMainWnd, 1, update_aspect_menus);
		update_coords();
		if (pOSD)
			pOSD->DisplayZoom(zoom_ratio);
		break;

	case IDM_ALLWDT:
		video_pan_mode = video_pan_mode == pm_AllWidth ? pm_Normal : pm_AllWidth;
		update_menus(hMainWnd, 1, update_aspect_menus);
		update_coords();
		break;

	case IDM_ALLHGT:
		video_pan_mode = video_pan_mode == pm_AllHeight ? pm_Normal : pm_AllHeight;
		update_menus(hMainWnd, 1, update_aspect_menus);
		update_coords();
		break;


	case IDM_AR_AUTO:
	case IDM_AR_1X1:
	case IDM_AR_5X4:
	case IDM_AR_4X3:
	case IDM_AR_16X10:
	case IDM_AR_16X9:
		// Supprimer le mode "tirer" s'il tait slectionn
		if (video_pan_mode == pm_Stretch)
			video_pan_mode = pm_Normal;
		aspect_ratio = static_cast<ArModes>(wID-IDM_AR_BASE);
		update_menus(hMainWnd, 1, update_aspect_menus);
		update_coords();
		break;

	case IDM_ETIRER:
		video_pan_mode = video_pan_mode == pm_Stretch ? pm_Normal : pm_Stretch;
		aspect_ratio = ar_Auto;	// Retour au ratio d'aspect automatique
		update_menus(hMainWnd, 1, update_aspect_menus);
		update_coords();
		break;

	case IDM_SCREENSHOT:
		do_screenshot();
		break;

	case IDM_RECORD_CHANNEL_TS:
		// On regarde si la chane n'est pas dj en cours d'enregistrement
		if (enregistrements_actuels.getRecordingDescr(ixChaineCourante) != -1)
			// Chane dj en cours d'enregistrement
			break;
		if (enregistrements_actuels.start_record_ts(ixChaineCourante)>=0)
			update_record_menus(hMainWnd);
		break;

	case IDM_RECORD_CHANNEL_PS:
		// On regarde si la chane n'est pas dj en cours d'enregistrement
		if (enregistrements_actuels.getRecordingDescr(ixChaineCourante) != -1)
			// Chane dj en cours d'enregistrement
			break;
		if (enregistrements_actuels.start_record_ps(ixChaineCourante, ixSonCourant)>=0)
			update_record_menus(hMainWnd);
		break;

	case IDM_RECORD_OTHER:
		if (CRecOtherDialog().Do(hMainWnd) == IDOK)
			update_record_menus(hMainWnd);
		break;

#ifdef _DIAGNOSTIC
	case IDM_RECORD_UNDEFSTREAM:
		if (CGetFreqDialog().Do(hMainWnd) == IDOK) {
			if (enregistrements_actuels.start_record_undefstream(mpx_frequence_diag) >= 0)
				update_record_menus(hMainWnd);
		}
		break;
#endif

	case IDM_STOP_RECORD:
		ixRecDescr = enregistrements_actuels.getRecordingDescr(ixChaineCourante);
		if (ixRecDescr>=0) {
			enregistrements_actuels[ixRecDescr].stop(lParam==NULL ? epr_interrompu : EtatProgramme(lParam));
			update_record_menus(hMainWnd);
			finalize_prog_change();
		}
		break;

	case IDM_DELAYED_STOP:
		if (enregistrements_actuels.recording()!=0)
			delayed_stop_dialog();
		else {
			// (si on arrive ici, c'est qu'une mise  jour des menus
			// a manqu quelque part et qu'il y a donc un bug)
			update_record_menus(hMainWnd);
		}
		break;

	case IDM_RECORD_INFO:
		DisplayRecordInfo(hMainWnd);
		break;

	case IDM_BORDERS:
		switch_borders();
		break;

	case IDM_VIDEOCONTROL:
		CVideoControlDialog().Do(hMainWnd);
		break;

	case IDM_FULLSCREEN: {
		static EtatsFenetre sauve_etat = etf_normal;
		EtatsFenetre etat_courant = etat_fenetre(hMainWnd);

		if (etat_courant==etf_fullscreen)
			switch_etat_fenetre(sauve_etat);
		else {
			sauve_etat = etat_courant;
			switch_etat_fenetre(etf_fullscreen);
		}
		break; }

	case IDM_MUTE:
		SetMute(!muted);
		if (!use_osd)
			update_status_bar();
		break;

	case IDM_EPG:
		CEpgDialog::ShowDialog(hMainWnd);
		break;

	case IDM_SIGNAL:
		CSignalDialog::ShowDialog(hMainWnd);
		break;

	case IDM_ALWAYS_ON_TOP:
		switch_on_top();
		break;

	case IDM_VOL_AUG:
		update_volume(true);
		if (!use_osd)
			update_status_bar();
		break;

	case IDM_VOL_DIM:
		update_volume(false);
		if (!use_osd)
			update_status_bar();
		break;

	case IDM_REALTIME_PRIORITY:
		change_priority(REALTIME_PRIORITY_CLASS);
		break;

	case IDM_HIGH_PRIORITY:
		change_priority(HIGH_PRIORITY_CLASS);
		break;

	case IDM_ABOVE_NORMAL_PRIORITY:
		change_priority(ABOVE_NORMAL_PRIORITY_CLASS);
		break;

	case IDM_NORMAL_PRIORITY:
		change_priority(NORMAL_PRIORITY_CLASS);
		break;

	case IDM_BELOW_NORMAL_PRIORITY:
		change_priority(BELOW_NORMAL_PRIORITY_CLASS);
		break;

	case IDM_IDLE_PRIORITY:
		change_priority(IDLE_PRIORITY_CLASS);
		break;

	case IDM_SHOW_PROGRAMME:
		// Affichage du nom de la chane et du programme en cours
		// (note : cette commande peut aussi tre appele via une fonction
		// "PostMessage", aprs changement de chane)
		if (pOSD && ixChaine_ok(ixChaineCourante)) {
			// myprint(COLR(LGREEN, TEXT("IDM_SHOW_PROGRAMME")) EOL);
			const Chaine &	canal	= Canaux[ixChaineCourante];
			bool			bZap	= lParam==TIMER_ZAPPING;	// true si on vient juste de zapper
			bool			bNameOk	= canal.getCurrentProgram(bZap).hasName();

			// Si le nom du programme n'est pas encore connu, on demande l'affichage
			// une seconde fois  l'expiration du dlai OSD.
			if (!bNameOk && bZap) {
				// myprint(COLR(LGREEN, TEXT("SetTimer, TIMER_ZAPPING2")) EOL);
				SetTimer(hMainWnd, MAKEWPARAM(TIMER_ZAPPING, 1), DUREE_OSD, NULL);
			}

			// Affichage du nom du programme (mais on n'affiche pas deux fois un
			// programme avec nom inconnu)
			if (bNameOk || lParam != MAKELPARAM(TIMER_ZAPPING, 1)) {
				// myprint(COLR(LGREEN, TEXT("Affichage des informations de la chane.")) EOL);
				pOSD->DisplayChannelInfo(canal);
			}
		}
		update_filtres_menus(hMainMenu);	// Mise  jour du menu des filtres (au cas o un
											// codec aurait t insr qui ne l'tait pas avant)
		break;

	case IDM_CLOSE:
	case IDM_MINIMIZE:
		SendMessage(hMainWnd, WM_SYSCOMMAND, SC_MINIMIZE, NULL);
		break;

	case IDM_RESTORE:
		SendMessage(hMainWnd, WM_SYSCOMMAND, SC_RESTORE, NULL);
		break;

	case IDM_CHAN_DIM:
		chaine_avance(-1);
		break;

	case IDM_CHAN_DIM_MPX:
		chaine_avance(-1, true);
		break;

	case IDM_CHAN_AUG:
		chaine_avance(1);
		break;

	case IDM_CHAN_AUG_MPX:
		chaine_avance(1, true);
		break;

	case IDM_LAST_CHAN:
		if (ixChaine_ok(ixDerniereChaine) && ixDerniereChaine != ixChaineCourante)
			zappe_index(ixDerniereChaine);
		break;

	case IDM_SON_AUG:
		son_avance(1);
		break;

	case IDM_SON_DIM:
		son_avance(-1);
		break;

	case IDM_UPDATE:
		do_web_update(hMainWnd);
		break;

#ifdef _DIAGNOSTIC
	case IDM_GRAPH_STATE:
		display_graph_state();
		break;

	case IDM_RUN_GRAPH:
		if (pGraph) {
			if (pOSD)
				pOSD->DisplayBotRightMsg(TEXT("Dmarrage graphe"));
			myprintf(TEXT("Mise en marche du graphe") EOL);
			pGraph->graph_Run();
		}
		break;

	case IDM_PAUSE_GRAPH:
		if (pGraph) {
			if (pOSD)
				pOSD->DisplayBotRightMsg(TEXT("Pause graphe"));
			myprintf(TEXT("Mise en pause du graphe") EOL);
			pGraph->graph_Pause();
		}
		break;

	case IDM_STOP_GRAPH:
		if (pGraph) {
			if (pOSD)
				pOSD->DisplayBotRightMsg(TEXT("Arrt graphe"));
			myprintf(TEXT("Arrt du graphe") EOL);
			pGraph->graph_Stop();
		}
		break;

	case IDM_BRANCHE:
		if (pGraph) {
			if (pOSD)
				pOSD->DisplayBotRightMsg(TEXT("Branchement"));
			rebranche(false);
		}
		break;

	case IDM_DEBRANCHE:
		if (pGraph) {
			if (pOSD)
				pOSD->DisplayBotRightMsg(TEXT("Dbranchement"));
			suspend();
		}
		break;
#endif	// #ifdef _DIAGNOSTIC

	case IDM_APP_RESTART: {
		int nRec = enregistrements_actuels.count_recording();

		if (nRec != 0) {
			MessageBox(hMainWnd, ReasonAsStr(nRec, false).c_str(), TEXT("Redmarrage du graphe impossible"), MB_ICONSTOP|MB_OK);
			break;	// On enregistre, on ne redmarre pas le graphe.
		}
		save_config();

		// Annuler des tentatives de redmarrage temporises qui pourraient tre en cours
		KillTimer(hMainWnd, TIMER_WAKEUP);

		StopGraph();
		if (!StartGraph(true, false))	// Avec dialogue de configuration si chec
			PostQuitMessage(0);
		break; }

	case IDM_CHAN_TUNE:
		// Sleep(300);
		myprintf(BOLD(TEXT("Resyntonisation force")) EOL);
		// Reprogrammer le timer de zapping s'il y en avait dj un
		if (KillTimer(hMainWnd, TIMER_ZAPPING) || KillTimer(hMainWnd, MAKELONG(TIMER_ZAPPING, 1)))
			SetTimer(hMainWnd, TIMER_ZAPPING, DUREE_SHOW_PROGRAMME, NULL);
		change_chaine(ixChaineCourante, true);
		break;

	default:
		// Items de menu crs dynamiquement

		if (wID>=IDM_CHAINES_BASE && wID<IDM_CHAINES_BASE+NB_MAX_CHAINES) {
			// cas changement chane
			zappe_index(wID-IDM_CHAINES_BASE);
		} else if (wID>=IDM_PISTES_BASE && wID<IDM_PISTES_BASE+NB_MAX_PISTES) {
			// cas changement piste
			switch_audio(wID-IDM_PISTES_BASE);
			update_menus(hMainWnd, 1, update_pistes_menu);
		} else if (wID>=IDM_STOP_RECORD_BASE && wID<IDM_STOP_RECORD_BASE+NB_MAX_ENREG) {
			EtatProgramme		eReason	= lParam==NULL ? epr_interrompu : EtatProgramme(lParam);
			Enregistrement &	sRec	= enregistrements_actuels[wID-IDM_STOP_RECORD_BASE];
			LPCSTR				pszChan	= NULL;

			if (eReason == epr_fileerr)
				pszChan = sRec.channelName();	// Le nom de la chane est  rcuprer **avant**

			sRec.stop(eReason);
			update_record_menus(hMainWnd);

			if (eReason == epr_fileerr) {
				MessageBox(hMainWnd,
					tstr_printf(
							TEXT("   %") A2t TEXT("     ") EOL
							TEXT("(   %u = %s)"),
							pszChan, sRec.nErrCode,

							SystemErrorToString(sRec.nErrCode).c_str()
						).c_str(),
					TEXT("  "), MB_ICONERROR);
			}
#ifdef _DIAGNOSTIC
			// Reslectionner une chane aprs fin d'enregistrement de multiplex inconnu
			if (!ixChaine_ok(ixChaineCourante))
				PostMessage(hMainWnd, WM_COMMAND, IDM_CHAINES_BASE, NULL);
#endif
			finalize_prog_change();
		}
	}
}

/// Gestion des vnements des diffrents timers
static void do_timer(DWORD wTimerID)
{
	KillTimer(hMainWnd, wTimerID);	// On n'a aucun timer dont on souhaite la rptition

	switch (LOWORD(wTimerID)) {		// Juste LOWORD, rservant ainsi d'utiliser HIWORD comme "paramtre"

	case TIMER_VIDEO_MOVE:
		// Gestion du timer d'actualisation de la position de la vido :
		update_coords();
		break;

	case TIMER_SLEEP:
		// Mise en veille (normale ou prolonge selon HIWORD(wTimerID)) de l'ordinateur
		StopGraph();
		Suspend(HIWORD(wTimerID) != 0);
		break;

	case TIMER_SHUTDOWN:
		PostMessage(hMainWnd, WM_COMMAND, IDM_QUIT,
			MAKELPARAM(HIWORD(wTimerID) != 0 ? em_ShutdownForced : em_Shutdown, REC_NOCONFIRM));
		break;

	case TIMER_QUIT:
		// Gestion du timer lanc  la fin de certains enregistrements, lorsque l'on
		// souhaite quitter l'application, avec ou sans extinction de l'ordinateur.
		PostMessage(hMainWnd, WM_COMMAND, IDM_QUIT, MAKEWPARAM(em_Quit, REC_NOCONFIRM));
		break;

	case TIMER_NUMBER:
		// Gestion du timer permettant de changer de chane en tapant le numro de la nouvelle
		// chane au clavier. Le timer est ncessaire pour permettre de taper un numro de chane
		//  plusieurs chiffres (tous les chiffres sont considrs saisis  l'issue du dlai
		// du timer).
		chaine_saisie();
		break;

	case TIMER_BORDER:
		// Gestion du timer qui permet, lors d'un clic sur l'cran, de distinguer entre le passage en plein
		// cran et l'activation / dsactivation des bordures de fentre.
		switch_borders();
		break;

	case TIMER_UPDATE_OSD:
		// myprint(COLR(LGREEN, TEXT("WM_TIMER TIMER_UPDATE_OSD")) EOL);
		if (pOSD)
			pOSD->UpdateRectsAndFont();
		else
			// Pas d'OSD : il faut quand mme effacer le contour noir (concerne surtout VMR7)
			InvalidateRect(hMainWnd, NULL, TRUE);
		break;

	case TIMER_OSD:
		// Gestion du timer supprimant l'affichage OSD aprs expiration du dlai.
		if (pOSD)
			pOSD->ProcessTimeEvent();
		break;

		update_coords();	// Recalcul requis si on s'est dplac sur un autre moniteur
		switch_etat_fenetre(configState);	// -> ajustement car l'tat peut ne pas s'tablir correctement
											//    si l'utilisateur modifie la fentre pendant le dmarrage
	case TIMER_ENABLE_OSD:
		if (pOSD)
			pOSD->Enable();	// Ractivation de scurit de l'OSD
		break;

	case TIMER_ZAPPING:
		// Gestion du timer qui affiche le nom et la programmation de la chane aprs un
		// certain dlai, une fois qu'on a chang de chane

		// Demander l'affichage effectif du nom
		if (ixChaineCourante == ixChaineAvance) {	// Ne pas afficher si on a dj slectionn une nouvelle chane
			// myprint(COLR(LGREEN, TEXT("PostMessage, WM_COMMAND, IDM_SHOW_PROGRAMME")) EOL);
			PostMessage(hMainWnd, WM_COMMAND, IDM_SHOW_PROGRAMME, wTimerID);
		}

		// Recalcul des coordonnes d'affichage pour tre srs d'avoir un positionnement correct
		update_coords();
		break;

	case TIMER_CURSOR:
		// Gestion du timer qui supprime le curseur en mode plein cran, s'il n'a pas t dplac
		// depuis le dlai programm.
		if (etat_curseur==etc_fullscreen_on)
			change_etat_curseur(etc_fullscreen_off);
		break;

	case TIMER_DELAYED_RECORD:
		if (pGraph && pGraph->graph_isRunning()) {
			// Gestion du timer des enregistrements :
			if (do_record_events())
				update_record_menus(hMainWnd);

			// Redemande le calcul du prochain enregistrement
			set_timer_record();
		}
		break;

	case TIMER_NOTIFY_UPDATE:
		// Mise  jour du message de l'icne de notification, ainsi que du nom de l'mission sous MSN.
		PostMessage(hMainWnd, WM_APP_EPG_UPDATED, 0, 0);
		break;

	case TIMER_WAKEUP:
		myprintf(TEXT("Attente %u secondes aprs rveil termine") EOL, DUREE_WAKEUP / DUREE_1SEC);
		if (!StartGraph(false, false)) {
			if (nWakeupRetryCount > 0) {
				// Nouvel essai si #nWakeupRetryCount est suprieur  0
				myprintf(TEXT("chec redmarrage du graphe, nouvel essai dans %u secondes, %u essai(s) restant(s)") EOL,
					DUREE_WAKEUP / DUREE_1SEC, nWakeupRetryCount);
				--nWakeupRetryCount;
				CoUninitialize();
				CoInitializeEx(NULL, COINIT_MULTITHREADED);
				SetTimer(hMainWnd, TIMER_WAKEUP, DUREE_WAKEUP, NULL);
			} else {
				affiche_erreurs(TEXT("Rdmarrage du graphe impossible en sortie de veille. L'application va tre quitte."));
				PostQuitMessage(0);
			}
		}
	}
}

// ====================================================================================
// Gestion des messages de la fentre principale
// ====================================================================================

/**
 * main window procedure
 **/
static LRESULT WINAPI MainWndProc (HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	static	POINT	coord_down;
	static	DWORD	nClickTime = 0;
	static	bool	bDoubleClick = false;
	LRESULT			lRet = 0;

	switch (uMsg) {

	case WM_CREATE: {
		myprintf(BOLD(TEXT("Cration fentre principale")) EOL);
		hMainWnd = hWnd;
		break; }

	case WM_PAINT: {
		PAINTSTRUCT	ps;
		HDC			hDC = BeginPaint(hWnd, &ps);

		do_paint(hDC);

		EndPaint(hWnd, &ps);
		break; }

	case WM_DISPLAYCHANGE:
		if (pVMR)
			pVMR->DisplayModeChanged();
		break;

	case WM_SIZE:
		if (hMainStatus != NULL) {
			SendMessage(hMainStatus, WM_SIZE, wParam, lParam);
			update_status_size();
		}
		update_coords();
		if (pVMR)
			pVMR->Resize();
		break;

	case WM_CLOSE:
		myprint(TEXT("WM_CLOSE") EOL);
		// Si on enregistre, affiche la fentre de confirmation
		if (!CConfirmCloseDialog::Check(hWnd)) {
			break;	// On enregistre et on a appuy sur Annuler, donc, on ne quitte pas
		}
		DestroyWindow(hWnd);
		break;

	case WM_DESTROY:
		save_config();
		PostQuitMessage(0);
		break;

	case WM_MENUSELECT:
		// Au cas o une application externe aurait chang la priorit :
		if (get_priority()!=configPriority)
			update_menus(hWnd, 1, update_priority_menu);
		lRet = DefWindowProc(hWnd, uMsg, wParam, lParam);
		break;

	case WM_COMMAND:
		do_command(LOWORD(wParam), lParam);
		break;

	case WM_APPCOMMAND: {
		WORD	wID	= 0;

		if (use_multimedia) {	// vnements standards sous ME/2k/XP/Vista
			int		cmd = GET_APPCOMMAND_LPARAM(lParam);

			switch (cmd) {

			case APPCOMMAND_MEDIA_CHANNEL_DOWN:	wID = IDM_CHAN_DIM;			break;	// Met la chane suivante
			case APPCOMMAND_MEDIA_CHANNEL_UP:	wID = IDM_CHAN_AUG;			break;	// Met la chane prcdente
			case APPCOMMAND_MEDIA_RECORD:		wID = IDM_RECORD_CHANNEL_PS;break;	// Lance l'enregistrement en mode PS (fichier MPG)
			case APPCOMMAND_VOLUME_DOWN:		wID = IDM_VOL_DIM;			break;	// Diminue le volume
			case APPCOMMAND_VOLUME_UP:			wID = IDM_VOL_AUG;			break;	// Augmente le volume
			case APPCOMMAND_VOLUME_MUTE:		wID = IDM_MUTE;				break;	// Coupe le son
			case APPCOMMAND_MEDIA_STOP:			wID = IDM_STOP_RECORD;		break;	// Arrte l'enregistrement
			case APPCOMMAND_MEDIA_NEXTTRACK:	wID = IDM_SON_AUG;			break;	// Piste son suivante
			case APPCOMMAND_MEDIA_PREVIOUSTRACK:wID = IDM_SON_DIM;			break;	// Piste son prcdente
			case APPCOMMAND_BROWSER_FAVORITES:	wID = IDM_EPG;				break;	// Affiche la liste des programmes EPG

			default:							// Non trait, juste pour le connaitre
				myprintf(TEXT("WM_APPCOMMAND non trait: %d / Device: %s / Touches: %d") EOL,
					cmd,
					(GET_DEVICE_LPARAM(lParam) == FAPPCOMMAND_KEY ? TEXT("Clavier") :
							(GET_DEVICE_LPARAM(lParam) == FAPPCOMMAND_MOUSE ? TEXT("Souris") :
									(GET_DEVICE_LPARAM(lParam) == FAPPCOMMAND_OEM ? TEXT("OEM") : TEXT("Inconnu"))
							)
					),
					GET_KEYSTATE_LPARAM(lParam));
			}
		}
		if (wID) {
			PostMessage(hWnd, WM_COMMAND, wID, 0);
			return TRUE;
		}

		return DefWindowProc(hWnd, uMsg, wParam, lParam); }

	case WM_KEYDOWN:
		switch (wParam) {

		case '0':
		case '1':
		case '2':
		case '3':
		case '4':
		case '5':
		case '6':
		case '7':
		case '8':
		case '9':
			traite_numero( (int) (wParam - '0') );
			break;

		case VK_NUMPAD0:
		case VK_NUMPAD1:
		case VK_NUMPAD2:
		case VK_NUMPAD3:
		case VK_NUMPAD4:
		case VK_NUMPAD5:
		case VK_NUMPAD6:
		case VK_NUMPAD7:
		case VK_NUMPAD8:
		case VK_NUMPAD9:
			traite_numero( (int) (wParam - VK_NUMPAD0) );
			break;

		}
		break;

	case WM_MOUSEWHEEL: {
		POINT	sPt = {GET_X_LPARAM(lParam), GET_Y_LPARAM(lParam)};
		RECT	sRect;

		// Prise en compte de la molette de la souris, seulement si le curseur est
		// sur la fentre.
		if (GetWindowRect(hWnd, &sRect) && PtInRect(&sRect, sPt)) {
			short	zDelta	= GET_WHEEL_DELTA_WPARAM(wParam);
			WORD	wID		= 0;
			bool	bIncr	= false;

			if (zDelta >= WHEEL_DELTA)
				bIncr = true;
			else if (zDelta > -WHEEL_DELTA)
				break;

			if (inversion_molette)
				bIncr = ! bIncr;

			if (bIncr) {
				switch (role_molette) {
				case mwu_Channels:		wID = IDM_CHAN_AUG;		break;
				case mwu_Channels_MX:	wID = IDM_CHAN_AUG_MPX;	break;
				case mvu_Soundtrack:	wID = IDM_SON_AUG;		break;
				case mwu_Volume:		wID = IDM_VOL_AUG;		break;
				case mvu_Zoom:			wID = IDM_ZOOM;
				}
			} else {
				switch (role_molette) {
				case mwu_Channels:		wID = IDM_CHAN_DIM;		break;
				case mwu_Channels_MX:	wID = IDM_CHAN_DIM_MPX;	break;
				case mvu_Soundtrack:	wID = IDM_SON_DIM;		break;
				case mwu_Volume:		wID = IDM_VOL_DIM;		break;
				case mvu_Zoom:			wID = IDM_DEZOOM;
				}
			}

			if (wID)
				PostMessage(hWnd, WM_COMMAND, wID, 0);
		}
		break; }

	case WM_TIMER:
		do_timer(DWORD(wParam));
		break;

	case WM_TIMECHANGE:
		// Un changement est intervenu dans l'heure systme ou la programmation des
		// fins d'enregistrements.
		set_timer_record();		// Recalculer le timer des enregistrements
		break;

	case WM_LBUTTONDBLCLK:
		bDoubleClick = true;
		KillTimer(hWnd, TIMER_BORDER);
		PostMessage(hWnd, WM_COMMAND, MAKEWPARAM(IDM_FULLSCREEN, 1), 0);
		// vite de passer en mode bordures
		coord_down.x = -1;
		break;

	case WM_LBUTTONDOWN:
		// regarde si on vient pas de sortir du menu contextuel
		nClickTime = GetTickCount();
		bDoubleClick = false;
		if ( (nClickTime - time_context_quit) > 50)
			GetCursorPos(&coord_down);
		else
			coord_down.x = -1;
		break;

	case WM_LBUTTONUP: {
		POINT	newPos;

		GetCursorPos(&newPos);

		if (
			! bDoubleClick && etat_fenetre(hMainWnd) != etf_fullscreen &&
			newPos.x == coord_down.x && newPos.y == coord_down.y
		) {
			// Traitement des basculements de bordures, si pas de double clic, et si la souris
			// n'a pas boug depuis que le bouton de la souris a t press

			// Si on est en plein cran, on ne fait rien, car sinon, il rinitialise le mode
			// prcdent utilis avant de passer en plein cran (depuis rvision 154)
			UINT	nDoubleClickTime = GetDoubleClickTime();
			UINT	nClickDelay = GetTickCount() - nClickTime;

			if (nClickDelay < nDoubleClickTime)
				SetTimer(hWnd, TIMER_BORDER, nDoubleClickTime-nClickDelay, NULL);
		}
		break; }

	case WM_MOUSEMOVE:

		switch (etat_fenetre(hWnd)) {

		case etf_normal:
		case etf_noborders:
			mouse_move(wParam, lParam);
			change_etat_curseur(etc_normal);
			break;

		case etf_fullscreen: {
			static POINTS ancien_emplacement = {0,0};
			POINTS nouvel_emplacement = MAKEPOINTS(lParam);

			// Dtermine si on a dplac le curseur de la souris de plus de 10 pixels d'un cot comme de l'autre
			if (abs(ancien_emplacement.x - nouvel_emplacement.x) > 10 ||
				abs(ancien_emplacement.y - nouvel_emplacement.y) > 10) {
				// Affiche le curseur de la souris
				change_etat_curseur(etc_fullscreen_on);
				// Laisse affich un nombre de secondes dfini par DUREE_CURSEUR
				SetTimer(hWnd, TIMER_CURSOR, DUREE_CURSEUR, NULL);

				// Mmorise le nouvel emplacement
				ancien_emplacement.x = nouvel_emplacement.x;
				ancien_emplacement.y = nouvel_emplacement.y;
			}
		}

		default:
			lRet = DefWindowProc(hWnd, uMsg, wParam, lParam);
		}
		break;

	case WM_SYSCOMMAND:
		switch(wParam & 0xFFF0) {

		// Ceci est une astuce cite un peu partout sur Internet pour empcher l'cran de veille sous
		// XP lorsque la fentre correspondante a le focus, en empchant que ces notifications soient
		// transmises au traitement par dfaut de la fentre.
		// Voir http://www.cppfrance.com/codes/VCPLUSPLUS-DEV-CPLUSPLUS-DESACTIVER-ECRANS-VEILLES-AUTRE-PENDANT_9622.aspx
		// Ou bien http://bobobobo.wordpress.com/2008/02/10/intercepting-the-screensaver/
		case SC_SCREENSAVE:
		case SC_MONITORPOWER:
			return 0;

		case SC_CLOSE:
			myprintf(TEXT("WM_SYSCOMMAND close") EOL);
			if (record_protect) {
				if (can_quit(temps_prochain_enregistrement()))
					break;
				// Mode "rduire  la fermeture" : on ne quitte pas, on se contente de rduire la fentre
				PostMessage(hWnd, WM_SYSCOMMAND, SC_MINIMIZE, NULL);
				return 0;
			}
			break;

		case SC_MINIMIZE:
			myprintf(TEXT("WM_SYSCOMMAND minimize") EOL);
			if (suspend_minimized)
				suspend();
			lRet = DefWindowProc(hWnd, uMsg, wParam, lParam);
			if (minimize_system_tray)
				ShowWindow(hWnd, SW_HIDE);
			configState = etat_fenetre(hMainWnd);
			return 0;

		case SC_RESTORE:
			if (suspend_minimized && !rebranche(false))
				return false;
			myprintf(TEXT("WM_SYSCOMMAND restore") EOL);
			if (minimize_system_tray)
				ShowWindow(hWnd, SW_SHOW);
			break;

		case SC_MAXIMIZE:
			myprintf(TEXT("WM_SYSCOMMAND maximize") EOL);
		}
		lRet = DefWindowProc(hWnd, uMsg, wParam, lParam);
		configState = etat_fenetre(hMainWnd);
		break;

	case WM_ACTIVATE:
		// Vrifier  chaque changement d'tat d'activation que l'attribut
		// "WS_EX_TOPMOST" (= Rester toujours devant) est correct.
		if (((GetWindowLongPtr(hMainWnd, GWL_EXSTYLE) & WS_EX_TOPMOST) != 0) != on_top) {
			SetWindowPos(hWnd, on_top ? HWND_TOPMOST : HWND_NOTOPMOST, 0, 0, 0, 0,
				SWP_NOSIZE|SWP_NOREDRAW|SWP_NOMOVE|SWP_NOACTIVATE);
		}
		// myprintf(TEXT("WM_ACTIVATE, wParam = %u") EOL, wParam);
		break;

#ifdef _DIAGNOSTIC
	//case WM_ENABLE:
	//	myprintf(TEXT("WM_ENABLE, wParam = %u, lParam=0x%08x") EOL, wParam);
	//	break;

	case WM_DEVICECHANGE:
		myprintf(TEXT("WM_DEVICECHANGE wParam=0x%x, lParam=0x%08x") EOL, wParam, lParam);
		break;
#endif

	case WM_QUERYENDSESSION:
		// Verrouillage de la fermeture de session et de l'arrt systme
		if (record_protect) {	// Seulement si la protection des enregistrements est active
			myprintf(TEXT("WM_QUERYENDSESSION lParam=0x%08x") EOL, lParam);
			if ((lParam & ENDSESSION_CRITICAL) != 0)
				return TRUE;
			if ((lParam & ENDSESSION_CLOSEAPP) != 0)
				return can_power_down();

			LONG nReason = CheckIfWeCanQuit(false, false);

			if (nReason != 0) {
				// Si l'affichage de la raison du blocage n'est pas disponible (Vista...),
				// alors l'application notifie elle-mme a posteriori
				if (!cShutBlockReason.isAvailable())
					PostMessage(hWnd, WM_APP_SHUTBLOCK_NOTIFY, nReason, NULL);
				return FALSE;
			}
		}
		return TRUE;

	case WM_ENDSESSION:
		myprintf(TEXT("WM_ENDSESSION, wParam=%u, lParam=0x%08x") EOL, wParam, lParam);
		// Fermeture de session et/ou extinction de Windows !
		set_running_state(false);
		break;

	case WM_POWERBROADCAST:
		// Gestion des vnements de mise en veille (normale ou prolonge)
		myprintf(TEXT("WM_POWERBROADCAST wParam=0x%x, lParam=0x%08x") EOL, wParam, lParam);
		switch (wParam) {

		case PBT_APMSUSPEND:
			StopGraph();
			break;

		case PBT_APMRESUMESUSPEND:
		case PBT_APMRESUMEAUTOMATIC:
		case PBT_APMRESUMECRITICAL:
			myprint(TEXT("Attente aprs rveil") EOL);
			nWakeupRetryCount = WAKEUP_RETRIES;
			SetTimer(hMainWnd, TIMER_WAKEUP, DUREE_WAKEUP, NULL);
			break;

		case PBT_APMQUERYSUSPEND: {
			LONG nReason = CheckIfWeCanQuit(false, false);

			if (nReason != 0) {
				// Si l'affichage de la raison du blocage n'est pas disponible (Vista...),
				// alors l'application notifie elle-mme a posteriori
				if (!cShutBlockReason.isAvailable())
					PostMessage(hWnd, WM_APP_SHUTBLOCK_NOTIFY, nReason, NULL);
				return BROADCAST_QUERY_DENY;
			}
			break; }
		}
		myprint(TEXT("WM_POWERBROADCAST (fin)") EOL);
		break;

	case WM_CONTEXTMENU:
		popup_menu(false);
		break;

	case WM_SETFOCUS:
		time_context_quit = GetTickCount();
		break;

	case WM_APP_TRAYICON_MSG:
		if (wParam==MY_TRAY_ICON_ID) {
			switch(lParam) {
			case WM_LBUTTONDBLCLK:
				if (IsMinimized(hWnd)) {
					if (suspend_minimized && !rebranche(false))
						return -1;
					ShowWindow(hWnd, SW_SHOW);
					ShowWindow(hWnd, SW_RESTORE);
				} else {
					SetForegroundWindow(hWnd);
				}

				break;
			case WM_LBUTTONDOWN:
				if (!IsMinimized(hWnd))
					SetForegroundWindow(hWnd);
				break;
			case WM_CONTEXTMENU:
			case WM_RBUTTONUP:
				popup_menu(true);
			}
		}
		break;

	case WM_APP_EPG_UPDATED:
		update_status_bar();
		update_msn_and_tip();
		break;

	case WM_APP_PMT_UPDATED:
		myprintf(COLR(HRED, TEXT("La configuration de la chane (PMT) a chang")) EOL);
		sauve_chaines();
		change_chaine_update(ixChaineCourante);
		// request_osd_update();	// dornavant inutile, un vnement du graphe va faire a de toutes faons via WM_APP_VIDEO_UPDATED
		break;

	case WM_APP_GRAPHNOTIFY: {
		IPTvM_GraphBase * pGrph = reinterpret_cast<IPTvM_GraphBase *>(lParam);

		if (pGrph)
			pGrph->HandleGraphEvent(hWnd);
		break; }

	case WM_APP_VIDEO_UPDATED:
		myprint(COLR(HWHITE, TEXT("WM_APP_VIDEO_UPDATED")) EOL);
		update_coords();	// Recalcul requis si on s'est dplac sur un autre moniteur
		if (pOSD)
			pOSD->Enable();	// Ractivation de l'OSD
		break;

	case WM_APP_NEW_FREQ:
	case WM_APP_RECORD_EVENT:
		CSignalDialog::PostDlgMessage(uMsg, wParam, lParam);
		break;

	case WM_APP_SHUTBLOCK_NOTIFY:
		MessageBox(hWnd,
			(TEXT("La raison du blocage est la suivante:") EOL EOL + ReasonAsStr((LONG)wParam, false) + TEXT(".")).c_str(),
			APPLICATION_NAME TEXT(" a empch un arrt systme"), MB_ICONSTOP);
		break;

	default:
		if (uMsg == nWM_TASKBARCREATED) {
			// Recration de l'icne de la barre des tches aprs un plantage de l'explorateur Windows
			myprint(TEXT("La barre des tches a t recre") EOL);
			niData.disable();
			niData.update_state();
			break;
		} else if (uMsg == nWM_APP_INSTANCES) {
			switch (wParam) {

			case iit_RequestPowerStateChange:
				if (lParam != (LPARAM)hWnd && !can_power_down()) {	// Ne pas rpondre  nos propres requtes
					myprint(TEXT("Envoi de refus d'arrt ou mise en veille") EOL);
					SendMessage((HWND)lParam, nWM_APP_INSTANCES, iit_DenyPowerStateChange, (LPARAM)hWnd);
				}
				break;

			case iit_DenyPowerStateChange:
				myprint(TEXT("Rception de refus d'arrt ou mise en veille") EOL);
				bPowerStateChangeDenied = true;
			}
			uMsg = uMsg;
		}
		lRet = DefWindowProc(hWnd, uMsg, wParam, lParam);
		break;
	}

	return lRet;
}

/// Demande de mise  jour de l'OSD
void request_osd_update()
{
	// myprint(COLR(LGREEN, TEXT("request_osd_update : SetTimer(TIMER_UPDATE_OSD)")) EOL);
	SetTimer(hMainWnd, TIMER_UPDATE_OSD, DUREE_UPDATE_OSD, NULL);
}

// ====================================================================================

static void verifie_repertoire_captures(LPTSTR dir, UINT dir_size, int csidl_default, LPCTSTR dirUsage)
{
	TCHAR message[0x300];
	TCHAR default_dir[0x200];

	HANDLE hdir = CreateFile(dir, NULL, FILE_SHARE_READ, NULL, OPEN_EXISTING, FILE_FLAG_BACKUP_SEMANTICS, NULL);

	if (hdir != INVALID_HANDLE_VALUE)
		CloseHandle(hdir);
	else {
		// Rpertoire inexistant

		SHGetFolderPath(NULL, csidl_default, NULL, SHGFP_TYPE_CURRENT, default_dir);

		_stprintf_s(message, _countof(message),
			TEXT("  %s ,    %s,  .") EOL EOL
			TEXT("Voulez-vous le crer ?"), dir, dirUsage);
		if (MessageBox(hMainWnd, message, TEXT("  "), MB_ICONQUESTION | MB_YESNO) == IDYES) {
			// On le cre donc
			if (!CreateDirectory(dir, NULL)) {
				// Erreur lors de la cration du dossier
				strcpy_T(dir, dir_size, default_dir);
				_stprintf_s(message, _countof(message),
					TEXT("    .") EOL
					TEXT("   %s  :\n\n%s"), dirUsage, dir);
				MessageBox(hMainWnd, message, TEXT("    "), MB_ICONEXCLAMATION);
			}
		} else {
			strcpy_T(dir, dir_size, default_dir);
			_stprintf_s(message, _countof(message),
				TEXT("  %s  :\n\n%s"), dirUsage, dir);
			MessageBox(hMainWnd, message, TEXT("  "), MB_ICONINFORMATION);
		}
	}
}

// Arrt de l'ordinateur :
static bool shutdown(bool bForced)
{
	HANDLE hToken;
	TOKEN_PRIVILEGES tkp;

	// Rcupre un jetton pour l'application.
	if (!OpenProcessToken(GetCurrentProcess(), TOKEN_ADJUST_PRIVILEGES | TOKEN_QUERY, &hToken))
		return false;

	// Obtient le LUID pour le privilge d'arrt.
	LookupPrivilegeValue(NULL, SE_SHUTDOWN_NAME, &tkp.Privileges[0].Luid);

	tkp.PrivilegeCount = 1;  // 1 privilge  obtenir
	tkp.Privileges[0].Attributes = SE_PRIVILEGE_ENABLED;

	// Obtient le privilge d'arrt pour ce processus.
	AdjustTokenPrivileges(hToken, FALSE, &tkp, 0, (PTOKEN_PRIVILEGES)NULL, 0);

	if (GetLastError() != ERROR_SUCCESS)
		return false;

	// Arrte Windows, et force toutes les applications  s'arrter.
	if (time_before_shutdown > 0) {
		if (!InitiateSystemShutdownEx(NULL,
			(LPTSTR)tstr_printf(
				TEXT("Fin d'enregistrement TV:") EOL
				TEXT("L'ordinateur s'arrtera dans %u secondes.") EOL
				TEXT("(excutez shutdown -a pour annuler l'arrt)"),
				time_before_shutdown).c_str(),
			time_before_shutdown, bForced ? TRUE : FALSE, FALSE,
			SHTDN_REASON_MAJOR_APPLICATION | SHTDN_REASON_MINOR_OTHER | SHTDN_REASON_FLAG_PLANNED))
			// Erreur, on annule tout
			return false;
	} else {
		if (!ExitWindowsEx(EWX_SHUTDOWN | (bForced ? EWX_FORCE : 0),
			SHTDN_REASON_MAJOR_APPLICATION | SHTDN_REASON_MINOR_OTHER |SHTDN_REASON_FLAG_PLANNED))
			// Erreur, on annule tout
			return false;
	}
	// Tout est OK, le systme s'arrte...
	return true;
}

/** \brief Table des options de ligne de commande
 *
 *	Contient les noms des options, avec les types de donnes correspondants et la localisation
 *	des valeurs  affecter dans la structure qui va contenir les valeurs rcupres.
 *
 *  \todo Concernant les options de ligne de commande :
 *   - Trouver une stratgie cohrente pour le cas o une option --recps ou --rects
 *    spcifie entrerait en conflit avec un enregistrement programm
 *    (par exemple : si par ailleurs un enregistrement programm existe et se trouve
 *      devoir tre lanc immdiatement)
 **/
static OneCmdOption aOpt[] = {
	// Ancienne option, pour compatibilit : certains peuvent dj
	// avoir cette option dans une tche planifie cre antrieurement,
	// d'autres peuvent avoir cr un raccourci incluant l'option.
	{TEXT("-minimize"),		cot_BOOL,	offsetof(CmdOptions, bMinimize)},
	//
	{TEXT("--minimize"),	cot_BOOL,	offsetof(CmdOptions, bMinimize)},
	{TEXT("--fs"),			cot_BOOL,	offsetof(CmdOptions, bFs)},
	{TEXT("--maxi"),		cot_BOOL,	offsetof(CmdOptions, bMax)},
	{TEXT("--noborders"),	cot_BOOL,	offsetof(CmdOptions, bNoborder)},
	{TEXT("--nocrashwarn"),	cot_BOOL,	offsetof(CmdOptions, bNocrashwarn)},
	{TEXT("--normal"),		cot_BOOL,	offsetof(CmdOptions, bNormal)},
	{TEXT("--recps"),		cot_BOOL,	offsetof(CmdOptions, bRecPS)},
	{TEXT("--rects"),		cot_BOOL,	offsetof(CmdOptions, bRecTS)},
	{TEXT("--ch"),			cot_INT,	offsetof(CmdOptions, nChaine)},
	{TEXT("--help"),		cot_BOOL,	offsetof(CmdOptions, bHelp)},
	{TEXT("--prefix"),		cot_STR,	offsetof(CmdOptions, strPrefix)},
	{TEXT("--muted"),		cot_BOOL,	offsetof(CmdOptions, bMuted)}
};

/**
 * Traite les options dans la ligne de commande
 **/
static CmdOptions * TraiteCmdLine()
{
	CmdOptions * pOptions = new CmdOptions;

	if (pOptions)
		ProcessCmdLine(aOpt, _countof(aOpt), pOptions);
	return pOptions;
}

/**
 * Effacement des fichiers dont le nom se finit par  _ , dans le dossier actuel de PTVM
 * \todo Faire en sorte que ces fichiers ne se trouvent pas dans le dossier de l'application.
 * Normalement, l'application ne doit pas pouvoir modifier quoi que ce soit dans son propre
 * dossier : en effet, le dossier d'application n'est normalement pas accessible en criture
 * dans les comptes limits (non administrateurs).
 * La mise  jour de l'application elle-mme devrait sans doute tre la seule exception possible.
 **/
static void CleanTemporaryFiles()
{
	// Regarde si un fichier dont le nom se finit par  _  existe dans le dossier actuel de PTVM
	WIN32_FIND_DATA	fd = {0};	// Pour rechercher les fichiers
	SZPathName		chemin;		// Dossier actuel + pattern pour la recherche de fichiers de mise  jour

	strcpy_T(chemin, pouchindir_prog);
	strcat_T(chemin, TEXT("*_"));
	// On recherche
	HANDLE hSearch = FindFirstFile(chemin, &fd);

	if (hSearch != INVALID_HANDLE_VALUE) {

		// On efface les fichiers dont le nom se finit par  _ 
		do {
			_stprintf_s(chemin, _countof(chemin), TEXT("%s\\%s"), pouchindir_prog, fd.cFileName);
			if (!DeleteFile(chemin))
				break;
		} while (FindNextFile(hSearch, &fd));
		FindClose(hSearch);
	}
}

/**
 * Recherche d'une instance existante de l'application.
 * \param[in]	bMinimize	\p true si option "--minimize" en ligne de commande
 * \param[in]	pszPrefix	Prfixe optionnel  ajouter au nom du mutex
 * \retval	\p true si cette instance existe, et que l'application doit quitter
 * \note Effets de bord :
 *	- si l'application tourne dj, restauration de sa fentre de travail au cas
 *	  o elle serait minimise (sauf si "--minimize" en ligne de commande)
 *	- effacement de fichiers temporaires
 * \todo D'autres options de ligne de commande que "--minimize" pourraient galement
 * tre renvoyes  une instance de PTVM dj en cours d'excution si une seconde
 * instance est appele avec ces options.
 **/
static bool CheckForExistingInstance(bool bMinimize, LPCTSTR pszPrefix)
{
	// Premire chose  faire :
	//   Vrifier si une autre instance ne fonctionne pas dj

	HANDLE	hMutex		= CreateMutex(NULL, TRUE, (tstring(pszPrefix)+szAppName).c_str());
	bool	bQuitte		= true; // Indique qu'il faut quitter
	bool	bMutexError	= (GetLastError()==ERROR_ALREADY_EXISTS); // Mutex dj existant

	// On attend 3 secondes la libration du mutex
	// Pas plus longtemps, car si ce n'est pas une mise  jour, l'utilisateur l'a appel 2 fois,
	// et donc, si c'est plus long avant de rafficher la fentre principale, celui-ci
	// risque de le rappeller une fois de plus
	DWORD res			= WaitForSingleObject(hMutex, 3*DUREE_1SEC);

	if (!bMutexError || res == WAIT_ABANDONED) {
		// Attente de 250ms (car si trop rapide, on n'efface pas les fichiers)
		Sleep(250);
		// On ne quitte plus PTVM, car on a rcupr pour la mise  jour
		bQuitte = false;

		// Efface les fichiers dont le nom se finit par  _  dans le dossier actuel de PTVM
		CleanTemporaryFiles();
	}

	if (bQuitte && bMutexError) {
		// On quitte (pas de mise  jour, ou attente trop longue), mais on
		// affiche la fentre de Pouchin TV Mod dj lance au premier plan
		HWND hOtherInstanceWnd = FindWindow(szAppName, titre_fenetre);

		if (hOtherInstanceWnd) {
			if (bMinimize)
				SendMessage(hOtherInstanceWnd, WM_SYSCOMMAND, SC_MINIMIZE, NULL);
			else {
				SendMessage(hOtherInstanceWnd, WM_SYSCOMMAND, SC_RESTORE, NULL);
				SetForegroundWindow(hOtherInstanceWnd);
			}
		}
		return true;
	}
	return false;
}

/**
 *	Identification de la version du systme d'exploitation (toujours Windows  ce jour)
 *	\return		\p false en cas d'chec
 **/
bool FindOSversion()
{
	// Rcupre la version de windows
	// et initialise la variable indiquant si on tourne sous Vista
	ZeroMemory(&windows_version, sizeof(windows_version));
	windows_version.dwOSVersionInfoSize = sizeof(windows_version);
	if (!GetVersionEx((OSVERSIONINFO *)&windows_version)) {
		// Erreur lors de la rcupration de la version de Windows
		return false;
	}

	// http://www.codeguru.com/cpp/w-p/system/systeminformation/article.php/c8973
	switch (windows_version.dwMajorVersion) {
	case 5:
		switch (windows_version.dwMinorVersion) {
		case 0:
			eWinVers = wv_2000;
			break;
		case 2:
			if (windows_version.wProductType > VER_NT_WORKSTATION) {
				eWinVers = wv_Server2003;
				break;
			}
			// ... (passage  travers) (cas particulier XP 64)
		case 1:
			eWinVers = wv_XP;
		}
		break;
	case 6:
		switch (windows_version.dwMinorVersion) {
		case 0:
			switch (windows_version.wProductType) {
			case 1: eWinVers = wv_Vista; break;
			case 3: eWinVers = wv_Server2008;
			}
			break;
		case 1:
			switch (windows_version.wProductType) {
			case 1: eWinVers = wv_Seven; break;
			case 3: eWinVers = wv_Server2008R2;
			}
			break;
		default:
			eWinVers = wv_Newer;
		}
		break;
	default:
		if (windows_version.dwMajorVersion < 5)
			eWinVers = wv_Older;
		else if (windows_version.dwMajorVersion > 6)
			eWinVers = wv_Newer;
	}
	return true;
}

/// Initialisation de l'instance d'application
/// \retval \p true si succs
static bool InitInstance()
{
	// Chargement des icnes du programme :
	hPrgIcon = LoadIcon(hAppInstance, MAKEINTRESOURCE(IDI_PROG_ICON));	// Icne normale
	hRecIcon = LoadIcon(hAppInstance, MAKEINTRESOURCE(IDI_ICON_REC));	// Icne d'enregistrement

	INITCOMMONCONTROLSEX icc = {
		sizeof(INITCOMMONCONTROLSEX),
		ICC_BAR_CLASSES | ICC_LISTVIEW_CLASSES | ICC_DATE_CLASSES
	};

	InitCommonControlsEx(&icc);

	// Register the frame class
	WNDCLASS wndclass = {
		CS_DBLCLKS,												// style
		MainWndProc,											// lpfnWndProc
		0, 0,													// cbClsExtra, cbWndExtra
		hAppInstance,											// hInstance
		LoadIcon(hAppInstance, MAKEINTRESOURCE(IDI_PROG_ICON)),	// hIcon
		LoadCursor(NULL, IDC_ARROW),							// hCursor
		CreateSolidBrush(colorKey),								// hbrBackground
		MAKEINTRESOURCE(IDR_MAIN_MENU),							// lpszMenuName
		szAppName												// lpszClassName
	};

	if (RegisterClass(&wndclass)==0)
		return false;

	// Vrifie l'existence du fichier "canaux.ini", et le cre  partir des ressources
	// s'il n'existe pas :
	cree_fichier_scan();

	myprintf(
		TEXT("Fonctionne sous ") BOLD(TEXT("Windows ") TEXT("%") A2t) EOL TEXT("[Windows version %s]") EOL
		BOLD(TEXT("################################################################")) EOL EOL,
		EnumToString(eWinVers,
			"(version inconnue)\0"
			"(version ancienne)\0"
			"2000\0"
			"XP\0"
			"2003 Serveur\0"
			"Vista\0"
			"2008 Serveur\0"
			"7\0"
			"2008 Serveur R2\0"
			"(version inconnue : trop rcente)\0"),
		GetWindowsVersionStr(windows_version).c_str());

	HRESULT hr = CoInitializeEx(NULL,COINIT_MULTITHREADED);
	if (FAILED(hr)) {
		affiche_erreurs(TEXT("   COM ."), hr);
		return false;
	}

	return true;
}

/// Nettoyage de l'instance d'application
static void CleanInstance()
{
	// Destruction de l'icne du programme (ncessaire ?) :
	if (hPrgIcon != NULL) {
		DestroyIcon(hPrgIcon);
		hPrgIcon = NULL;
	}

	// Destruction de l'icne d'enregistrement (ncessaire ?) :
	if (hRecIcon != NULL) {
		DestroyIcon(hRecIcon);
		hRecIcon = NULL;
	}

	// On efface la liste des raccourcis
	DestroyAcceleratorTable(hMainAccel);

	// On libre toutes les ressources initialises
	CoUninitialize();
}

static bool CreateMainWindow(int nCmdShow, bool bMinimize)
{
	HWND hWnd = CreateWindowEx(0,
			szAppName,
			titre_fenetre,
			WS_OVERLAPPEDWINDOW | WS_CLIPCHILDREN | WS_CLIPSIBLINGS,
			CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT,
			NULL,
			NULL,
			hAppInstance,
			NULL);

	// make sure window was created
	if (!hWnd)
		return false;

	// Restauration de la position de la fentre. Noter que l'on ne passe pas la position  CreateWindowEx
	// car, cette valeur ayant t sauvegarde par GetWindowPlacement, elle n'est pas dfinie en termes de
	// coordonnes d'cran absolues, et en particulier sa valeur peut tre inexacte si la barre des tches
	// de Windows se trouve  gauche ou bien en haut de l'cran.
	winPos.RestoreWindowPos(hWnd, bMinimize ? SW_MINIMIZE : nCmdShow);

	UpdateWindow(hWnd);
	if (bMinimize && minimize_system_tray)
		ShowWindow(hWnd, SW_HIDE);	// Doit apparemment tre excut dans un second temps : mettre SW_HIDE
									// dans WINDOWPLACEMENT ne fonctionne pas comme on le voudrait.
	hMainMenu = GetMenu(hWnd);
	return true;
}

/**
 * Cration d'une instance de redmarrage de l'application (en principe aprs
 * remplacement de l'excutable suite  une mise  jour), juste avant de quitter
 **/
void Restart()
{
	STARTUPINFO			si = { sizeof(STARTUPINFO) };
	PROCESS_INFORMATION	pi;
	SZPathName			szTmp;

	// Synthse du nom avec ajout du prfixe si on est en train de redmarrer une instance secondaire de l'application
	// (il restera quand mme  l'utilisateur la responsabilit de redmarrer les ventuelles autres ...)
	_stprintf_s(szTmp, TEXT("\"%s\"%s"), szAppFileName, GetInstancePrefix().c_str());

	if (!CreateProcess(NULL, szTmp, NULL, NULL, FALSE, NULL, NULL, NULL, &si, &pi))
		affiche_erreurs(SystemErrorToString(GetLastError()).c_str(), 0,  TEXT(""));
}

int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow)
{
#ifdef _DEBUG
	// Programme la dtection des fuites mmoire
	_CrtSetDbgFlag ( _CRTDBG_ALLOC_MEM_DF | _CRTDBG_LEAK_CHECK_DF );
#endif

	hAppInstance = hInstance;

	/// Rcupration du nom de l'application
	GetModuleFileName(NULL, szAppFileName, _countof(szAppFileName));

	// Dfinition des rpertoires de base de l'application
	ini_app_paths();

	// Dcodage des options
	cmdOptions.reset(TraiteCmdLine());

	if (cmdOptions.get()) {
		// Aide en ligne pour les options de ligne de commande
		if (cmdOptions->bHelp) {
			MessageBox(NULL,
				TEXT("--ch\t\t(+ N  )     ") EOL
				TEXT("--fs\t\t   ") EOL
				TEXT("--help\t\t  ") EOL
				TEXT("--maxi\t\t  ") EOL
				TEXT("--minimize\t  ") EOL
				TEXT("--muted\t\t  ") EOL
				TEXT("--noborders\t   ") EOL
				TEXT("--nocrashwarn\t     ") EOL
				TEXT("--normal\t\t    ") EOL
				TEXT("--prefix\t\t()     ,") EOL
				TEXT("\t\t         ") EOL
				TEXT("\t\t       ") EOL
				TEXT("--recps\t\t    PS   ") EOL
				TEXT("--rects\t\t    TS   "),
				TEXT("    "), 0);
			return 0;
		}

		// Traitement de l'option "--prefix"
		if (!cmdOptions->strPrefix.empty() && cmdOptions->strPrefix.size() < _countof(prefix_conf)) {
			NomProtege<TCHAR, _countof(prefix_conf)> prefix_prot(cmdOptions->strPrefix.c_str(), TCHAR('_'));

			prefix_prot.Extract(prefix_conf, _countof(prefix_conf));
		}
	}

	// Rcupre la version de Windows (certaines valeurs par dfaut de la configuration peuvent en avoir besoin)
	if (!FindOSversion())
		return -1;	// si chec

	// On lit la config maintenant, besoin des coords de la fentre (surtout si la console
	// de debugging est prsente, y compris pendant le test d'instance prexistante)
	load_config();

	// Vrifier si une autre instance ne fonctionne pas dj
	if (CheckForExistingInstance(cmdOptions.get() && cmdOptions->bMinimize, prefix_conf))
		return 0;

	bCrashDetected = get_running_state();

	if (!bCrashDetected)
		set_running_state(true);

	// Mise  jour du registre pour le dmarrage avec windows
	update_start_with_windows();

	// Sauvegarde de l'tat temporaire des rglages de la vido
	memcpy(video_sav, video_ctl, sizeof(video_sav));

#if USE_CONSOLE
	// Affichage des messages de debugging initiaux, avec ouverture ventuelle de la console
	startConsole(TEXT("  MMzinTV"), TEXT("mmzintv.log"));

	// Affiche divers renseignements pour le debogguage
	myprintf(TEXT("Compil le        : ") TEXT(__DATE__) TEXT(" ") TEXT(__TIME__) EOL);
	myprintf(TEXT("Version           : ") BOLD(TEXT("%s")) EOL , szAppVersion);

	myprintf(
		TEXT("Tuner             : ") COLR(HYELLOW, TEXT("%s")) EOL
		TEXT("Codec Vido MPEG2 : ") COLR(HCYAN,   TEXT("%s")) EOL
		TEXT("Codec Vido H264  : ") COLR(HCYAN,   TEXT("%s")) EOL
		TEXT("Codec Audio MPEG2 : ") COLR(HGREEN,  TEXT("%s")) EOL
		TEXT("Codec Audio AC3   : ") COLR(HGREEN,  TEXT("%s")) EOL
		TEXT("Codec Audio E-AC3 : ") COLR(HGREEN,  TEXT("%s")) EOL
		TEXT("Canaux  explorer : ") COLR(HBLUE,   TEXT("%s")) EOL,
		nom_tuner,
		filtreMPEG2,
		filtreH264,
		filtreAudio,
		filtreAc3,
		filtreEac3,
		choixCanaux);
	myprintf(
		TEXT("Ligne de commande :") EOL
		COLR(LGREEN, TEXT("%s")) EOL,
		GetCommandLine());
#endif // #if USE_CONSOLE

	// Initialisations d'instance d'application
	if (!InitInstance())
		return -1;

	// Rcupration du code du message indiquant la recration de la barre des tches
	// http://msdn.microsoft.com/en-us/library/cc144179%28VS.85%29.aspx
	// http://www.codeguru.com/forum/archive/index.php/t-303732.html
	nWM_TASKBARCREATED = RegisterWindowMessage(TEXT("TaskbarCreated"));

	nWM_APP_INSTANCES  = RegisterWindowMessage(tstr_printf(TEXT("%s_instance"), szAppName).c_str());

	// Lecture du fichier des chanes (avant celle de la programmation, car on peut avoir
	// besoin de la liste des chanes dans celle-ci).
	lit_chaines();

	// Chargement et purge de la liste des programmes
	init_programmation();

	if (!CreateMainWindow(nCmdShow, cmdOptions.get() && cmdOptions->bMinimize))
		return -1;

	// On vrifie si les rpertoires spcifis pour l'enregistrement sont valides ou non :
	verifie_repertoire_captures(video_dir, _countof(video_dir),
		CSIDL_MYVIDEO /* = Rpertoire "Mes vidos" */, TEXT(" "));
	verifie_repertoire_captures(screenshots_dir, _countof(screenshots_dir),
		CSIDL_MYPICTURES /* = Rpertoire "Mes images" */, TEXT("  "));

	int nRetCode = -1;	// par dfaut

	// Ignorer la sauvegarde de l'tat minimis (vrifier si c'est adquat)
	if (configState == etf_minimized)
		configState = etf_normal;

	// Comme marqu, on cre la barre d'tat ("status") de la fentre
	if (hMainStatus == NULL)
		create_status_bar();

	// Ajustements de l'tat de dpart de la fentre :
	if (Canaux.size()==0) {
		// Pas question d'utiliser un tat de fentre sauvegard dans la config
		// ou de tourner en mode minimis si aucune chane n'a encore t dtecte
		if (cmdOptions.get())
			cmdOptions->bMinimize = false;
		configState = etf_normal;
	}

	if (cmdOptions.get()) {
		if (cmdOptions->bMinimize)
			configState = etf_minimized;
		else if (cmdOptions->bFs)
			configState = etf_fullscreen;
		else if (cmdOptions->bMax)
			configState = etf_maximized;
		else if (cmdOptions->bNoborder)
			configState = etf_noborders;
		else if (cmdOptions->bNormal)
			configState = etf_normal;

		if (cmdOptions->bMuted)
			muted = true;
	}

	// Presser la touche "majuscules"  ce moment force l'ouverture du dialogue de configuration.
	bool	bForceConfig = GetKeyState(VK_SHIFT) < 0;

	// Vrifier si l'application n'a pas t quitte incorrectement. SI c'est le cas, on propose l'ouverture
	// du dialogue de configuration (sauf en mode minimis : on ne va quand mme pas risquer de rater un
	// enregistrement programm  cause de l'ouverture de ce message d'avertissement !)
	if (!bForceConfig && bCrashDetected && !(cmdOptions.get() && (cmdOptions->bMinimize || cmdOptions->bNocrashwarn))) {
		if (
			MessageBox(hMainWnd,
				TEXT("MMzinTV       .") EOL
				TEXT("        .") EOL
				TEXT("    ,       ?"),
				TEXT(""), MB_ICONQUESTION | MB_YESNO) == IDYES
		)
			bForceConfig = true;
	}

	bCrashDetected = false;	// Ne plus redemander avant, heu ... le plantage suivant.

	// Dterminer la chane qu'on va slectionner au dmarrage
	if (cmdOptions.get() && cmdOptions->nChaine > 0) {
		// Un numro de chane pass en paramtre est prioritaire sur la chane
		// courante sauvegarde dans la configuration
		ixChaineAvance = Canaux.trouve_par_no((WORD)cmdOptions->nChaine);
	} else
		ixChaineAvance = Canaux.trouve_restauration_par_cle(cleChaineConfig);

	if (StartGraph(true, bForceConfig)) {
		if (cmdOptions.get()) {
			// Traiter les enregistrements demands par ligne de commande :
			if (cmdOptions->bRecTS) {
				if (enregistrements_actuels.start_record_ts(ixChaineCourante)>=0){
					update_record_menus(hMainWnd);
				}
			}

			if (cmdOptions->bRecPS) {
				if (enregistrements_actuels.start_record_ps(ixChaineCourante, ixSonCourant)>=0){
					update_record_menus(hMainWnd);
				}
			}
		}

		cmdOptions.reset();

		MSG			msg;

		// liminer tout message WM_POWERBROADCAST antrieur qui tranerait
		while (PeekMessage(&msg, 0, WM_POWERBROADCAST, WM_POWERBROADCAST, PM_REMOVE));

		myprintf(BOLD(TEXT("Entre dans la boucle de messages")) EOL);

		// boucle qui traite les messages clavier
		while (GetMessage(&msg, NULL, 0, 0)) {
			if (IsModelessDialogMessage(msg))
				continue;
			if (TranslateAccelerator(hMainWnd, hMainAccel, &msg))
				continue;
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}

		StopGraph();
		nRetCode = (int)msg.wParam;
	}

#if USE_PIPE_SERVER
	cPipeServ.Stop();
#endif
	CleanInstance();

	set_running_state(false);

	switch (eExitMode) {

	case em_Shutdown:
	case em_ShutdownForced:
		if (!shutdown(eExitMode == em_ShutdownForced))
			myprintf(TEXT("Impossible d'arrter l'ordinateur"));
		else
			myprintf(BOLD(TEXT("On quitte, pour redmarrer !")) EOL);
		break;

	case em_FullRestart:
		// Redmarrage demand
		myprintf(BOLD(TEXT("On redmarre !")) EOL);
		Restart();
		break;

	default:
		// Pour tmoigner qu'on est bien passs par ici en quittant :
		myprintf(BOLD(TEXT("On quitte !")) EOL);
	}

	return nRetCode;
}
