/*
 * $Id: settings.cpp 277 2011-03-13 10:24:41Z gingko $
 * Copyright (C) 2007 gingko - http://gingko.homeip.net/
 *
 * This file is part of Pouchin TV Mod, a free DVB-T viewer.
 * See http://www.pouchintv.fr/ for updates.
 *
 * Pouchin TV Mod is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Pouchin TV Mod is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 *
 *
 * Most files of this project have been changed from the Pouchin TV
 * project (Copyright (C) 2006 Pouchin <pouchinteve@free.fr>),
 * to use with a modified version of this software.
 * See http://pouchinteve.free.fr/ for the original project.
 */

/** \file
 *	\brief Fonctionnalits lies aux dialogues de configuration
 *
 *	Ce fichier contient les diffrentes fonctions ncessaires au
 *	traitement des dialogues de configuration de Pouchin TV Mod, ainsi que
 *	de leurs pages de proprits.
 **/

#include "stdafx.h"
#include "resource.h"
#include "settings.h"
#include "dlgutils.h"
#include "scan.h"
#include "search.h"
#include "trayicon.h"
#include "recprog.h"	// pour remplit_liste_audio
#include <Prsht.h>
#include <commdlg.h>

/**
 * Rcupre la valeur textuelle slectionne dans la liste droulante
 *
 * \param[in]		hItm		handle de la liste droulante dans la boite de dialogue
 * \param[in,out]	nom			nom de la valeur prcdement slectionne, et nouvelle valeur de celle-ci
 * \param[in]		size		nombre de caractres dans la chane de caractres
 * \param[in]		bWithNone	\p true si l'item 0 indique "Aucun"
 * \retval			true		la slection a t modifie
 * \retval			false		pas de changement de la slection
 **/
static bool extrait_combo(HWND hItm, LPTSTR nom, UINT size, bool bWithNone)
{
	// nom temporaire de l'lment slectionn dans la liste droulante
	TCHAR temp[256];

	// Rcupre l'index de l'lment slectionn
	INT		nIdx = (INT)SendMessage(hItm, CB_GETCURSEL, 0, 0);

	if (bWithNone && nIdx == 0) {
		// "Aucun"
		strcpy_T(temp, size, NULL_NAME);
	} else {
		LRESULT lRet = SendMessage(hItm, CB_GETLBTEXT, nIdx, (LPARAM)temp);

		if (lRet == CB_ERR)
			// La liste est vide :(
			// ou aucun lment n'est slectionn
			return false;
	}

	return copy_check(nom, size, temp);
}

/**
 * Dtermine si le combo n'a qu'un seul lment. Le slectionne si oui.
 *
 * \param[in]  hItem  handle de l'item devant tre test
 * \deprecated Vraisemblablement obsolte : maintenant, s'il n'y a qu'un seul lment dans la
 * liste, c'est l'lment "Aucun", et il devrait dj avoir t slectionn.
 **/
static void selectionne_si_solo(HWND hItem)
{
	// Rcupre le nombre d'lments dans le combo
	if (SendMessage(hItem, CB_GETCOUNT, 0, 0) == 1) {
		// Un seul lment dans le combo. Le slectionne
		SendMessage(hItem, CB_SETCURSEL, 0, 0);
	}
}

/**
 * \brief Ajustement du pas d'un contrle up down de temps.
 *
 * Ajustement de manire  avoir un pas de progression plus lev, un alignement au pas le plus proche,
 * et une valeur de pas qui augmente avec la valeur courante (valeurs : 10, 20, 50, 100, 200, 500).
 * \param[in]	sUpDn	Valeur  ajuster
 **/
static void AdjustDelayUpDown(NMUPDOWN & sUpDn)
{
	bool	bUp	 = sUpDn.iDelta > 0;
	int		nTmp = sUpDn.iPos + (int)(bUp);
	int		nInc =
				nTmp > 5000 ? 500 :
				nTmp > 2000 ? 200 :
				nTmp > 1000 ? 100 :
				nTmp >  500 ? 50 :
				nTmp >  200 ? 20 : 10;

	sUpDn.iDelta = ((sUpDn.iPos + (bUp ? nInc : -1)) / nInc * nInc) - sUpDn.iPos;
}

/// Slection de la couleur de l'OSD
static void ChooseOsdColor(HWND hDlg)
{
	CHOOSECOLOR	sChooseColor = {
		sizeof(sChooseColor),	// lStructSize
		hDlg,					// hwndOwner
		NULL,					// hInstance
		osd_color,				// rgbResult
		aCustomColors,			// lpCustColors
		CC_ANYCOLOR|CC_RGBINIT|CC_FULLOPEN,
								// Flags
		NULL,					// lCustData
		NULL,					// lpfnHook
		NULL					// lpTemplateName
	};

	if (ChooseColor(&sChooseColor))
		osd_color = sChooseColor.rgbResult;
}

/**
 * Remplissage de la "combo box" de mthodes de rendu vido
 * \param[in]	hDlg		Handle de la bote de dialogue contenant les contrles  interroger
 * \param[in]	eCurrMode	Mthode de rendu  slectionner
 **/
static void SetVMRSel(HWND hDlg, RdrTypes eCurrMode)
{
	// En correspondance avec le type numr 'RdrTypes'
	static LPCTSTR aRenderInitList[] = {
		TEXT("  "),
		TEXT("VMR7 windowless"),
		TEXT("VMR7 renderless"),
		TEXT("VMR9 windowless"),
#if NEW_VMR9_OUTPUT
		TEXT("VMR9 windowless (v2)"),
#endif
		TEXT("Enhanced Video Renderer"),
		TEXT("Enhanced Video Renderer (v2)"),
		NULL
	};

	ComboBox_Init(GetDlgItem(hDlg, IDC_COMBO_RENDERING), aRenderInitList, eCurrMode);
}

/**
 * Rcuprer le choix VMR selon la slection de la "combo box"
 * \param[in]	hDlg	Handle de la bote de dialogue contenant les contrles  interroger
 * \retval				Mthode de rendu slectionne
 **/

static RdrTypes GetVMRSel(HWND hDlg)
{
	return (RdrTypes)SendDlgItemMessage(hDlg, IDC_COMBO_RENDERING, CB_GETCURSEL, 0, 0);
}

/**
 * Accorder l'tat de la configuration OSD  celle du VMR et de la variable #eWinVers.
 **/
static void SetOSDState(CDialogBase & cDlg)
{
	RdrTypes eSelMode	= GetVMRSel(cDlg.hDlg);
	bool	bCanUseOSD	= eWinVers < wv_Vista || eSelMode!=rdt_VMR7_renderless && eSelMode!=rdt_VMR7_windowless;

	cDlg.EnableDlgItem(IDC_USE_OSD, bCanUseOSD);
	cDlg.EnableDlgItem(IDC_USE_OSD_S, bCanUseOSD);
}

// ====================================================================================
// Renumrotation des chanes
// ====================================================================================
/** \defgroup CRenumDialog Classe CRenumDialog
 * Fonctions pour grer la fentre de renumrotation des chanes
 * @{ */

class CRenumDialog : public CModalDialogBase
{
	/// Traitement des messages de la bote de dialogue
	virtual INT_PTR DialogProc(UINT uMsg, WPARAM wParam, LPARAM lParam);

public:

	/// Constructeur :
	CRenumDialog() :
		CModalDialogBase(IDD_RENUMBER)
	{
	}
};

/// Traitement des messages de la bote de dialogue
INT_PTR CRenumDialog::DialogProc(UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch (uMsg) {
	case WM_INITDIALOG:
		// Initialisation de la fentre
		CheckDlgButton(hDlg, IDC_RENUM_SEQ, BST_CHECKED);
		return TRUE;

	case WM_COMMAND:
		// Commande reue (clic, etc...)
		switch(LOWORD(wParam)) {

		case IDOK:
			// Clic sur le bouton OK
			EndDialog(hDlg,
				IsDlgButtonChecked(hDlg, IDC_RENUM_REST)==BST_CHECKED ?
				IDC_RENUM_REST : IDC_RENUM_SEQ);
			return TRUE;

		case IDCANCEL:
			// Clic sur le bouton Annuler
			EndDialog(hDlg, IDCANCEL);
			return TRUE;
		}
	}

	// Autre message, trait par Windows
	return FALSE;
}

/** @} */

// ====================================================================================
// Config des chanes
// ====================================================================================
/** \defgroup ChannelsDialogProc Fonctions pour fentre ChannelsDialogProc
 * Fonctions pour grer la fentre de gestion des chanes
 * @{ */

#define CHAN_COL_NO		0	///< Colonne indiquant le numro de la chane
#define CHAN_COL_CANAL	1	///< Colonne indiquant le numro du canal
#define CHAN_COL_FREQ	2	///< Colonne indiquant la frquence de la chane
#define CHAN_COL_GROUP	3	///< Colonne indiquant le groupe du mutiplex
#define CHAN_COL_NAME	4	///< Colonne indiquant le nom de la chane
#define CHAN_COL_STATE	5	///< Colonne indiquant l'tat de la chane
#define CHAN_COL_MPEG	6	///< Colonne indiquant le type de codage vido de diffusion (MPEG2, MPEG4 pour le moment)
#define CHAN_COL_TSID	7	///< Colonne indiquant le TSID (Transport Stream ID) de la chane
#define CHAN_COL_ONID	8	///< Colonne indiquant le ONID (Original Network ID) de la chane
#define CHAN_COL_SID	9	///< Colonne indiquant le SID (Service ID) de la chane
#define CHAN_COL_VIDPID	10	///< Colonne indiquant le PID vido de la chane
#define CHAN_COL_PMTPID	11	///< Colonne indiquant le PID PMT de la chane

/**
 * Dfinit l'tat de la chane dans la liste
 *
 * \param[in]  hListItem  handle de la liste
 * \param[in]  iItem      numro de l'item (position de la chane) dans la liste qui doit tre modifie
 * \param[in]  etat       EtatChaine dfinissant l'tat de la chane
 **/
static void State2LV(HWND hListItem, int iItem, EtatChaine etat)
{
	const LVITEM item = {
		LVIF_TEXT,								// mask
		iItem,									// iItem
		CHAN_COL_STATE,							// iSubItem
		0, 0,									// state, stateMask
		LPTSTR(dword2str(aChStateTable, etat))	// pszText
		// le reste  zro (implicite)
	};

	ListView_SetItem(hListItem, &item);
}

/**
 * Ajoute une chane dans la liste des chanes, en dfinisant ses informations
 *
 * \param[in]  hListItem  handle de la liste recevant la chane
 * \param[in]  iItem      numro de l'item (position de la chane) dans la liste qui doit tre insre
 * \param[in]  canal      chane  ajouter (contient toutes les informations)
 **/
void InsertChannelItem(HWND hListItem, int iItem, const Chaine & canal)
{
	TCHAR buffer[64];
	LVITEM item = {
		LVIF_PARAM | LVIF_TEXT,	// mask
		iItem,					// iItem
		CHAN_COL_NO,			// iSubItem
		0, 0,					// state, stateMask
		buffer,					// pszText
		0, 0,					// cchTextMax, iImage
		canal.cle()				// lParam
		// le reste  zro (implicite)
	};

	// Ajoute le numro de la chane
	_stprintf_s(buffer, _countof(buffer), TEXT("%u"), canal.nNumeroChaine);
	ListView_InsertItem(hListItem, &item);

	// On ne met que le texte dans les colonnes suivantes
	item.mask = LVIF_TEXT;
	item.lParam = 0;

	// Numro du canal
	item.iSubItem = CHAN_COL_CANAL;
	_stprintf_s(buffer, _countof(buffer), TEXT("%u"), canal.canal_no);
	ListView_SetItem(hListItem, &item);

	// Frquence de la chane
	item.iSubItem = CHAN_COL_FREQ;
	_stprintf_s(buffer, _countof(buffer), TEXT("%03u,%03u"), canal.khz/1000, canal.khz%1000);
	ListView_SetItem(hListItem, &item);

	// Groupe du multiplex de diffusion
	item.iSubItem = CHAN_COL_GROUP;
	strcpy_T(buffer, canal.groupe);
	ListView_SetItem(hListItem, &item);

	// Nom de la chane
	item.iSubItem = CHAN_COL_NAME;
	strcpy_T(buffer, canal.nom);
	ListView_SetItem(hListItem, &item);

	// tat de la chane (prfr, inactive, ...)
	State2LV(hListItem, iItem, canal.etat);

	// Codec vido de diffusion
	item.iSubItem = CHAN_COL_MPEG;
	_stprintf_s(buffer, _countof(buffer), TEXT("%s"), canal.norme());
	ListView_SetItem(hListItem, &item);

	// TSID
	item.iSubItem = CHAN_COL_TSID;
	_stprintf_s(buffer, _countof(buffer), TEXT("%u"), canal.TSID);
	ListView_SetItem(hListItem, &item);

	// ONID
	item.iSubItem = CHAN_COL_ONID;
	_stprintf_s(buffer, _countof(buffer), TEXT("%u"), canal.ONID);
	ListView_SetItem(hListItem, &item);

	// SID
	item.iSubItem = CHAN_COL_SID;
	_stprintf_s(buffer, _countof(buffer), TEXT("%u"), canal.SID);
	ListView_SetItem(hListItem, &item);

	// PID vido
	item.iSubItem = CHAN_COL_VIDPID;
	_stprintf_s(buffer, _countof(buffer), TEXT("%u"), canal.video_pid);
	ListView_SetItem(hListItem, &item);

	// PID PMT
	item.iSubItem = CHAN_COL_PMTPID;
	_stprintf_s(buffer, _countof(buffer), TEXT("%u"), canal.pmt_pid);
	ListView_SetItem(hListItem, &item);
}

/**
 * Remplit la liste des chanes, en la vidant et en y remettant toutes les chanes
 *
 * \param[in]  hListItem  handle de la liste recevant les chanes
 **/
static void remplit_liste_chaines(HWND hListItem)
{
	// Vide la liste des chanes
	ListView_DeleteAllItems(hListItem);

	int nbChaines = (int)Canaux.size();

	// Ajoute les chanes
	for (int i = 0; i<nbChaines; i++) {
		const Chaine & canal = Canaux[i];

		InsertChannelItem(hListItem, i, canal);
	}
}

/**
 * Fonction utilise pour trier la liste des chanes
 *
 * \param[in]  lParam1     index du permier lment
 * \param[in]  lParam2     index du second lment
 * \param[in]  lParamSort  paramtres de tri
 * \retval     <0          lParam1 < lParam2
 * \retval     0           lParam1 = lParam2
 * \retval     >0          lParam1 > lParam2
 **/
int CALLBACK LVCompareProc(LPARAM lParam1, LPARAM lParam2,  LPARAM lParamSort)
{
	LPNMLISTVIEW pnml = ((LPNMLISTVIEW)lParamSort);
	int iCol = pnml->iSubItem;	// N colonne
	TCHAR buf1[64] = TEXT("");
	TCHAR buf2[64] = TEXT("");
	LVITEM item1 = {
		LVIF_PARAM | LVIF_TEXT,	// mask
		(int)lParam1,			// iItem
		iCol,					// iSubItem
		0, 0,					// state, stateMask
		buf1,					// pszText
		_countof(buf1)			// cchTextMax
		// le reste  zro (implicite)
	};
	LVITEM item2 = {
		LVIF_PARAM | LVIF_TEXT,	// mask
		(int)lParam2,			// iItem
		iCol,					// iSubItem
		0, 0,					// state, stateMask
		buf2,					// pszText
		_countof(buf2)			// cchTextMax
		// le reste  zro (implicite)
	};
	int res = 0; // rsultat

	ListView_GetItem(pnml->hdr.hwndFrom, &item1);
	ListView_GetItem(pnml->hdr.hwndFrom, &item2);

	switch (iCol) {

	case CHAN_COL_NO:		// N
	case CHAN_COL_CANAL:	// Canal
	case CHAN_COL_TSID:
	case CHAN_COL_ONID:
	case CHAN_COL_SID:
	case CHAN_COL_VIDPID:
	case CHAN_COL_PMTPID: {
		int no1 =_tstoi(buf1);
		int no2 =_tstoi(buf2);

		res = no1<no2 ? -1 : no1>no2 ? 1 : 0;
		break; }

	case CHAN_COL_FREQ:		// Frquence
	case CHAN_COL_GROUP:	// Groupe
	case CHAN_COL_NAME:		// Nom
	case CHAN_COL_STATE:	// tat
	case CHAN_COL_MPEG:		// MPEG
		res = _tcsicmp(buf1, buf2);
	}

	// Tri descendant si pnml->lParam != 0
	if (pnml->lParam!=0)
		res = -res;
	return res;
}

/**
 * Permet de faire glisser les chanes pour en rorganiser l'ordre
 **/
class ChannelsLVDrag {
	/// Handle de la fentre de dialogue
	HWND hDlg;
	/// Handle de la liste
	HWND hListItem;
	/// Image affiche durant le clic&drop
	HIMAGELIST hDragImageList;
	int iItemMrq;	///< Item couramment point par le marqueur d'insertion
	int iItemEmpty;	///< Item vide insr  la fin
	DWORD iSaveTicks;
	bool scrolldown;
	//
	HIMAGELIST CreateDragImage();
	void DrawInsertMark(int iItem, bool erase);
	int ItemHit(LONG x, LONG y);
public:
	ChannelsLVDrag(HWND hDg, HWND hLstItem);
	void Begin(LONG x, LONG y);
	void Move(LONG x, LONG y);
	int End(LONG x, LONG y);	// (retourne le point d'insertion)
	void Scroll();
	~ChannelsLVDrag();
};

/**
 * Constructeur
 *
 * \param[in]  hDg       handle de la boite de dialogue
 * \param[in]  hLstItem  handle de la liste
 **/
ChannelsLVDrag::ChannelsLVDrag(HWND hDg, HWND hLstItem) :
	hDlg(hDg),
	hListItem(hLstItem),
	hDragImageList(NULL),
	iItemMrq(-1),
	iItemEmpty(0),
	iSaveTicks(0),
	scrolldown(false)
{
	if (iItemEmpty==0) {
		// Ajout d'un item vide  la fin pour permettre l'insertion aprs le dernier item :
		LVITEM item = {
			LVIF_PARAM | LVIF_TEXT,	// mask
			0x7fffffff,				// iItem (assignation  la fin)
			0,						// iSubItem
			0, 0,					// state, stateMask
			TEXT(""),				// pszText
			// le reste  zro (implicite)
		};

		iItemEmpty = ListView_InsertItem(hListItem, &item);
	}
	myprintf(TEXT("ChannelsLVDrag construit") EOL);
}

/**
 * Gnre l'image affiche durant le drag&drop, indiquant la position
 *
 * \return le handle de l'image gnre
 **/
HIMAGELIST ChannelsLVDrag::CreateDragImage()
{
	HIMAGELIST hDragImageList = NULL;
	int	iHeight = 0;

	// Cration d'une image "drag" pour chacun des items slectionns

	for (int iPos = ListView_GetNextItem(hListItem, -1, LVNI_SELECTED);
		iPos!= -1; iPos = ListView_GetNextItem(hListItem, iPos, LVNI_SELECTED)
	) {
		POINT p;

		if (hDragImageList==NULL) {
			// For the first selected item,
			// we simply create a single-line drag image
			hDragImageList = ListView_CreateDragImage(hListItem, iPos, &p);
		} else {
			// For the rest selected items,
			// we create a single-line drag image, then
			// append it to the bottom of the complete drag image
			HIMAGELIST	hTempImageList = hDragImageList;
			HIMAGELIST	hOneImageList =
				ListView_CreateDragImage(hListItem, iPos, &p);

			hDragImageList =
				ImageList_Merge(hTempImageList, 0, hOneImageList, 0, 0, iHeight);
			ImageList_Destroy(hTempImageList);
			ImageList_Destroy(hOneImageList);
		}

		IMAGEINFO imf;

		ImageList_GetImageInfo(hDragImageList, 0, &imf);
		iHeight = imf.rcImage.bottom;
	}

	return hDragImageList;
}

/**
 * On commence le drag&drop
 *
 * \param[in]  x  position x de la gnration de l'vnement
 * \param[in]  y  position y de la gnration de l'vnement
 **/
void ChannelsLVDrag::Begin(LONG x, LONG y)
{
	// Initialisation et dmarrarage du glisser-dposer
	// (implment d'aprs http://www.codeproject.com/listctrl/jianghong.asp)

	hDragImageList = CreateDragImage();

	ImageList_BeginDrag(hDragImageList, 0, 0, 0);

	POINT p = {x, y};
	ClientToScreen(hListItem, &p);
	ImageList_DragEnter(GetDesktopWindow(), p.x, p.y);

	SetCapture(hDlg);			// Capture de la souris
	myprintf(TEXT("ChannelsLVDrag::Begin") EOL);
}

/**
 * Dessine la flche d'insertion
 *
 * \param[in]  iItem  index correspondant  la ligne sur laquelle il faut dessiner la flche
 * \param[in]  erase  indique si l'on doit effacer la prcdente flche
 **/
void ChannelsLVDrag::DrawInsertMark(int iItem, bool erase)
{
	if (iItem<0)
		return;

	RECT rc;

	// Dterminer la position du marqueur, dans la marge de gauche du dialogue :
	ListView_GetItemRect(hListItem, iItem, &rc, LVIR_BOUNDS);	// Position dans la LV
	POINT pt = {rc.left-4, rc.top};	// Position de la pointe du marqueur
	ClientToScreen(hListItem, &pt);	// Conversion de coordonnes
	ScreenToClient(hDlg, &pt);

	if (erase) {
		// Effacement :
		RECT irc = {pt.x-10, pt.y-4, pt.x+1, pt.y+5};

		InvalidateRect(hDlg, &irc, true);
	} else {
		// Dessin du marqueur :
		POINT ptv[] = {
			{pt.x,		pt.y},
			{pt.x-4,	pt.y+4},
			{pt.x-5,	pt.y+4},
			{pt.x-5,	pt.y+2},
			{pt.x-10,	pt.y+2},
			{pt.x-10,	pt.y-4},
			{pt.x-7,	pt.y-4},
			{pt.x-7,	pt.y-2},
			{pt.x-5,	pt.y-2},
			{pt.x-5,	pt.y-4},
			{pt.x-4,	pt.y-4}
		};

		AutoDC				cDC(hDlg);
		AutoHandle<HPEN>	cPen(CreatePen(PS_SOLID, 1, RGB(255,255,255)));	// blanc
		AutoHandle<HBRUSH>	cBrush(CreateSolidBrush(RGB(0,0,0)));			// noir
		AutoSelect<HPEN>	cPS(cDC, cPen);
		AutoSelect<HBRUSH>	cBS(cDC, cBrush);

		Polygon(cDC, ptv, _countof(ptv));
	}
}

/**
 * Fonction retournant le numro de l'item point par la souris
 *
 * \param[in]  x  position x de la gnration de l'vnement
 * \param[in]  y  position y de la gnration de l'vnement
 * \return        le numro de l'index point
 * \retval    -1  si la souris ne pointe pas un item
 * \retval    -2  si on pointe au-dessus
 * \retval    -3  si on pointe en-dessous
 * \todo Vrifier/revoir fonctionnement sous Windows Vista : avec cet OS, les flags
 * \p LVHT_ABOVE et \p LVHT_BELOW ne semblent pas tre retourns correctement.
 **/
int ChannelsLVDrag::ItemHit(LONG x, LONG y)
{
	// Trouver l'item sous la souris :
	LVHITTESTINFO lvhti = {
		{x, y}
		// le reste  zro (implicite)
	};

	ClientToScreen(hDlg, &lvhti.pt);
	ScreenToClient(hListItem, &lvhti.pt);
	ListView_HitTest(hListItem, &lvhti);

	// En dehors de la liste, et pas dans un item ?
	if (lvhti.iItem != -1) {
		// Vrifier si on ne pointe pas dans les en-ttes de colonnes :
		RECT hdrect;

		GetClientRect(ListView_GetHeader(hListItem), &hdrect);
		if (PtInRect(&hdrect, lvhti.pt))
			return -2;		// si dans les en-ttes
		return lvhti.iItem;	// item trouv
	}
	if (lvhti.flags & LVHT_ABOVE)
		return -2;			// trop en haut
	if (lvhti.flags & LVHT_BELOW)
		return -3;			// trop en bas
	return -1;				// ailleurs
}

/**
 * Fait dfiler la liste
 **/
void ChannelsLVDrag::Scroll()
{
	ListView_Scroll(hListItem, 0, scrolldown ? 16 : -16);
	InvalidateRect(hListItem, NULL, TRUE);	// Ajout  cause d'un certain problme de corruption d'affichage ...
	iSaveTicks = GetTickCount();
	SetTimer(hDlg, TIMER_CHAN_LVDRAG, 200, NULL);
}

/**
 * Gre les dplacements de la souris
 *
 * \param[in]  x  position x de la gnration de l'vnement
 * \param[in]  y  position y de la gnration de l'vnement
 **/
void ChannelsLVDrag::Move(LONG x, LONG y)
{
	POINT pt = {x, y};

	ClientToScreen(hDlg, &pt);
	ScreenToClient(hListItem, &pt);
	pt.x = 0;
	ClientToScreen(hListItem, &pt);
    ImageList_DragMove(pt.x, pt.y);

	int iItem = ItemHit(x, y);			// Dterminer l'item choisi

	if (iItemMrq!=iItem) {				// si item point modifi :
		DrawInsertMark(iItemMrq, true);	// Retirer le marqueur de slection existant
		iItemMrq = -1;
	}

	switch (iItem) {
	case -1:
		KillTimer(hDlg, TIMER_CHAN_LVDRAG);
		return;			// Aucun item point
	case -2:
	case -3:
		// Trop haut ou trop bas
		if (GetTickCount() - iSaveTicks >= 250) {
			scrolldown = iItem==-3;
			Scroll();
		}
		break;
	default:
		if (iItemMrq!=iItem) {				// si item point modifi :
			KillTimer(hDlg, TIMER_CHAN_LVDRAG);
			DrawInsertMark(iItem, false);	// Dessiner le nouveau marqueur de slection
			iItemMrq = iItem;
		}
	}
}

/**
 * Fin du drag&drop
 *
 * \param[in]  x  position x de la gnration de l'vnement
 * \param[in]  y  position y de la gnration de l'vnement
 * \return        la nouvelle position de l'item dans la liste
 * \retval    -1  erreur
 **/
int ChannelsLVDrag::End(LONG x, LONG y)
{
	myprintf(TEXT("ChannelsLVDrag::End") EOL);

	KillTimer(hDlg, TIMER_CHAN_LVDRAG);
	ImageList_DragLeave(hListItem);
	ImageList_EndDrag();
	ImageList_Destroy(hDragImageList);

	ReleaseCapture();

	// Effacement du dernier marqueur d'insertion :
	DrawInsertMark(iItemMrq, true);
	iItemMrq = -1;

	int iItem = ItemHit(x, y);
	if (iItem <0)
		return -1;

	// Est-ce que l'item de chute est slectionn ?
	LVITEM lvi = {
		LVIF_STATE,		// mask
		iItem,			// iItem
		0,				// iSubItem
		0,				// state,
		LVIS_SELECTED	// stateMask
		// le reste  zro (implicite)
	};

	ListView_GetItem(hListItem, &lvi);

	if (lvi.state & LVIS_SELECTED)
		return -1;
	return iItem;

}

/**
 * Destructeur
 **/
ChannelsLVDrag::~ChannelsLVDrag()
{
	if (iItemEmpty>0) {
		// Suppression de l'item vide qui avait t ajout  la fin au dbut de l'opration :
		ListView_DeleteItem(hListItem, iItemEmpty);
		iItemEmpty = 0;
	}
	// Redessiner tout le contrle contenant les chanes :
	InvalidateRect(hListItem, NULL, TRUE);
	myprintf(TEXT("ChannelsLVDrag dtruit") EOL);
}

/**
 * Application d'un changement de numro de chane
 *
 * \param[in]  hListItem   handle de la liste
 * \param[in]  iItem       numro de la ligne contenant la chane
 * \param[in]  str_newnum  nouveau numro de la chane
 **/
static void channel_number_changed(HWND hListItem, int iItem, LPCTSTR str_newnum)
{
	// Convertit le nouveau numro en nombre
	WORD canal_no = (WORD)_tstoi(str_newnum);

	// Numro valide ?
	if (canal_no>0 && canal_no<1000) {
		TCHAR		buf1[16], buf2[16];
		bool		confirm = true;
		HWND hDlg = GetParent(hListItem);

		_stprintf_s(buf1, _countof(buf1), TEXT("%u"), canal_no);	// reformater
		LVFINDINFO lvfi = {
			LVFI_STRING,
			buf1
			// le reste  zro (implicite)
		};

		int iItemFound = ListView_FindItem(hListItem, -1, &lvfi);

		if (iItemFound>=0 && iItemFound != iItem) {
			TCHAR chnom1[32], chnom2[32], bufmsg[256];

			ListView_GetItemText(hListItem, iItem, CHAN_COL_NAME, chnom1, _countof(chnom1));
			ListView_GetItemText(hListItem, iItemFound, CHAN_COL_NAME, chnom2, _countof(chnom2));
			ListView_GetItemText(hListItem, iItem, CHAN_COL_NO, buf2, _countof(buf2));
			_stprintf_s(bufmsg, _countof(bufmsg),
				TEXT(" ,     %s    ")
				TEXT("%s.\n  ?") EOL
				TEXT("(    :  %s    %s)"),
				chnom1, chnom2, chnom2, buf2);

			confirm = MessageBox(hDlg, bufmsg,
				TEXT("  "), MB_OKCANCEL)==IDOK;
			if (confirm)
				ListView_SetItemText(hListItem, iItemFound, CHAN_COL_NO, buf2);
		}

		if (confirm) {
			ListView_SetItemText(hListItem, iItem, CHAN_COL_NO, buf1);
			PropSheet_Changed(GetParent(hDlg), hDlg);
		}
	}
}

/**
 * Permet de savoir l'tat de la chaine  partir d'un index
 *
 * \param[in]  hListItem  handle de la liste
 * \param[in]  iItem      numro de l'item
 * \return                l'tat de la chane
 **/
static EtatChaine ItemToEtat(HWND hListItem, int iItem)
{
	TCHAR buf[16];

	// Mise  jour des cases  cocher lies  l'tat :
	ListView_GetItemText(hListItem, iItem, CHAN_COL_STATE, buf, _countof(buf));

	return EtatChaine(str2dword(aChStateTable, buf, ec_preferee));
}

/// ====================================================================================

/// Dialogue (feuilles de proprits) de configuration
class CConfigPropSheet : public CPropSheetBase
{
	/// Initialisation : modification du titre
	virtual void Init();

public:
	static bool	bNeedRestart;

	/// Constructeur
	CConfigPropSheet();
};

/// ====================================================================================

/// Gestion de la page des chanes dans le dialogue de configuration
class CChannelsPropSheet : public CPropSheetPageBase
{
	/// Traitement des messages de la page de proprits
	virtual INT_PTR DialogProc(UINT uMsg, WPARAM wParam, LPARAM lParam);

	HWND						hListItem;
	int							iItemEdit;
	UINT32						cleSauveChaineCourante;
	auto_ptr<ChannelsLVDrag>	m_Drag;
public:

	/// Constructeur :
	CChannelsPropSheet() :
		CPropSheetPageBase(),
		hListItem(NULL),
		iItemEdit(-1),
		cleSauveChaineCourante(0)
	{
	}
};

/**
 * Fonction grant les vnements de la fentre des chanes
 **/
INT_PTR CChannelsPropSheet::DialogProc(UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	// Nom et largeur des diffrentes colonnes
	static const struct Tab_cols t_cols[] = {
		{TEXT("N"), 28},					// CHAN_COL_NO
		{TEXT("C."), 28},					// CHAN_COL_CANAL
		{TEXT("MHz"), 54},					// CHAN_COL_FREQ
		{TEXT(""), 60},				// CHAN_COL_GROUP
		{TEXT("  "), 120},	// CHAN_COL_NAME
		{TEXT(""), 60},					// CHAN_COL_STATE
		{TEXT(""), 48},				// CHAN_COL_MPEG
		{TEXT("TsId"), 40},					// CHAN_COL_TSID
		{TEXT("OnId"), 40},					// CHAN_COL_ONID
		{TEXT("SId"), 40},					// CHAN_COL_SID
		{TEXT("vPId"), 40},					// CHAN_COL_VIDPID
		{TEXT("pPId"), 40},					// CHAN_COL_PMTPID
		{NULL, 0}		// Terminateur
	};

	TCHAR			buf[16];
	int				i;

	switch (uMsg) {

	case WM_INITDIALOG:	// Initialisation de la fentre
		cleSauveChaineCourante = indexToCle(ixChaineCourante);	// Rcupre l'index de la chaine courante
		hListItem = GetDlgItem(hDlg, IDC_CHANNEL_LIST);			// Mmorise le handle de la liste
		ListView_AjouteColonnes(hListItem, t_cols);

		// Charge la liste des chanes
		remplit_liste_chaines(hListItem);
		return TRUE;

	case WM_COMMAND:
		switch(wParam) {

		case _cmd_(IDC_SCAN, BN_CLICKED): {
			// Lance la rechercher des chanes
			static bool rechercheEncours = false;

			if (!rechercheEncours) {
				rechercheEncours = true;
				BYTE res = scan_dialog(hDlg);

				if (res==IDABORT)
					MessageBoxCheck(hDlg,
						TEXT(" :       "),
						TEXT(" "), MB_ICONINFORMATION | MB_OK,
						IDOK, TEXT("    "));
				rechercheEncours = false;
				remplit_liste_chaines(hListItem);
			}
			break; }

		case _cmd_(IDC_RENUMBER, BN_CLICKED): {
			// Boite de dialogue permettant de renumroter les chanes
			int res = (int)CRenumDialog().Do(hDlg);

			if (res==IDC_RENUM_SEQ || res==IDC_RENUM_REST) {
				int nbItems = ListView_GetItemCount(hListItem);
				WORD iCount = 0;

				for (i=0; i<nbItems; i++) {
					WORD newNumChaine = ++iCount;
					TCHAR buf[16];

					if (res==IDC_RENUM_REST) {
						// Rcupration du numro initial :
						LVITEM lvi = {
							LVIF_PARAM,	// mask
							i						// iItem
							// le reste  zro (implicite)
						};

						ListView_GetItem(hListItem, &lvi);

						int ixChaine = Canaux.trouve_par_cle(UINT32(lvi.lParam));

						if (!ixChaine_ok(ixChaine))
							break;	// gros bug si on arrive l
						newNumChaine = Canaux[ixChaine].nNumeroAuto;
					}

					_stprintf_s(buf, _countof(buf), TEXT("%u"), newNumChaine);	// formater
					ListView_SetItemText(hListItem, i, CHAN_COL_NO, buf);
				}
			}
			break; }

		// On a chang l'tat de la chane
		case _cmd_(IDM_CHAN_INACTIVE, 0):
		case _cmd_(IDM_CHAN_ACTIVE, 0):
		case _cmd_(IDM_CHAN_PREFERRED, 0):
			if (iItemEdit>=0)
				State2LV(hListItem, iItemEdit, EtatChaine(LOWORD(wParam)-IDM_CHAN_STATE_BASE));
			break;
		}
		return FALSE;

	case WM_NOTIFY:
		switch (_ncode_(lParam)) {

		case LVN_ENDLABELEDIT: {	// Fin de modification manuelle des numros de chanes
			NMLVDISPINFO & nmlvdi = *(NMLVDISPINFO *)lParam;

			if (nmlvdi.item.iItem>=0 && nmlvdi.item.pszText!=NULL)
				channel_number_changed(hListItem, nmlvdi.item.iItem, nmlvdi.item.pszText);
			break; }

		case LVN_BEGINDRAG:			// Dbut de la rorganisation de l'ordre des chanes
			if (wParam==IDC_CHANNEL_LIST) {
				if (!m_Drag.get())
					m_Drag.reset(new ChannelsLVDrag(hDlg, hListItem));
				if (m_Drag.get())
					m_Drag->Begin(LPNMLISTVIEW(lParam)->ptAction.x, LPNMLISTVIEW(lParam)->ptAction.y);
			}
			break;

		case LVN_COLUMNCLICK:		// Clic sur la colonne => tri des chanes
			if (wParam==IDC_CHANNEL_LIST)  {
				// Tri de la liste par clic sur un en-tte de colonne :
				static BOOL		tblSortDir[_countof(t_cols)] = {0};
				LPNMLISTVIEW	pnml = ((LPNMLISTVIEW)lParam);
				int				iCol = pnml->iSubItem;	// N de colonne concerne
				BOOL &			bColFlag = tblSortDir[iCol];

				pnml->lParam = bColFlag;	// on rquisitionne ce champ pour transmettre le sens du tri
				bColFlag = !bColFlag;		// le tri suivant sera dans le sens inverse
				ListView_SortItemsEx(hListItem, LVCompareProc, lParam);
				PropSheet_Changed(GetParent(hDlg), hDlg);
				iItemEdit = -1;
			}
			break;

		case PSN_APPLY: {			// Application des changements
			int nbItems = ListView_GetItemCount(hListItem);

			for (i=0; i<nbItems; i++) {
				LVITEM lvi = {
					LVIF_TEXT | LVIF_PARAM,	// mask
					i,						// iItem
					0,						// iSubItem
					0, 0,					// state, stateMask,
					buf,					// pszText,
					_countof(buf)			// cchTextMax
					// le reste  zro (implicite)
				};

				ListView_GetItem(hListItem, &lvi);

				int ixChaine = Canaux.trouve_par_cle(UINT32(lvi.lParam));

				if (ixChaine_ok(ixChaine)) {
					Chaine &	canal = Canaux[i];

					if (ixChaine != i) {
						// Permutation des deux canaux :
						Chaine temp = canal;
						Chaine & canal2 = Canaux[ixChaine];

						canal = canal2;
						canal2 = temp;
					}

					canal.nNumeroChaine = (UINT16)_tstoi(buf);
					lvi.iSubItem = CHAN_COL_STATE;
					ListView_GetItem(hListItem, &lvi);
					canal.etat = EtatChaine(str2dword(aChStateTable, buf, ec_preferee));
				}
			}

			// Actualiser l'index de la chane courante, puisque celui-ci a probablement t modifi :
			if (cleSauveChaineCourante!=0)
				ixChaineCourante = Canaux.trouve_par_cle(cleSauveChaineCourante);

			sauve_chaines();    // sauvegarde les chanes dans le fichier xml
			iItemEdit = -1;
			break; }
		}
		return FALSE;

	case WM_MOUSEMOVE:
		// Gestion du mouvement de la souris dans le glisser-dposer :
		if (m_Drag.get())
			m_Drag->Move(LOWORD(lParam), HIWORD(lParam));

		break;

	case WM_TIMER:
		if (wParam == TIMER_CHAN_LVDRAG && m_Drag.get())
			m_Drag->Scroll();
		break;

	case WM_LBUTTONUP:	{
		// Fin du processus de glisser-dposer
		int iItem = -1;

		if (m_Drag.get()) {
			iItem = m_Drag->End(LOWORD(lParam), HIWORD(lParam));
			m_Drag.reset();
		}

		if (iItem<0)
			break;

		// Rarrangement des items
		for (int iPos = ListView_GetNextItem(hListItem, -1, LVNI_SELECTED);
			iPos != -1; iPos = ListView_GetNextItem(hListItem, -1, LVNI_SELECTED)
		) {
			TCHAR buf[64];
			LVITEM lvi = {
				LVIF_STATE | LVIF_IMAGE | LVIF_INDENT | LVIF_PARAM | LVIF_TEXT,	// mask
				iPos,					// iItem
				0,						// iSubItem
				0,						// state
				(UINT)~LVIS_SELECTED,	// stateMask
				buf,					// pszText
				_countof(buf),			// cchTextMax
				// le reste  zro (implicite)
			};

			// Premirement, copier un item
			ListView_GetItem(hListItem, &lvi);
			lvi.iItem = iItem;
			// Insrer l'item principal
			int iRet = ListView_InsertItem(hListItem, &lvi);
			if (lvi.iItem < iPos)
				iItem++;
			if (iRet <= iPos)
				iPos++;
			// Insrer le texte des sous-items :
			for (i = 1; i < _countof(t_cols); i++) {
				ListView_GetItemText(hListItem, iPos, i, buf, _countof(buf));
				ListView_SetItemText(hListItem, iRet, i, buf);
			}
			// Supprimer de la position initiale
			ListView_DeleteItem(hListItem, iPos);
		}
		PropSheet_Changed(GetParent(hDlg), hDlg);
		break; }

	case WM_CONTEXTMENU: {
		POINT	sPt = {GET_X_LPARAM(lParam), GET_Y_LPARAM(lParam)};
		INT		iItem, iSubItem;

		if (FindListViewSubItem(hListItem, sPt, false, iItem, iSubItem) && iSubItem == CHAN_COL_STATE) {
			// Change l'tat de la chane : Active / Inactive / Prfre
			static const struct Tab_states {
				LPCTSTR nom;
				UINT    uID;
			} t_states[] = {
				{TEXT(""), IDM_CHAN_INACTIVE},
				{TEXT(""),	IDM_CHAN_ACTIVE},
				{TEXT(""), IDM_CHAN_PREFERRED}
			};

			EtatChaine	etat		= ItemToEtat(hListItem, iItem);
			HMENU		hPopMenu	= CreatePopupMenu();

			for (i=0; i<_countof(t_states); i++) {
				const Tab_states e_stat = t_states[i];

				AppendMenu(hPopMenu, i==etat ? MF_STRING|MF_CHECKED : MF_STRING,
					e_stat.uID, e_stat.nom);
			}
			iItemEdit = iItem;    // se souvenir quel item on traite
			TrackPopupMenuEx(hPopMenu, TPM_LEFTALIGN | TPM_LEFTBUTTON, sPt.x, sPt.y, hDlg, NULL);

			DestroyMenu(hPopMenu);
		}
		break; }
	}
	return FALSE;
}
/** @} */

// ====================================================================================
// Dialogue de proprits du tuner
// ====================================================================================

class CTunerPropsDialog : public CModalDialogBase
{
	LPCTSTR			pszNomTuner;
	TunerProps &	sProps;

	/// Traitement des messages de la bote de dialogue
	virtual INT_PTR DialogProc(UINT uMsg, WPARAM wParam, LPARAM lParam);

	/// Construction du nom du dialogue incluant le nom du tuner
	void SetWindowName();

	/// Construction de la zone d'information sur le tuner utilis
	void SetTunerInfo();

	/// Activation / dsactivation de l'option "Si chec, essayer dcalage +5 kHz"
	void AdjustAdd5kState();

public:

	/// Constructeur :
	CTunerPropsDialog(LPCTSTR pszNomTunr, TunerProps & sPrps) :
		CModalDialogBase(IDD_TUNERPROPS),
		pszNomTuner(pszNomTunr),
		sProps(sPrps)
	{
	}
};

/// Construction du nom du dialogue incluant le nom du tuner
void CTunerPropsDialog::SetWindowName()
{
	TCHAR	szWrk[256];

	get_text(hDlg, szWrk);
	strcat_T(szWrk, TEXT(" "));
	strcat_T(szWrk, pszNomTuner);
	strcat_T(szWrk, TEXT(""));

	set_text(hDlg, szWrk);
}

/// Construction de la zone d'information sur le tuner utilis
void CTunerPropsDialog::SetTunerInfo()
{
	NamesAndDevPaths	sNnP;
	bool				bOk = CRegControlSet().GetAllNamesAndPaths(sProps, sNnP);
	RegFilterInfo		sInfo;
	tstring				strWrk;

	if (
		bOk && !sNnP.strTunerPath.empty() &&
		CRegControlSet().GetDeviceInfo(sNnP.strTunerPath.c_str(), sInfo)
	) {
		// Espace + tiret cadratin + espace inscable
		#ifdef _UNICODE
			#define INFO_SEP L" \x2014\x00a0"
		#else
			#define INFO_SEP " \x97\xa0"
		#endif

		if (!sInfo.strDriverDesc.empty())
			strWrk  = INFO_SEP TEXT("\xa0: ") + sInfo.strDriverDesc;
		if (!sInfo.strProviderName.empty())
			strWrk += INFO_SEP TEXT("\xa0: ") + sInfo.strProviderName;
		if (!sInfo.strDriver.empty())
			strWrk += INFO_SEP TEXT("\xa0: ") + sInfo.strDriver;
		if (!sInfo.strDriverVersion.empty())
			strWrk += INFO_SEP TEXT("  \xa0: ") + sInfo.strDriverVersion;
		if (!sInfo.strDriverDate.empty())
			strWrk += INFO_SEP TEXT("\xa0: ") + sInfo.strDriverDate;
		if (!sNnP.strCaptureName.empty()) {
			if (!sNnP.strDemodName.empty())
				strWrk += INFO_SEP TEXT("Demodulator filter\xa0: ") + sNnP.strDemodName;
			strWrk += INFO_SEP TEXT("Capture filter\xa0: ") + sNnP.strCaptureName;
		} else
			strWrk += INFO_SEP TEXT("  ");
		strWrk += TEXT(".");

		// Chargement du contrle statique, en ignorant les trois premiers caractres, qui sont des
		// sparateurs par rapport  un item prcdent qui n'existe pas en dbut de chane.
		SetItemText(IDC_TUNER_INFO, strWrk.c_str()+3);

		#undef INFO_SEP
	}
}

/// Activation / dsactivation de l'option "Si chec, essayer dcalage +5 kHz"
void CTunerPropsDialog::AdjustAdd5kState()
{
	// L'option n'est pas fonctionnelle si les deux cases "Ne pas se fier  l'indicateur
	// et de prsence du signal" et "Ne pas se fier  l'indicateur de signal verrouill"
	// sont simultanment coches
	bool bCanAdd5khz =
			!get_check(hDlg, IDC_SCAN_IGNORE_PRESENCE) || !get_check(hDlg, IDC_SCAN_IGNORE_LOCK);

	set_check(hDlg, IDC_TRY_PLUS5KHZ, sProps.try_adding_5khz && bCanAdd5khz);
	EnableDlgItem(IDC_TRY_PLUS5KHZ, bCanAdd5khz);
}

/// Traitement des messages de la bote de dialogue
INT_PTR CTunerPropsDialog::DialogProc(UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch (uMsg) {

	case WM_INITDIALOG: {
		// Nom de la fentre
		SetWindowName();

		// "Correction de frquence (kHz)"
		InitUpDownItems(IDC_TUNER_OFFSET, IDC_TUNER_OFFSET_UPDOWN, sProps.offset_tuner, -999, 999);

		// "Ne pas se fier  l'indicateur de prsence du signal"
		set_check(hDlg, IDC_SCAN_IGNORE_PRESENCE, sProps.scan_ignore_presence);
		// "Ne pas se fier  l'indicateur de signal verrouill"
		set_check(hDlg, IDC_SCAN_IGNORE_LOCK, sProps.scan_ignore_lock);
		// "Ne pas se fier  l'indicateur de qualit du signal"
		set_check(hDlg, IDC_SCAN_IGNORE_QUALITY, sProps.scan_ignore_quality);

		// Slection "Si chec, essayer dcalage +5 kHz"
		AdjustAdd5kState();

		// "Temps avant test de prsence et verrouillage du signal"
		InitUpDownItems(IDC_SCAN_DELAY1, IDC_SCAN_DELAY1_UPDOWN, sProps.delai_tuner1, 0, 10000);

		// "Temps de stabilisation avant test qualit du signal et analyse"
		InitUpDownItems(IDC_SCAN_DELAY2, IDC_SCAN_DELAY2_UPDOWN, sProps.delai_tuner2, 0, 10000);

		// Calibration de la force du signal :
		InitUpDownItems(IDC_SIGNAL_ZERO, IDC_SIGNAL_ZERO_UPDOWN, sProps.force_signal_zero, -999999, 999999);
		InitUpDownItems(IDC_SIGNAL_MAX, IDC_SIGNAL_MAX_UPDOWN, sProps.force_signal_max, 1, 999999);

		// "Convertir en pourcentage" :
		CheckRadioButton(hDlg, IDC_SIGNAL_DISP_REAL, IDC_SIGNAL_DISP_PERCENT,
			IDC_SIGNAL_DISP_REAL + sProps.force_signal_aff_pct);

		// Resyntonisaton automatique en cas d'erreur du graphe
		set_check(hDlg, IDC_AUTO_RETUNE, sProps.auto_resyntonisation);

		// Informations du tuner
		SetTunerInfo();
		return TRUE; }

	case WM_NOTIFY:
		if (wParam==IDC_SCAN_DELAY1_UPDOWN || wParam==IDC_SCAN_DELAY2_UPDOWN)
			AdjustDelayUpDown(*reinterpret_cast<LPNMUPDOWN>(lParam));
		return TRUE;

	case WM_COMMAND:
		// Commande reue (clic, etc...)
		switch(wParam) {

		case _cmd_(IDC_USE_DEFAULTS, BN_CLICKED):
			if (MessageBox(hDlg,
				TEXT("          .") EOL
				TEXT(" "),
				TEXT("  "), MB_YESNO)==IDYES)
			{
				SetDlgItemInt(hDlg, IDC_TUNER_OFFSET, 0, TRUE);
				set_check(hDlg, IDC_TRY_PLUS5KHZ, INI_TRY_ADDING_5KHZ_DEFAULT);
				set_check(hDlg, IDC_SCAN_IGNORE_PRESENCE, false);
				set_check(hDlg, IDC_SCAN_IGNORE_LOCK, false);
				set_check(hDlg, IDC_SCAN_IGNORE_QUALITY, false);

				SetDlgItemInt(hDlg, IDC_SCAN_DELAY1, (UINT)INI_DELAI_TUNER1_DEFAULT, TRUE);
				SetDlgItemInt(hDlg, IDC_SCAN_DELAY2, (UINT)INI_DELAI_TUNER2_DEFAULT, TRUE);

				SetDlgItemInt(hDlg, IDC_SIGNAL_ZERO, 0, TRUE);
				SetDlgItemInt(hDlg, IDC_SIGNAL_MAX, 100, FALSE);
				CheckRadioButton(hDlg, IDC_SIGNAL_DISP_REAL, IDC_SIGNAL_DISP_PERCENT, IDC_SIGNAL_DISP_REAL);
				set_check(hDlg, IDC_AUTO_RETUNE, true);
			}
			return TRUE;

		case _cmd_(IDC_SCAN_IGNORE_PRESENCE, BN_CLICKED):
		case _cmd_(IDC_SCAN_IGNORE_LOCK, BN_CLICKED):
			AdjustAdd5kState();
			return TRUE;

		case _cmd_(IDC_TUNER_OFFSET, EN_CHANGE):
		case _cmd_(IDC_SCAN_DELAY1, EN_CHANGE):
		case _cmd_(IDC_SCAN_DELAY2, EN_CHANGE):
		case _cmd_(IDC_SIGNAL_ZERO, EN_CHANGE):
		case _cmd_(IDC_SIGNAL_MAX, EN_CHANGE):
			CheckUpDownValue(wParam);
			return TRUE;

		case _cmd_(IDOK, BN_CLICKED):
			// Clic sur le bouton OK
			get_int(hDlg, IDC_TUNER_OFFSET, sProps.offset_tuner, TRUE);
			sProps.try_adding_5khz = get_check(hDlg, IDC_TRY_PLUS5KHZ);
			get_int(hDlg, IDC_SCAN_DELAY1, sProps.delai_tuner1, FALSE);
			get_int(hDlg, IDC_SCAN_DELAY2, sProps.delai_tuner2, FALSE);

			sProps.scan_ignore_presence = get_check(hDlg, IDC_SCAN_IGNORE_PRESENCE);
			sProps.scan_ignore_lock = get_check(hDlg, IDC_SCAN_IGNORE_LOCK);
			sProps.scan_ignore_quality = get_check(hDlg, IDC_SCAN_IGNORE_QUALITY);

			get_int(hDlg, IDC_SIGNAL_ZERO, sProps.force_signal_zero, TRUE);
			get_int(hDlg, IDC_SIGNAL_MAX, sProps.force_signal_max, FALSE);
			sProps.force_signal_aff_pct	= get_check(hDlg, IDC_SIGNAL_DISP_PERCENT);
			sProps.auto_resyntonisation	= get_check(hDlg, IDC_AUTO_RETUNE);
			EndDialog(hDlg, IDOK);
			return TRUE;

		case _cmd_(IDCANCEL, BN_CLICKED):
			// Clic sur le bouton Annuler
			EndDialog(hDlg, IDCANCEL);
			return TRUE;
		}
	}

	// Autre message, trait par Windows
	return FALSE;
}

// ====================================================================================
// Dialogue d'options vido initiales
// ====================================================================================
/** \defgroup DriversDialogProc Fonctions pour fentre DriversDialogProc
 * Fonctions pour grer la fentre de configuration des codecs, tuners
 * @{ */

class CVideoOptionsDialog : public CModalDialogBase
{
	/// Traitement des messages de la bote de dialogue
	virtual INT_PTR DialogProc(UINT uMsg, WPARAM wParam, LPARAM lParam);

public:

	/// Constructeur :
	CVideoOptionsDialog() :
		CModalDialogBase(IDD_VIDEOOPTIONS)
	{
	}
};


/// Traitement des messages de la bote de dialogue
INT_PTR CVideoOptionsDialog::DialogProc(UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch (uMsg) {

	case WM_INITDIALOG:
		// Mthodes de rendu vido :
		SetVMRSel(hDlg, vmr_mode);

		// Sous Vista, la configuration OSD n'est valide qu'en mode VMR9
		SetOSDState(*this);
		set_check(hDlg, IDC_USE_OSD, use_osd);
		return TRUE;

	case WM_COMMAND:
		// Commande reue (clic, etc...)
		switch(LOWORD(wParam)) {

		case _cmd_(IDC_OSD_COLOR, BN_CLICKED):
			ChooseOsdColor(hDlg);
			return 0;

		case IDOK:
			// Clic sur le bouton OK
			vmr_mode	= GetVMRSel(hDlg);
			use_osd		= get_check(hDlg, IDC_USE_OSD);
			EndDialog(hDlg, IDOK);
			return TRUE;

		case IDCANCEL:
			// Clic sur le bouton Annuler
			EndDialog(hDlg, IDCANCEL);
			return TRUE;
		}
	}

	// Autre message, trait par Windows
	return FALSE;
}

/** @} */

// ====================================================================================
// Config des pilotes
// ====================================================================================
/** \defgroup DriversDialogProc Fonctions pour fentre DriversDialogProc
 * Fonctions pour grer la fentre de configuration des codecs, tuners
 * @{ */

class CFiltersDialog : public CModalDialogBase
{
	/// Raisons possibles de prsences dans la liste d'exclusion
	enum ExclusionReason
	{
		er_NoReason,
		er_Unknown,
		er_Protected,
		er_Incompatible
	};

	/// Item de la liste d'exclusion
	struct ExclusionItem
	{
		LPCTSTR			pszFilterName;
		ExclusionReason	eReason;
	};

	/// Liste d'exclusion pour les codecs vido Mpeg2
	static const ExclusionItem aMp2vExclusionList[];

	/// Liste d'exclusion pour les codecs vido H264
	static const ExclusionItem aH264ExclusionList[];

	/// Liste d'exclusion pour les codecs audio Mpeg2
	static const ExclusionItem aMp2aExclusionList[];

	/// Liste d'exclusion pour les codecs audio AC3
	static const ExclusionItem aAc3ExclusionList[];

	/// Liste d'exclusion pour les codecs audio E-AC3
	static const ExclusionItem aEac3ExclusionList[];

	/**
	 * Slection du nom dans une liste droulante, avec gestion de l'item "Aucun".
	 *
	 * Si \p nom n'est pas dans la liste, il y est ajout  la fin et slectionn.
	 * \param[in]	hItm		handle de la liste droulante recevant les noms de filtres
	 * \param[in]	nom		nom de l'lment  slectionner
	  * \return		\p true, ou bien \p false si le nom  slectionner n'tait pas dans
	 *				la liste et a donc d tre ajout.
	 **/
	static bool select_combo_name(HWND hItm, LPCTSTR nom);

	/// Comparaison insensible  la casse de deux chanes \p tstring
	static bool str_i_comp(tstring & elem1, tstring & elem2);

	/**
	 * Transfert d'une liste de filtres place dans une table de chanes de caractres
	 * vers une liste droulante. La liste est pralablement trie, et un item "Aucun"
	 * est ajout au dbut de la liste droulante, et l'lment dont le nom est donn
	 * en paramtre est slectionn. Si celui-ci n'existe plus, il est ajout  la
	 * liste et slectionn, mais la fonction retourne \p false
	 *
	 * \param[in]	vStrFiltres	table contenant les noms de filtres
	 * \param[in]	hItm		handle de la liste droulante recevant les noms de filtres
	 * \param[in]	nom			nom de l'lment  slectionner
	 * \return		\p true, ou bien \p false si le filtre  slectionner n'tait pas dans
	 *				la table et a donc d tre ajout.
	 **/
	static bool vectorstr_to_combo(vector<tstring> & vStrFiltres, HWND hItm, LPCTSTR nom);

	/**
	 * Remplit la liste droulante contenant les tuners, et y slectionne le tuner
	 * prcdemment dfini. Si celui-ci n'existe plus, il est ajout  la liste et
	 * slectionn, mais la fonction retourne \p false.
	 * \param[in]	bLegacy		\p true si ancien graphe  utiliser.
	 * \return		\p true ou \p false selon succs ou chec
	 **/
	bool remplit_combo_tuner(bool bLegacy);

	/**
	 * Remplit la liste droulante contenant les filtres, et y slectionne le filtre
	 * prcdemment dfini. Si celui-ci n'existe plus, il est ajout  la liste et
	 * slectionn, mais la fonction retourne \p false.
	 *
	 * \param[in]	hItm	HANDLE de la liste droulante dans la boite de dialogue
	 * \param[in]	type	\p GUID permettant d'effectuer la recherche des filtres installs (AUDIO, VIDEO, etc.)
	 * \param[in]	subtype	\p GUID permettant d'affiner la recherche (MPRG2_AUDIO, AC3, etc.)
	 * \param[in]	nom		nom du filtre prcdement slectionn
	 * \retval		false	si le prcdent tuner n'a pas pu tre trouv
	 * \return				La position dans la liste droulante
	 **/
	static bool remplit_combo_filtre(HWND hItm, GUID type, GUID subtype, LPCTSTR nom);

	/**
	 * \brief Validation d'un codec avec test de la liste d'exclusion
	 **/
	void ValidateCodec(HWND hCtl, LPCTSTR pszCategory, const ExclusionItem * paExclList, LPCTSTR pszPrevious);

protected:
	DeviceInfoList	vTunerList;
	HWND			hTuner;
	HWND			hMpeg2;
	HWND			hH264;
	HWND			hAudio;
	HWND			hAc3;
	HWND			hEac3;
	bool			bWarnFfdshowDone;

	/// Traitement des messages de la bote de dialogue
	virtual INT_PTR DialogProc(UINT uMsg, WPARAM wParam, LPARAM lParam);

	/**
	 * Met  jour les diffrents paramtres des drivers, et dtermine si il y a eu une modification ou pas
	 *
	 * \retval		true		un lment a t modifi
	 * \retval		false		aucune modification
	 **/
	bool drivers_apply();

public:
	/// Constructeur :
	CFiltersDialog() :
		CModalDialogBase(IDD_FILTERS_INIT),
		hTuner(NULL),
		hMpeg2(NULL),
		hH264(NULL),
		hAc3(NULL),
		hEac3(NULL),
		bWarnFfdshowDone(false)
	{
	}
};

/// Liste d'exclusion pour les codecs vido Mpeg2
const CFiltersDialog::ExclusionItem CFiltersDialog::aMp2vExclusionList[] =
{
	{TEXT("Nero DVD Decoder"), er_Protected},
	{TEXT("Nero Mpeg Video Decoder"), er_Protected},
	{TEXT("Nero Video Decoder"), er_Protected},
	{TEXT("Nero Video Decoder HD"), er_Protected},
	{0}
};

/// Liste d'exclusion pour les codecs vido H264
const CFiltersDialog::ExclusionItem CFiltersDialog::aH264ExclusionList[] =
{
	{0}
};

/// Liste d'exclusion pour les codecs audio Mpeg2
const CFiltersDialog::ExclusionItem CFiltersDialog::aMp2aExclusionList[] =
{
	{0}
};

/// Liste d'exclusion pour les codecs audio AC3
const CFiltersDialog::ExclusionItem CFiltersDialog::aAc3ExclusionList[] =
{
	{TEXT("Microsoft DTV-DVD Audio Decoder"), er_Protected},
	{0}
};

/// Liste d'exclusion pour les codecs audio E-AC3
const CFiltersDialog::ExclusionItem CFiltersDialog::aEac3ExclusionList[] =
{
	{TEXT("Microsoft DTV-DVD Audio Decoder"), er_Protected},
	{0}
};

/**
 * Slection du nom dans une liste droulante, avec gestion de l'item "Aucun".
 *
 * Si \p nom n'est pas dans la liste, il y est ajout  la fin et slectionn.
 * \param[in]	hItm		handle de la liste droulante recevant les noms de filtres
 * \param[in]	nom		nom de l'lment  slectionner
  * \return		\p true, ou bien \p false si le nom  slectionner n'tait pas dans
 *				la liste et a donc d tre ajout.
 **/
bool CFiltersDialog::select_combo_name(HWND hItm, LPCTSTR nom)
{
	// Slection de l'item "nom"
	if (IsNullName(nom))
		SendMessage(hItm, CB_SETCURSEL, 0, 0);
	else if (SendMessage(hItm, CB_SELECTSTRING, (WPARAM)-1, (LPARAM)nom)==CB_ERR) {
		// L'lment prcdemment slectionn n'existe plus ! On l'ajoute quand mme,
		// avec un inquitant <?>  en prfixe :
		tstring strMissing = TEXT("<?> ");

		strMissing += nom;
		SendMessage(hItm, CB_ADDSTRING, 0, (LPARAM)strMissing.c_str());
		// ... et on le slectionne :
		SendMessage(hItm, CB_SELECTSTRING, (WPARAM)-1, (LPARAM)strMissing.c_str());
		return false;
	}
	return true;
}

/// Comparaison insensible  la casse de deux chanes \p tstring
bool CFiltersDialog::str_i_comp(tstring & elem1, tstring & elem2)
{
	return _tcsicmp(elem1.c_str(), elem2.c_str()) < 0;
}

/**
 * Transfert d'une liste de filtres place dans une table de chanes de caractres
 * vers une liste droulante. La liste est pralablement trie, et un item "Aucun"
 * est ajout au dbut de la liste droulante, et l'lment dont le nom est donn
 * en paramtre est slectionn. Si celui-ci n'existe plus, il est ajout  la
 * liste et slectionn, mais la fonction retourne \p false
 *
 * \param[in]	vStrFiltres	table contenant les noms de filtres
 * \param[in]	hItm		handle de la liste droulante recevant les noms de filtres
 * \param[in]	nom			nom de l'lment  slectionner
 * \return		\p true, ou bien \p false si le filtre  slectionner n'tait pas dans
 *				la table et a donc d tre ajout.
 **/
bool CFiltersDialog::vectorstr_to_combo(vector<tstring> & vStrFiltres, HWND hItm, LPCTSTR nom)
{
	// Tri de la table
	std::sort(vStrFiltres.begin(), vStrFiltres.end(), str_i_comp);

	// Ajout de l'item "Aucun" en dbut de liste
	SendMessage(hItm, CB_ADDSTRING, 0, reinterpret_cast<LPARAM>(TEXT("")));

	// Ajout des items tris, juste derrire
	for (ITERATE_CONST_VECTOR(vStrFiltres, tstring, it))
		SendMessage(hItm, CB_ADDSTRING, 0, reinterpret_cast<LPARAM>(it->c_str()));

	// Slection de l'item "nom"
	return select_combo_name(hItm, nom);
}

/**
 * Remplit la liste droulante contenant les tuners, et y slectionne le tuner
 * prcdemment dfini. Si celui-ci n'existe plus, il est ajout  la liste et
 * slectionn, mais la fonction retourne \p false.
 *
 * \param[in]	bLegacy		\p true si ancien graphe  utiliser.
 * \return		\p true ou \p false selon succs ou chec
 **/
bool CFiltersDialog::remplit_combo_tuner(bool bLegacy)
{
	// Remplissage de la table des tuners disponibles
	vTunerList.clear();	// Nettoyage de tout contenu antrieur (prcaution)
	CSearchTuner(vTunerList, bLegacy).Do();

	// Rinitialisation de la combo box
	SendMessage(hTuner, CB_RESETCONTENT, 0, NULL);

	// Ajout des items tris, dans la combo box
	for (ITERATE_CONST_VECTOR(vTunerList, DeviceInfo, it))
		SendMessage(hTuner, CB_ADDSTRING, 0, reinterpret_cast<LPARAM>(it->TunerDisplayName().c_str()));

	// Slection du nom du tuner
	return select_combo_name(hTuner, nom_tuner);
}

/**
 * Remplit la liste droulante contenant les filtres, et y slectionne le filtre
 * prcdemment dfini. Si celui-ci n'existe plus, il est ajout  la liste et
 * slectionn, mais la fonction retourne \p false.
 *
 * \param[in]	hItm	HANDLE de la liste droulante dans la boite de dialogue
 * \param[in]	type	\p GUID permettant d'effectuer la recherche des filtres installs (AUDIO, VIDEO, etc.)
 * \param[in]	subtype	\p GUID permettant d'affiner la recherche (MPRG2_AUDIO, AC3, etc.)
 * \param[in]	nom		nom du filtre prcdement slectionn
 * \retval		false	si le prcdent tuner n'a pas pu tre trouv
 * \return				La position dans la liste droulante
 **/
bool CFiltersDialog::remplit_combo_filtre(HWND hItm, GUID type, GUID subtype, LPCTSTR nom)
{
	vector<tstring>	vStrFiltres;

	CSearchByType_FillVector(type, subtype, exact_match, vStrFiltres).Do();

	return vectorstr_to_combo(vStrFiltres, hItm, nom);
}

/**
 * \brief Validation d'un codec avec test de la liste d'exclusion
 **/
void CFiltersDialog::ValidateCodec(HWND hCtl, LPCTSTR pszCategory, const ExclusionItem * paExclList, LPCTSTR pszPrevious)
{
	SZFilterName	szTmpFilter;

	if (extrait_combo(hCtl, szTmpFilter, _countof(szTmpFilter), true)) {
		while (paExclList->pszFilterName) {
			if (CompareString(LOCALE_NEUTRAL, NORM_IGNORECASE, paExclList->pszFilterName, -1, szTmpFilter, -1) == CSTR_EQUAL) {
				static LPCTSTR apszReasons[] = {
					TEXT("< >"),
					TEXT("  "),
					TEXT("   "),
					TEXT("   ")
				};

				if (MessageBox(hDlg,
					tstr_printf(
					TEXT("  %s   %s  ,  %s,   %s.") EOL
						TEXT("       ?"),
						pszCategory,
						szTmpFilter,
						apszReasons[paExclList->eReason],
						szAppName).c_str(),
					TEXT("  "), MB_YESNO |MB_ICONWARNING | MB_DEFBUTTON2) == IDNO
				) {
					select_combo_name(hCtl, pszPrevious);
					return;
				}
				break;
			}
			++paExclList;
		}
		PropSheet_Changed(GetParent(hDlg), hDlg);
	}
}

/**
 * Met  jour les diffrents paramtres des drivers, et dtermine si il y a eu une modification ou pas
 *
 * \retval		true		un lment a t modifi
 * \retval		false		aucune modification
 **/
bool CFiltersDialog::drivers_apply()
{
	bool	changed	= false;
	INT		nIdx	= (INT)SendMessage(GetDlgItem(hDlg, IDC_COMBO_TUNER), CB_GETCURSEL, 0, 0);

	if (nIdx >= 0 && nIdx < (INT)vTunerList.size()) {
		const DeviceInfo &	sNewTunerConf	= vTunerList[nIdx];
		tstring				strNewTuner		= sNewTunerConf.TunerDisplayName();

		changed	 = copy_check(nom_tuner,				_countof(nom_tuner),				strNewTuner.c_str());

		load_config_tuner(nom_tuner);	// Charger les paramtres de ce tuner

		changed |= copy_check(sTunProps.nom_receiver,	_countof(sTunProps.nom_receiver),	sNewTunerConf.ReceiverDisplayName().c_str());
		changed |= copy_check(sTunProps.szPathTuner,	_countof(sTunProps.szPathTuner),	sNewTunerConf.asDev[dt_Tuner].strPath.c_str());
		changed |= copy_check(sTunProps.szPathDemod,	_countof(sTunProps.szPathDemod),	sNewTunerConf.asDev[dt_Demod].strPath.c_str());
		changed |= copy_check(sTunProps.szPathCapture,	_countof(sTunProps.szPathCapture),	sNewTunerConf.asDev[dt_Capture].strPath.c_str());
	}

	// Vrifie si des paramtres ont t changs, et rcupre les nouvelles versions
	if (get_check(hDlg, IDC_ALT_GRAPH)!=autre_graphe) {
		autre_graphe = !autre_graphe;
		changed = true;
	}
	changed |= extrait_combo(hMpeg2, filtreMPEG2, _countof(filtreMPEG2), true);
	changed |= extrait_combo(hH264, filtreH264, _countof(filtreH264), true);
	changed |= extrait_combo(hAudio, filtreAudio, _countof(filtreAudio), true);
	changed |= extrait_combo(hAc3, filtreAc3, _countof(filtreAc3), true);
	changed |= extrait_combo(hEac3, filtreEac3, _countof(filtreEac3), true);

	myprintf(
		TEXT("Nouvelle configuration :") EOL
		TEXT("Tuner    : %s") EOL
		TEXT("Video    : %s") EOL
		TEXT("- H264   : %s") EOL
		TEXT("Audio    : %s") EOL
		TEXT("- Ac3    : %s") EOL
		TEXT("- Eac3   : %s") EOL,
		nom_tuner,
		filtreMPEG2, filtreH264,
		filtreAudio, filtreAc3, filtreEac3);

	if (changed && _tcsstr(filtreMPEG2, TEXT("ffdshow")) && _tcsstr(filtreH264, TEXT("ffdshow"))) {
		MessageBoxCheck(hDlg,
			TEXT(" ffdshow     MPEG2 ")
			TEXT(" H264,          ")
			TEXT("multiple instances,      . ")
			TEXT("   0x80004005     ."),
			TEXT("    ffdshow"), MB_OK|MB_ICONWARNING,
			MB_OK, TEXT("  ffdshow"), &bWarnFfdshowDone);
	}
	return changed;
}

/**
 * Fonction de traitement des vnements pour la fentre des drivers
 **/
INT_PTR CFiltersDialog::DialogProc(UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch (uMsg) {

	case WM_INITDIALOG:	 {	// Initialise les valeurs
		hTuner    = GetDlgItem(hDlg, IDC_COMBO_TUNER);
		hMpeg2    = GetDlgItem(hDlg, IDC_COMBO_MPEG2);
		hH264     = GetDlgItem(hDlg, IDC_COMBO_H264);
		hAudio    = GetDlgItem(hDlg, IDC_COMBO_AUDIO);
		hAc3      = GetDlgItem(hDlg, IDC_COMBO_AC3);
		hEac3     = GetDlgItem(hDlg, IDC_COMBO_EAC3);
		set_check(hDlg, IDC_ALT_GRAPH, autre_graphe);

		// Liste des filtres dfinis dans la configuration mais qui ne sont plus disponibles
		// sur l'ordinateur :
		tstring	strListeAbsents;

		// Remplissage de la table des tuners disponibles
		if (!remplit_combo_tuner(autre_graphe))
			strListeAbsents = tstr_printf(EOL TEXT("- %s"), nom_tuner);
		if (!remplit_combo_filtre(hMpeg2, MEDIATYPE_Video, MEDIASUBTYPE_MPEG2_VIDEO, filtreMPEG2))
			strListeAbsents += tstr_printf(EOL TEXT("- %s"), filtreMPEG2);
		if (!remplit_combo_filtre(hH264, MEDIATYPE_Video, MEDIASUBTYPE_H264, filtreH264))
			strListeAbsents += tstr_printf(EOL TEXT("- %s"), filtreH264);
		if (!remplit_combo_filtre(hAudio, MEDIATYPE_Audio, MEDIASUBTYPE_MPEG2_AUDIO, filtreAudio))
			strListeAbsents += tstr_printf(EOL TEXT("- %s"), filtreAudio);
		if (!remplit_combo_filtre(hAc3, MEDIATYPE_Audio, MEDIASUBTYPE_DOLBY_AC3, filtreAc3))
			strListeAbsents += tstr_printf(EOL TEXT("- %s"), filtreAc3);
		if (!remplit_combo_filtre(hEac3, MEDIATYPE_Audio, MEDIASUBTYPE_DOLBY_DDPLUS, filtreEac3))
			strListeAbsents += tstr_printf(EOL TEXT("- %s"), filtreEac3);

		// Cherche s'il n'y a qu'un lment dans la liste
		// Le slectionne si oui
		selectionne_si_solo(hTuner);
		selectionne_si_solo(hMpeg2);
		selectionne_si_solo(hH264);
		selectionne_si_solo(hAudio);
		selectionne_si_solo(hAc3);
		selectionne_si_solo(hEac3);

		if (!strListeAbsents.empty()) {
			strListeAbsents.insert(0,
				TEXT("       .") EOL
				TEXT("        USB ") EOL
				TEXT("    :"));
			MessageBox(hDlg, strListeAbsents.c_str(), TEXT(" "),
				MB_OK|MB_ICONEXCLAMATION);
		}
		return TRUE; }

	case WM_COMMAND:

		switch(wParam) {

		// Changement dans un combo
		case _cmd_(IDC_COMBO_TUNER, CBN_SELCHANGE):
			PropSheet_Changed(GetParent(hDlg), hDlg);
			break;

		case _cmd_(IDC_COMBO_MPEG2, CBN_SELCHANGE):
			ValidateCodec(hMpeg2, TEXT("Vido Mpeg2"), aMp2vExclusionList, filtreMPEG2);
			break;

		case _cmd_(IDC_COMBO_H264, CBN_SELCHANGE):
			ValidateCodec(hH264, TEXT("Vido H264"), aH264ExclusionList, filtreH264);
			break;

		case _cmd_(IDC_COMBO_AUDIO, CBN_SELCHANGE):
			ValidateCodec(hAudio, TEXT("Audio Mpeg2"), aMp2aExclusionList, filtreAudio);
			break;

		case _cmd_(IDC_COMBO_AC3, CBN_SELCHANGE):
			ValidateCodec(hAc3, TEXT("Audio AC3"), aAc3ExclusionList, filtreAc3);
			break;

		case _cmd_(IDC_COMBO_EAC3, CBN_SELCHANGE):
			ValidateCodec(hEac3, TEXT("Audio E-AC3"), aEac3ExclusionList, filtreEac3);
			break;

		case _cmd_(IDC_TUNERPROPS, BN_CLICKED): {
			SZFilterName			szNomTunerLocal;

			extrait_combo(hTuner, szNomTunerLocal, _countof(szNomTunerLocal), false);

			auto_ptr<TunerProps>	pLocalProps(new TunerProps(sTunProps));	// Copie des paramtres du tuner

			if (pLocalProps.get()) {
				bool bCurrent = _tcscmp(nom_tuner, szNomTunerLocal) == 0;

				if (!bCurrent) {
					// Mthode "sauvage" pour charger (et ensuite sauvegarder) les paramtres d'un
					// tuner qui n'est pas le tuner courant, en attendant le jour ventuel o le
					// chargement et la sauvegarde de ces paramtres pourra s'effectuer sur des
					// donnes qui ne sont pas places dans des variables globales.
					load_config_tuner(szNomTunerLocal);
					swap(sTunProps, *pLocalProps);		// change avec les paramtres courants
				}
				if (CTunerPropsDialog(szNomTunerLocal, *pLocalProps).Do(hDlg) == IDOK) {
					if (!bCurrent) {
						swap(sTunProps, *pLocalProps);	// change avec les paramtres courants
						save_config_tuner(szNomTunerLocal);
						sTunProps = *pLocalProps;		// Remise en place des paramtres courants
					} else {
						sTunProps = *pLocalProps;		// criture des paramtres courants
						save_config_tuner(szNomTunerLocal);
					}
				}
			}
			break; }

		case _cmd_(IDC_ALT_GRAPH, BN_CLICKED):
			// Reconstruction de la liste des tuners (celle-ci pouvant diffrer si le type
			// de graphe est chang)
			remplit_combo_tuner(get_check(hDlg, IDC_ALT_GRAPH)!=0);
			PropSheet_Changed(GetParent(hDlg), hDlg);
			break;

		// Les trois cas suivants ne s'appliquent que lorsque le dialogue
		// ne fait PAS partie d'une feuille de proprits :
		case _cmd_(IDC_VIDEOOPTIONS, BN_CLICKED):
			CVideoOptionsDialog().Do(hDlg);
			break;

		case _cmd_(IDOK, BN_CLICKED):
			// sauvegarder le changement
			drivers_apply();
			// ... (passage  travers)
		case _cmd_(IDCANCEL, BN_CLICKED):

			vTunerList.clear(); // On n'a plus besoin de la liste des tuners
			EndDialog(hDlg, LOWORD(wParam));
			return TRUE;
		}
	}
	return FALSE;
}

/// Gestion de la page des pilotes et des filtres dans le dialogue de configuration
class CFiltersPropSheet : public CPropSheetPageBase, public CFiltersDialog
{
	/// Traitement des messages de la bote de dialogue
	virtual INT_PTR DialogProc(UINT uMsg, WPARAM wParam, LPARAM lParam);

};

/**
 * Fonction de traitement des vnements pour la fentre des drivers
 **/
INT_PTR CFiltersPropSheet::DialogProc(UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch (uMsg) {

	case WM_NOTIFY:
		switch (_ncode_(lParam)) {

		case PSN_KILLACTIVE: {
			SZFilterName	szWrk;	// Variable de travail

			strcpy_T(szWrk, nom_tuner);
			if (extrait_combo(hTuner, szWrk, _countof(szWrk), false) && !IsNullName(szWrk)) {
				if (ePSN_KILLACTIVE_discriminator == epkd_pagechange) {
					MessageBox(hDlg, TEXT("    .") EOL
						TEXT("       ."),
						TEXT(" "), MB_OK);
					return psn_result(TRUE);
				}
			}
			return psn_result(PSNRET_NOERROR); }

		case PSN_APPLY:			// Application des changements
			if (drivers_apply()) {
				save_config_tuner(nom_tuner);
				CConfigPropSheet::bNeedRestart = true;
			}
			return psn_result(PSNRET_NOERROR);

		case PSN_RESET:	// Destruction de la page de proprits
			vTunerList.clear(); // On n'a plus besoin de la liste des tuners
			return FALSE;
		}
		break;
	}

	return CFiltersDialog::DialogProc(uMsg, wParam, lParam);
}

/**
 * Appel du dialogue de configuration en mode normal en attendant l'implmentation
 * ventuelle d'un "wizard".
 *
 * \retval IDOK     si la configuration a t valide
 * \retval IDCANCEL si elle a t annule
 * \todo Implmenter ventuellement un "wizard" (assistant).
 **/
BYTE drivers_dialog()
{
	INT_PTR res = CFiltersDialog().Do(hMainWnd);
	save_config();		// sauvegarde la configuration
	return (BYTE) res;
}

/** @} */

// ====================================================================================
// Config du scan des chanes
// ====================================================================================
/** \defgroup CScanDialog Classe ScanDialogProc
 * Fonctions pour grer la fentre de recherche des chanes
 * @{ */

/// Structure utilise par la recherche du bloc-notes
struct SearchLogWindowRes {
	TCHAR	szName[256];
	HWND	hWnd;
	size_t	nLen;

	SearchLogWindowRes() :
		hWnd(NULL)
	{
		strcpy_T(szName, prefix_conf);
		strcat_T(szName, TEXT("scan.log"));

		nLen = _tcslen(szName);
	}
};

/// Fonction "callback" utilise par la recherche du bloc-notes
static BOOL CALLBACK EnumWindowsProc(HWND hWnd, LPARAM lParam)
{
	SearchLogWindowRes &	sRes = *reinterpret_cast<SearchLogWindowRes *>(lParam);
	TCHAR					szWrk[256];

	if (GetClassName(hWnd, szWrk, _countof(szWrk)) == 0)
		return TRUE;
	if (_tcsicmp(szWrk, TEXT("Notepad")) != 0)
		return TRUE;	// Ignorer toute fentre dont la classe n'est pas "Notepad"
	if (GetWindowText(hWnd, szWrk, _countof(szWrk)) == 0)
		return TRUE;
	if (_tcsncmp(szWrk, sRes.szName, sRes.nLen) != 0)
		return TRUE;	// Ignorer toute fentre dont le titre ne commence pas par notre
						// nom de journal (en tenant compte du prfixe ventuel d'instance)

	sRes.hWnd = hWnd;	// Retourner la "handle" de cette fentre
	return FALSE;		// Arrter la recherche
}

/// Recherche s'il existe une fentre "bloc-notes" ouverte avec notre fichier journal.
static HWND LookForNotepadWithScanLog()
{
	SearchLogWindowRes sRes;

	EnumWindows(EnumWindowsProc, reinterpret_cast<LPARAM>(&sRes));
	return sRes.hWnd;
}

class CScanListDialog : public CModalDialogBase
{
	tstring &	strResult;
	bool &		bScanLoop;

	/// Traitement des messages de la bote de dialogue
	virtual INT_PTR DialogProc(UINT uMsg, WPARAM wParam, LPARAM lParam);

public:

	/**
	 * Constructeur
	 *
	 * \param[in]	strRes	Rfrence sur chane destine  recevoir la liste
	 * \param[in]	bLoop	Rfrence sur variable destine  recevoir l'option de boucle
	 **/
	CScanListDialog(tstring & strRes, bool & bLoop) :
		CModalDialogBase(IDD_SCAN_FREQS),
		strResult(strRes),
		bScanLoop(bLoop)
	{
	}
};

/// Traitement des messages de la bote de dialogue
INT_PTR CScanListDialog::DialogProc(UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch (uMsg) {

	case WM_INITDIALOG:	// Initialisation de la fentre
		SetItemText(IDC_SCAN_CHANLIST, strResult.c_str());
		set_check(hDlg, IDC_SCAN_LOOP, bScanLoop);
		return TRUE;

	case WM_COMMAND:
		switch (wParam) {

		case _cmd_(IDOK, BN_CLICKED): {
			TCHAR szTmp[256];

			bScanLoop = get_check(hDlg, IDC_SCAN_LOOP);
			get_text(GetDlgItem(hDlg, IDC_SCAN_CHANLIST), szTmp);
			strResult = szTmp;
			EndDialog(hDlg, IDOK);			// IDOK = pas d'erreur
			return TRUE;
		}

		case _cmd_(IDCANCEL, BN_CLICKED):
			EndDialog(hDlg, IDCANCEL);
			return TRUE;
		}
	}

	return FALSE;
}

class CScanOptionsDialog : public CModalDialogBase
{
	/**
	 * Remplissage de la "combo box" de mthodes de rendu vido
	 * \param[in]	eCurrOfs	Position  slectionner
	 **/
	void	InitOffsetsCombo(OffsetsToUse eCurrOfs);

	/// Traitement des messages de la bote de dialogue
	virtual INT_PTR DialogProc(UINT uMsg, WPARAM wParam, LPARAM lParam);

public:

	CScanOptionsDialog() :
		CModalDialogBase(IDD_SCAN_OPTIONS)
	{
	}
};

/**
 * Remplissage de la "combo box" de mthodes de rendu vido
 * \param[in]	eCurrOfs	Position  slectionner
 **/
void CScanOptionsDialog::InitOffsetsCombo(OffsetsToUse eCurrOfs)
{
	static LPCTSTR aOffsetsInitList[] = {
		TEXT("-167"),
		TEXT("0"),
		TEXT("+167"),
		TEXT("+333"),
		TEXT("+500"),
		NULL
	};

	ComboBox_Init(GetDlgItem(hDlg, IDC_SCAN_OFFSETS), aOffsetsInitList, eCurrOfs);
}

/// Traitement des messages de la bote de dialogue
INT_PTR CScanOptionsDialog::DialogProc(UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch (uMsg) {

	case WM_INITDIALOG:	// Initialisation de la fentre
		if (methode_recherche>=sm_incremental)
			CheckRadioButton(hDlg, IDC_SCAN_INCREMENTAL, IDC_SCAN_FULL,
				IDC_SCAN_INCREMENTAL-sm_incremental+methode_recherche);

		InitUpDownItems(IDC_AUTONUM_UNDEF, IDC_AUTONUM_UNDEF_UPDOWN, autonum_undef, 1, 950);
		InitUpDownItems(IDC_AUTONUM_DUPLICATE, IDC_AUTONUM_DUPLICATE_UPDOWN, autonum_duplicate, 10, 950);
		InitOffsetsCombo(use_offsets);
		return TRUE;

	case WM_COMMAND:
		switch (wParam) {

		case _cmd_(IDC_TUNERPROPS, BN_CLICKED):
			if (CTunerPropsDialog(nom_tuner, sTunProps).Do(hDlg) == IDOK)
				save_config_tuner(nom_tuner);
			break;

		case _cmd_(IDC_SHOW_LOG, BN_CLICKED): {
			// Chercher si la fentre existe dj, pour ne pas l'ouvrir deux fois (en fait, on
			// cherche ici n'importe quelle fentre de bloc-notes en spcifiant son nom de classe).
			HWND	hLogWnd = LookForNotepadWithScanLog();

			if (hLogWnd == NULL)
				ShellOpen(hDlg, TEXT("Notepad"), NomFichierConf<>(TEXT("scan.log")));
			else {
				// Bloc-notes dj ouvert : on le ramne devant
				ShowWindow(hLogWnd, SW_RESTORE);
				BringWindowToTop(hLogWnd);
			}
			break; }

		case _cmd_(IDC_AUTONUM_UNDEF, BN_CLICKED):
		case _cmd_(IDC_AUTONUM_DUPLICATE, BN_CLICKED):
			CheckUpDownValue(wParam);
			return TRUE;

		case _cmd_(IDC_USE_DEFAULTS, BN_CLICKED):
			if (MessageBox(hDlg,
				TEXT("        .") EOL
				TEXT(" "),
				TEXT(""), MB_YESNO)==IDYES)
			{
				CheckRadioButton(hDlg, IDC_SCAN_INCREMENTAL, IDC_SCAN_FULL,
					IDC_SCAN_INCREMENTAL-sm_incremental+INI_METHODE_RECH_DEFAULT);

				SetDlgItemInt(hDlg, IDC_AUTONUM_UNDEF, (UINT)INI_AUTONUM_UNDEF_DEFAULT, TRUE);
				SetDlgItemInt(hDlg, IDC_AUTONUM_DUPLICATE, (UINT)INI_AUTONUM_DUP_DEFAULT, TRUE);
				SendDlgItemMessage(hDlg, IDC_SCAN_OFFSETS, CB_SETCURSEL, INI_USE_OFFSETS_DEFAULT, 0);
			}
			return TRUE;

		case _cmd_(IDCANCEL, BN_CLICKED):
			EndDialog(hDlg, IDCANCEL);
			return TRUE;

		case _cmd_(IDOK, BN_CLICKED):
			// Transfert des options depuis les items de la bote de dialogue vers
			// les variables respectives qui les concernent
			get_int(hDlg, IDC_AUTONUM_UNDEF, autonum_undef, FALSE);
			get_int(hDlg, IDC_AUTONUM_DUPLICATE, autonum_duplicate, FALSE);
			methode_recherche = (SearchMethod)get_radio_check(hDlg,
				IDC_SCAN_INCREMENTAL, IDC_SCAN_FULL, sm_prefs, sm_incremental);
			use_offsets = (OffsetsToUse)SendDlgItemMessage(hDlg,
				IDC_SCAN_OFFSETS, CB_GETCURSEL, 0, 0);

			EndDialog(hDlg, IDOK);			// IDOK = pas d'erreur
			return TRUE;
		}
	}

	return FALSE;
}

class CScanDialog : public CModalDialogBase
{
	auto_ptr<ChannelsScanner>
			pScan;
	UINT32	cleChaine;
	HWND	hCanauxItem;
	HWND	hChanText;
	HWND	hListItem;
	HWND	hOptBut;
	HWND	hStartBut;
	HWND	hCancelBut;
	HWND	hOkBut;
	HWND	hInfoText;
	HWND	hProgBar;
	HWND	hStatPText;
	HWND	hStatLText;
	HWND	hStatSText;
	HWND	hStatQText;
	HWND	hSLevelBar;
	HWND	hQLevelBar;
	int		cancelCode;
	int		nNewChannels;		//!< Nouvelle chanes trouves
	int		nNewChannelsTotal;	//!< Nouvelle chanes trouves (cumul pour cette session)
	tstring	strListe;			//!< Liste de canaux (copie temporaire)
	bool	bScanLoop;			//!< Option de recherche en boucle (non sauvegarde, dure de vie
								//!< intentionnellement limite  la session de recherche)

	/// Informations utilisateur rcupres et mises en attente
	CInfoUser		cInfoUser;

	/**
	 * Remplit la liste droulante contenant les listes de canaux  explorer du fichier canaux.ini
	 *
	 * \param[in]	nom		nom du filtre prcdement slectionn
	 * \retval		CB_ERR	si le prcdent tuner n'a pas pu tre trouv
	 * \return		La position dans la liste droulante
	 **/
	LRESULT			remplit_combo_canaux(LPCTSTR nom);

	UINT			extrait_canaux();

	void			set_info_methode();

	void			update_signal_items(const SignalStatistics & sSStats) const;

	/// Rinitialisation aprs recherche
	void			reset();


	/// Traitement des messages de la bote de dialogue
	virtual INT_PTR DialogProc(UINT uMsg, WPARAM wParam, LPARAM lParam);

public:

	/// Constructeur
	CScanDialog();
};

/// Constructeur
CScanDialog::CScanDialog() :
	CModalDialogBase(IDD_SCAN),
	pScan(NULL),
	cleChaine(0),
	hCanauxItem(NULL),
	hChanText(NULL),
	hListItem(NULL),
	hOptBut(NULL),
	hStartBut(NULL),
	hCancelBut(NULL),
	hOkBut(NULL),
	hInfoText(NULL),
	hProgBar(NULL),
	hStatPText(NULL),
	hStatLText(NULL),
	hStatSText(NULL),
	hStatQText(NULL),
	hSLevelBar(NULL),
	hQLevelBar(NULL),
	nNewChannels(0),
	nNewChannelsTotal(0),
	strListe(liste_canaux),
	bScanLoop(false)
{
	// Sauvegarde des prfrences utilisateur pour chaque chane :
	for (ITERATE_CONST_VECTOR(Canaux, Chaine, it))
		cInfoUser.push_back(*it);
}

/**
 * Remplit la liste droulante contenant les listes de canaux  explorer du fichier canaux.ini
 *
 * \param[in]	nom		nom du filtre prcdement slectionn
 * \retval		CB_ERR	si le prcdent tuner n'a pas pu tre trouv
 * \return		La position dans la liste droulante
 **/
LRESULT CScanDialog::remplit_combo_canaux(LPCTSTR nom)
{
	static LPCTSTR aScanInitList[] = {
		TEXT(" :    "), // Option pour scanner tous les canaux
		TEXT("    ..."),
		NULL
	};

	ComboBox_Init(hCanauxItem, aScanInitList);

	remplit_canaux(hCanauxItem);
	if (IsNullName(nom))
		return SendMessage(hCanauxItem, CB_SETCURSEL, 0, 0);
	if (_tcscmp(nom, LIST_NAME)==0)
		return SendMessage(hCanauxItem, CB_SETCURSEL, 1, 0);
	return SendMessage(hCanauxItem, CB_SELECTSTRING, (WPARAM)-1, (LPARAM)nom);
}

UINT CScanDialog::extrait_canaux()
{
	// nom temporaire de l'lment slectionn dans la liste droulante
	LRESULT lRet;

	// Rcupre l'index de l'lment slectionn
	switch (lRet = SendMessage(hCanauxItem, CB_GETCURSEL, 0, 0)) {

	case 0:
		strcpy_T(choixCanaux, NULL_NAME);	// "Aucun"
		break;

	case 1:
		strcpy_T(choixCanaux, LIST_NAME);	// Liste
		break;

	default:
		if (SendMessage(hCanauxItem, CB_GETLBTEXT, lRet, (LPARAM)choixCanaux) == CB_ERR)
			// La liste est vide :(
			// ou aucun lment n'est slectionn
			return 0;
	}

	return (UINT)lRet;		// false = Aucun changement
}

void CScanDialog::set_info_methode()
{
	TCHAR szWrk[64];

	_stprintf_s(szWrk,
		TEXT(" : %" A2t),
		EnumToString(methode_recherche,
			"()\0"	// -> ne devrait pas arriver
			"  \0"
			"  \0"
			"  \0"));
	SetWindowText(hInfoText, szWrk);
}

void CScanDialog::update_signal_items(const SignalStatistics & sSStats) const
{
	TCHAR szWrk[32];
	TCHAR szTmp[16];

	_stprintf_s(szWrk, TEXT("=%s"), sSStats.getPresentStr());
	SetWindowText(hStatPText, szWrk);

	_stprintf_s(szWrk, TEXT("=%s"), sSStats.getLockedStr());
	SetWindowText(hStatLText, szWrk);

	sSStats.SetLevelBars(hSLevelBar, hQLevelBar);

	sSStats.getStrengthStr(sTunProps.force_signal_aff_pct, szTmp, _countof(szTmp), TEXT("%"));
	_stprintf_s(szWrk, TEXT("=%s"), szTmp);
	SetWindowText(hStatSText, szWrk);

	sSStats.getQualityStr(szTmp, _countof(szTmp), TEXT("%"));
	_stprintf_s(szWrk, TEXT("=%s"), szTmp);
	SetWindowText(hStatQText, szWrk);
}

/// Rinitialisation aprs recherche
void CScanDialog::reset()
{
	// Supprimer les donnes alloues en mmoire :
	pScan.reset();

	// Rtablissement du timer de qualit du signal :
	SetTimer(hDlg, TIMER_SIGNAL, DUREE_SIGNAL_STATS, NULL);

	EnableWindow(hOptBut, TRUE);
	EnableWindow(hStartBut, TRUE);
	SetWindowText(hStartBut, TEXT(""));
	EnableWindow(hCanauxItem, TRUE);	// Ractiver la liste des canaux  explorer
}

/// Traitement des messages de la bote de dialogue
INT_PTR CScanDialog::DialogProc(UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	static INT a_tabs[5] = {24, 40, 95, 146, 238}; // Positions des tabulations dans la list box
	//
	switch (uMsg) {

	case WM_INITDIALOG:	// Initialisation de la fentre
		// Obtenir les handles de certains items :
		hCanauxItem	= GetDlgItem(hDlg, IDC_COMBO_CANAUX);			// Combo des canaux  explorer
		hChanText	= GetDlgItem(hDlg, IDC_CHANNEL_FOUND);			// "Chanes trouves "
		hListItem	= GetDlgItem(hDlg, IDC_CHANNEL_LIST);			// List box
		hOptBut		= GetDlgItem(hDlg, IDC_SCAN_OPTIONS);			// Bouton "Options..."
		hStartBut	= GetDlgItem(hDlg, IDC_START);					// Bouton "Lancer"
		hCancelBut	= GetDlgItem(hDlg, IDCANCEL);					// Bouton "Annuler"
		hOkBut		= GetDlgItem(hDlg, IDOK);						// Bouton "Accepter"
		hInfoText	= GetDlgItem(hDlg, IDC_SCAN_INFO);				// Champ d'informations
		hProgBar	= GetDlgItem(hDlg, IDC_SCAN_PROGRESS);			// Barre de progression
		hStatPText	= GetDlgItem(hDlg, IDC_SIGNAL_PRESENT);			// Stats : signal prsent
		hStatLText	= GetDlgItem(hDlg, IDC_SIGNAL_LOCK);			// Stats : signal verrouill
		hStatSText	= GetDlgItem(hDlg, IDC_SIGNAL_STRENGTH);		// Stats : force du signal
		hStatQText	= GetDlgItem(hDlg, IDC_SIGNAL_QUALITY);			// Stats : qualit du signal
		hSLevelBar	= GetDlgItem(hDlg, IDC_SIGNAL_STRENGTH_BAR);	// Barre de force du signal
		hQLevelBar	= GetDlgItem(hDlg, IDC_SIGNAL_QUALITY_BAR);		// Barre de qualit du signal

		// Initialiser les barres de qualit du signal :
		SendMessage(hSLevelBar, PBM_SETRANGE32, 0, sTunProps.force_signal_max);
		SendMessage(hQLevelBar, PBM_SETRANGE32, 0, 100);

		// Cration d'un timer pour la qualit du signal, pour ne pas laisser les
		// barres de niveau vide en dehors de la recherche (ce timer est dsactiv
		// pendant la recherche).
		SetTimer(hDlg, TIMER_SIGNAL, DUREE_SIGNAL_STATS, NULL);

		cleChaine = indexToCle(ixChaineCourante);
		remplit_combo_canaux(choixCanaux);

		// Dfinir les tabulations dans la list box :
		SendMessage(hListItem, LB_SETTABSTOPS, (WPARAM)_countof(a_tabs), (LPARAM)a_tabs);
		AddLineToList(hListItem, TEXT("       "));
		cancelCode = IDCANCEL;

		// Rappeler le mode de recherche  venir
		set_info_methode();
		return TRUE;

	case WM_TIMER:
		if (wParam == TIMER_SIGNAL) {
			SignalStatistics sSStats;

			sSStats.getStats();
			update_signal_items(sSStats);
		}
		return TRUE;

	case WM_COMMAND:
		switch (wParam) {

		case _cmd_(IDC_SCAN_OPTIONS, BN_CLICKED):	// Clic sur le bouton des options de recherche
			CScanOptionsDialog().Do(hDlg);
			set_info_methode();						// Peut avoir chang ...
			break;

		case _cmd_(IDC_START, BN_CLICKED):			// Clic sur le bouton commencer
			if (CConfigPropSheet::bNeedRestart) {
				MessageBox(hDlg,
					TEXT("     .") EOL
					TEXT("    ,  ") EOL
					TEXT(" ."),
					TEXT(""), MB_OK);
			} else if (!pScan.get()) {
				// Si un bloc-notes avec journal de recherches est ouvert, le fermer
				// (car le contenu du journal va changer).
				HWND	hLogWnd = LookForNotepadWithScanLog();

				if (hLogWnd)
					::PostMessage(hLogWnd, WM_CLOSE, 0, NULL);

				int		nListe = extrait_canaux();

				if (nListe==1) {
					if (CScanListDialog(strListe, bScanLoop).Do(hDlg)==IDCANCEL)
						return TRUE;

				}

				// Arrt du timer de qualit du signal (la recherche va rapporter ses
				// valeurs de son ct, en lieu et place)
				KillTimer(hDlg, TIMER_SIGNAL);

				cancelCode = IDABORT;
				nNewChannels = 0;
				SendMessage(hListItem, LB_RESETCONTENT, 0, 0);		// Vider la liste
				EnableWindow(hOptBut, FALSE);						// Dsactiver le bouton "Options..."
				EnableWindow(hOkBut, FALSE);						// Dsactiver le bouton "Accepter"
				EnableWindow(hCanauxItem, FALSE);					// Dsactiver la liste des canaux  explorer
				SetWindowText(hStartBut, TEXT(""));			// Change le texte sur le bouton "Dmarrer"

				// Initialisation recherche
				zappe_index(-1);									// Dsactiver la chane courante

				pScan.reset(new ChannelsScanner(hDlg, cInfoUser));	// Crer les donnes de recherche

				if (!pScan.get())
					return TRUE;

				switch (nListe) {
				case 0:
					pScan->liste_scan.add_fullscan();
					break;

				case 1: {
					ListeCanaux::ParseListRes eRes;

					pScan->liste_scan.bScanLoop = bScanLoop;
					eRes = pScan->liste_scan.add_list(strListe.c_str());

					if (eRes!=ListeCanaux::plr_noerror){
						MessageBox(hDlg,
							tstr_printf(TEXT("%" A2t),
							EnumToString(eRes, "\1\0"
								" \0"
								"     \0"
								"  \0")).c_str(),
							TEXT("    "), MB_OK|MB_ICONEXCLAMATION);

						// Rinitialisation :
						reset();
						// Simuler une commande pour relancer :
						PostDlgMessage(WM_COMMAND, _cmd_(IDC_START, BN_CLICKED));
						return TRUE;
					}

					// Liste de canaux valide, on sauve pour la postrit
					strcpy_T(liste_canaux, strListe.c_str());
					break; }

				default:
					pScan->liste_scan.lit_fichier_scan(choixCanaux);
				}
				//
				pScan->StartThread();
			} else if (pScan->eScanState==ss_running) {
				// Une recherche est dj en cours (appui sur arrt)
				EnableWindow(hStartBut, FALSE);
				AddLineToList(hListItem, TEXT(""));
				pScan->eScanState = ss_stopping;		// On change l'tat en arrt
			}
			break;

		case _cmd_(IDCANCEL, BN_CLICKED):
			EnableWindow(hCancelBut, FALSE);
			EnableWindow(hStartBut, FALSE);
			if (pScan.get()) {
				AddLineToList(hListItem, TEXT(""));
				pScan->eScanState = ss_cancelling;			// On change l'tat en "annuler"
				return FALSE;
			}
			if (cancelCode==IDABORT) {
				zappe_index(-1);
				Canaux.clear();
				ixChaineCourante = -1;
				// Si la recherche est annule par l'utilisateur, alors on recharge
				// le fichier de chanes prcdent :
				lit_chaines();
				zappe_index(Canaux.trouve_restauration_par_cle(cleChaine));
			}
			EndDialog(hDlg, cancelCode);	// IDCANCEL = annul sans scan,
											// IDABORT  = annul aprs tentative de scan
			return TRUE;

		case _cmd_(IDOK, BN_CLICKED):
			extrait_canaux();
			sauve_chaines();				// sauvegarde les chanes dans le fichier xml
			EndDialog(hDlg, IDOK);			// IDOK = pas d'erreur
			return TRUE;

		case _cmd_(IDC_COMBO_CANAUX, CBN_SELCHANGE):
			extrait_canaux();
			return TRUE;
		}
		return FALSE;

	case WM_APP_SCAN_NOTIFY:
		// Traitement des messages d'information envoys au cours de la recherche :
		switch (LOBYTE(wParam)) {

		case sn_init:
			// L'intervalle de progression comporte une tape de plus qu'il y a de frquences,
			// car le simple dmarrage constitue une tape (plus rassurant pour l'utilisateur).
			SendMessage(hProgBar, PBM_SETRANGE, 0, MAKELPARAM(0, WORD(lParam)+1));
			SendMessage(hProgBar, PBM_SETPOS, 1, 0);
			if (HIWORD(wParam)>=2)
				AddLineToList(hListItem,
					tstr_printf(TEXT(",  N %u"), HIWORD(wParam)).c_str());
			break;

		case sn_info_freq: {
			INT32	khz			= INT32(lParam);
			BYTE	canal_no	= LOBYTE(HIWORD(wParam));

			SetWindowText(hInfoText,
				tstr_printf(
					TEXT("   %u %03u,%03u MHz"),
						canal_no, khz/1000, khz%1000).c_str());
			break; }

		case sn_info_stats:
			update_signal_items(*reinterpret_cast<const SignalStatistics *>(lParam));
			break;

		case sn_completed: {
			UINT	nbChaines = (int)Canaux.size();

			// Affichage des canaux qui viennent d'tre ajouts :
			for (UINT nInx=pScan->prev_size; nInx<nbChaines; nInx++) {
				const Chaine & canal = Canaux[nInx];

				if (canal.nom[0]) {
					LPCTSTR	norme		= canal.norme();
					TCHAR	strnum[4]	= TEXT("-");	// afficher '-' si zro
					LPCSTR	groupe		= *canal.groupe ? canal.groupe : "< >";

					if (canal.nNumeroChaine > 0)
						_itot_s(canal.nNumeroChaine, strnum, _countof(strnum), 10);
					AddLineToList(hListItem,
						tstr_printf(
							TEXT("%s\t%u\t%03u,%03u MHz\t%") A2t TEXT("\t%") A2t TEXT("\t%s"),
							strnum, canal.canal_no, canal.khz/1000, canal.khz%1000,
							groupe, canal.nom, norme).c_str());
					SetWindowText(hChanText,
						tstr_printf(TEXT("  : %u"), ++nNewChannels).c_str());
					++nNewChannelsTotal;
				}
			}

			// Incrmente la barre de progression
			SendMessage(hProgBar, PBM_SETPOS, pScan->nIndex+2, 0);
			break; }

		case sn_info_nit:
			SetWindowText(hInfoText, TEXT("    "));
			break;

		case sn_info_clear:
			set_info_methode();
			break;

		case sn_restore:
			AddLineToList(hListItem,  TEXT("  "));
			break;

		case sn_scan_end:
			// Rinitialisation gnrale :
			reset();

			if (HIWORD(wParam)!=ss_running)
				AddLineToList(hListItem, TEXT(""));

			if (HIWORD(wParam)==ss_cancelling) {
				// Annulation demande :
				PostDlgMessage(WM_COMMAND, MAKELONG(IDCANCEL, BN_CLICKED));
				break;
			}

			if (HIWORD(wParam)==ss_running) {
				// S'assurer que la barre de progression indique toujours ici le maximum, mme si aucune
				// chane n'a t trouve :
				SendMessage(hProgBar, PBM_SETPOS, UINT(lParam)+1, 0);
				AddLineToList(hListItem, TEXT("   "));
			}
			if (Canaux.empty()) {
				static LPCTSTR msg_aucune = TEXT("   ");

				affiche_erreurs(msg_aucune);
				AddLineToList(hListItem, msg_aucune);
			} else {
				EnableWindow(hOkBut, nNewChannelsTotal!=0);

				zappe_index(Canaux.trouve_restauration_par_cle(cleChaine));
				if (nNewChannels==0) {
					if (methode_recherche==sm_incremental)
						AddLineToList(hListItem, TEXT("    "));
				} else if (HIWORD(wParam)!=ss_running)
					AddLineToList(hListItem, TEXT("    "));
			}
		}
	}

	return FALSE;
}

/**
 * Appel du dialogue de recherche des chanes en mode normal en attendant l'implmentation
 * ventuelle d'un "wizard".
 *
 * \param[in]	hParentWnd	\p HANDLE de la fentre parente
 * \retval		IDOK		si la configuration a t valide
 * \retval		IDCANCEL	si annul sans scan
 * \retval		IDABORT		si elle a t annule avec un scan
 **/
BYTE scan_dialog(HWND hParentWnd)
{
	BYTE res = (BYTE)CScanDialog().Do(hParentWnd);

	save_config();	// sauvegarde la configuration (des options peuvent avoir chang)
	return res;
}
/** @} */

// ====================================================================================
// Config des rpertoires
// ====================================================================================
/** \defgroup BrowseCallbackProc Fonctions pour fentre BrowseCallbackProc
 * Fonctions pour grer la fentre de configuration des dossiers, ainsi que le motif pour le nom des enregistrements
 * @{ */

/**
 * Fonction slectionnant le rpertoire initial lors de la demande d'un dossier
 **/
static int CALLBACK BrowseCallbackProc(HWND hWnd, UINT uMsg, LPARAM lParam, LPARAM lpData)
{
	if(uMsg == BFFM_INITIALIZED)
		SendMessage(hWnd, BFFM_SETSELECTION, 1, lpData);
	return 0;
}

/**
 * Fonction permettant d'afficher la fentre de slection des dossiers
 *
 * \param[in,out]  dir    rpertoire de base/rpertoire slectionn
 * \param[in]      title  titre de la fentre
 **/
static void do_select_dir(LPTSTR dir, LPCTSTR title)
{
	SZPathName	display_name;
	BROWSEINFO	bi = {
			hMainWnd,				// hwndOwner - Handle de la fentre
			NULL,					// pidlRoot -  partir de quel dossier faut-il rechercher ?
			display_name,			// pszDisplayName - Variable recevant le dossier slectionn
			title,					// lpszTitle - Message  afficher dans la fentre
			BIF_RETURNONLYFSDIRS,	// ulFlags - N'afiche que les dossiers
			BrowseCallbackProc,		// lpfn - Adresse de la fonction de CallBack
			(LPARAM)dir,			// lParam - Paramtre pass  la fonction de CallBack
			0						// iImage
		};

	ShowCursor(TRUE);

	LPITEMIDLIST iil = SHBrowseForFolder(&bi);

	ShowCursor(FALSE);

	if (iil != NULL) {
		SHGetPathFromIDList(iil, dir);
		CoTaskMemFree(iil);
	}
}

/**
 * Fonction appele depuis les boutons permettant de parcourir pour rechercher un dossier
 *
 * \param[in]  hDlg   handle de la fentre parente
 * \param[in]  hItem  handle de la zone de texte recevant le chemin slectionn
 * \param[in]  title  titre de la fentre de dialogue
 **/
static void change_dir(HWND hDlg, HWND hItem, LPCTSTR title)
{
	SZPathName dir, save_dir;

	get_text(hItem, dir);
	strcpy_T(save_dir, dir);
	do_select_dir(dir, title);
	if (_tcscmp(dir, save_dir)!=0) {
		set_text(hItem, dir);
		PropSheet_Changed(GetParent(hDlg), hDlg);
	}
}

/// Gestion de la page de capture dans le dialogue de configuration
class CCapturePropSheet : public CPropSheetPageBase
{
	/// Traitement des messages de la page de proprits
	virtual INT_PTR DialogProc(UINT uMsg, WPARAM wParam, LPARAM lParam);

	HWND	hMotifNomsEnreg;
	HWND	hVideoDirItem;
	HWND	hSshotDirItem;
	bool	bWarnAwake;
public:

	/// Constructeur :
	CCapturePropSheet() :
		hMotifNomsEnreg(NULL),
		hVideoDirItem(NULL),
		hSshotDirItem(NULL),
		bWarnAwake(false)
	{
	}
};

/**
 * Fonction grant les vnements dans la fentre des captures
 **/
INT_PTR CCapturePropSheet::DialogProc(UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch (uMsg) {

	case WM_INITDIALOG:
		hMotifNomsEnreg	= GetDlgItem(hDlg, IDC_MOTIF_NOM_REC);
		hVideoDirItem	= GetDlgItem(hDlg, IDC_MPEG2_DIR_VIEW);
		hSshotDirItem	= GetDlgItem(hDlg, IDC_SCREENSHOTS_DIR_VIEW);

		// Item de motif de nom d'enregistrements
		set_text(hMotifNomsEnreg, motif_noms_enreg);

		set_check(hDlg, IDC_ALLOW_WAKEUP, allow_wakeup);

		// Items d'avance et de retard sur les dbuts et fins d'enregistrements
		InitUpDownItems(IDC_AVANCE_ENREG, IDC_AVANCE_ENREG_UPDOWN, avance_enreg, 0, 99);
		InitUpDownItems(IDC_RETARD_ENREG, IDC_RETARD_ENREG_UPDOWN, retard_enreg, 0, 99);

		// Items de rpertoires
		set_text(hVideoDirItem, video_dir);
		set_text(hSshotDirItem, screenshots_dir);
		return TRUE;

	case WM_COMMAND:
		switch(wParam) {

		case _cmd_(IDC_MOTIF_NOM_REC, EN_CHANGE): {
			SZPathName	szTmp;

			get_text(hMotifNomsEnreg, szTmp);
			if (_tcscmp(motif_noms_enreg, szTmp)!=0)
				PropSheet_Changed(GetParent(hDlg), hDlg);
			return 0; }

		case _cmd_(IDC_AVANCE_ENREG, EN_CHANGE):
		case _cmd_(IDC_RETARD_ENREG, EN_CHANGE):
			// Changement de l'offset
			// On vrifie si la valeur entre est valide
			CheckUpDownValue(wParam);
			PropSheet_Changed(GetParent(hDlg), hDlg);
			return 0;

		case _cmd_(IDC_ALLOW_WAKEUP, BN_CLICKED):
			if (get_check(hDlg, IDC_ALLOW_WAKEUP)) {
				MessageBoxCheck(hDlg,
					TEXT("   ,       ,    ")
					TEXT(" Standby(  ) .          ")
					TEXT("  ,    .") EOL
					TEXT("   ,   ,       ")
					TEXT("      ,        ."),
					TEXT("   "), MB_OK|MB_ICONWARNING,
					MB_OK, TEXT(""), &bWarnAwake);
			}
			PropSheet_Changed(GetParent(hDlg), hDlg);
			return 0;

		case _cmd_(IDC_MPEG2_DIR, BN_CLICKED):
			change_dir(hDlg, hVideoDirItem,
				TEXT("      :"));
			return 0;

		case _cmd_(IDC_SCREENSHOTS_DIR, BN_CLICKED):
			change_dir(hDlg, hSshotDirItem,
				TEXT("       :"));
			return 0;

		}
		return FALSE;

	case WM_NOTIFY:
		switch (_ncode_(lParam)) {

		case PSN_APPLY: {		// Application des changements
			// Traitement du nom du fichier d'enregistrement
			get_text(hMotifNomsEnreg, motif_noms_enreg);
			clean_string(motif_noms_enreg, _countof(motif_noms_enreg), TEXT(MOTIF_NOMS_ENREG_DEFAULT));

			bool bAllowWakeUp = get_check(hDlg, IDC_ALLOW_WAKEUP);

			get_int(hDlg, IDC_AVANCE_ENREG, avance_enreg, FALSE);
			get_int(hDlg, IDC_RETARD_ENREG, retard_enreg, FALSE);

			// Traitement des noms de rpertoires
			get_text(hVideoDirItem, video_dir);
			get_text(hSshotDirItem, screenshots_dir);

			if (bAllowWakeUp != allow_wakeup) {
				// Actualiser les dialogues ouverts qui pourraient tre affects :
				allow_wakeup = bAllowWakeUp;
				CRecordDialog::PostDlgMessage(WM_APP_PROG_CHANGED);
			}

			return psn_result(PSNRET_NOERROR); }
		}
		return FALSE;

	default:
		return FALSE;
	}
}
/** @} */

// ====================================================================================
// Config des raccourcis du programme
// ====================================================================================
/** \defgroup ShortcutDialogProc Fonctions pour fentre ShortcutDialogProc
 * Fonctions pour grer la fentre de modification des raccourcis
 * @{ */

/// Gestion de la page des raccourcis dans le dialogue de configuration
class CShortcutPropSheet : public CPropSheetPageBase
{
	/// Traitement des messages de la page de proprits
	virtual INT_PTR DialogProc(UINT uMsg, WPARAM wParam, LPARAM lParam);

	HHOOK			hHook;			//!< Handle du hook
	ShortcutItem *	pScutSel;		//!< Dernier raccourci slectionn
	int				iColonne;		//!< Numro de la colonne modifie
	bool			bDescending;	//!< \p true si le dernier tri tait descendant
	HWND			hList;			//!< Handle de la liste
	int				dx;				//!< Position du scrolling horizontal
	int				dy;				//!< Position du scrolling vertical

	static HWND	hShortcutDiag; //!< Handle (statique) de la fentre de gestion des raccourcis

	/**
	 * Obtenir le descripteur de raccourci correspondant  un item
	 **/
	ShortcutItem *			GetShortcutItem(int iItem);

	/**
	 * Trouver un sous-item par position de souris
	 **/
	bool					FindSubItem(POINT sPt, bool bRelative);

	/**
	 * Mise en place de raccourci avec recherche de doublon
	 **/
	bool					SetShortcut(ShortcutItem & sToReplace, Raccourci sToFind, bool bAlt);

	/**
	 * Ajoute un lment de raccourci
	 **/
	void					InsertShortcutItem(int iItem, const ShortcutItem & shortcut);

	/**
	 * Rgnre la liste des raccourcis
	 **/
	void					remplit_liste_shorcut();

	/**
	 * Met  jour les valeurs des raccourcis
	 **/
	void					update_liste_shorcut();

	/**
	 * Remplissage de la "combo box" de rles de la molette de la souris
	 * \param[in]	eMWheel		Rle  slectionner
	 **/
	void					SetMWheelUsage(MouseWheelUsage eMWheel);

	/**
	 * Fonction de hook interceptant les touches du clavier
	 **/
	static LRESULT CALLBACK	LowLevelKeyboardProc(int nCode, WPARAM wParam, LPARAM lParam);

	/**
	 * Fonction utilise pour trier la liste des raccourcis
	 *
	 * \param[in]  lParam1     Pointeur sur Pointeur (EmissionPtr) sur le 1er lment
	 * \param[in]  lParam2     Pointeur sur Pointeur (EmissionPtr) sur le 2nd lment
	 * \param[in]  lParamSort  paramtres de tri
	 * \retval     <0          lParam1 < lParam2
	 * \retval     0           lParam1 = lParam2
	 * \retval     >0          lParam1 > lParam2
	 **/
	static int CALLBACK LVShctCompareProc(LPARAM lParam1, LPARAM lParam2,  LPARAM lParamSort);

public:

	/// Constructeur :
	CShortcutPropSheet() :
		hHook(NULL),
		pScutSel(NULL),
		iColonne(0),
		bDescending(false),
		hList(NULL),
		dx(0),
		dy(0)
	{
	}

	~CShortcutPropSheet()
	{
		if (hHook) {
			// On lve le Hook plac
			EnableWindow(hDlg, true);
			UnhookWindowsHookEx(hHook);
			// hHook = NULL;
		}
	}
};

/// Handle de la fentre de gestion des raccourcis
HWND CShortcutPropSheet::hShortcutDiag = 0;

#define SHORT_COL_NOM			0	///< Colonne indiquant le nom de l'option
#define SHORT_COL_RACCOURCIS	1	///< Colonne indiquant le raccourci
#define SHORT_COL_ALTERNATIF	2	///< Colonne indiquant un raccourci alternatif

/**
 * Ajoute un lment de raccourci
 **/
void CShortcutPropSheet::InsertShortcutItem(int iItem, const ShortcutItem & shortcut)
{
	TCHAR			buffer[64];				// Buffer pour les textes

	LVITEM item = {
		LVIF_PARAM | LVIF_TEXT,	// mask
		iItem,					// iItem
		SHORT_COL_NOM,			// iSubItem
		0, 0,					// state, stateMask
		buffer,					// pszText
		0, 0,					// cchTextMax, iImage
		(LPARAM)&shortcut		// lParam
		// le reste  zro (implicite)
	};

	// Nom de l'action
	strcpy_T(buffer, shortcut.nom);
	ListView_InsertItem(hList, &item);

	//
	item.mask = LVIF_TEXT;
	item.lParam = 0;

	// Premier raccourci
	item.iSubItem = SHORT_COL_RACCOURCIS;
	if (shortcut.raccourci.isSet()) {
		shortcut.raccourci.ToString(buffer);
		ListView_SetItem(hList, &item);
	}

	// Raccourci alternatif
	item.iSubItem = SHORT_COL_ALTERNATIF;
	if (shortcut.alternatif.isSet()) {
		shortcut.alternatif.ToString(buffer);
		ListView_SetItem(hList, &item);
	}
}

/**
 * Rgnre la liste des raccourcis
 **/
void CShortcutPropSheet::remplit_liste_shorcut()
{
	// Vide la liste
	ListView_DeleteAllItems(hList);

	int count = get_shortcuts_count();

	// Ajoute les raccourcis
	for (int i=0; i<count; i++) {
		const ShortcutItem &item = shortcuts[i];
		if (item.nom[0] != 0)
			InsertShortcutItem(i, item);
	}

	// Se repositionne sur la mme vue
	ListView_Scroll(hList, dx, dy*16);
}

/**
 * Met  jour les valeurs des raccourcis
 **/
void CShortcutPropSheet::update_liste_shorcut()
{
	int nCount = ListView_GetItemCount(hList);

	for (int i=0; i<nCount; ++i) {
		ShortcutItem *	pScItm = GetShortcutItem(i);
		TCHAR			szTmp[64];

		if (pScItm) {
			pScItm->raccourci.ToString(szTmp);
			ListView_SetItemText(hList, i, SHORT_COL_RACCOURCIS, szTmp);
			pScItm->alternatif.ToString(szTmp);
			ListView_SetItemText(hList, i, SHORT_COL_ALTERNATIF, szTmp);
		}
	}

}

/**
 * Remplissage de la "combo box" de rles de la molette de la souris
 * \param[in]	eMWheel		Rle  slectionner
 **/
void CShortcutPropSheet::SetMWheelUsage(MouseWheelUsage eMWheel)
{
	static LPCTSTR apszWheelUsage[] = {
		TEXT(" "),
		TEXT("  "),
		TEXT("  "),
		TEXT("   "),
		TEXT("  "),
		TEXT("  "),
		NULL
	};

	ComboBox_Init(GetDlgItem(hDlg, IDC_WHEEL_USAGE), apszWheelUsage, eMWheel);
}

/**
 * Fonction de hook interceptant les touches du clavier
 **/
LRESULT CALLBACK CShortcutPropSheet::LowLevelKeyboardProc(int nCode, WPARAM wParam, LPARAM lParam)
{
	if (nCode < 0) // On ne fait rien, on transmet simplement
		return CallNextHookEx(NULL /* ignor */, nCode, wParam, lParam);

	if (nCode == HC_ACTION && (wParam == WM_KEYDOWN || wParam == WM_SYSKEYDOWN)) {
		// On traite les vnements
		KBDLLHOOKSTRUCT *kbhook = (KBDLLHOOKSTRUCT*) lParam;

		// On ignore si il s'agit de touches de fonctions
		switch (kbhook->vkCode) {
			// Maj
			case VK_SHIFT:
			case VK_LSHIFT:
			case VK_RSHIFT:
			// Ctrl
			case VK_CONTROL:
			case VK_LCONTROL:
			case VK_RCONTROL:
			// Alt
			case VK_MENU:
			case VK_LMENU:
			case VK_RMENU:
			// Win
			case VK_LWIN:
			case VK_RWIN:
				break;

			// Autre touche, on traite
			default: {
				Raccourci r;
				r.maj	= (GetAsyncKeyState(VK_SHIFT) & 0xFFFF0000 ? true : false);
				r.ctrl	= (GetAsyncKeyState(VK_CONTROL) & 0xFFFF0000 ? true : false);
				r.alt	= (GetAsyncKeyState(VK_MENU) & 0xFFFF0000 ? true : false);
				r.touche= kbhook->vkCode;
				myprintf(TEXT("Touche raccourci = 0x%02x") EOL, r.touche);

				if(!r.maj && !r.ctrl && !r.alt && r.touche == VK_ESCAPE)
					PostMessage(hShortcutDiag, WM_APP_HOOK_CANCEL, NULL, NULL);
				else {
					Raccourci *param = new Raccourci(r);
					PostMessage(hShortcutDiag, WM_APP_HOOK_OK, (WPARAM)param, NULL);
				}
			}
		};

	}
	return 1;
}

/**
 * Fonction utilise pour trier la liste des raccourcis
 *
 * \param[in]  lParam1     Pointeur sur Pointeur (EmissionPtr) sur le 1er lment
 * \param[in]  lParam2     Pointeur sur Pointeur (EmissionPtr) sur le 2nd lment
 * \param[in]  lParamSort  paramtres de tri
 * \retval     <0          lParam1 < lParam2
 * \retval     0           lParam1 = lParam2
 * \retval     >0          lParam1 > lParam2
 **/
int CALLBACK CShortcutPropSheet::LVShctCompareProc(LPARAM lParam1, LPARAM lParam2,  LPARAM lParamSort)
{
	CShortcutPropSheet & cSpsht = *reinterpret_cast<CShortcutPropSheet *>(lParamSort);
	const ShortcutItem & sSCut1 = *reinterpret_cast<const ShortcutItem *>(lParam1);
	const ShortcutItem & sSCut2 = *reinterpret_cast<const ShortcutItem *>(lParam2);
	int					res		= 0; // rsultat
	TCHAR				szTmp1[64];
	TCHAR				szTmp2[64];

	switch (cSpsht.iColonne) {

	case SHORT_COL_NOM:
		res = CompareString(LOCALE_NEUTRAL, NORM_IGNORECASE, sSCut1.nom, -1, sSCut2.nom, -1);
		if (res != 0)
			res -= 2;
		break;

	case SHORT_COL_RACCOURCIS:
		sSCut1.raccourci.ToString(szTmp1);
		sSCut2.raccourci.ToString(szTmp2);
		res = CompareString(LOCALE_NEUTRAL, NORM_IGNORECASE, szTmp1, -1, szTmp2, -1);
		if (res != 0)
			res -= 2;
		break;

	case SHORT_COL_ALTERNATIF:
		sSCut1.alternatif.ToString(szTmp1);
		sSCut2.alternatif.ToString(szTmp2);
		res = CompareString(LOCALE_NEUTRAL, NORM_IGNORECASE, szTmp1, -1, szTmp2, -1);
		if (res != 0)
			res -= 2;
	}

	// Tri descendant si pnml->lParam != 0
	if (cSpsht.bDescending)
		res = -res;
	return res;
}

/**
 * Obtenir un pointeur sur le descripteur de raccourci correspondant  un item
 **/
ShortcutItem * CShortcutPropSheet::GetShortcutItem(int iItem)
{
	LVITEM lvi = {
		LVIF_PARAM,
		iItem
		// Le reste  zro (implicite)
	};

	if (ListView_GetItem(hList, &lvi))
		return reinterpret_cast<ShortcutItem *>(lvi.lParam);
	return NULL;
}

/**
 * Trouver un sous-item par position de souris
 **/
bool CShortcutPropSheet::FindSubItem(POINT sPt, bool bRelative)
{
	INT iItem, iSubItem;

	// On ne fait rien si on n'est pas dans la seconde/troisime colonne
	if (FindListViewSubItem(hList, sPt, bRelative, iItem, iSubItem) && iSubItem == SHORT_COL_RACCOURCIS || iSubItem == SHORT_COL_ALTERNATIF) {
		iColonne = iSubItem;
		pScutSel = GetShortcutItem(iItem);
		return true;
	}
	return false;
}

/**
 * Mise en place de raccourci avec recherche de doublon
 **/
bool CShortcutPropSheet::SetShortcut(ShortcutItem & sToReplace, Raccourci sToFind, bool bAlt)
{
	int					count		= get_shortcuts_count();
	TCHAR				szRaccourci[32];
	TCHAR				buffer[256];

	for (int i=0; i<count; ++i) {
		ShortcutItem &sScutCmp = shortcuts[i];

		if (sScutCmp.raccourci == sToFind || sScutCmp.alternatif == sToFind) {
			if (&sScutCmp == &sToReplace)
				break;

			sToFind.ToString(szRaccourci);
			_stprintf_s(buffer,
				TEXT("  %s      %s.") EOL EOL
					TEXT("     ?"), szRaccourci, sScutCmp.nom);

			if (MessageBox(hDlg, buffer, TEXT(""), MB_YESNO) != IDYES)
				return false;	// On ne fait pas de modification

			// Suppression du raccourci trouv en vue de son affectation  l'autre commande
			sScutCmp.clear(sScutCmp.raccourci != sToFind);
			break;
		}
	}

	// Rien trouv, ou bien remplacement autoris
	if (!bAlt)
		sToReplace.raccourci = sToFind;
	else
		sToReplace.alternatif = sToFind;
	return true;
}

/**
 * Fonction de traitement des vnements de la fentre des tlcommandes
 **/
INT_PTR CShortcutPropSheet::DialogProc(UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	// Nom et largeur des diffrentes colonnes
	static const struct Tab_cols t_cols[] = {
		{TEXT(""),		198},
		{TEXT(""),		96},
		{TEXT(""),	96},
		{NULL, 0}				// Terminateur
	};

	switch (uMsg) {

	case WM_INITDIALOG:
		hShortcutDiag = hDlg;
		hList = GetDlgItem(hDlg, IDC_SHORTCUT);	// Handle de la liste
		ListView_AjouteColonnes(hList, t_cols);
		remplit_liste_shorcut();
		SetMWheelUsage(role_molette);
		set_check(hDlg, IDC_WHEEL_INVERT, inversion_molette);
		set_check(hDlg, IDC_USE_MULTIMEDIA, use_multimedia);
		return TRUE;

	case WM_COMMAND:
		switch(wParam) {

		case _cmd_(IDC_SHORTCUT_REINIT, BN_CLICKED):
			if (MessageBox(hDlg, TEXT("        ?"), TEXT("  "), MB_YESNO) == IDYES) {
				load_config_raccourcis(true);
				update_liste_shorcut();
				PropSheet_Changed(GetParent(hDlg), hDlg);
			}
			return 0;

		case _cmd_(IDM_SHORT_DEFAUT, 0):{	// On remet la valeur par dfaut
			if (!pScutSel)
				break;
			Raccourci			sNewScut = pScutSel->defaut;

			if (pScutSel->raccourci == pScutSel->defaut && !pScutSel->alternatif.isSet())
				break;	// La valeur par dfaut est dj attribue

			if (SetShortcut(*pScutSel, pScutSel->defaut, false)) {
				pScutSel->alternatif.clear();
				// Indique une modification
				PropSheet_Changed(GetParent(hDlg), hDlg);
			}
			// raffiche la liste
			update_liste_shorcut();
			break;} //case _cmd_(IDM_SHORT_DEFAUT, 0)

		case _cmd_(IDM_SHORT_EFFACER, 0):	// On efface le raccourci
			if (!pScutSel)
				break;
			// Teste si la case est vide
			if (iColonne == SHORT_COL_RACCOURCIS && !pScutSel->raccourci.isSet())
				break;
			else if (iColonne == SHORT_COL_ALTERNATIF && !pScutSel->alternatif.isSet())
				break;

			// Efface le raccourci
			pScutSel->clear(iColonne == SHORT_COL_ALTERNATIF);

			// Raffiche la liste
			update_liste_shorcut();
			// ... (passage  travers)
		case _cmd_(IDC_WHEEL_USAGE, CBN_SELCHANGE) :
		case _cmd_(IDC_WHEEL_INVERT, BN_CLICKED) :
		case _cmd_(IDC_USE_MULTIMEDIA, BN_CLICKED) :
			// Indique une modification
			PropSheet_Changed(GetParent(hDlg), hDlg);
			break;

		} // switch(wParam)
		return FALSE;

	case WM_NOTIFY:
		switch (_ncode_(lParam)) {

		case PSN_APPLY:			// Application des changements
			save_config_raccourcis();
			load_config_raccourcis();
			update_shortcuts_menus();
			use_multimedia = get_check(hDlg, IDC_USE_MULTIMEDIA);
			role_molette = (MouseWheelUsage)SendDlgItemMessage(hDlg, IDC_WHEEL_USAGE, CB_GETCURSEL, 0, 0);
			inversion_molette = get_check(hDlg, IDC_WHEEL_INVERT);
			return psn_result(PSNRET_NOERROR);

		case PSN_KILLACTIVE:	// Validation des changements
			psn_result(FALSE);
			break;

		case PSN_RESET:			// On a appuy sur le bouton Annuler
			load_config_raccourcis();
			break;

		case NM_DBLCLK:
			if (hHook == NULL && _nidfr_(lParam) == IDC_SHORTCUT) {
				// Notification envoye de la liste des raccourcis
				const NMITEMACTIVATE & sNmIa = *reinterpret_cast<LPNMITEMACTIVATE>(lParam);

				// Double clic sur un raccourci
				if (sNmIa.iItem >= 0 && FindSubItem(sNmIa.ptAction, true)) {
					// Ne pas placer le raccourci alternatif si le raccourci principal est vide
					if (pScutSel && !pScutSel->raccourci.isSet() && iColonne == SHORT_COL_ALTERNATIF)
						break;

					// On change le message dans la colonne
					ListView_SetItemText(hList, sNmIa.iItem, iColonne, TEXT("..."));

					// On met un hook sur le clavier pour capturer les touches
					hHook = SetWindowsHookEx(WH_KEYBOARD_LL, LowLevelKeyboardProc, hAppInstance, NULL);
					EnableWindow(hDlg, false);
				}
			}
			break;

		case LVN_COLUMNCLICK: {		// Clic sur la colonne => tri des raccourcis
			// Tri de la liste par clic sur un en-tte de colonne :
			NMLISTVIEW &	sNmLv		= *reinterpret_cast<LPNMLISTVIEW>(lParam);
			int				iCol		= sNmLv.iSubItem;	// N de colonne concerne

			if (iColonne != iCol) {
				// Colonne diffrente : tri ascendant
				iColonne	= iCol;
				bDescending	= false;
			} else
				// Mme colonne : inversion de l'ordre de tri
				bDescending	= !bDescending;
			ListView_SortItems(hList, LVShctCompareProc, this); // Tri
			break; }

		case LVN_ENDSCROLL: {
			// On a descendu le scroll
			const NMLVSCROLL & sNmLs = *reinterpret_cast<LPNMLVSCROLL>(lParam);

			dx += sNmLs.dx;
			dy += sNmLs.dy; }
		} // switch (_ncode_(lParam))

		return FALSE;

	case WM_APP_HOOK_OK:{
		Raccourci			sNewScut	= *reinterpret_cast<const Raccourci *>(wParam);

		delete reinterpret_cast<const Raccourci *>(wParam);	// Supprimer l'allocation en mmoire

		SetShortcut(*pScutSel, sNewScut, iColonne == SHORT_COL_ALTERNATIF);

		// On vrifie si un raccourci n'est pas attribu

		// On notifie de la modification
		PropSheet_Changed(GetParent(hDlg), hDlg); }
		// ... (passage  travers)
	case WM_APP_HOOK_CANCEL:
		// On a appuy sur la touche "chap", on annule le hook sans rien modifier donc
		update_liste_shorcut();

		if (hHook) {
			// On lve le Hook plac
			EnableWindow(hDlg, true);
			UnhookWindowsHookEx(hHook);
			hHook = NULL;
		}
		return 0;

	case WM_CONTEXTMENU: {	// On a effectu un clic avec le bouton droit (menu contextuel)
		// Rcupre la position de la souris par rapport  la fentre
		POINT		sPt;

		GetCursorPos(&sPt);

		if (FindSubItem(sPt, false)) {
			// Permet de modifier les raccourcis
			HMENU	hPopMenu = CreatePopupMenu();
			TCHAR	szBuffer[128];
			TCHAR	szRaccourci[32];

			pScutSel->defaut.ToString(szRaccourci);
			_stprintf_s(szBuffer, TEXT("  : %s"), szRaccourci);
			AppendMenu(hPopMenu, MF_STRING, IDM_SHORT_DEFAUT,	szBuffer);
			AppendMenu(hPopMenu, MF_STRING, IDM_SHORT_EFFACER,	TEXT(" "));

			TrackPopupMenuEx(hPopMenu, TPM_LEFTALIGN | TPM_LEFTBUTTON, sPt.x, sPt.y, hDlg, NULL);
			DestroyMenu(hPopMenu);
		}

		} // case WM_CONTEXTMENU

	default:
		return FALSE;
	}
}
/** @} */

// ====================================================================================
// Config des options
// ====================================================================================
/** \defgroup OptionsDialogProc Fonctions pour fentre OptionsDialogProc
 * Fonctions pour grer la fentre de configuration des options de PTVM
 * @{ */

/// Gestion de la page des options dans le dialogue de configuration
class COptionsPropSheet : public CPropSheetPageBase
{
	HWND	hTitreFenetre;
	bool	bWarnRecProtDone;

	/// Traitement des messages de la page de proprits
	virtual INT_PTR	DialogProc(UINT uMsg, WPARAM wParam, LPARAM lParam);

	/// Mise  jour de l'tat d'activation de certains items
	void			UpdateStates();

public:

	/// Constructeur :
	COptionsPropSheet() :
		hTitreFenetre(NULL),
		bWarnRecProtDone(false)
	{
	}
};

void COptionsPropSheet::UpdateStates()
{
	bool	bSuspendMinimized	= get_check(hDlg, IDC_SUSPEND);
	bool	bAllowRecProtect	= bSuspendMinimized && !get_check(hDlg, IDC_SYSTEM_TRAY);

	if (!bSuspendMinimized)
		set_check(hDlg, IDC_START_WITH_WINDOWS, false);
	if (!bAllowRecProtect)
		set_check(hDlg, IDC_RECORD_PROTECT, false);
	EnableDlgItem(IDC_RECORD_PROTECT, bAllowRecProtect);
	EnableDlgItem(IDC_START_WITH_WINDOWS, bSuspendMinimized);

	bool	bAllowRecProtSet	= get_check(hDlg, IDC_RECORD_PROTECT);
	bool	bStartWithWindows	= get_check(hDlg, IDC_START_WITH_WINDOWS);

	EnableDlgItem(IDC_SUSPEND, !bAllowRecProtSet && !bStartWithWindows);
	EnableDlgItem(IDC_SYSTEM_TRAY, !bAllowRecProtSet);
}

/**
 * Fonction de traitement des vnements de la fentre des options
 **/
INT_PTR COptionsPropSheet::DialogProc(UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch (uMsg) {

	case WM_INITDIALOG:
		hTitreFenetre	= GetDlgItem(hDlg, IDC_WINDOW_TITLE);

		set_check(hDlg, IDC_START_WITH_WINDOWS, start_with_windows);
		set_check(hDlg, IDC_RECORD_PROTECT, record_protect);
		set_check(hDlg, IDC_SUSPEND, suspend_minimized);
		set_check(hDlg, IDC_SYSTEM_TRAY, !minimize_system_tray);
		remplit_liste_audio(GetDlgItem(hDlg, IDC_AUDIO), audio_pref);

		// Mthodes de rendu vido :
		SetVMRSel(hDlg, vmr_mode);

		// Sous Vista, la configuration OSD n'est valide qu'en mode VMR9
		SetOSDState(*this);
		set_check(hDlg, IDC_USE_OSD, use_osd);

		// "Temps avant test de prsence et verrouillage du signal"
		InitUpDownItems(IDC_ZAPPING_DELAY, IDC_ZAPPING_DELAY_UPDOWN, duree_zapping, 0, 3000);

		set_check(hDlg, IDC_MSN, use_msn);
		set_check(hDlg, IDC_ALLOW_STREAM_RECORD, allow_stream_record);

		// Titre de la fentre
		SendMessage(hTitreFenetre, EM_SETLIMITTEXT, _countof(titre_fenetre)-1, 0);
		set_text(hTitreFenetre, titre_fenetre);

		UpdateStates();
		return TRUE;

	case WM_COMMAND:
		switch(wParam) {
		case _cmd_(IDC_COMBO_RENDERING, CBN_SELCHANGE):
			SetOSDState(*this);
			// ... (passage  travers)
		case _cmd_(IDC_AUDIO, CBN_SELCHANGE):
		case _cmd_(IDC_USE_OSD, BN_CLICKED):
		case _cmd_(IDC_MSN, BN_CLICKED):
		case _cmd_(IDC_ALLOW_STREAM_RECORD, BN_CLICKED):
			PropSheet_Changed(GetParent(hDlg), hDlg);
			return 0;

		case _cmd_(IDC_RECORD_PROTECT, BN_CLICKED):
			if (get_check(hDlg, IDC_RECORD_PROTECT))
				MessageBoxCheck(hDlg,
					TEXT("    ,       24 .")
					TEXT("            ")
					TEXT("  System tray.") EOL
					TEXT("      ,     ")
					TEXT(" 䠻         System tray."),
					TEXT(" "), MB_OK|MB_ICONWARNING,
					MB_OK, TEXT("   "), &bWarnRecProtDone);
			// ... (passage  travers)
		case _cmd_(IDC_START_WITH_WINDOWS, BN_CLICKED):
		case _cmd_(IDC_SUSPEND, BN_CLICKED):
		case _cmd_(IDC_SYSTEM_TRAY, BN_CLICKED):
			UpdateStates();
			PropSheet_Changed(GetParent(hDlg), hDlg);
			return 0;

		case _cmd_(IDC_ZAPPING_DELAY, EN_CHANGE):
			CheckUpDownValue(IDC_ZAPPING_DELAY, IDC_ZAPPING_DELAY_UPDOWN);
			PropSheet_Changed(GetParent(hDlg), hDlg);
			return 0;

		case _cmd_(IDC_OSD_COLOR, BN_CLICKED):
			ChooseOsdColor(hDlg);
			return 0;

		case _cmd_(IDC_WINDOW_TITLE, EN_CHANGE): {
			TCHAR szTmp[_countof(titre_fenetre)];

			get_text(hTitreFenetre, szTmp);
			if (_tcscmp(titre_fenetre, szTmp)!=0)
				PropSheet_Changed(GetParent(hDlg), hDlg);
			return 0; }

		}
		return FALSE;

	case WM_NOTIFY:
		switch (_ncode_(lParam)) {

		case UDN_DELTAPOS:
			if (wParam==IDC_ZAPPING_DELAY_UPDOWN)
				AdjustDelayUpDown(*reinterpret_cast<LPNMUPDOWN>(lParam));
			return TRUE;

		case PSN_APPLY:	{		// Application des changements
			// Sauvegarde de ces deux options afin de pouvoir dtecter
			// la ncessit de relancer le programme
			RdrTypes	old_vmr_mode	= vmr_mode;
			bool		old_use_osd		= use_osd;

			start_with_windows		= get_check(hDlg, IDC_START_WITH_WINDOWS);
			record_protect			= get_check(hDlg, IDC_RECORD_PROTECT);
			suspend_minimized		= get_check(hDlg, IDC_SUSPEND);
			minimize_system_tray	= !get_check(hDlg, IDC_SYSTEM_TRAY);
			audio_pref				= (AudioMode)SendDlgItemMessage(hDlg, IDC_AUDIO, CB_GETCURSEL, 0, 0);
			vmr_mode				= GetVMRSel(hDlg);
			use_osd					= get_check(hDlg, IDC_USE_OSD);
			use_msn					= get_check(hDlg, IDC_MSN);
			allow_stream_record		= get_check(hDlg, IDC_ALLOW_STREAM_RECORD);

			// Titre de la fentre
			get_text(hTitreFenetre, titre_fenetre);
			clean_string(titre_fenetre, _countof(titre_fenetre), szAppName);

			// Dlai de zapping
			get_int(hDlg, IDC_ZAPPING_DELAY, duree_zapping, FALSE);

			niData.update_state();

			// Avertir de la ncessit de relancer l'application si un des paramtres
			// de sortie vido a t modifi :
			if (vmr_mode!=old_vmr_mode || use_osd!=old_use_osd)
				CConfigPropSheet::bNeedRestart = true;
			return psn_result(PSNRET_NOERROR); }
		}
		return FALSE;

	default:
		return FALSE;
	}
}
/** @} */

// ====================================================================================

/// Constructeur
CConfigPropSheet::CConfigPropSheet()
{
	AddPage(MakePropSheetPage<CChannelsPropSheet>	(IDD_CHANNELS,	TEXT("")));
	AddPage(MakePropSheetPage<CFiltersPropSheet>	(IDD_FILTERS,	TEXT("")));
	AddPage(MakePropSheetPage<CCapturePropSheet>	(IDD_CAPTURE,	TEXT("")));
	AddPage(MakePropSheetPage<COptionsPropSheet>	(IDD_OPTIONS,	TEXT("")));
	AddPage(MakePropSheetPage<CShortcutPropSheet>	(IDD_SHORTCUT,	TEXT("")));
}

bool CConfigPropSheet::bNeedRestart = false;

/// Initialisation : modification du titre
void CConfigPropSheet::Init()
{
	TCHAR szWrk[128] = TEXT("  ");

	strcat_T(szWrk, szAppName);
	set_text(hDlg, szWrk);
	bNeedRestart = false;
}

/**
 * Appel du dialogue de configuration
 *
 * \param[in]  hwndOwner  handle de la fentre parente
 * \return	\p true si redmarrage du graphe ncessaire
 **/
bool do_config(HWND hwndOwner)
{
	CConfigPropSheet().Do(hwndOwner, szAppName);
	save_config();		// sauvegarde la configuration
	return CConfigPropSheet::bNeedRestart;
}
