/*
 * $Id: trayicon.cpp 266 2010-11-19 17:43:17Z gingko $
 * Copyright (C) 2007 gingko - http://gingko.homeip.net/
 *
 * This file is part of Pouchin TV Mod, a free DVB-T viewer.
 * See http://www.pouchintv.fr/ for updates.
 *
 * Pouchin TV Mod is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Pouchin TV Mod is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 *
 *
 * Most files of this project have been changed from the Pouchin TV
 * project (Copyright (C) 2006 Pouchin <pouchinteve@free.fr>),
 * to use with a modified version of this software.
 * See http://pouchinteve.free.fr/ for the original project.
 */

/** \file
 *	\brief Implmentations lies  l'icne de la zone de notification
 *
 *	Ce fichier implmente les membres de  la classe \p MyNOTIFYICONDATA, qui
 *	contient les fonctionnalits ncessaires  la gnration de l'icne qui
 *	s'affiche dans la zone de nofification (zone improprement nomme tray).
 **/

#include "stdafx.h"
#include "trayicon.h"
#include "record.h"

MyNOTIFYICONDATA niData;


MyNOTIFYICONDATA::MyNOTIFYICONDATA() :
	alreadyset(false)
{
	ZeroMemory(this, sizeof(NOTIFYICONDATA));
	cbSize = sizeof(NOTIFYICONDATA);
}

bool MyNOTIFYICONDATA::notify(DWORD dwMessage)
{
	if (hMainWnd != NULL) {
		hWnd = hMainWnd;
		uID = MY_TRAY_ICON_ID;
		//myprintf(TEXT("MyNOTIFYICONDATA::notify() - msg=%u, flg=0x%02x") EOL, dwMessage, uFlags);
		return Shell_NotifyIcon(dwMessage, this)==TRUE;
	}
	return false;
}

void MyNOTIFYICONDATA::set_tip()
{
	if (ixChaine_ok(ixChaineCourante)) {
		const Chaine &	canal_courant = Canaux[ixChaineCourante];

		EmissionPtr	pEmis			= canal_courant.getCurrentProgram();
		tstring		strProgramName	= pEmis.getName(TEXT(""));
		tstring		strTmp;

		strcpy_T(strTmp, canal_courant.nom);		// Nom de la chane
		if (!strProgramName.empty())
			strTmp += TEXT(" - ") + strProgramName;	// Ajout du nom de l'mission, si disponible

		// Ajout du nom de la fentre de l'application, si et seulement si le nom de cette fentre n'est
		// pas le nom par dfaut. De plus, si l'ajout de ce nom produirait une chane plus grande que
		// le tampon "szTip", c'est ce nom qui serait tronqu, en priorit sur le nom de l'mission.
		// Ceci afin de permettre de distinguer entre elles des icnes de zone de notification multiples
		// dans le cas o plusieurs instances de l'application existent simultanment.
		if (_tcscmp(titre_fenetre, szAppName)!= 0) {
			INT nSize		= (INT)strTmp.size();
			INT nAvailLen	= _countof(szTip) - nSize - 1;

			if (nAvailLen >= 6) {	// Ignorer le nom de la fentre si la troncature, qui s'accompagnerait
									// en plus d'un point de suspension, laisserait moins de 4 (6 - 2)
									// caractres  afficher.
				INT nAppNLen	= strlen_T(titre_fenetre);

				strTmp.insert(0, EOL); nAvailLen -= 1;
				if (nAppNLen <= nAvailLen)
					strTmp.insert(0, titre_fenetre);				// Insrer nom de la fentre
				else {
					strTmp.insert(0, TEXT("")); nAvailLen -= 1;	// Insrer points de suspension
					strTmp.insert(0, titre_fenetre, nAvailLen);		// Insrer nom de la fentre tronqu
				}
			}
		}
		_tcsncpy(szTip, strTmp.c_str(), _countof(szTip)-1);
		szTip[_countof(szTip)-1] = 0;
		uFlags |= NIF_TIP;
	}
}

void MyNOTIFYICONDATA::set_icon()
{
	if (hCurIcon != NULL && hCurIcon != hIcon) {
		hIcon = hCurIcon;
		uFlags |= NIF_ICON;
	}
}

void MyNOTIFYICONDATA::enable()
{
	if (!alreadyset) {
		uFlags = NIF_MESSAGE;
		uCallbackMessage = WM_APP_TRAYICON_MSG;
		set_icon();
		set_tip();
		if (notify(NIM_ADD))
			alreadyset = true;
	}

}

void MyNOTIFYICONDATA::update_icon()
{
	if (alreadyset) {
		uFlags = 0;
		set_icon();
		notify(NIM_MODIFY);
	}
}

void MyNOTIFYICONDATA::update_tip()
{
	if (alreadyset) {
		uFlags = 0;
		set_tip();
		notify(NIM_MODIFY);
	}
}

void MyNOTIFYICONDATA::disable()
{
	if (alreadyset) {
		notify(NIM_DELETE);
		hIcon = NULL;
		alreadyset = false;
	}
}

void MyNOTIFYICONDATA::update_state()
{
	if (minimize_system_tray)
		enable();
	else
		disable();
}

