/*
 * $Id: update.cpp 276 2011-03-06 10:17:01Z gingko $
 * Copyright (C) 2008 lolo_32 <pouchintv-dev@baysse.fr>
 *
 * This file is part of Pouchin TV Mod, a free DVB-T viewer.
 * See http://www.pouchintv.fr/ for updates.
 *
 * Pouchin TV Mod is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Pouchin TV Mod is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 *
 *
 * Most files of this project have been changed from the Pouchin TV
 * project (Copyright (C) 2006 Pouchin <pouchinteve@free.fr>),
 * to use with a modified version of this software.
 * See http://pouchinteve.free.fr/ for the original project.
 */

/** \file
 *	\brief Traitement des mises  jour par Internet (implmentation)
 *
 *	Ce fichier contient les fonctionnalits ncessaires au
 *	traitement des mises  jour par Internet.
 **/

#include "stdafx.h"
#include "resource.h"
#include "dlgutils.h"
#include "internet.h"		// Pour tlcharger les mises  jour
#include "update.h"
#include "record.h"			// Si des enregistrements sont en cours
#include "version.h"		// Pour avoir la version compile
#include "xmlutils.h"		// Pour le XML
#include "lzma.h"
#include <shellapi.h>
#include <shlwapi.h>

/// Version maximale supporte pour les mises  jour
#define UPDATE_VERSION_MAX	1

/// URL pour la mise  jour
#ifdef _DEBUG
	#define UPDATE_URL	"http://mmzin.com/mmzintv_upd_debug.xml"
#else
	#define UPDATE_URL	"http://mmzin.com/mmzin_upd.xml"
#endif

#ifdef _WIN64
	#define REQ_APP_TYPE	"app64"
#else
	#define REQ_APP_TYPE	"app32"
#endif

class CWebUpdateDialog;

enum UpdateState
{
	us_Undef,
	us_NeedCheck,
	us_Checking,
	us_NeedUpdate,
	us_Updating,
	us_NeedRestart
};

class CUpdateThread : public WorkingThread
{
protected:

	/// Structure associant un document tlcharg  sa date de dernire modification.
	struct InternetFile : public ByteVector
	{
		SYSTEMTIME sTime;	// Date et heure du fichier

		/**
		 * Enregistre un fichier depuis un tableau d'octets
		 *
		 * \param[in]	nom	Nom du fichier  sauvegarder
		 * \retval		0 si tout s'est bien droul
		 * \retval		Code d'erreur systme en cas d'erreur
		 **/
		DWORD SauveFichier(LPCTSTR nom) const;

#if UPDATE_VIA_EXTERNAL == 0
		/**
		 * \brief Remplace le fichier par le nouveau, en sauvegardant l'ancien
		 *
		 * \param[in]  nouveau   nom du fichier venant d'tre tlcharg
		 * \param[in]  original  ancien fichier devant tre remplac
		 * \param[in]  backup    si true, on ne sauvegarde pas l'ancien fichier, on l'efface tout simplement
		 * \retval	0 si tout s'est bien droul
		 * \retval	Code d'erreur systme en cas d'erreur
		 **/
		static DWORD RemplaceFichier(LPCTSTR nouveau, LPCTSTR original, bool backup);

		/**
		 * \brief Remplace le fichier par le nouveau, en sauvegardant l'ancien
		 *
		 * \param[in]  original	Ancien fichier devant tre remplac
		 * \param[in]  backup	Si true, on ne sauvegarde pas l'ancien fichier, on l'efface tout simplement
		 * \retval	0 si tout s'est bien droul
		 * \retval	Code d'erreur systme en cas d'erreur
		 **/
		DWORD RemplaceFichier(LPCTSTR original, bool backup) const;
#endif
	};

	CWebUpdateDialog &	cUpdDlg;
	bool				bWithBeta;

	/// Arrte du tlchargement des fichiers demand
	bool				stop_maj;

#if UPDATE_VIA_EXTERNAL == 0
	/// Indique si l'avertissement concernant le mode administrateur a dj t affich
	bool				bAdminWarnDisplayed;
#endif
	/**
	 * \brief Ajoute le message dans la liste
	 *
	 * \param[in]  pszFmt		message ou chane de formatage devant tre affich dans la liste de statut
	 **/
	void CUpdateThread::AddMsg(LPCTSTR pszFmt, ...);

	/**
	 * Lance la rcupration du fichier, dont l'url est passe en paramtre,
	 * et stocke le fichier rcupr dans \p vData
	 *
	 * \param[in]	url		Adresse contenant les fichiers  rcuprer
	 * \param[out]	vData	Vecteur d'octets contenant le fichier rcupr, initialis dans la fonction
	 * \return	Taille du fichier rcupr ( = vData.size() )
	 **/
	DWORD GetInternetFile(LPCTSTR url, InternetFile & vData);

	/**
	 * Lance la rcupration du fichier compress, dont l'url est passe en paramtre,
	 * et stocke le fichier rcupr dans \p vData
	 *
	 * \param[in]	url		Adresse contenant les fichiers  rcuprer
	 * \param[out]	vData	Vecteur d'octets contenant le fichier rcupr, initialis dans la fonction
	 * \return		Taille du fichier rcupr ( = vData.size() )
	 **/
	INT GetCompressedInternetFile(LPCTSTR url, InternetFile & vData);

	/**
	 * \brief Traitement pour une mise  jour possible du fichier d'application lui-mme
	 * \param[in]	sTiXml		Noeud XML du dcrivant le fichier
	 * \param[in]	pszDispType	Type de noeud recherch ("stable" ou "beta") pour affichage
	 **/
	virtual bool UpdateTraiteApp(const TiXmlElement & sTiXml, LPCTSTR pszDispType) = 0;

	/**
	 * \brief Traitement pour une mise  jour d'un fichier secondaire (autre qu'icne)
	 * \param[in]	strUrl		URL de tlchargement du fichier
	 * \param[in]	strPath		Chemin complet pour enregistrer le fichier
	 * \param[in]	strFile		Nom du fichier, seul
	 * \return		\p true ou \p false selon succs ou chec
	 **/
	virtual bool UpdateTraiteFile(const tstring & strUrl, const tstring & strPath, const tstring & strFile) = 0;

	/**
	 * \brief Traitement pour une icne dont la mise  jour est requise
	 * \param[in]	strUrl		URL de tlchargement
	 * \param[in]	strPays		Pays concern par l'icne
	 * \param[in]	strDirPays	Rpertoire (relatif) des icnes de ce pays
	 * \param[in]	strIcone	Nom de base de l'icne
	 * \param[in]	strIconFile	Nom de fichier de l'icne
	 * \param[in]	strFound	Chemin de l'icne dj existante  mettre  jour, le cas chant
	 * \return		\p true ou \p false selon succs ou chec
	 **/
	virtual bool UpdateTraiteIcon(const tstring & strUrl, const tstring & strPays, const tstring & strDirPays,
								  const tstring & strIcone, const tstring & strIconFile, const tstring & strFound) = 0;

	/**
	 * \brief Vrification de la date d'un fichier
	 *
	 * (en vue d'un remplacement si plus rcent disponible)
	 * \param[in]	pszPath	Chemin d'accs au fichier  tester
	 * \param[in]	sTiXml	Noeud XML contenant un attribut de date
	 * \return		\p true si une version plus rcente ( tlcharger) existe
	 **/
	static bool CheckFileDate(LPCTSTR pszPath, const TiXmlElement & sTiXml);

	/**
	 * \brief Traitement d'un noeud d'application
	 * \param[in]	pszReqType	Type de noeud recherch ("stable" ou "beta")
	 * \param[in]	pszDispType	Type de noeud recherch (pour affichage)
	 **/
	bool UpdateTraiteAppNode(LPCSTR pszReqType, LPCTSTR pszDispType);

	bool UpdateTraiteFileNode(const TiXmlElement & sTiXml);

	bool UpdateTraiteIcons(const TiXmlElement & sTiXml);

	/**
	 * Fonction traitant le fichier XML dcrivant la mise  jour
	 *
	 * \return	Cumul de \p UPDATE_APP_FLAG (si une mise  jour d'application a t dtecte ou ralise)
				ou \p UPDATE_ICON_FLAG (si une mise  jour d'icne(s) a t dtecte ou ralise)
	 **/
	BYTE UpdateTraiteXml();

public:
	CUpdateThread(CWebUpdateDialog &	cUpDlg, bool bWBeta) :
		WorkingThread(false),
		cUpdDlg(cUpDlg),
		bWithBeta(bWBeta),
		stop_maj(false)
#if UPDATE_VIA_EXTERNAL == 0
		, bAdminWarnDisplayed(false)
#endif
	{
	}

	/// Ordonne au thread de s'arrter
	void Stop() {
		stop_maj = true;
		// Patienter 4 secondes pour l'arrt du thread
		WaitThreadExit(4 * 1000); }

};

class CWebUpdateDialog : public CModalDialogBase
{
	friend class CUpdateThread;
	friend class CUpdateCheckThread;
	friend class CUpdateTelechargeThread;


	HWND					hUpdateStatut;			//!< Handle de la liste indiquant la progression
	HWND					hProgressStatut;		//!< Handle de la barre de progression
	HWND					hUpdateButton;			//!< Handle du bouton "Mettre  jour"
	HWND					hUpdateBetaCheckBox;	//!< Handle du bouton "Inclure les version beta"
	auto_ptr<CUpdateThread>	pUpdateThread;			//!< Pointeur sur le "thread" actif
	TiXmlDocument			xmlVersionDocument;		//!< Objet TinyXML pour traiter le fichier XML reu
	UpdateState				eCurrentState;			//!< tat courant de la mise  jour

	/**
	 * \brief Passage  un nouvel tat
	 *
	 * \param[in]  eNewState	tat auquel passer
	 **/
	void SetState(UpdateState eNewState);

	/**
	 * \brief Incrmente la barre de progression
	 *
	 * \param[in]  nMaxPos	Index maximal
	 **/
	void InitProgress(int nMaxPos);

	/**
	 * \brief Incrmente la barre de progression
	 *
	 * \param[in]  iNewDeltaPos	index indiquant la nouvelle position de la barre de progression
	 *							(utiliser -1 pour aller  la valeur maximale)
	 **/
	void CWebUpdateDialog::AddProgress(int iNewDeltaPos=1);

	virtual INT_PTR DialogProc(UINT uMsg, WPARAM wParam, LPARAM lParam);

public:
	/// Constructeur
	CWebUpdateDialog() :
		CModalDialogBase(IDD_UPDATE),
		hUpdateStatut(NULL),
		hProgressStatut(NULL),
		hUpdateButton(NULL),
		hUpdateBetaCheckBox(NULL),
		eCurrentState(us_Undef)
	{
	}

	/// Destructrut
	~CWebUpdateDialog() {
		if (pUpdateThread.get())
			pUpdateThread->Stop();
	}
};

class CUpdateCheckThread : public CUpdateThread
{
	/// Fonction d'excution principale effective du 'thread'.
	virtual DWORD ThreadProc();

	/**
	 * \brief Traitement pour une mise  jour possible du fichier d'application lui-mme
	 * \param[in]	sTiXml		Noeud XML du dcrivant le fichier
	 * \param[in]	pszDispType	Type de noeud recherch ("stable" ou "beta") pour affichage
	 **/
	virtual bool UpdateTraiteApp(const TiXmlElement & sTiXml, LPCTSTR pszDispType);

	/**
	 * \brief Traitement pour une mise  jour d'un fichier secondaire (autre qu'icne)
	 * \param[in]	strUrl		URL de tlchargement du fichier
	 * \param[in]	strPath		Chemin complet pour enregistrer le fichier
	 * \param[in]	strFile		Nom du fichier, seul
	 * \return		\p true ou \p false selon succs ou chec
	 **/
	virtual bool UpdateTraiteFile(const tstring & strUrl, const tstring & strPath, const tstring & strFile);

	/**
	 * \brief Traitement pour une icne dont la mise  jour est requise
	 * \param[in]	strUrl		URL de tlchargement
	 * \param[in]	strPays		Pays concern par l'icne
	 * \param[in]	strDirPays	Rpertoire (relatif) des icnes de ce pays
	 * \param[in]	strIcone	Nom de base de l'icne
	 * \param[in]	strIconFile	Nom de fichier de l'icne
	 * \param[in]	strFound	Chemin de l'icne dj existante  mettre  jour, le cas chant
	 * \return		toujours \p true
	 **/
	virtual bool UpdateTraiteIcon(const tstring & strUrl, const tstring & strPays, const tstring & strDirPays,
								  const tstring & strIcone, const tstring & strIconFile, const tstring & strFound);

public:
	CUpdateCheckThread(CWebUpdateDialog &	cUpDlg, bool bWBeta) :
		CUpdateThread(cUpDlg, bWBeta)
	{
		StartThread();
	}
};

class CUpdateTelechargeThread : public CUpdateThread
{
#if UPDATE_VIA_EXTERNAL
	tstring			strArgs;	//!< Liste de mouvements de fichiers  effectuer

	/**
	 * \brief Sauvegarde d'un fichier  mettre  jour dans un fichier temporaire
	 *
	 * Le contenu du fichier est sauvegarder dans un fichier temporaire, et les informations
	 * ncessaires pour le mettre ensuite en place sont ajoutes  la liste
	 * \param[in]	vData		Donnes  sauvegarder
	 * \param[in]	pszFileName	Nom de fichier final (aprs mise en place)
	 *`\return		\p true ou \p false selon succs ou erreur
	 **/
	bool			SaveUpdateInTemp(InternetFile & vData, LPCTSTR pszFileName);

	/**
	 * \brief Appel du processus externe de finalisation de la mise  jour
	 *
	 * Extraction du processus des ressources, dcompression, et lancement avec lvation
	 * de privilges (cette lvation tant paramtre dans le manifeste du processus externe).
	 *
	 * \param[in]	bAppIncluded	\p true si la mise  jour inclut l'application elle-mme
	 * \return		0 si succs, ou bien code d'erreur systme
	 **/
	DWORD			CallMoveProcess(bool bAppIncluded);
#endif

	/// Fonction d'excution principale effective du 'thread'.
	virtual DWORD	ThreadProc();

	/**
	 * \brief Traitement pour une mise  jour possible du fichier d'application lui-mme
	 * \param[in]	sTiXml		Noeud XML du dcrivant le fichier
	 * \param[in]	pszDispType	Type de noeud recherch ("stable" ou "beta") pour affichage
	 **/
	virtual bool	UpdateTraiteApp(const TiXmlElement & sTiXml, LPCTSTR pszDispType);

	/**
	 * \brief Traitement pour une mise  jour d'un fichier secondaire (autre qu'icne)
	 * \param[in]	strUrl		URL de tlchargement du fichier
	 * \param[in]	strPath		Chemin complet pour enregistrer le fichier
	 * \param[in]	strFile		Nom du fichier, seul
	 * \return		\p true ou \p false selon succs ou chec
	 **/
	virtual bool	UpdateTraiteFile(const tstring & strUrl, const tstring & strPath, const tstring & strFile);

	/**
	 * \brief Traitement pour une icne dont la mise  jour est requise
	 * \param[in]	strUrl		URL de tlchargement
	 * \param[in]	strPays		Pays concern par l'icne
	 * \param[in]	strDirPays	Rpertoire (relatif) des icnes de ce pays
	 * \param[in]	strIcone	Nom de base de l'icne
	 * \param[in]	strIconFile	Nom de fichier de l'icne
	 * \param[in]	strFound	Chemin de l'icne dj existante  mettre  jour, le cas chant
	 * \return		\p true ou \p false selon succs ou chec
	 **/
	virtual bool UpdateTraiteIcon(const tstring & strUrl, const tstring & strPays, const tstring & strDirPays,
								  const tstring & strIcone, const tstring & strIconFile, const tstring & strFound);

public:
	CUpdateTelechargeThread(CWebUpdateDialog &	cUpDlg, bool bWBeta) :
		CUpdateThread(cUpDlg, bWBeta)
	{
		StartThread();
	}
};

/// Indique si une mise  jour est en attente de redmarrage
static bool bUpdateNeedRestart		= false;

/// Structure dcrivant une version de l'application
struct AppVersion
{
	int		nMajor;
	int		nMinor;
	int		nRev;
	CHAR	szSvn[8];	// Des caractres alphanumriques peuvent exister sur ce champ

	/// Constructeur chargeant la version avec une chane de version
	AppVersion(LPCSTR pszVersion) :
		nMajor(0),
		nMinor(0),
		nRev(0)
	{
		szSvn[0] = '0';	szSvn[1] = 0;
		// On rcupre la version de PTVM fonctionnant actuellement
		sscanf_s(PRODUCT_VERSION_STR, "%i.%i.%i.%s", &nMajor, &nMinor, &nRev, szSvn, _countof(szSvn));
	}

	/// Constructeur chargeant la version avec les attributs d'un lment TiXML
	AppVersion(const TiXmlElement & sTiXml) :
		nMajor(0),
		nMinor(0),
		nRev(0)
	{
		szSvn[0] = '0';	szSvn[1] = 0;
		sTiXml.QueryIntAttribute("major", &nMajor);
		sTiXml.QueryIntAttribute("minor", &nMinor);
		sTiXml.QueryIntAttribute("rev", &nRev);

		LPCSTR pszSvn = sTiXml.Attribute("svn");

		if (pszSvn && *pszSvn)
			strcpy_T(szSvn, pszSvn);
	}

	bool operator > (const AppVersion & sV2) const {
		return
			nMajor>sV2.nMajor ||
			nMajor==sV2.nMajor && (
				nMinor>sV2.nMinor ||
				nMinor==sV2.nMinor &&
					nRev > sV2.nRev ||
					nRev == sV2.nRev &&
						atoi(szSvn) > atoi(sV2.szSvn));
	}

	bool operator < (const AppVersion & sV2) const {
		return ! operator > (sV2);}

	/// Conversion en chane (le rsultat tant maintenu sur une variable statique,
	/// ne pas utiliser cette fonction deux fois dans la mme instruction)
	LPCTSTR toString() const {
		static TCHAR szTmp[20];

		_stprintf_s(szTmp, TEXT("%d.%d.%d.%") A2t, nMajor, nMinor, nRev, szSvn);
		return szTmp;}
};

#if UPDATE_VIA_EXTERNAL == 0
/**
 * \brief Remplace le fichier par le nouveau, en sauvegardant l'ancien
 *
 * \param[in]  nouveau   nom du fichier venant d'tre tlcharg
 * \param[in]  original  ancien fichier devant tre remplac
 * \param[in]  backup    si true, on ne sauvegarde pas l'ancien fichier, on l'efface tout simplement
 * \retval	0 si tout s'est bien droul
 * \retval	Code d'erreur systme en cas d'erreur
 **/
DWORD CUpdateThread::InternetFile::RemplaceFichier(LPCTSTR nouveau, LPCTSTR original, bool backup) // static
{
	SZPathName	fichierBackup;		// Fichier temporaire

	myprintf(TEXT("Remplace %s par %s en %s") EOL,  original, nouveau, backup ? TEXT("sauvegardant") : TEXT("effaant"));

	// Nom du fichier de backup
	strcpy_T(fichierBackup, original);
	strcat_T(fichierBackup, TEXT("_"));

	// On renomme l'actuel PTVM
	BOOL ret =
		ReplaceFile(
			original,  nouveau,
			backup ? fichierBackup : NULL,
			REPLACEFILE_WRITE_THROUGH|REPLACEFILE_IGNORE_MERGE_ERRORS,
			0,  0);

	if (!ret)
		return GetLastError();

	return 0;
}
#endif

/**
 * Enregistre un fichier depuis un tableau d'octets
 *
 * \param[in]	nom	Nom du fichier  sauvegarder
 * \retval		0 si tout s'est bien droul
 * \retval		Code d'erreur systme en cas d'erreur
 **/
DWORD CUpdateThread::InternetFile::SauveFichier(LPCTSTR nom) const
{
	myprintf(TEXT("Sauvegarde %s depuis un tlchargement de %u octets") EOL, nom, size());

	return SaveAsFile(nom, &sTime);
}

#if UPDATE_VIA_EXTERNAL == 0
/**
 * \brief Remplace le fichier par le nouveau, en sauvegardant l'ancien
 *
 * \param[in]  original	Ancien fichier devant tre remplac
 * \param[in]  backup	Si true, on ne sauvegarde pas l'ancien fichier, on l'efface tout simplement
 * \retval	0 si tout s'est bien droul
 * \retval	Code d'erreur systme en cas d'erreur
 **/
DWORD CUpdateThread::InternetFile::RemplaceFichier(LPCTSTR original, bool backup) const
{
	SZPathName	fichierTemp;			// Fichier temporaire

	myprintf(TEXT("Remplace %s depuis un tlchargement de %u octets") EOL, original, size());

	// Nom du fichier de mise  jour
	strcpy_T(fichierTemp, original);
	strcat_T(fichierTemp, TEXT(".update"));

	DWORD nRes = SauveFichier(fichierTemp);

	if (nRes==0)
		nRes = RemplaceFichier(fichierTemp, original, backup);

	return nRes;
}
#endif

/**
 * \brief Ajoute le message dans la liste
 *
 * \param[in]  pszFmt		message ou chane de formatage devant tre affich dans la liste de statut
 **/
void CUpdateThread::AddMsg(LPCTSTR pszFmt, ...)
{
	FMT_BUF(szBuf, 512, pszFmt, pszFmt);

	AddLineToList(cUpdDlg.hUpdateStatut, szBuf);
}

/**
 * \brief Passage  un nouvel tat
 *
 * \param[in]  eNewState	tat auquel passer
 **/
void CWebUpdateDialog::SetState(UpdateState eNewState)
{
	if (eNewState != eCurrentState) {
		eCurrentState = eNewState;

		switch (eCurrentState) {

		case us_NeedCheck:
			SetWindowText(hUpdateButton, TEXT(""));
			EnableWindow(hUpdateButton, TRUE);	// Active le bouton
			break;

		case us_Checking:
			EnableWindow(hUpdateButton, FALSE);
			InitProgress(4);	// On met 4 tapes pour la vrification de la version
			break;

		case us_NeedUpdate:
			EnableWindow(hUpdateBetaCheckBox, FALSE);
			SetWindowText(hUpdateButton, TEXT("  "));
			EnableWindow(hUpdateButton, TRUE);
			break;

		case us_Updating:
			EnableWindow(hUpdateButton, FALSE);
			InitProgress(1);	// Une seule tape, en attendant d'en connatre le nombre
			break;

		case us_NeedRestart:
			SetWindowText(hUpdateButton, TEXT(""));
			EnableWindow(hUpdateButton, TRUE);	// Active le bouton
		}
	}
}

/**
 * \brief Incrmente la barre de progression
 *
 * \param[in]  nMaxPos	Index maximal
 **/
void CWebUpdateDialog::InitProgress(int nMaxPos)
{
	if (hProgressStatut) {
		// On met les tapes
		SendMessage(hProgressStatut, PBM_SETRANGE, 0, MAKELPARAM(0, nMaxPos));

		// On se positionne au dbut de la barre de progression
		SendMessage(hProgressStatut, PBM_SETPOS, 0, 0);
	}
}

/**
 * \brief Incrmente la barre de progression
 *
 * \param[in]  iNewDeltaPos	index indiquant la nouvelle position de la barre de progression
 *							(utiliser -1 pour aller  la valeur maximale)
 **/
void CWebUpdateDialog::AddProgress(int iNewDeltaPos)
{
	int nMax = (int)SendMessage(hProgressStatut, PBM_GETRANGE, false, 0);
	int nPos = (int)SendMessage(hProgressStatut, PBM_GETPOS, 0, 0);
	int	nDif = nMax-nPos;

	if (iNewDeltaPos < 0 || nDif < iNewDeltaPos)
		iNewDeltaPos = nMax-nPos;
	if (iNewDeltaPos > 0)
		SendMessage(hProgressStatut, PBM_DELTAPOS, iNewDeltaPos, 0);
}

/**
 * Lance la rcupration du fichier, dont l'url est passe en paramtre,
 * et stocke le fichier rcupr dans \p vData
 *
 * \param[in]	url		Adresse contenant les fichiers  rcuprer
 * \param[out]	vData	Vecteur d'octets contenant le fichier rcupr, initialis dans la fonction
 * \return	Taille du fichier rcupr ( = vData.size() )
 **/
DWORD CUpdateThread::GetInternetFile(LPCTSTR url, InternetFile & vData)
{
	// Si vData est renseign, on quitte
	if (!vData.empty())
		return 0;

	bool		stoppe			= false;

	// En version Unicode, la conversion en UTF-8 est implicite
	ConnexionInternet maj(	// Objet traitant de la connexion Internet
		tstr_printf(TEXT("%s / %s"), szAppName, szAppVersion).c_str(),
#ifdef _UNICODE
		url
#else
		ConvertChar2UTF8(url).c_str()
#endif
		);

	myprintf(TEXT("Tlchargement de %s") EOL, url);

	int http_statut = maj.EnvoieRequete();

	if (http_statut != maj.HTTP_ERR_OK) {
		if (http_statut == 0) {
			AddMsg(TEXT("  ."));
			myprintf(ERRX(TEXT("Erreur connexion")));
		} else {
			AddMsg(TEXT(" %d     ."), http_statut);
			AddMsg(TEXT("  -..."));
			myprintf(ERRX(TEXT("Erreur connexion: erreur HTTP: %d")), http_statut);
		}
		stoppe = true;
	}

	if (stop_maj || stoppe)
		return 0;

	if (cUpdDlg.eCurrentState == us_Checking)
		AddMsg(TEXT("    ..."));

	// Tout est OK, on rcupre le fichier
	BYTE	temp[8*1024];	// Buffer de 8 Ko
	DWORD	lu = sizeof(temp), dwLuTotal = 0;

	if (cUpdDlg.eCurrentState == us_Updating) {
		// On initialise la barre de progression, avec autant de morceaux que ncessaire
		cUpdDlg.InitProgress(int(maj.GetTaille() / sizeof(temp))+1);
	}

	while (!stop_maj && maj.RecupereFichier(temp, &lu)) {
		vData.resize(dwLuTotal+lu);			// Redimensionnement
		memcpy_s(&vData.get()[dwLuTotal], lu, temp, lu);

		dwLuTotal += lu;

		// Remet le compteur de lecture  sa valeur max
		lu = sizeof(temp);

		if (stop_maj) // Arrt demand, on quitte
			break;

		// Si on tlcharge, on met  jour la barre de progression
		if (cUpdDlg.eCurrentState == us_Updating)
			cUpdDlg.AddProgress();
	}

	vData.sTime = maj.GetModifTime();
	return dwLuTotal;
}

/**
 * Lance la rcupration du fichier compress, dont l'url est passe en paramtre,
 * et stocke le fichier rcupr dans \p vData
 *
 * \param[in]	url		Adresse contenant les fichiers  rcuprer
 * \param[out]	vData	Vecteur d'octets contenant le fichier rcupr, initialis dans la fonction
 * \return		Taille du fichier rcupr ( = vData.size() )
 **/
INT CUpdateThread::GetCompressedInternetFile(LPCTSTR url, InternetFile & vData)
{
	INT				nSize = 0;
	InternetFile	vCompressedData;

	if (GetInternetFile(url, vCompressedData) > 0) {
		// Des donnes ont t rcupres
		bool		bResLzma = DecompressLzma(vData, vCompressedData);

		vData.sTime = vCompressedData.sTime;

		if (bResLzma)
			nSize = (INT)vData.size();
		else
			return -1;	// Erreur  la dcompression
	}
	return nSize;
}

/**
 * \brief Traitement pour une mise  jour possible du fichier d'application lui-mme
 * \param[in]	sTiXml		Noeud XML du dcrivant le fichier
 * \param[in]	pszDispType	Type de noeud recherch ("stable" ou "beta") pour affichage
 **/
bool CUpdateCheckThread::UpdateTraiteApp(const TiXmlElement & sTiXml, LPCTSTR pszDispType)
{
	AppVersion	sAppVers(PRODUCT_VERSION_STR);	// Version courante de PTVM
	AppVersion	sXmlVers(sTiXml);			// Version du noeud XML
	bool		bUpdate	= false;

	if (sXmlVers > sAppVers) {
		// Nouvelle version disponible

		// Affiche simplement le message informatif dans la fentre de statut
		AddMsg(
			TEXT("  %s  %s   : %s"),
			pszDispType, szAppName, sXmlVers.toString());
		AddMsg(TEXT("  : %s"), sAppVers.toString());
		bUpdate  = true;
	} else
		AddMsg(
			TEXT("   %s  %s"),
			pszDispType, szAppName);

	return bUpdate;
}

#if UPDATE_VIA_EXTERNAL
/**
 * \brief Sauvegarde d'un fichier  mettre  jour dans un fichier temporaire
 *
 * Le contenu du fichier est sauvegarder dans un fichier temporaire, et les informations
 * ncessaires pour le mettre ensuite en place sont ajoutes  la liste
 * \param[in]	vData		Donnes  sauvegarder
 * \param[in]	pszFileName	Nom de fichier final (aprs mise en place)
 *`\return		\p true ou \p false selon succs ou erreur
 **/
bool CUpdateTelechargeThread::SaveUpdateInTemp(InternetFile	& vData, LPCTSTR pszFileName)
{
	TempPathName sTmpFile;

	// Des donnes ont t rcupres
	// Tout semble bon, on sauvegarde le nouveau fichier
	DWORD nRes = vData.SauveFichier(sTmpFile);

	if (nRes!=0) {
		// Des erreurs d'criture du fichier se sont produites. L'indiquer.
		AddMsg(TEXT(" %u : %s"), nRes, SystemErrorToString(nRes).c_str());
		AddMsg(TEXT("   %s."), sTmpFile.szPath);
		return false;
	}

	// Ajouter le fichier enregistr  la liste des fichiers qui doivent tre mis en place
	// par l'utilitaire de mise  jour qui va tre lanc avec des droits d'administration
	// levs.
	strArgs += tstr_printf(TEXT("\"%s\"=\"%s\"") EOL, sTmpFile.szPath, pszFileName);
	return true;
}
#endif

/**
 * \brief Traitement pour une mise  jour possible du fichier d'application lui-mme
 * \param[in]	sTiXml		Noeud XML du dcrivant le fichier
 * \param[in]	pszDispType	Type de noeud recherch ("stable" ou "beta") pour affichage
 **/
bool CUpdateTelechargeThread::UpdateTraiteApp(const TiXmlElement & sTiXml, LPCTSTR pszDispType)
{
	AppVersion	sAppVers(PRODUCT_VERSION_STR);	// Version courante de PTVM
	AppVersion	sXmlVers(sTiXml);			// Version du noeud XML
	bool		bUpdate	= false;

	if (sXmlVers > sAppVers) {
		// Nouvelle version disponible

		// Tlcharge la nouvelle version de PTVM
		AddMsg(TEXT("     %s..."), szAppName);

		tstring			strUrl	= ConvertUTF82Char(sTiXml.FirstChild()->Value());
		InternetFile	vData;
		INT				nSize	= GetCompressedInternetFile(strUrl.c_str(), vData);

		if (nSize > 0) {
#if UPDATE_VIA_EXTERNAL
			bUpdate = SaveUpdateInTemp(vData, szAppFileName);
#else
			// Des donnes ont t rcupres
			// Tout semble bon, on sauvegarde le nouveau fichier
			DWORD nRes = vData.RemplaceFichier(szAppFileName, true);

			if (nRes==0) {
				AddMsg(TEXT("%s  ."), szAppName);
				AddMsg(TEXT("  %s    ."), szAppName);
				bUpdate  = true;
			} else {
				// Des erreurs d'criture du fichier se sont produite. L'indiquer
				AddMsg(TEXT(" %u : %s"), nRes, SystemErrorToString(nRes).c_str());
				AddMsg(TEXT("    %s."), szAppName);
				if (nRes==ERROR_ACCESS_DENIED && !bAdminWarnDisplayed) {
					AddMsg(TEXT(""));
					AddMsg(TEXT("##          "));
					AddMsg(TEXT("## /     "));
					AddMsg(TEXT("## ( ,   Windows Vista / 7)."));
					AddMsg(TEXT(""));
					bAdminWarnDisplayed = true;
				}
			}
#endif
		} else if (nSize < 0)
			AddMsg(TEXT("   .  -."));
		else
			AddMsg(TEXT("      ."));
	}

	return bUpdate;
}

/**
 * \brief Traitement pour une mise  jour d'un fichier secondaire (autre qu'icne)
 * \param[in]	strUrl		URL de tlchargement du fichier
 * \param[in]	strPath		Chemin complet pour enregistrer le fichier
 * \param[in]	strFile		Nom du fichier, seul
 * \return		\p true ou \p false selon succs ou chec
 **/
bool CUpdateCheckThread::UpdateTraiteFile(const tstring & strUrl, const tstring & strPath, const tstring & strFile)
{
	AddMsg(TEXT("    %s"), strFile.c_str());
	return true;
}

/**
 * \brief Traitement pour une icne dont la mise  jour est requise
 * \param[in]	strUrl		URL de tlchargement
 * \param[in]	strPays		Pays concern par l'icne
 * \param[in]	strDirPays	Rpertoire (relatif) des icnes de ce pays
 * \param[in]	strIcone	Nom de base de l'icne
 * \param[in]	strIconFile	Nom de fichier de l'icne
 * \param[in]	strFound	Chemin de l'icne dj existante  mettre  jour, le cas chant
 * \return		toujours \p true
 **/
bool CUpdateCheckThread::UpdateTraiteIcon(const tstring & strUrl, const tstring & strPays, const tstring & strDirPays,
										  const tstring & strIcone, const tstring & strIconFile, const tstring & strFound)
{
	AddMsg(
		strFound.empty() ?
			TEXT("    %s disponible: %s") :
			TEXT(" : icne %s %s"),
		strPays.c_str(), strIcone.c_str() );
	return true;
}

/**
 * \brief Traitement pour une mise  jour d'un fichier secondaire (autre qu'icne)
 * \param[in]	strUrl		URL de tlchargement du fichier
 * \param[in]	strPath		Chemin complet pour enregistrer le fichier
 * \param[in]	strFile		Nom du fichier, seul
 * \return		\p true ou \p false selon succs ou chec
 **/
bool CUpdateTelechargeThread::UpdateTraiteFile(const tstring & strUrl, const tstring & strPath, const tstring & strFile)
{
	InternetFile	vData;

	AddMsg(TEXT("  %s"), strFile.c_str());

	// Envoie la connexion
	if (GetInternetFile(strUrl.c_str(), vData) > 0) {
#if UPDATE_VIA_EXTERNAL
		return SaveUpdateInTemp(vData, strPath.c_str());
#else
		DWORD nRes = vData.SauveFichier(strPath.c_str());

		if (nRes==0) {
			AddMsg(TEXT(" %s    ."), strFile.c_str());
			return true;
		} else {
			// Des erreurs d'criture du fichier se sont produites. L'indiquer.
			AddMsg(TEXT(" %u : %s"), nRes, SystemErrorToString(nRes).c_str());
			AddMsg(TEXT("     %s ."), strFile.c_str());
			AddMsg(TEXT(" ,      ."));
		}
#endif
	}
	return false;
}

/**
 * \brief Traitement pour une icne dont la mise  jour est requise
 * \param[in]	strUrl		URL de tlchargement
 * \param[in]	strPays		Pays concern par l'icne
 * \param[in]	strDirPays	Rpertoire (relatif) des icnes de ce pays
 * \param[in]	strIcone	Nom de base de l'icne
 * \param[in]	strIconFile	Nom de fichier de l'icne
 * \param[in]	strFound	Chemin de l'icne dj existante  mettre  jour, le cas chant
 * \return		\p true ou \p false selon succs ou chec
 **/
bool CUpdateTelechargeThread::UpdateTraiteIcon(const tstring & strUrl, const tstring & strPays, const tstring & strDirPays,
											   const tstring & strIcone, const tstring & strIconFile, const tstring & strFound)
{
	InternetFile	vData;

	AddMsg(TEXT("   %s"), strIconFile.c_str());

	// Envoie la connexion
	if (GetInternetFile(strUrl.c_str(), vData) > 0) {
		// Des donnes ont t rcupres
#if UPDATE_VIA_EXTERNAL
		if (!strFound.empty()) {
			// Mise  jour d'une icne existante par une version plus rcente :
			// Utiliser le dossier dans lequel elle a t trouve
			return SaveUpdateInTemp(vData, strFound.c_str());
		} else {
			// Enregistrement d'une nouvelle icne dans le premier rpertoire
			// trouv disponible et adquat :
			return SaveUpdateInTemp(vData, (strDirPays + TEXT("\\") + strIconFile).c_str());
		}
#else	// #if UPDATE_VIA_EXTERNAL
		DWORD nRes;

		if (!strFound.empty()) {
			// Mise  jour d'une icne existante par une version plus rcente :
			// Utiliser le dossier dans lequel elle a t trouve
			nRes = vData.SauveFichier(strFound.c_str());
		} else {
			// Enregistrement d'une nouvelle icne dans le rpertoire de l'application
			tstring	strIconPath	= tstr_printf(
									TEXT("%sIcones\\%s%s"),
									pouchindir_prog, strDirPays.c_str(), strIconFile.c_str());

			nRes = vData.SauveFichier(strIconPath.c_str());

			if (nRes!=0) {
				// Impossible de sauvegarder dans le rpertoire de l'application :
				// utiliser le rpertoire de configuration
				strIconPath	= tstr_printf(
								TEXT("%sIcones\\%s%s"),
								pouchindir_conf, strDirPays.c_str(), strIconFile.c_str());
				nRes = vData.SauveFichier(strIconPath.c_str());
			}
		}

		if (nRes==0) {
			AddMsg(TEXT("  %s    ."), strIcone.c_str());
			return true;
		} else {
			// Des erreurs d'criture du fichier se sont produites. L'indiquer.
			AddMsg(TEXT(" %u : %s"), nRes, SystemErrorToString(nRes).c_str());
			AddMsg(TEXT("     %s ."), strIcone.c_str());
			AddMsg(TEXT(" ,      ."));
		}
#endif	// #if UPDATE_VIA_EXTERNAL
	}
	return false;
}

bool CUpdateThread::UpdateTraiteFileNode(const TiXmlElement & sTiXml)
{
	tstring		strUrl	= ConvertUTF82Char(sTiXml.GetText());
	tstring		strFile = ConvertUTF82Char(sTiXml.Attribute("nom"));;

	if (strUrl.empty() || strFile.empty())
		return false;

	tstring		strPath = pouchindir_prog + strFile;

	if (PathFileExists(strPath.c_str())) {
		// si trouv : vrifier si plus rcent disponible
		if (!CheckFileDate(strPath.c_str(), sTiXml))
			return false;	// Plus rcent non disponible
	}
	return UpdateTraiteFile(strUrl, strPath, strFile);
}

bool CUpdateThread::UpdateTraiteIcons(const TiXmlElement & sTiXml)
{
	bool		bUpdate	= false;
	// on parse les icones. tablissement de la liste locale

	// on liste les packs d'icnes
	for (ITERATE_XML_ELEMENTS_NAMED(&sTiXml, xmlIconesPack, "pack")) {
		// Pays du pack (pour classer les icnes)
		tstring	strPays		= ConvertUTF82Char(xmlIconesPack->Attribute("pays"));
		tstring	strDirPays; // 	= strPays==TEXT("France") ? TEXT(".") : (strPays + TEXT("\\")).c_str();
		tstring	strFound;	// Rceptacle gnrique pour les rsultats de recherches

		if (strPays==TEXT("France")) {
			// Si le sous-rpertoire de pays n'existe pas, c'est que l'utilisateur n'a pas
			// slectionn le pack correspondant  l'installation. On ignore donc.
			if (
				FindAppFile(TEXT(""), TEXT("Icones"),
					strDirPays, FF_DIRECTORY) == pd_notfound
			) {
				continue;
			}
		} else {
			// Si le sous-rpertoire de pays n'existe pas, c'est que l'utilisateur n'a pas
			// slectionn le pack correspondant  l'installation. On ignore donc.
			if (
				FindAppFile(TEXT("Icones"), strPays.c_str() /* strPays car on ne veut pas du "\" */,
					strDirPays, FF_DIRECTORY) == pd_notfound
			) {
				continue;
			}
		}

		// On parcourt les diffrentes icnes (au cas o certaines seraient manquantes)
		for (ITERATE_XML_ELEMENTS_NAMED(xmlIconesPack, xmlIconesIcone, "icone")) {
			tstring	strIcone	= ConvertUTF82Char(xmlIconesIcone->GetText());
			tstring	strIconFile	= strIcone + TEXT(".bmp");
			bool	bNewVers	= false;

			// Recherche dans les rpertoires de l'application :
			if (FindAppFile(TEXT("Icones"), strIconFile.c_str(), strFound)!=pd_notfound) {
				// si trouv : vrifier si plus rcent disponible
				if (!CheckFileDate(strFound.c_str(), *xmlIconesIcone))
					continue;	// Plus rcent non disponible
				bNewVers	= true;
			}

			tstring	strUrlBase	= ConvertUTF82Char(sTiXml.Attribute("base"));
			tstring	strUrlPath	= ConvertUTF82Char(xmlIconesPack->Attribute("path"));

			// Ajustement du rpertoire dans l'URL
			if (strUrlPath==TEXT("/"))
				strUrlPath.clear();
			else
				strUrlPath += TEXT("/");

			tstring	strUrl		= strUrlBase + strUrlPath + strIconFile;

			if (UpdateTraiteIcon(strUrl, strPays, strDirPays, strIcone, strIconFile, strFound))
				bUpdate  = true;
		} // For (icone)
	} // For (pack)
	// Plus d'cones  traiter
	return bUpdate;
}

/// Vrification de la version du noeud XML
/// Retourne \p true si le nud est valide
static bool CheckVersionVersion(const TiXmlElement & sTiXml)
{
	// Rcupre la version de la branche
	int updateversion;
	int	nQueryResult = sTiXml.QueryIntAttribute("version", &updateversion);

	if (
		nQueryResult == TIXML_NO_ATTRIBUTE ||	// anciennes versions de PTVM, on ignore
		updateversion > UPDATE_VERSION_MAX		// nud plus rcent, on le saute, il sera lu une fois PTVM mis  jour
	) {
		return false;
	}
	return true;
}

/**
 * \brief Vrification de la date d'un fichier
 *
 * (en vue d'un remplacement si plus rcent disponible)
 * \param[in]	pszPath	Chemin d'accs au fichier  tester
 * \param[in]	sTiXml	Noeud XML contenant un attribut de date
 * \return		\p true si une version plus rcente ( tlcharger) existe
 **/
bool CUpdateThread::CheckFileDate(LPCTSTR pszPath, const TiXmlElement & sTiXml)
{
	INT64		nRmtFTime	= 0;	// Date et heure du fichier distant
	INT64		nLocFTime	= 0;	// Date et heure du fichier local
	LPCSTR		pszUtime	= sTiXml.Attribute("utime");	// Timestamp Unix maintenant inclus

	if (pszUtime) {
		// http://msdn.microsoft.com/en-us/library/ms724228%28v=vs.85%29.aspx
		nRmtFTime = Int32x32To64(atoi(pszUtime), 10000000) + 116444736000000000I64;
	} else {
		// Plus vraiment utile mais on laisse quand mme pour le moment
		LPCSTR		pszDate		= sTiXml.Attribute("date");

		if (pszDate == NULL)
			return false;

		UINT nYear, nMonth, nDay;

		if (sscanf_s(pszDate, "%u-%u-%u", &nYear, &nMonth, &nDay) != 3)
			return false;

		SYSTEMTIME	sTmpTime	= {		// Date et heure du fichier distant
			(WORD)nYear,	// wYear
			(WORD)nMonth,	// wMonth
			0,				// wDayOfWeek
			(WORD)nDay		// wDay
			// Le reste  zro (implicite)
		};

		if (!SystemTimeToFileTime(&sTmpTime, reinterpret_cast<LPFILETIME>(&nRmtFTime)))
			return false;
	}

	// Ouverture du fichier pour rcupration de la date
	HANDLE		hFile		= CreateFile(pszPath, GENERIC_READ, FILE_SHARE_READ, NULL,
								OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);

	if (hFile != INVALID_HANDLE_VALUE) {
		BOOL		bRes = GetFileTime(hFile, NULL, NULL, reinterpret_cast<LPFILETIME>(&nLocFTime));

		CloseHandle(hFile);	// Fermeture du fichier
		if (!bRes)
			return false;
	}

#ifdef _DIAGNOSTIC
	SYSTEMTIME	sDebugRmt, sDebugLoc;

	FileTimeToSystemTime((LPFILETIME)&nRmtFTime, &sDebugRmt);
	FileTimeToSystemTime((LPFILETIME)&nLocFTime, &sDebugLoc);
	myprintf(TEXT("CheckFileDate (%s)") EOL
		TEXT("\tlocal=%04u-%02u-%02u %02u:%02u:%02u, distant=%04u-%02u-%02u %02u:%02u:%02u") EOL, pszPath,
		sDebugLoc.wYear, sDebugLoc.wMonth, sDebugLoc.wDay, sDebugLoc.wHour, sDebugLoc.wMinute, sDebugLoc.wSecond,
		sDebugRmt.wYear, sDebugRmt.wMonth, sDebugRmt.wDay, sDebugRmt.wHour, sDebugRmt.wMinute, sDebugRmt.wSecond);
#endif
	return (nRmtFTime - nLocFTime) > 50000000I64;	// Cinq secondes de marge d'erreur
}

/**
 * \brief Traitement d'un noeud d'application
 * \param[in]	pszReqType	Type de noeud recherch ("stable" ou "beta")
 * \param[in]	pszDispType	Type de noeud recherch (pour affichage)
 **/
bool CUpdateThread::UpdateTraiteAppNode(LPCSTR pszReqType, LPCTSTR pszDispType)
{
	bool bUpdateNodeFound = false;
	bool bUpdateFound = false;

	myprintf(TEXT("Vrification pour la version %s") EOL, pszDispType);
	for (ITERATE_XML_ELEMENTS_NAMED(cUpdDlg.xmlVersionDocument.FirstChildElement("Versions"), xmlVersionElem, "version")) {
		if (!CheckVersionVersion(*xmlVersionElem))
			continue;

		// On ignore les versions dont le type de vrification n'a pas t choisi
		if (_stricmp(xmlVersionElem->Attribute("type"), pszReqType)!=0)
			continue;

		bUpdateNodeFound = true;		// On a trouv un descripteur de mise  jour (ce qui n'implique
										// pas ncessairement qu'on ait trouv une mise  jour ...)

		// Rcupration des mises  jour de l'application
		for (ITERATE_XML_ELEMENTS_NAMED(xmlVersionElem, xmlTypeMaj, REQ_APP_TYPE)) {
			if (UpdateTraiteApp(*xmlTypeMaj, pszDispType)) {
				bUpdateFound = true;	// Voil, maintenant on a trouv une mise  jour
				break;	// Une seule mise  jour possible, donc on sort
			}
		}
		if (bUpdateFound)	// toujours pas de "break" multi-niveaux en C/C++ ...
			break;
	}
	if (!bUpdateNodeFound)
		AddMsg(TEXT("     : %s"), pszDispType);
	myprintf(TEXT("%s version %s") EOL, (bUpdateFound ? TEXT("Trouv") : TEXT("Pas trouv")), pszDispType);
	return bUpdateFound;
}

#define UPDATE_APP_FLAG		0x01
#define UPDATE_ICON_FLAG	0x02
#define UPDATE_HELP_FLAG	0x04

/**
 * Fonction traitant le fichier XML dcrivant la mise  jour
 *
 * \return	Cumul de \p UPDATE_APP_FLAG (si une mise  jour d'application a t dtecte ou ralise)
 *			ou \p UPDATE_ICON_FLAG (si une mise  jour d'icne(s) a t dtecte ou ralise)
 *			ou \p UPDATE_HELP_FLAG (si une mise  jour de fichier d'aide a t dtecte ou ralise)
 **/
BYTE CUpdateThread::UpdateTraiteXml()
{
	// Rcupration de la version de PTVM fonctionnant actuellement, pour comparaison :
	AppVersion	sAppVers(PRODUCT_VERSION_STR);

	if (!cUpdDlg.xmlVersionDocument.FirstChildElement("Versions")) {
		// Fichier bien XML, mais pas la bonne racine, on quitte donc
		AddMsg(TEXT("   .   -..."));
		return 0;
	}

	// Commence le parcours des versions dans le fichier XML pars
	BYTE		retval			= 0;

	if (UpdateTraiteAppNode("stable", TEXT("stable")))
		retval |= UPDATE_APP_FLAG;

	myprintf(TEXT("retval=0x%02x, bWithBeta=%u") EOL, retval, bWithBeta);

	if (retval==0 && bWithBeta) {
		// Si (et seulement si) aucune version stable n'a t trouve, et que l'utilisateur
		// souhaite aussi les versions beta, alors on recherche celles-ci.
		// S'il y a  la fois une version stable et une version beta, on ne prend que la
		// stable. La version beta sera ventuellement mise  jour  la recherche suivante.
		if (UpdateTraiteAppNode("beta", TEXT("beta")))
			retval |= UPDATE_APP_FLAG;
	}

	for (ITERATE_XML_ELEMENTS_NAMED(cUpdDlg.xmlVersionDocument.FirstChildElement("Versions"), xmlVersionElem, "version")) {
		if (!CheckVersionVersion(*xmlVersionElem))
			continue;

		// On ignore les versions dont le type de vrification n'a pas t choisi
		if (_stricmp(xmlVersionElem->Attribute("type"), "common")!=0)
			continue;

		// Rcupration des mises  jour des icnes et du fichier d'aide
		for (ITERATE_XML_ELEMENTS(xmlVersionElem, xmlTypeMaj)) {
			if (_stricmp(xmlTypeMaj->Value(), "icones")==0) {
				if (UpdateTraiteIcons(*xmlTypeMaj))
					retval |= UPDATE_ICON_FLAG;
			} else if (_stricmp(xmlTypeMaj->Value(), "fichier")==0) {
				if (UpdateTraiteFileNode(*xmlTypeMaj))
					retval |= UPDATE_HELP_FLAG;
			}
		}
	}

	return retval;
}

#if UPDATE_VIA_EXTERNAL
/**
 * \brief Appel du processus externe de finalisation de la mise  jour
 *
 * Extraction du processus des ressources, dcompression, et lancement avec lvation
 * de privilges (cette lvation tant paramtre dans le manifeste du processus externe).
 *
 * \param[in]	bAppIncluded	\p true si la mise  jour inclut l'application elle-mme
 * \return		0 si succs, ou bien code d'erreur systme
 **/
DWORD CUpdateTelechargeThread::CallMoveProcess(bool bAppIncluded)
{
	TempPathName	sArgFile;
	DWORD			dwErr = 0, nWritten;
	HANDLE			hArgFile = CreateFile(sArgFile, GENERIC_WRITE, FILE_SHARE_READ|FILE_SHARE_DELETE,
								NULL, CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);

	if (hArgFile == INVALID_HANDLE_VALUE)
		return GetLastError();

	if (!WriteFile(hArgFile, strArgs.c_str(), DWORD(strArgs.length()*sizeof(TCHAR)), &nWritten, NULL))
		dwErr = GetLastError();

	CloseHandle(hArgFile);
	if (dwErr)
		return dwErr;

	TempPathName	sExeFile(TEXT("PTvM_Autoupdate.exe"));

	{
		RawResource	vCompressed;
		ByteVector	vUpdApp;

		// Extraction de l'application de mise  jour
		if (
			vCompressed.Extraction(hAppInstance, IDR_AUTOUPDATE_EXE) &&
			DecompressLzma(vUpdApp, vCompressed)
		) {
			dwErr = vUpdApp.SaveAsFile(sExeFile);
		}

		// (les variables vCompressed et vUpdApp sont dtruites ici)
	}

	if (dwErr != 0) {
		AddMsg(TEXT("    ,  %u"), dwErr);
		AddMsg(TEXT("-> %s"), SystemErrorToString(dwErr).c_str());
		CloseHandle(hArgFile);
		return dwErr;
	}

	tstring				strArgFile = tstr_printf(TEXT("\"%s\""), sArgFile.szPath);
	SHELLEXECUTEINFO	sShExInf = {
		sizeof(sShExInf),			// cbSize
		SEE_MASK_NOCLOSEPROCESS,	// fMask
		cUpdDlg.hDlg,				// hwnd
		TEXT("open"),				// lpVerb
		sExeFile,					// lpFile
		strArgFile.c_str(),			// lpParameters
		NULL,						// lpDirectory
		SW_SHOW,					// nShow
		NULL,						// hInstApp
		NULL,						// lpIDList
		NULL,						// lpClass
		NULL,						// hkeyClass
		0,							// dwHotKey
		{NULL},						// hIcon /  hMonitor
		NULL,						// hProcess
	};

	if (ShellExecuteEx(&sShExInf)) {
		// Attente de sortie du processus de mise  jour
		if (WaitForSingleObject(sShExInf.hProcess, 10000) == WAIT_OBJECT_0) {
			// Processus termin : rcupration du code retour
			GetExitCodeProcess(sShExInf.hProcess, &dwErr);
			if (dwErr != 0) {
				AddMsg(TEXT("    ,  %u"), dwErr);
				AddMsg(TEXT("-> %s"), SystemErrorToString(dwErr).c_str());
			} else {
				AddMsg(TEXT("  ."), szAppName);
				if (bAppIncluded)
					AddMsg(TEXT("  %s    ."), szAppName);
			}
		} else {
			dwErr = WAIT_TIMEOUT;
			// Time out
			AddMsg(TEXT(" :    "));
			if (!TerminateProcess(sShExInf.hProcess, 0)) {
				dwErr = GetLastError();
				AddMsg(TEXT("    ,  %u"), dwErr);
				AddMsg(TEXT("-> %s"), SystemErrorToString(dwErr).c_str());
			}
		}
		CloseHandle(sShExInf.hProcess);
	} else {
		dwErr = GetLastError();
		AddMsg(TEXT("    ,  %u"), dwErr);
		AddMsg(TEXT("-> %s"), SystemErrorToString(dwErr).c_str());
	}

	DeleteFile(sArgFile);
	DeleteFile(sExeFile);
	return dwErr;
}
#endif	// #if UPDATE_VIA_EXTERNAL


/**
 * On a lanc le tlchargement des nouveaux fichiers
 **/
DWORD CUpdateTelechargeThread::ThreadProc()
{
#if UPDATE_VIA_EXTERNAL
	DWORD			nRes = 0;
#else
	bAdminWarnDisplayed = false;
#endif

	cUpdDlg.SetState(us_Updating);

	BYTE	ret			= UpdateTraiteXml();

#if UPDATE_VIA_EXTERNAL
	if (!strArgs.empty()) {
		// Appel du processus externe de finalisation de la mise  jour avec lvation de
		// privilges, si des arguments existent  lui passer
		nRes = CallMoveProcess((ret & UPDATE_APP_FLAG)!=0);
	}
	if (ret & UPDATE_APP_FLAG && nRes == 0) {
#else
	if (ret & UPDATE_APP_FLAG) {
#endif
		bUpdateNeedRestart = true;
		if (enregistrements_actuels.recording())
			AddMsg(TEXT("     .  -."));
		else
			cUpdDlg.SetState(us_NeedRestart);
	} else
		cUpdDlg.SetState(us_NeedCheck);

	return 0;
}

/**
 * Fonction ralisant la connection internet, et les tests pour la vrification de la version,
 * le tout dans un nouveau Thread pour ne pas geler PTVM
 **/
DWORD CUpdateCheckThread::ThreadProc()
{
	InternetFile	vBytVersion;	// Tableau d'octets recevant le fichier XML de mise  jour

	cUpdDlg.SetState(us_Checking);

#ifdef _DEBUG
	{	// Le debugging de modules d'accs Internet pose de gros problmes si Visual Studio a t
		// configur pour mettre  jour sa bibliothque de symboles via le serveur de symboles de
		// Microsoft, au fil du chargement des modules : cette mise  jour se faisant par Internet,
		// un "deadlock" peut en rsulter, provoquant un plantage de Visual Studio.
		// Les symboles ne sont normalement chargs qu'une fois, et rechargs seulement
		// si une mise  jour "Windows Update" a chang le module ".dll" correspondant, donc
		// le problme ne survient qu'au premier chargement, et ensuite trs occasionnellement.
		// Le code qui suit vise  prvenir ce blocage en forant le chargement des modules incrimins
		// (donc provoquant le chargement des symboles) pralablement  tout accs Internet rel.
		// Si d'autres modules provoquent le mme type de blocage (on voit lesquels dans la barre d'tat
		// de Visual Studio au moment du "deadlock"), il suffit de les rajouter dans la table qui suit.
		static LPCTSTR aszModules[] = {
			TEXT("rasapi32.dll"),
			TEXT("credssp.dll"),
			TEXT("schannel.dll"),
			TEXT("sensapi.dll"),
			TEXT("rtutils.dll")
		};

		for (int nInx = 0; nInx < _countof(aszModules); ++nInx) {
			HMODULE hLib = LoadLibrary(aszModules[nInx]);

			if (hLib)
				FreeLibrary(hLib);
		}
	}
#endif

	cUpdDlg.AddProgress();
	GetInternetFile(TEXT(UPDATE_URL), vBytVersion); // Objet traitant de la connexion Internet
#ifdef _DIAGNOSTIC
	if (!vBytVersion.empty()) {
		// Enregistrement du fichier XML reu, en compilation _DIAGNOSTIC seulement
		TempPathName sUpdateFileName("mmzintv_update.xml");

		vBytVersion.SauveFichier(sUpdateFileName);
	}
#endif

	UpdateState eNextState = us_NeedCheck;	// tat suivant par dfaut

	if (! stop_maj && ! vBytVersion.empty()) {
		AddMsg(TEXT("  ..."));
		cUpdDlg.AddProgress();
		// On ajoute un caractre '\0'  la fin (car c'est un fichier texte dont il s'agit)
		vBytVersion.resize(vBytVersion.size()+1, 0);

		// Analyse du fichier rcupr
		cUpdDlg.xmlVersionDocument.Clear();
		cUpdDlg.xmlVersionDocument.Parse(vBytVersion, 0, TIXML_ENCODING_UTF8);
		vBytVersion.clear();

		if (cUpdDlg.xmlVersionDocument.Error()) {
			AddMsg(TEXT("   ."));
			AddMsg(TEXT("        !!!"));
		} else {
			if (UpdateTraiteXml())
				eNextState = us_NeedUpdate;	// mise  jour trouve

			AddMsg(TEXT(" "));
		}
	}

	cUpdDlg.AddProgress(-1);
	cUpdDlg.SetState(eNextState);
	return 0;
}

/**
 * Fonction de gestion des vnements
 **/
INT_PTR CWebUpdateDialog::DialogProc(UINT uMsg, WPARAM wParam, LPARAM lParam)
{


	switch (uMsg) {
	case WM_INITDIALOG:
		hUpdateStatut		= GetDlgItem(hDlg, IDC_UPDATE_LIST);
		hProgressStatut		= GetDlgItem(hDlg, IDC_UPDATE_PROGRESS);
		hUpdateButton		= GetDlgItem(hDlg, IDC_UPDATE_CHECK);
		hUpdateBetaCheckBox = GetDlgItem(hDlg, IDC_UPDATE_BETA);

		SetState(us_NeedCheck);

		AddLineToList(hUpdateStatut, TEXT("     ."));
		AddLineToList(hUpdateStatut, TEXT(""));
#if UPDATE_VIA_EXTERNAL == 0
		AddLineToList(hUpdateStatut, TEXT("##########  ##########"));
		AddLineToList(hUpdateStatut, TEXT("     ,  "));
		AddLineToList(hUpdateStatut, TEXT("   ."));
		AddLineToList(hUpdateStatut, TEXT("-,   Windows Vista  Windows 7"));
		AddLineToList(hUpdateStatut, TEXT("  User Access Control (UAC), MMzinTV  "));
		AddLineToList(hUpdateStatut, TEXT("    ,      "));
		AddLineToList(hUpdateStatut, TEXT("."));
#endif

		return TRUE; // WM_INITDIALOG

	case WM_COMMAND:

		switch (LOWORD(wParam)) {

		case IDC_UPDATE_CHECK: {
			SendMessage(hUpdateStatut, LB_RESETCONTENT, 0, 0);	// Efface les messages dans la liste

			bool bWithBeta = IsDlgButtonChecked(hDlg, IDC_UPDATE_BETA)==BST_CHECKED;

			switch (eCurrentState) {

			case us_NeedCheck:
				// On lance la recherche de mise  jour
				pUpdateThread.reset(new CUpdateCheckThread(*this, bWithBeta));
				break;

			case us_NeedUpdate:
				// On lance le tlchargement des fichiers
				pUpdateThread.reset(new CUpdateTelechargeThread(*this, bWithBeta));
				break;

			case us_NeedRestart:
				EndDialog(hDlg, IDOK);
				// Redmarrage (complet) de l'application
				PostMessage(hMainWnd, WM_COMMAND, IDM_QUIT, em_FullRestart);
			}

			return TRUE; }	// IDC_UPDATE_CHECK

		case IDOK:
		case IDCANCEL:
			if (pUpdateThread.get()) {
				AddLineToList(hUpdateStatut, TEXT("  "));
				// Ordonne au thread de s'arrter
				pUpdateThread->Stop();
			}
			EndDialog(hDlg, LOWORD(wParam));
			return TRUE;
		}
		break; // case WM_COMMAND

	} // switch (uMsg)
	return FALSE;
}

/**
 * Traitement de la mise  jour par Internet
 *
 * \param[in]  hOwner  handle de la fentre parente
 **/
void do_web_update(HWND hOwner)
{
	if (bUpdateNeedRestart) {
		MessageBox(hOwner, TEXT("  ,       "),
			TEXT(""), MB_ICONERROR);
		return;
	}

	CWebUpdateDialog().Do(hOwner);
}
